Package: neotoma2
Title: Working with the Neotoma Paleoecology Database
Date: 2026-04-23
Version: 1.0.12
Authors@R: 
    c(person(given = "Dominguez Vidana",
             family = "Socorro",
             role = c("aut", "cre"),
             email = "dominguezvid@wisc.edu",
             comment = c(ORCID="0000-0002-7926-4935")),
      person(given = "Simon",
             family = "Goring",
             role = c("aut"),
             email = "goring@wisc.edu",
             comment = c(ORCID="0000-0002-2700-4605")))
URL: https://github.com/NeotomaDB/neotoma2
BugReports: https://github.com/NeotomaDB/neotoma2/issues
Description: Access and manipulation of data using the Neotoma Paleoecology Database.
        <https://api.neotomadb.org/api-docs/>.
        Examples in functions that require API access are not executed during CRAN checks.
        Vignettes do not execute as to avoid API calls during CRAN checks.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.3
Suggests: testthat, knitr, pkgdown, rmarkdown, ggplot2, Bchron, covr,
Imports: rlang, magrittr, sf, digest, geojsonsf, purrr, httr, jsonlite,
        methods, dplyr, leaflet, assertthat, stringr, progress, uuid,
        utils, tidyr
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-23 19:21:41 UTC; sedv8808
Author: Dominguez Vidana Socorro [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7926-4935>),
  Simon Goring [aut] (ORCID: <https://orcid.org/0000-0002-2700-4605>)
Maintainer: Dominguez Vidana Socorro <dominguezvid@wisc.edu>
Repository: CRAN
Date/Publication: 2026-04-23 19:50:03 UTC
Built: R 4.5.2; ; 2026-04-24 13:28:47 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(check_contacts,contacts)
S3method(clean,collunits)
S3method(clean,datasets)
S3method(clean,sites)
S3method(filter,NULL)
S3method(filter,data.frame)
S3method(filter,sites)
S3method(get_contacts,default)
S3method(get_contacts,numeric)
S3method(get_datasets,NULL)
S3method(get_datasets,default)
S3method(get_datasets,numeric)
S3method(get_datasets,site)
S3method(get_datasets,sites)
S3method(get_downloads,NULL)
S3method(get_downloads,numeric)
S3method(get_downloads,sites)
S3method(get_publications,default)
S3method(get_publications,numeric)
S3method(get_publications,publication)
S3method(get_publications,publications)
S3method(get_publications,sites)
S3method(get_sites,NULL)
S3method(get_sites,default)
S3method(get_sites,numeric)
S3method(get_speleothems,numeric)
S3method(get_speleothems,sites)
S3method(get_taxa,default)
S3method(get_taxa,numeric)
S3method(get_taxon,default)
S3method(get_taxon,numeric)
S3method(getids,collunit)
S3method(getids,collunits)
S3method(getids,sites)
S3method(neotoma2::getids,site)
export("%>%")
export(check_contacts)
export(chroncontrols)
export(chronologies)
export(cite_data)
export(clean)
export(collunits)
export(coordinates)
export(datasets)
export(doi)
export(filter)
export(get_contacts)
export(get_datasets)
export(get_documentation)
export(get_downloads)
export(get_manual)
export(get_publications)
export(get_sites)
export(get_speleothems)
export(get_stats)
export(get_table)
export(get_taxa)
export(get_taxon)
export(getids)
export(getids.sites)
export(pingNeotoma)
export(plotLeaflet)
export(samples)
export(set_chronology)
export(set_collunit)
export(set_contact)
export(set_dataset)
export(set_default)
export(set_publication)
export(set_sample)
export(set_server)
export(set_site)
export(set_speleothem)
export(speleothemdetails)
export(speleothems)
export(taxa)
export(toWide)
exportClasses(author)
exportClasses(authors)
exportClasses(chronologies)
exportClasses(chronology)
exportClasses(collunit)
exportClasses(collunits)
exportClasses(contact)
exportClasses(contacts)
exportClasses(dataset)
exportClasses(datasets)
exportClasses(publication)
exportClasses(publications)
exportClasses(repositories)
exportClasses(repository)
exportClasses(sample)
exportClasses(samples)
exportClasses(site)
exportClasses(sites)
exportClasses(specimen)
exportClasses(specimens)
exportClasses(speleothem)
exportClasses(speleothems)
exportClasses(taxa)
exportClasses(taxon)
exportMethods("$")
exportMethods("$<-")
exportMethods("[")
exportMethods("[<-")
exportMethods("[[")
exportMethods("[[<-")
exportMethods(add_chronology)
exportMethods(as.data.frame)
exportMethods(c)
exportMethods(chroncontrols)
exportMethods(chronologies)
exportMethods(cite_data)
exportMethods(collunits)
exportMethods(coordinates)
exportMethods(datasets)
exportMethods(doi)
exportMethods(length)
exportMethods(names)
exportMethods(plot)
exportMethods(plotLeaflet)
exportMethods(samples)
exportMethods(set_default)
exportMethods(speleothemdetails)
exportMethods(speleothems)
exportMethods(summary)
exportMethods(taxa)
importFrom(assertthat,assert_that)
importFrom(digest,digest)
importFrom(dplyr,all_of)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,coalesce)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,full_join)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_at)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,rename)
importFrom(dplyr,row_number)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(geojsonsf,geojson_sf)
importFrom(geojsonsf,sf_geojson)
importFrom(geojsonsf,sfc_geojson)
importFrom(httr,GET)
importFrom(httr,HEAD)
importFrom(httr,POST)
importFrom(httr,add_headers)
importFrom(httr,content)
importFrom(httr,http_error)
importFrom(httr,modify_url)
importFrom(httr,stop_for_status)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(leaflet,addCircleMarkers)
importFrom(leaflet,addTiles)
importFrom(leaflet,leaflet)
importFrom(leaflet,markerClusterOptions)
importFrom(leaflet,markerOptions)
importFrom(magrittr,"%>%")
importFrom(methods,"slot<-")
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(progress,progress_bar)
importFrom(purrr,flatten)
importFrom(purrr,imap)
importFrom(purrr,map)
importFrom(purrr,map_df)
importFrom(purrr,map_lgl)
importFrom(purrr,pluck)
importFrom(purrr,reduce)
importFrom(rlang,.data)
importFrom(rlang,is_interactive)
importFrom(sf,st_as_sf)
importFrom(sf,st_as_sfc)
importFrom(sf,st_bbox)
importFrom(sf,st_coordinates)
importFrom(sf,st_crs)
importFrom(sf,st_read)
importFrom(sf,st_sfc)
importFrom(stats,na.omit)
importFrom(stringr,str_detect)
importFrom(stringr,str_extract_all)
importFrom(stringr,str_match)
importFrom(stringr,str_match_all)
importFrom(tidyr,pivot_wider)
importFrom(utils,browseURL)
importFrom(uuid,UUIDgenerate)
importMethodsFrom(methods,show)

<!-- badges: start -->
[![lifecycle](https://img.shields.io/badge/lifecycle-experimental-orange.svg)](https://lifecycle.r-lib.org/articles/stages.html#experimental)
[![NSF-1948926](https://img.shields.io/badge/NSF-1948926-blue.svg)](https://www.nsf.gov/awardsearch/show-award/?AWD_ID=1948926)
[![Codecov test coverage](https://codecov.io/gh/NeotomaDB/neotoma2/branch/production/graph/badge.svg)](https://app.codecov.io/gh/NeotomaDB/neotoma2?branch=production)
[![DOI](https://zenodo.org/badge/265675258.svg)](https://zenodo.org/badge/latestdoi/265675258) 
[![status](https://joss.theoj.org/papers/80636d1e6d32175f13edb7e7ed14089a/status.svg)](https://joss.theoj.org/papers/10.21105/joss.05561)
<!-- badges: end -->

# `neotoma2` R Package

The `neotoma2` R package represents a set of breaking changes with the original `neotoma` R package. The `neotoma` package was deprecated following end-of-life for the Neotoma Windows Server in 2020 and the migration of the Neotoma backend infrastructure to a PostgreSQL database and JavaScript API.

The `neotoma2` package is built on the new [Neotoma API](https://api.neotomadb.org/api-docs) and is intended as a starting point for a fully interactive experience with the [Neotoma Paleoecology Database](https://www.neotomadb.org), to support both data access and data input through R.

## Contributors

This project is an open project, and contributions are welcome from any individual.  All contributors to this project are bound by a [code of conduct](https://github.com/NeotomaDB/neotoma2/blob/production/CODE_OF_CONDUCT.md).  Please review and follow this code of conduct as part of your contribution.

* [![ORCID](https://img.shields.io/badge/orcid-0000--0002--7926--4935-brightgreen.svg)](https://orcid.org/0000-0002-7926-4935) [Socorro Dominguez Vidana](https://ht-data.com/)

* [![ORCID](https://img.shields.io/badge/orcid-0000--0002--2700--4605-brightgreen.svg)](https://orcid.org/0000-0002-2700-4605) [Simon Goring](http://www.goring.org)

### Tips for Contributing

Issues and bug reports are always welcome.  Code clean-up, and feature additions can be done either through pull requests to [project forks](https://github.com/NeotomaDB/neotoma2/network/members) or [project branches](https://github.com/NeotomaDB/neotoma2/branches).

Please direct development questions to Simon Goring by email: [goring@wisc.edu](mailto:goring@wisc.edu).

All products of the Neotoma Paleoecology Database are licensed under an [MIT License](LICENSE) unless otherwise noted.

## How to use this repository

All R functions for the package should be written in the `R` folder.  Any documentation should be added to `.R` files using [`roxygen2`](https://cran.r-project.org/package=roxygen2) notation.  Because we are using `roxygen2` for documentation in this package, all edits to documentation should take place in the associated functions `.R` file. The files in the `man` folder should not be manually changed.

Class definitions and new methods should be added to the files `01_classDefinitions.R` and `02_genericDefinitions.R` respectively, to ensure they are properly loaded during the package build process.

### Development Workflow Overview

The `neotoma2` package is built for R.  Build tools include elements from the `usethis`, `devtools` and `testthat` R packages, and build and compilation occurs within (and outside) the RStudio IDE environment.

Package use requires the use of the `devtools::install_github()` function, to pull this working repository into a user's environment:

```r
devtools::install_github('NeotomaDB/neotoma2', build_vignettes = TRUE)
```

The expectation for this repository is that all commits to the `prod` branch will support a clean package build.  This is supported through [GitHub Actions](https://github.com/NeotomaDB/neotoma2/actions) in the `.github` folder of the repository.

### Analysis Workflow Overview

There is considerable information in the vignettes for the package, which can be [accessed directly](https://github.com/NeotomaDB/neotoma2/tree/production/vignettes).

### Report of Sites Statistics

In order to see the total available sites that can be queried by this package, use the function:

```
neotoma2::get_stats(start=0, end=1)
```

### System Requirements

This project is built with R > v4.0.  The packages needed for proper package use are detailed in the `DESCRIPTION` file for this repository.

### Data Requirements

The `neotoma2` R package pulls data from the [Neotoma Paleoecology Database](https://www.neotomadb.org).  Neotoma maintains a [permissive data use policy](https://www.neotomadb.org/data/data-use-and-embargo-policy).  Within the data use policy there is a statement on co-authorship which reads:

> Normal ethics apply to co-authorship of scientific publications. Paleoecological datasets are labor-intensive and complex: they take years to generate and always have additional attributes and metadata not captured in Neotoma. Neotoma data quality also relies on expert curation by data stewards, each associated with one or more Constituent Databases. **Users of data stored in Neotoma’s Constituent Databases should consider inviting the original data contributor, or Constituent Database steward(s), to be a co-author(s) of any resultant publications if that contributor’s data are a major portion of the dataset analyzed, or if a data contributor or steward makes a significant contribution to the analysis of the data or to the interpretation of results.** For large-scale studies using many Neotoma records, contacting all contributors or stewards or making them co-authors will not be practical, possible, or reasonable. Under no circumstance should authorship be attributed to data contributors or stewards, individually or collectively, without their explicit consent.

## Metrics

This project is to be evaluated using the following metrics:

* CRAN release 1.0.8 with speleothem data support and improved documentation.**DONE** November 18, 2025
* Maintenance Github 1.0.7 release **DONE** June 15, 2025
* Maintenance Github 1.0.6 release **DONE** June 23, 2025
* Maintenance Github 1.0.5 release, **DONE** December 27, 2024
* Maintenance Github 1.0.4 release, **DONE** July 23, 2024
* Updated production Branch with CRAN changes, May 24, 2024.
* Maintenance Github 1.0.3 release, **DONE** February 28, 2024
* Published JOSS paper, **DONE** 28 November, 2023
* Submitted paper to JOSS **DONE** 03 May, 2023
* Completion CRAN 1.0.0 release **DONE** April 23, 2023
* Completion of core functionality for data access **DONE** Feb 10, 2022
* Completion of core functionality for data presentation **DONE**
* Completion of clear vignettes for major data types or Constituent Databases represented within the Neotoma Database.

## ----setup, include=FALSE-----------------------------------------------------
library(sf)
library(geojsonsf)
library(dplyr)
library(neotoma2)

## ----setOpts, include = FALSE-------------------------------------------------
knitr::opts_chunk$set(eval = FALSE,
                      error = TRUE)

## ----setup_md, include=FALSE--------------------------------------------------
# safe_eval <- function(expr, fallback = "N/A") {
#   tryCatch(eval(expr, envir = .GlobalEnv), error = function(e) fallback)
# }

## ----getSiteBySiteID----------------------------------------------------------
# # Search for site by a single numeric ID:
# alex <- get_sites(24)
# alex
# 
# # Search for sites with multiple IDs using c():
# multiple_sites <- get_sites(c(24, 47))
# multiple_sites

## ----getsitename--------------------------------------------------------------
# alex <- get_sites(sitename = "Alexander Lake")
# alex

## ----sitewithwildcardname-----------------------------------------------------
# alex <- get_sites(sitename = 'Alex%')
# alex

## ----agebounds----------------------------------------------------------------
# # Note, we are using the `all_data = TRUE` flag here to avoid the default limit of 25 records, discussed below.
# # Because these queries are searching through every record they are slow and and are not
# # run in knitting this vignette.
# get_sites(ageof = 8200, all_data = TRUE) %>% length()
# get_sites(ageyounger = 5000, ageolder = 8000, all_data = TRUE) %>% length()
# get_sites(minage = 5000, maxage = 8000, all_data = TRUE) %>% length()

## ----extractElement-----------------------------------------------------------
# alex <- get_sites(sitename = "Alexander Lake")
# alex[[1]]$siteid

## ----showallNamesSite---------------------------------------------------------
# names(alex[[1]])
# 
# # Modify a value using $<- assignment:
# alex[[1]]$area
# alex[[1]]$area <- 100
# alex[[1]]$area
# 
# # Modify a value using [<- assignment:
# alex[[1]]["area"] <- 30
# alex[[1]]$area
# # alex[[1]][7] <- 30  This fails because the `Notes` field expects a character string.

## ----setsitefunction----------------------------------------------------------
# my_site <- set_site(sitename = "My Lake",
#                     geography = st_sf(a = 3, st_sfc(st_point(1:2))),
#                     description = "my lake",
#                     altitude = 30)
# my_site

## ----addtosites---------------------------------------------------------------
# # Add a new site that's been edited using set_site()
# longer_alex <- c(alex, my_site)
# # Or replace an element within the existing list of sites
# # with the newly created site.
# longer_alex[[2]] <- my_site
# 
# # Or append to the `sites` list with assignment:
# longer_alex[[3]] <- my_site

## ----getdatasetsbyid----------------------------------------------------------
# # Getting datasets by ID
# my_datasets <- get_datasets(c(5, 10, 15, 20))
# my_datasets

## ----getdatasetsbytype--------------------------------------------------------
# # Getting datasets by type
# my_pollen_datasets <- get_datasets(datasettype = "pollen", limit = 25)
# my_pollen_datasets

## ----all_data-----------------------------------------------------------------
# allSites_dt <- get_sites(datasettype = "diatom")
# allSites_dt_all <- get_sites(datasettype = "diatom", all_data = TRUE)
# 
# # Because we used the `all_data = TRUE` flag, there will be more sites
# # in allSites_dt_all, because it represents all sites containing diatom datasets.
# length(allSites_dt_all) > length(allSites_dt)

## ----boundingBox--------------------------------------------------------------
# brazil <- '{"type": "Polygon",
#             "coordinates": [[
#                 [-73.125, -9.102],
#                 [-56.953, -33.138],
#                 [-36.563, -7.711],
#                 [-68.203, 13.923],
#                 [-73.125, -9.102]
#               ]]}'
# 
# # We can make the geojson a spatial object if we want to use the
# # functionality of the `sf` package.
# brazil_sf <- geojsonsf::geojson_sf(brazil)
# 
# 
# brazil_datasets <- get_datasets(loc = brazil_sf)

## ----leafletBrazil------------------------------------------------------------
# plotLeaflet(brazil_datasets)

## ----filterBrazil-------------------------------------------------------------
# 
# brazil_dates <- neotoma2::filter(brazil_datasets,
#                                    datasettype == "geochronologic")
# # or:
# brazil_dates <- brazil_datasets %>%
#     neotoma2::filter(datasettype == "geochronologic")
# 
# # With boolean operators:
# brazil_space <- brazil_datasets %>% neotoma2::filter(lat > -18 & lat < -16)

## ----filterAndShowTaxa--------------------------------------------------------
# brazil <- '{"type": "Polygon",
#             "coordinates": [[
#                 [-73.125, -9.102],
#                 [-56.953, -33.138],
#                 [-36.563, -7.711],
#                 [-68.203, 13.923],
#                 [-73.125, -9.102]
#               ]]}'
# 
# # We can make the geojson a spatial object if we want to use the
# # functionality of the `sf` package.
# brazil_sf <- geojsonsf::geojson_sf(brazil)
# 
# brazil_records <- get_datasets(loc = brazil_sf, all_data=TRUE) %>%
#     neotoma2::filter(datasettype == "pollen" & age_range_young <= 1000 & age_range_old >= 10000) %>%
#     get_downloads()
# 
# 
# count_by_site <- samples(brazil_records) %>%
#   dplyr::filter(elementtype == "pollen" & units == "NISP") %>%
#   group_by(siteid, variablename) %>%
#   summarise(n = n()) %>%
#   group_by(variablename) %>%
#   summarise(n = n()) %>%
#   arrange(desc(n))
# 

## ----pubsbyid-----------------------------------------------------------------
# one <- get_publications(12)
# two <- get_publications(c(12, 14))

## ----showSinglePub------------------------------------------------------------
# two[[2]]

## ----fulltestPubSearch--------------------------------------------------------
# michPubs <- get_publications(search = "Michigan", limit = 2)

## ----nonsenseSearch-----------------------------------------------------------
# noise <- get_publications(search = "Canada Banada Nanada", limit = 5)

## ----getSecondPub-------------------------------------------------------------
# two[[1]]

## ----subsetPubs---------------------------------------------------------------
# # Select publications with Neotoma Publication IDs 1 - 10.
# pubArray <- get_publications(1:10)
# # Select the first five publications:
# subPub <- pubArray[[1:5]]
# subPub

## ----setNewPub----------------------------------------------------------------
# new_pub <- set_publications(
# articletitle = "Myrtle Lake: a late- and post-glacial pollen diagram from northern Minnesota",
# journal = "Canadian Journal of Botany",
# volume = 46)

## ----setPubValue--------------------------------------------------------------
# new_pub@pages <- "1397-1410"

#' @title get_sites
#' @name get_sites
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @param x Use a single integer or vector of integers representing 
#' unique Neotoma site identifiers (siteids) to extract site information.
#' @param ... accepted arguments, see details for more information.
#' @description
#' The `get_sites` function is a wrapper for the Neotoma `sites` API
#' endpoint.
#' The function takes parameters defined by the user and returns a list
#' of site information supplied by the Neotoma Paleoecological Database.
#' The user may define all or none of the possible fields.
#' @details
#' A `site` object in Neotoma is a physical location at which one or more
#' collection units are located. Each collection unit may have one or more
#' datasets within it, defined by the dataset type. The `get_sites()`
#' function searches for each site within Neotoma that matches the query
#' parameters, and returns them as a `sites` object, a list of `site`
#' objects. The `get_sites()` command wraps the Neotoma API
#' ([api.neotomadb.org](https://api.neotomadb.org)) call for `sites`.
#' The call itself uses a SQL query which accepts any one of the following
#' parameters:
#'  * `siteid`  The unique site ID (integer) in Neotoma. Can be passed as a
#' vector of site IDs.
#'  * `sitename`  The site name, or approximate match using the % wildcard.
#'  * `database`  The constituent database for the record.
#' See `get_table("constituentdatabases")`
#'  * `altmin`  The minimum altitude range for site elevation (in meters).
#'  * `altmax`  The maximum altitude range for site elevation (in meters).
#'  * `datasetid`  The unique dataset ID (integer) in Neotoma. Can be passed
#' as a vector of dataset IDs.
#'  * `datasettype`  Neotoma contains data for a number of datasettypes.
#'  This returns a subset of data types. For a complete list of available 
#'  datasettypes, run `neotoma2::get_table('datasettypes')`
#'  * `doi`  The dataset DOI for a dataset contained within a site. Can be
#' passed as a vector of DOIs.
#'  * `gpid`  The geopolitical name or identifier containing a site. Can be
#' passed as a vector of names.
#'  * `keywords`  Keywords for samples within a set of sites. For example
#' "modern" indicates a sample within the record uses the keyword "modern".
#'  * `contacts`  Contact names or IDs associated with a site.
#'  * `ageyoung`  A minimum spanning age for the record, in years before
#' radiocarbon present (1950).
#'  * `ageold`  A maximum spanning age for the record, in years before
#' radiocarbon present (1950).
#'  * `ageof`  An age which must be contained within the range of sample ages
#' for a site.
#'  * `taxa`  The names of taxa which must be present within samples in a
#' record.
#'  * `all_data`  The API only downloads the first 25 records of the query. 
#'  For the complete records, use `all_data=TRUE`
#' This call will then return a data object that contains site metadata for one
#' or more sites, along with limited metadata describing the collection units
#' and datasets located at that site.
#' @returns The function returns either a single item of class `"try-error"`
#' describing the reason for failure (either misdefined parameters or an error
#' from the Neotoma API), or a table of sites, with rows corresponding to the
#' number of individual sites returned by the Neotoma API.
#' Each "site" object contains 6 parameters that can be accessed as well:
#' siteid, sitename, location, altitude, description,
#' limited collection units information.
#'  * `loc` An `sf` object that describes site's location.
#'  * `description`
#'  * `collunits` limited information on collunits
#' @examples \dontrun{
#' ## Find sites with a min altitude of 12m and a max altitude of 25m
#' sites_12to25 <- get_sites(altmin=12, altmax=25)
#' sites_2500 <- get_sites(altmin=2500, all_data = TRUE)
#' ## To find sites in Brazil
#' brazil <- '{"type": "Polygon",
#' "coordinates": [[
#'  [-73.125, -9.102096738726443],
#'  [-56.953125,-33.137551192346145],
#'  [-36.5625,-7.710991655433217],
#'  [-68.203125,13.923403897723347],
#'  [-73.125,-9.102096738726443]]]}'
#' brazil_sites <- get_sites(loc = brazil[1])
#' }
#' @md
#' @export
get_sites <- function(x = NA, ...) {
  if (missing(x)) {
    UseMethod("get_sites", "default")
  } else {
    UseMethod("get_sites", x)
  }
}

#' @rdname get_sites
#' @method get_sites numeric
#' @exportS3Method get_sites numeric
get_sites.numeric <- function(x, ...) {
  if (length(x) > 0) {
    siteids <- paste0(x, collapse = ",")
  }
  baseURL <- paste0("data/sites/", siteids)
  result <- tryCatch(
    parseURL(baseURL, ...),
    error = function(e) {
      stop("API call failed: ", e$message)
      NULL
    }
  )
  if (!is.null(result) && length(result[2]$data) > 0) {
    output <- parse_site(result)
    return(output)
  } else {
    return(NULL)
  }
}

#' @rdname get_sites
#' @method get_sites default
#' @exportS3Method get_sites default
get_sites.default <- function(...) {
  cl <- as.list(match.call())
  cl[[1]] <- NULL
  cl <- lapply(cl, eval, envir = parent.frame())
  if ("siteid" %in% names(cl)) {
    # redirect to numeric method
    if ("all_data" %in% names(cl)) {
      all_data <- cl$all_data
    } else {
      all_data <- FALSE
    }
    output <- get_sites(x = cl$siteid, all_data = all_data)
    return(output)
  }
  params <- get_params("sites")
  if (!all(names(cl) %in% params)) {
    warning("Some parameters seem invalid.
             The current accepted parameters are: ",
            paste(unlist(params), collapse = ", "))
  }
  oo <- options(scipen = 9999)
  on.exit(options(oo))
  baseURL <- paste0("data/sites")
  result <- tryCatch(
    parseURL(baseURL, ...),
    error = function(e) {
      stop("API call failed: ", e$message)
      NULL
    }
  )
  if (is.null(result) || is.null(result$data[1][[1]])) {
    return(NULL)
  } else {
    result <- result %>%
      cleanNULL()
    output <- parse_site(result)
    return(output)
  }
}

#' @rdname get_sites
#' @exportS3Method get_sites NULL
get_sites.NULL <- function(x, ...) {
  message("Input is NULL, returning NULL")
  return(NULL)
}

library("testthat")
library("neotoma2")

test_that("collunits()`", {
  skip_on_cran()
  dls <- get_sites() %>%
    get_downloads()
  sumDl <- summary(dls)
  ids <- getids(dls)
  cus <- collunits(dls)
  testthat::expect_identical(length(unique(cus$collectionunitid)),
                             length(unique(ids$collunitid)))
  testthat::expect_true(all(sumDl$collunit_name %in% cus$handle))
})

library("testthat")
library("neotoma2")

test_that("`datasets()`", {
  skip_on_cran()
  dls <- get_sites() %>%
    get_downloads()
  sumDl <- summary(dls)
  ids <- getids(dls)
  dss <- datasets(dls)
  cus <- collunits(dls)
  testthat::expect_identical(sum(sumDl$n_datasets), nrow(ids))
  testthat::expect_identical(length(unique(sumDl$siteid)), length(dls))
  testthat::expect_true(all(sumDl$collunit_name %in% cus$handle))
  testthat::expect_true(all(cus$handle %in% sumDl$collunit_name))
})

library("testthat")
library("neotoma2")

context("Test that filter receives a sites object
        and filters using dplyr's syntax")
test_that("filter datasettype", {
  skip_on_cran()
  sts <- get_sites()
  pollen_sts <-
    sts %>%
    neotoma2::filter(datasettype == "pollen")
  sum <- summary(pollen_sts)
  pollen_types <- unique(sum$dataset_types)
  testthat::expect_equal(pollen_types, "pollen")
})

test_that("filter datasettype + loc", {
  skip_on_cran()
  brazil <- '{"type": "Polygon",
            "coordinates": [[
                [-73.125, -9.102],
                [-56.953, -33.138],
                [-36.563, -7.711],
                [-68.203, 13.923],
                [-73.125, -9.102]
              ]]}'
  brazil_sf <- geojsonsf::geojson_sf(brazil)
  brazil_datasets <- get_datasets(loc = brazil[1], limit=10)
  brazil_pollen <- neotoma2::filter(brazil_datasets, datasettype == "pollen")
  brazil_summary <- summary(brazil_pollen)
  brazil_datatype <- unique(brazil_summary$dataset_types)
  testthat::expect_equal(brazil_datatype, "pollen")
})

test_that("filter lat & long.", {
  skip_on_cran()
  sts <- get_sites()
  lat_sts <-
    sts %>%
    filter(lat >= 50 & lat <= 90)
  latitudes <- as.data.frame(lat_sts)$lat
  testthat::expect_true(all(latitudes <= 90))
  testthat::expect_true(all(latitudes >= 50))
  testthat::expect_lt(length(lat_sts), length(sts))
  testthat::expect_true(all(getids(lat_sts)$siteid %in% getids(sts)$siteid))
  testthat::expect_true(all(getids(lat_sts)$datasetid %in% getids(sts)$datasetid))
  testthat::expect_true(all(getids(lat_sts)$collunitid %in% getids(sts)$collunitid))
})

test_that("filter get_datasets('datasettype') by datasettype
          does not change sites", {
            skip_on_cran()
            ds <- get_datasets(datasettype = "pollen")
            pollen_ds <- neotoma2::filter(ds, datasettype == "pollen")
            testthat::expect_equal(length(ds), length(pollen_ds))
            # test ids in datastid for ds and pollen ds are the same
            testthat::expect_setequal(getids(ds)$datasetid,
                                      getids(pollen_ds)$datasetid)
          })

test_that("filter by 2 datasettypes", {
  skip_on_cran()
  sts <- get_sites()
  filtered <- sts %>%
    neotoma2::filter(datasettype == "pollen" | datasettype == "charcoal")
  dst <- datasets(filtered) %>%
    as.data.frame() %>%
    select(datasettype) %>%
    unlist()
  testthat::expect_true(all(dst %in% c("pollen", "charcoal")))
})

test_that("filter datasets returns a clean object", {
  skip_on_cran()
  sts <- get_sites() %>%
    neotoma2::filter(datasettype == "pollen" | datasettype == "charcoal")
  ids <- getids(sts) %>%
    select(siteid) %>%
    unique() %>%
    nrow()
  testthat::expect_equal(ids, length(sts))
  testthat::expect_lte(length(sts), nrow(summary(sts)))
})

test_that("filter by collunitid removes samples", {
  skip_on_cran()
  meer <- get_sites(sitename = "meerfeld%") %>%
    get_downloads()
  meerDS42510 <- meer %>%
    neotoma2::filter(collunitid == 29576) %>%
    samples()
  testthat::expect_true(all(meerDS42510$collectionunitid == 29576))
})

test_that("filter by datasetid keeps all other collection units
           where the dataset does not belong", {
             skip_on_cran()
             meer <- get_sites(sitename = "meerfeld%") %>%
               get_downloads()
             meerDS42510 <- meer %>%
               neotoma2::filter(datasetid == 42510)
             meerId <-  as.data.frame(collunits(meerDS42510))$collectionunitid
             testthat::expect_equal(meerId, 29576)
           })

test_that("filter works before/after get_downloads", {
  skip_on_cran()
  core_sites <- c(13949, 11904, 13319, 728, 
                  13248, 2625, 2806,
                  13280, 519, 11745, 273, 13956,
                  11880, 13321, 9801, 13698, 11816,
                  13909, 13921)
  core_wc_dl_f <- get_sites(core_sites) %>%
    get_datasets() %>%
    get_downloads() %>%
    neotoma2::filter(datasettype == "water chemistry")
  
  core_wc_f_dl <- get_sites(core_sites) %>%
    get_datasets() %>%
    neotoma2::filter(datasettype == "water chemistry") %>%
    get_downloads()
  core_dl_f_ids <- getids(core_wc_dl_f)
  core_f_dl_ids <- getids(core_wc_f_dl)
  testthat::expect_setequal(core_dl_f_ids$siteid, core_f_dl_ids$siteid)
  testthat::expect_setequal(core_dl_f_ids$collunitid, core_f_dl_ids$collunitid)
  testthat::expect_setequal(core_dl_f_ids$datasetid, core_f_dl_ids$datasetid)
})

library("testthat")
library("neotoma2")

context("Creating sites from scratch or updating existing site objects.")
test_that("`set_sites()` for new site.", {
  skip_on_cran()
  newSite <- set_site(sitename = "My New Site",
                      geography = sf::st_as_sf(
                        sf::st_sfc(
                          sf::st_point(c(-100.0, 40.0)),
                          crs = 4326)),
                      altitude = 250)
  testthat::expect_is(newSite, "site")
  testthat::expect_equal(newSite@sitename, "My New Site")
  testthat::expect_equal(newSite@altitude, 250)
  testthat::expect_equal(sf::st_coordinates(newSite@geography)[1], -100.0)
  testthat::expect_equal(sf::st_coordinates(newSite@geography)[2], 40.0)
})

test_that("Updating existing site.", {
  skip_on_cran()
  # Invalid input:
  testthat::expect_error(set_site(x = 12, altitude = 33))
  # Cannot update over a sites object:
  downloadSite <- get_sites(limit = 1)
  testthat::expect_error(set_site(x = downloadSite, altitude = 33))
  # Update single site
  updatedSample <- set_site(x = downloadSite[[1]], altitude = -9999)
  testthat::expect_equal(updatedSample@altitude, -9999)
  # Cannot use invalid data types
  testthat::expect_error(set_site(x = downloadSite[[2]], altitude = "33a"))
})

testthat::skip("Skipping all tests in this file")
test_that("Manual loading fails if interactive is false.", {
  skip_on_cran()
  get_specimens(datasetid = c(19832, 41610))
})

# Get specimens by specimenID
test_that("Manual loading fails if interactive is false.", {
  skip_on_cran()
  get_specimens(c(7, 8))
})

test_that("Manual loading fails if interactive is false.", {
  skip_on_cran()
  my_sites <- get_sites(c(13296, 5663, 24))
  my_specimens <- get_specimens(my_sites)
})

test_that("Building a specimen works.", {
  skip_on_cran()
  dataset <- data.frame(datasetid = 12,
    repo = c("Manhattan", "Banana"),
    taxonid = seq(1, 12))
  # Build error when we pass multiple rows.
  new_spec <- testthat::expect_error(build_specimen(dataset))
  new_spec <- testthat::expect_is(build_specimen(dataset[1, ]), "specimen")
})

library("testthat")
library("neotoma2")

context("Creating datasets from scratch or updating existing objects.")
test_that("`set_dataset()` for a new dataset", {
  skip_on_cran()
  newDS <- set_dataset(datasetid = 56,
                       datasetname = "My Dataset",
                       database = "neotoma",
                       recdatecreated = as.Date("2025-11-10"),
                       datasettype = "pollen")
  testthat::expect_is(newDS, "dataset")
  testthat::expect_equal(newDS@datasetname, "My Dataset")
  testthat::expect_equal(newDS@datasettype, "pollen")
  testthat::expect_equal(newDS@datasetid, 56)
  testthat::expect_equal(newDS@database, "neotoma")
  testthat::expect_equal(newDS@recdatecreated, as.Date("2025-11-10"))
  testthat::expect_error(set_dataset(x = 12, waterdepth = 33))
})

test_that("Updating existing dataset", {
  skip_on_cran()
  # Cannot update over a sites object:
  downloadSite <- get_sites(limit = 1)
  testthat::expect_error(set_dataset(x = downloadSite, datasetid = -9999))
  # Update single site
  updatedSample <- set_dataset(x = downloadSite[[1]]@collunits[[1]]@datasets[[1]], datasetid = -9999)
  testthat::expect_equal(updatedSample@datasetid, -9999)
  # Cannot use invalid data types
  testthat::expect_error(set_dataset(x = downloadSite[[1]]@collunits[[1]]@datasets[[1]], waterdepth = "33a"))
})

library("testthat")
library("neotoma2")

context("`coordinates()` function")
testthat::test_that("coordinates() returns correct coordinates", {
  skip_on_cran()
  sts <- get_sites(c(101, 102))
  sts_df <- as.data.frame(sts)
  coords <- coordinates(sts)
  testthat::expect_is(coords, "data.frame")
  testthat::expect_equal(nrow(coords), 2)
  testthat::expect_equal(ncol(coords), 3)
  testthat::expect_equal(coords$siteid, sts_df$siteid)
  testthat::expect_equal(coords$latitude, sts_df$latitude, tolerance = 0.01)
  testthat::expect_equal(coords$longitude, sts_df$longitude, tolerance = 0.01)
})

library("testthat")
library("neotoma2")

context("Creating collection units from scratch or updating existing objects.")
test_that("`set_collunit()` for new collection unit", {
  skip_on_cran()
  newCU <- set_collunit(notes = "CU set up",
                          handle = "My CU",
                          colldate = as.Date("2025-11-14"),
                          gpslocation = sf::st_as_sf(
                            sf::st_sfc(
                              sf::st_point(c(-100.0, 40.0)),
                              crs = 4326)),
                          collunittype = "magical")
                          datasets = new("datasets", datasets = list(new("dataset")))
  testthat::expect_is(newCU, "collunit")
  testthat::expect_equal(newCU@handle, "My CU")
  testthat::expect_equal(newCU@collunittype, "magical")
  testthat::expect_equal(sf::st_coordinates(newCU@gpslocation)[1], -100.0)
  testthat::expect_equal(sf::st_coordinates(newCU@gpslocation)[2], 40.0)
})

test_that("Updating existing collection unit", {
  skip_on_cran()
  # Invalid input:
  testthat::expect_error(set_collunit(x = 12, waterdepth = 33))
  # Cannot update over a sites object:
  downloadSite <- get_sites(limit = 1)
  testthat::expect_error(set_collunit(x = downloadSite, waterdepth = 33))
  # Update single site
  updatedSample <- set_collunit(x = downloadSite[[1]]@collunits[[1]], waterdepth = -9999)
  testthat::expect_equal(updatedSample@waterdepth, -9999)
  # Cannot use invalid data types
  testthat::expect_error(set_collunit(x = downloadSite[[1]]@collunits[[1]], waterdepth = "33a"))
})

testthat::skip("Skipping all tests in this file")
library("testthat")
library("neotoma2")

context("Run Neotoma `test_speleothems` only when not on CRAN")
test_that("get_speleothems runs with numeric datasetid", {
  skip_on_cran()
  kesang <- get_speleothems(68552)
  testthat::expect_is(kesang, "sites")
  testthat::expect_true(length(kesang) > 0)
})

test_that("get_speleothems works with vector of datasetids", {
  skip_on_cran()
  spel_multi <- get_speleothems(c(68552, 68512, 67718, 69557))
  testthat::expect_is(spel_multi, "sites")
  testthat::expect_true(length(spel_multi) >= 1)
})

test_that("get_speleothems works with sites object", {
  skip_on_cran()
  test_sites <- get_sites(37302)
  spel_sites <- get_speleothems(test_sites)
  testthat::expect_is(spel_sites, "sites")
})


# test_that("get_speleothems handles invalid datasetid", {
#   skip_on_cran()
#   skip_if(TRUE, "Speleothem functionality needs verification")
#   
#   # Test with non-existent datasetid
#   spel_invalid <- get_speleothems(999999999)
#   
#   # Should return NULL or empty result
#   testthat::expect_true(is.null(spel_invalid) || length(spel_invalid) == 0)
# })

test_that("get_speleothems returns correct structure", {
  skip_on_cran()
  spel_data <- get_speleothems(68552)
  if (!is.null(spel_data) && length(spel_data) > 0) {
    summary <- summary(spel_data)
    testthat::expect_true("speleothem" %in% summary$types)
  }
})

test_that("get_speleothems handles empty results", {
  skip_on_cran()
  no_spel <- get_speleothems(1)
  testthat::expect_true(is.null(no_spel) || length(no_spel) >= 0)
  testthat::expect_is(no_spel, "sites")
})

test_that("if API has error, get_speleothems also breaks", {
  skip_on_cran()
  testthat::expect_error(get_speleothems("invalid"))
})

test_that("get_speleothems integrates with summary", {
  skip_on_cran()
  spel_data <- get_speleothems(68552)
  if (!is.null(spel_data) && length(spel_data) > 0) {
    sum_data <- summary(spel_data)
    testthat::expect_is(sum_data, "data.frame")
    testthat::expect_true(nrow(sum_data) > 0)
  }
})

testthat::skip("Skipping all tests in this file")
library("testthat")
library("neotoma2")

context("Run Neotoma `test_speleothems` only when not on CRAN")
test_that("speleothems method extracts data correctly", {
  skip_on_cran()
  spel_data<- get_datasets(datasettype='speleothem', limit=2)
  spel_data <- get_speleothems(spel_data)
  if (!is.null(spel_data) && length(spel_data) > 0) {
    spel_extracted <- speleothems(spel_data)
    testthat::expect_is(spel_extracted, "data.frame")
    testthat::expect_equal(nrow(spel_extracted), 2)
  }
})

test_that("no duplicate rows in speleothems data frame", {
  skip_on_cran()
  spel_data<- get_datasets(datasettype='speleothem', limit=10)
  spel_data <- speleothems(get_speleothems(spel_data))
  testthat::expect_equal(nrow(spel_data), nrow(unique(spel_data)))
})

# Tests for speleothemdetails() function
test_that("speleothemdetails method works on sites object", {
  skip_on_cran()
  spel_data<- get_datasets(datasettype='speleothem', limit=10)
  spel_sites <- get_speleothems(spel_data)
  if (!is.null(spel_sites) && length(spel_sites) > 0) {
    testthat::expect_warning(
      result <- speleothemdetails(spel_sites)
    )
    testthat::expect_is(result, "data.frame")
    testthat::expect_true(nrow(result) >= 0)
  }
})

test_that("speleothemdetails returns distinct rows for sites", {
  skip_on_cran()
  spel_sites <- get_speleothems(c(66775, 66789))
  if (!is.null(spel_sites) && length(spel_sites) > 0) {
    testthat::expect_warning(
      result <- speleothemdetails(spel_sites)
    )
    testthat::expect_equal(nrow(result), nrow(unique(result)))
  }
})

test_that("speleothemdetails warns when no speleothems found in sites", {
  skip_on_cran()
  non_spel_sites <- get_sites(siteid = 1)
  if (!is.null(non_spel_sites) && length(non_spel_sites) > 0) {
    testthat::expect_warning(
      speleothemdetails(non_spel_sites),
      regexp = "No assigned speleothems"
    )
  }
})

test_that("speleothemdetails method works on single site object", {
  skip_on_cran()
  spel_data <- get_datasets(66789) %>%
    get_speleothems()
  if (!is.null(spel_data) && length(spel_data) > 0) {
    result <- speleothemdetails(spel_data[[1]])
    testthat::expect_is(result, "data.frame")
  }
})

test_that("speleothemdetails includes siteid in site method", {
  skip_on_cran()
  spel_data <- get_datasets(66789) %>%
    get_speleothems()
  if (!is.null(spel_data) && length(spel_data) > 0) {
    result <- speleothemdetails(spel_data[[1]])
    if (nrow(result) > 0) {
      testthat::expect_true("siteid" %in% names(result))
      testthat::expect_true(all(!is.na(result$siteid)))
    }
  }
})

test_that("speleothemdetails method works on collunits object", {
  skip_on_cran()
  spel_data <- get_datasets(66789) %>%
    get_speleothems()
  if (!is.null(spel_data) && length(spel_data) > 0) {
    result <- speleothemdetails(spel_data[[1]]@collunits[[1]])
    if (nrow(result) > 0) {
      testthat::expect_is(result, "data.frame")
      testthat::expect_true(length(unique(result$collunitid.x)) == 1)
      testthat::expect_true(length(unique(result$datasetid)) == 1)
    }
  }
})  

test_that("speleothemdetails joins speleothems and samples for collunit", {
  skip_on_cran()
  spel_site <- get_datasets(datasettype="speleothem", limit=1) %>%
    get_speleothems()
  if (!is.null(spel_site) && length(spel_site) > 0 && 
      length(spel_site[[1]]@collunits@collunits) > 0) {
    collunit_obj <- spel_site[[1]]@collunits@collunits[[1]]
    testthat::expect_warning(
      result <- speleothemdetails(collunit_obj)
    )
    if (nrow(result) > 0) {
      testthat::expect_true("datasetid" %in% names(result))
      # Sample-related columns
      sample_cols <- c("age", "depth", "sampleid")
      has_sample_cols <- all(sample_cols %in% names(result))
      testthat::expect_true(has_sample_cols)
    }
  }
})

library("testthat")
library("neotoma2")

context("Verifying that neotoma objects do not have duplicates and
         are nested properly respecting Neotoma's data object:
        site <- cu <- ds")
test_that("Doubling a set of records results and cleaning results in a clean set.", {
  # c calls clean internally but we can call it again to be sure
  skip_on_cran()
  alex <- get_sites(sitename = "Alex%")
  alex2 <- get_sites(24)
  alex_sets <- c(alex, alex2)
  clean_al <- clean(alex_sets)
  fiftyds <- get_datasets(limit = 50)
  nextds <- get_datasets(limit = 50)
  doubled <- c(fiftyds, nextds)
  testthat::expect_equal(getids(doubled), getids(nextds))
  testthat::expect_equal(length(doubled), length(fiftyds))
})

library("testthat")
library("neotoma2")

context("Creating samples from scratch or updating existing sample objects.")
test_that("Creating samples for the neotoma2 package.", {
  skip_on_cran()
  testthat::expect_error(set_sample(x = 12, depth = 33))
  sm <- set_sample(depth = 140,
                   sampleid = 42,
                   thickness = 32,
                   samplename = "My Sample")
  testthat::expect_is(sm, "sample")
  testthat::expect_equal(sm@depth, 140)
  testthat::expect_equal(sm@sampleid, 42)
  testthat::expect_equal(sm@thickness, 32)
})

test_that("Updating samples for the neotoma2 package.", {
  skip_on_cran()
  downloadSample <- get_sites(24) %>% get_downloads()
  # Cannot update on sites object
  testthat::expect_error(set_sample(downloadSample, depth=40))
  # Update single sample
  oneSample <- downloadSample[[1]]@collunits[[1]]@datasets[[1]]@samples[[1]]
  testthat::expect_true(is.na(oneSample@depth))
  updatedSample <- set_sample(x = oneSample, depth = 40)
  testthat::expect_equal(updatedSample@depth, 40)
  # Cannot use invalid data types
  testthat::expect_error(set_sample(x = oneSample, depth = "33a"))
})

library("testthat")
library("neotoma2")

context("`samples()` retrieves a data.frame of all data.")
test_that("`samples` retrieve df.", {
  skip_on_cran()
  dl <- get_downloads(4716) %>% samples()
  singlechron <- testthat::expect_true({dl; TRUE})
  # expect the return is a data frame
  testthat::expect_is(dl, "data.frame")
  testthat::expect_gt(nrow(dl), 0)
  # expect at least one row
  
  dl2 <- get_downloads(21007) %>% samples()
  testthat::expect_true({dl2; TRUE})
  testthat::expect_is(dl2, "data.frame")
  testthat::expect_gt(nrow(dl2), 0)
})

test_that("Get the samples out of dataset 15692.", {
  skip_on_cran()
  df <- get_downloads(15692) %>% samples()
  testthat::expect_gt(nrow(df), 1)
  testthat::expect_is(df, "data.frame")
})

test_that("ggplot2 on samples", {
  skip_on_cran()
  my_datasets <- get_datasets(40945)
  my_sites <- get_downloads(my_datasets)
  my_counts <- neotoma2::samples(my_sites)
  aa <- my_counts %>%
    dplyr::filter(taxongroup == "Vascular plants") %>%
    group_by(age, ecologicalgroup) %>%
    dplyr::summarize(count = sum(value)) %>%
    ggplot2::ggplot() +
    ggplot2::geom_path(ggplot2::aes(x = age,
                                    y = count,
                                    color = ecologicalgroup))
  testthat::expect_is(aa, "gg")
})

# Test that no duplicated sampleids exist for a variety of datasetids
# c(48891, 52833 52742) dups
datasetids <- c(41625, 46798, 48891, 47771,
                41620, 46689, 52833, 48756, 
                52742, 46700, 47608, 46841, 
                49238)
for (i in datasetids) {
  test_that(paste0("Duplicated sampleids for Dataset ID, ",
                   i, " don't exist (in the APD)"), {
                     skip_on_cran()
                     L <- get_datasets(i) %>%
                       get_downloads()
                     my_counts <- samples(L)
                     assertthat::assert_that(!any(duplicated(my_counts)))
                     counts <- my_counts %>%
                       mutate(age_chr = ifelse(is.na(age), "__NA__",
                                               as.character(age))) %>%
                       pivot_wider(
                         id_cols = age_chr,
                         names_from = variablename,
                         values_from = value,
                         values_fn = sum,
                         values_fill = NA
                       ) %>%
                       mutate(age = as.numeric(dplyr::na_if(age_chr, "__NA__"))) %>%
                       select(-age_chr)
                     
                     counts <- counts%>%
                       tryCatch(.data,
                                error = function(e) e,
                                warning = function(w) w)
                     testthat::expect_false(any(duplicated(my_counts)))
                     testthat::expect_false(is(counts, "warning"))
                   })
}

test_that("Samples of all sites has the same nrow as samples of each site combined", {
  skip_on_cran()
  si <- get_sites(limit=3)
  dl <- get_downloads(si)
  df1 <- nrow(samples(dl[[1]]))
  df2 <- nrow(samples(dl[[2]]))
  df3 <- nrow(samples(dl[[3]]))
  
  all_df <- nrow(samples(dl))
  testthat::expect_equal(all_df, df1 + df2 + df3)
})

testthat::skip("Skipping all tests in this file")
library("testthat")
library("neotoma2")

context("Creating speleothems from scratch or updating existing objects.")
test_that("`set_speleothem()` for a new publication", {
  skip_on_cran()
  newSpeleo <- set_speleothem(entityid = 56,
                           entityname = "My Speleothem",
                           siteid = 54,
                           collectionunitid = 32)
  testthat::expect_is(newSpeleo, "speleothem")
  testthat::expect_equal(newSpeleo@entityid, 56)
  testthat::expect_equal(newSpeleo@entityname, "My Speleothem")
  testthat::expect_equal(newSpeleo@siteid, 54)
  testthat::expect_equal(newSpeleo@collectionunitid, 32)
})

test_that("Updating existing publication", {
  skip_on_cran()
  newSpeleo <- set_speleothem(entityid = 56,
                              entityname = "My Speleothem",
                              siteid = 54,
                              collectionunitid = 32)
  site <- get_sites(3)
  siteid <- site[[1]]@siteid
  site[[1]]@collunits[[1]]@speleothems <- new("speleothems", speleothems = list(newSpeleo))
  
  testthat::expect_error(set_speleothems(x = site, siteid = siteid))
  testthat::expect_is(newSpeleo, "speleothem")
  updatedSpeleo <- set_speleothem(x = site[[1]]@collunits[[1]]@speleothems[[1]], siteid = siteid)
  testthat::expect_equal(updatedSpeleo@siteid, site[[1]]@siteid)
})

library("testthat")
library("neotoma2")

test_that("`datasets()`", {
  skip_on_cran()
  dls <- get_sites() %>%
    get_downloads()
  sumDl <- summary(dls)
  ids <- getids(dls)
  dss <- datasets(dls)
  testthat::expect_identical(length(dss), nrow(ids))
  testthat::expect_identical(sum(sumDl$n_datasets), nrow(ids))
  testthat::expect_identical(length(unique(dss$datasetid)),
                             length(unique(ids$datasetid)))
})

library("testthat")
library("neotoma2")

context("get_downloads() retrieves data from /download endpoint 
        and parses content as expected.")
test_that("get_downloads numeric", {
  skip_on_cran()
  download_1001 <- neotoma2::get_downloads(1001)
  datasets_ids <- neotoma2::getids(download_1001) %>%
    dplyr::select(datasetid) %>%
    unique()
  # Downloads using dsid, verify that we only have
  # objects belonging to that dsid
  testthat::expect_equivalent(datasets_ids, 1001)
})

test_that("get_downloads from get_datasets()
          sites object.", {
            skip_on_cran()
            brazil <- '{"type": "Polygon",
            "coordinates": [[
                [-73.125, -9.102],
                [-56.953, -33.138],
                [-36.563, -7.711],
                [-68.203, 13.923],
                [-73.125, -9.102]
              ]]}'
            brazil_sf <- geojsonsf::geojson_sf(brazil)
            brazil_datasets <- get_datasets(loc = brazil_sf, all_data = TRUE)
            brazil_dl <- get_downloads(brazil_datasets)
            testthat::expect_identical(nrow(getids(brazil_datasets)),
                                       nrow(getids(brazil_dl)))
            testthat::expect_equal(getids(brazil_datasets), 
                                   getids(brazil_dl))
          })

test_that("get_downloads from get_sites sites", {
  skip_on_cran()
  core_sites <- c(13949, 11904, 13319, 728,
                  13248, 2625, 2806, 13280, 519, 11745,
                  273, 13956, 11880, 13321, 9801, 13698,
                  11816, 13909, 13921)
  df1 <- get_sites(core_sites) %>%
    get_downloads() %>%
    getids()
  df2 <- get_sites(core_sites) %>%
    getids()
  # The get_downloads limit happens, so we have fewer rows in df1
  testthat::expect_lte(nrow(df1), nrow(df2))
  testthat::expect_true(all(unique(df1$datasetid) %in% unique(df2$datasetid)))
})

test_that("Faunmap dataset", {
  skip_on_cran()
  mydataset <- get_downloads(7032)
  testthat::expect_is(mydataset, "sites")
})

test_that("get_downloads with or without all_data works.", {
  skip_on_cran()
  uk_bbox_geojson <- "{\n\"type\": \"FeatureCollection\",\n\"name\": \"out\",\n\"crs\": { \"type\": \"name\", \"properties\": { \"name\": \"urn:ogc:def:crs:OGC:1.3:CRS84\" } },\n\"features\": [\n{ \"type\": \"Feature\", \"properties\": { }, \"geometry\": { \"type\": \"Polygon\", \"coordinates\": [ [ [ -10.390234374999977, 50.021386718749994 ], [ 1.74658203125, 50.021386718749994 ], [ 1.74658203125, 60.831884765624991 ], [ -10.390234374999977, 60.831884765624991 ], [ -10.390234374999977, 50.021386718749994 ] ] ] } }\n]\n}"
  uk_datasets <- get_datasets(
    loc = uk_bbox_geojson,
    datasettype = 'pollen',
    limit=5
  )
  uk_dl <- get_downloads(uk_datasets, all_data=TRUE)
  uk_dl2 <- get_downloads(uk_datasets)
  testthat::expect_equal(nrow(as.data.frame(datasets(uk_dl))), 
                         nrow(as.data.frame(datasets(uk_dl2))))
  # expect no error
  testthat::expect_error(get_downloads(uk_datasets, all_data=TRUE), NA)
  testthat::expect_error(get_downloads(uk_datasets, all_data=FALSE), NA)
  testthat::expect_error(get_downloads(uk_datasets), NA)
})

test_that("get_downloads handles empty result", {
  skip_on_cran()
  gpids <- c(7326, 6442, 7923, 7990, 7368, 8480, 8981, 7934)
  ne_sites <- c() 
  for (id in gpids) {
    search_1 <- get_sites(gpid = id)
    ne_sites <- c(ne_sites, search_1)
  }
  diatom_list <- c("diatom", "diatom top-bottom", "diatom bottom sample")
  bottom_diat_info <- c()
  for (type in diatom_list) {
    ne_sites 
    bottom_diatoms <- c(bottom_diat_info, ne_sites) 
  } 
  bottom_diatoms
  testthat::expect_error(get_downloads(bottom_diatoms), NA)
})

library("testthat")
library("neotoma2")

context("`getids()` retrieves all 'site', 'collection unit', and
        'dataset' IDs from Neotoma2 objects")
test_that("getids gets correct site IDs", {
  skip_on_cran()
  sts <- get_sites(c(1001, 2001, 15, 24))
  ids <- getids(sts)
  testthat::expect_setequal(ids$siteid, c(1001, 2001, 15, 24))
})

test_that("getids gets correct dataset IDs", {
  skip_on_cran()
  dss <- get_datasets(c(1001, 2001, 15, 24))
  ids <- getids(dss)
  testthat::expect_setequal(ids$datasetid, c(1001, 2001, 15, 24))
})

test_that("getids gets correct IDs and correct alignment", {
  skip_on_cran()
  sts <- get_sites(c(1001, 2001, 15, 24))
  ids <- getids(sts)
  sts_ids_df <- purrr::map_df(sts@sites, function(x) {
    siteid <- x@siteid
    purrr::map(x@collunits, function(y) {
      cuid <- y@collectionunitid
      dsids <- purrr::map(y@datasets, function(z) z$datasetid) %>% unlist()
      # Return one row per datasetid
      data.frame(
        siteid           = siteid,
        collectionunitid = cuid,
        datasetid        = dsids
      )}) %>% 
      do.call(rbind, .)
  })
  testthat::expect_equal(nrow(ids), nrow(sts_ids_df))
  testthat::expect_setequal(ids$siteid, sts_ids_df$siteid)
  testthat::expect_setequal(ids$datasetid, sts_ids_df$datasetid)
  testthat::expect_setequal(ids$collunitid, sts_ids_df$collectionunitid)
  sts_cu <- sts_ids_df$collectionunitid %>% unique()
  st15_ids <- ids %>%
    dplyr::filter(siteid == 15)
  st15_map <- sts_ids_df %>%
    dplyr::filter(siteid == 15)
  testthat::expect_setequal(st15_ids$datasetid, st15_map$datasetid)
  testthat::expect_setequal(st15_ids$collunitid, st15_map$collectionunitid)
  st2001_ids <- ids %>%
    dplyr::filter(siteid == 2001)
  st2001_map <- sts_ids_df %>%
    dplyr::filter(siteid == 2001)
  testthat::expect_setequal(st2001_ids$datasetid, st2001_map$datasetid)
  testthat::expect_setequal(st2001_ids$collunitid, st2001_map$collectionunitid)
})

library("testthat")
library("neotoma2")
library("dplyr")
library("stringr")

context("`cite_data()` function")
test_that("cite_data() returns dataframe", {
  skip_on_cran()
  st <- get_sites(c(24, 100))
  citation <- cite_data(st)
  testthat::expect_is(citation, "data.frame")
})

test_that("cite_data() returns correct DOIs for each dataset", {
  skip_on_cran()
  st <- get_datasets(c(24, 100))
  citation <- cite_data(st)
  cit24 <- citation %>% 
    dplyr::filter(datasetid == 24)
  cit24 <- cit24$citation
  doi24 <- st[[1]]@collunits[[1]]@datasets[[1]]@doi %>% unlist()
  # expect that doi is in citation string
  testthat::expect_true(any(str_detect(cit24, fixed(doi24))))
  cit100 <- citation %>% 
    filter(datasetid == 100)
  cit100 <- cit100$citation
  doi100 <- st[[2]]@collunits[[1]]@datasets[[1]]@doi %>% unlist()
  # expect that doi is in citation string
  testthat::expect_true(any(str_detect(cit100, fixed(doi100))))
  testthat::expect_false(any(str_detect(cit100, fixed(doi24))))
  testthat::expect_false(any(str_detect(cit24, fixed(doi100))))
})

library("testthat")
library("neotoma2")

context("Test plots")
testthat::test_that("Plot records with plot", {
  skip_on_cran()
  ten <- get_sites(limit = 10)
  testthat::expect_error(plot(ten), NA)
})

testthat::test_that("Plot a single record with plot.", {
  skip_on_cran()
  five <- get_sites(limit = 5)
  testthat::expect_error(plot(five[[1]]), NA)
})

library("testthat")
library("neotoma2")

context("Test leaflet plots")

testthat::test_that("Plot records with plotLeaflet.", {
  skip_on_cran()
  ten <- get_sites(limit = 10)
  output <- plotLeaflet(ten)
  testthat::expect_is(output, "leaflet")
})

testthat::test_that("Plot a single record with plotLeaflet.", {
  skip_on_cran()
  five <- get_sites(limit = 5)
  output <- plotLeaflet(five[[1]])
  testthat::expect_is(output, "leaflet")
})

library("testthat")
library("neotoma2")

context("get_datasets() retrieves data from /dataset endpoint 
        and parses content as expected.")
test_that("get_datasets numeric", {
  skip_on_cran()
  dataset_1001 <- neotoma2::get_datasets(1001)
  datasets_ids <- neotoma2::getids(dataset_1001) %>%
    dplyr::select(datasetid) %>%
    unique()
  testthat::expect_equivalent(datasets_ids, 1001)
})

test_that("get_datasets numeric vector", {
  skip_on_cran()
  datasets_ob <- get_datasets(c(1001, 2001, 15, 24))
  testthat::expect_length(datasets_ob, 4)
  datasets_vec <- getids(datasets_ob) %>%
    dplyr::select(datasetid) %>%
    unique() %>%
    unlist()
  testthat::expect_setequal(datasets_vec, c(1001, 2001, 15, 24))
  datasets_long <- get_datasets(seq(1, 1000), limit = 10)
  testthat::expect_length(datasets_long, 10)
})

test_that("get_datasets with loc attribute", {
  skip_on_cran()
  brazil <- '{"type": "Polygon",
            "coordinates": [[
                [-73.125, -9.102],
                [-56.953, -33.138],
                [-36.563, -7.711],
                [-68.203, 13.923],
                [-73.125, -9.102]
              ]]}'
  brazil_sf <- geojsonsf::geojson_sf(brazil)
  brazil_datasets <- get_datasets(loc = brazil[1], datasettype = "pollen")
  # Check that datasset types names are only pollen
  sum <- summary(brazil_datasets)
  testthat::expect_lte(nrow(summary(brazil_datasets)), nrow(sum))
  testthat::expect_equivalent(nrow(sum), nrow(getids(brazil_datasets)))
  # All datasets should be pollen:
  testthat::expect_equivalent(unique(sum$dataset_types), "pollen")
  # Check that all siteids are unique
  brazil_unique_sites <- length(unique(getids(brazil_datasets)$siteid))
  brazil_sites_length <- length(brazil_datasets)
  testthat::expect_equal(brazil_sites_length, brazil_unique_sites)
  # Check that all datasetids are unique
  brazil_unique_dsids <- length(unique(getids(brazil_datasets)$datasetid))
  brazil_ds_length <- length(datasets(brazil_datasets))
  testthat::expect_equal(brazil_ds_length, brazil_unique_dsids)
})

test_that("all_data + loc", {
  skip_on_cran()
  europe_json <- '{"type": "Polygon",
            "coordinates": [[
                [-73.125, -9.102],
                [-56.953, -33.138],
                [-36.563, -7.711],
                [-68.203, 13.923],
                [-73.125, -9.102]
              ]]}'
  data_short <- get_datasets(loc = europe_json[1])
  data_long <- get_datasets(loc = europe_json[1], all_data = TRUE)
  testthat::expect_gte(length(data_long), length(data_short))
  eur_ids <- getids(data_long)
  # check that all datasetids in datasets df are in eur_ids
  ds_ids <- as.data.frame(datasets(data_short)) %>%
    dplyr::select(datasetid) %>%
    unique() %>%
    unlist()
  testthat::expect_true(all(ds_ids %in% eur_ids$datasetid))
  cu_ids <- as.data.frame(collunits(data_short)) %>%
    dplyr::select(collectionunitid) %>%
    unique() %>%
    unlist()
  testthat::expect_true(all(cu_ids %in% eur_ids$collunitid))
})

test_that("get_datasets with or without all_data works.", {
  skip_on_cran()
  uk_bbox_geojson <- "{\n\"type\": \"FeatureCollection\",\n\"name\": \"out\",\n\"crs\": { \"type\": \"name\", \"properties\": { \"name\": \"urn:ogc:def:crs:OGC:1.3:CRS84\" } },\n\"features\": [\n{ \"type\": \"Feature\", \"properties\": { }, \"geometry\": { \"type\": \"Polygon\", \"coordinates\": [ [ [ -10.390234374999977, 50.021386718749994 ], [ 1.74658203125, 50.021386718749994 ], [ 1.74658203125, 60.831884765624991 ], [ -10.390234374999977, 60.831884765624991 ], [ -10.390234374999977, 50.021386718749994 ] ] ] } }\n]\n}"
  uk_sts <- get_sites(
    loc = uk_bbox_geojson,
    datasettype = 'pollen',
    limit=5
  )
  uk_dl <- get_datasets(uk_sts, all_data=TRUE)
  uk_dl2 <- get_datasets(uk_sts)
  testthat::expect_equal(nrow(as.data.frame(datasets(uk_dl))), 
                         nrow(as.data.frame(datasets(uk_dl2))))
  # expect no error
  testthat::expect_error(get_datasets(uk_sts, all_data=TRUE), NA)
  testthat::expect_error(get_datasets(uk_sts, all_data=FALSE), NA)
  testthat::expect_error(get_datasets(uk_sts), NA)
})

library("testthat")
library("neotoma2")

context("Concatenation of c()  in `neotoma2` objects works appropriately.")
test_that("c() concats properly.", {
  skip_on_cran()
  alex <- get_sites(24)
  site5 <- get_sites(5)
  sites <- c(alex, site5)
  testthat::expect_equal(length(sites), 2)
})

test_that("c() removes duplicates properly.", {
  skip_on_cran()
  alex <- get_sites(24)
  alex2 <- get_datasets(7870)
  sites <- c(alex, alex2)
  testthat::expect_equal(length(sites), 1)
  
  site5 <- get_sites(5)
  sites <- c(sites, site5)
  testthat::expect_equal(length(sites), 2)
})

library("testthat")
library("neotoma2")

context("Test `get_sites()` function.")
test_that("get_sites numeric", {
  skip_on_cran()
  site_1001 <- neotoma2::get_sites(1001)
  sites_ids <- neotoma2::getids(site_1001) %>%
    dplyr::select(siteid) %>%
    unique()
  testthat::expect_equivalent(sites_ids, 1001)
})

test_that("get_sites numeric vector", {
  skip_on_cran()
  sites_ob <- get_sites(c(1001, 2001, 15, 24))
  testthat::expect_length(sites_ob, 4)
  sites_vec <- getids(sites_ob) %>%
    dplyr::select(siteid) %>%
    unique() %>%
    unlist()
  testthat::expect_setequal(sites_vec, c(1001, 2001, 15, 24))
  sites_long <- get_sites(seq(1, 1000), limit = 10)
  testthat::expect_length(sites_long, 10)
})

test_that("get_sites with loc attribute", {
  skip_on_cran()
  brazil <- '{"type": "Polygon",
            "coordinates": [[
                [-73.125, -9.102],
                [-56.953, -33.138],
                [-36.563, -7.711],
                [-68.203, 13.923],
                [-73.125, -9.102]
              ]]}'
  brazil_sf <- geojsonsf::geojson_sf(brazil)
  brazil_sites <- get_sites(loc = brazil[1], datasettype = "pollen")
  sum <- summary(brazil_sites)
  testthat::expect_lte(length(brazil_sites), nrow(sum))
  testthat::expect_equivalent(nrow(sum), nrow(getids(brazil_sites)))
  # All datasets should be pollen:
  testthat::expect_equivalent(unique(sum$dataset_types), "pollen")
  # Check that all siteids are unique
  brazil_unique_sites <- length(unique(getids(brazil_sites)$siteid))
  brazil_sites_length <- length(brazil_sites)
  testthat::expect_equal(brazil_sites_length, brazil_unique_sites)
})

test_that("all_data + loc", {
  skip_on_cran()
  europe_json <- '{"type": "Polygon",
            "coordinates": [[
                [-73.125, -9.102],
                [-56.953, -33.138],
                [-36.563, -7.711],
                [-68.203, 13.923],
                [-73.125, -9.102]
              ]]}'
  data_short <- get_sites(loc = europe_json[1])
  data_long <- get_sites(loc = europe_json[1], all_data = TRUE)
  testthat::expect_gt(length(data_long), length(data_short))
  eur_ids <- getids(data_long)
  # check that all siteids are in eur_ids
  st_ids <- as.data.frame(data_short) %>%
    dplyr::select(siteid) %>%
    unique() %>%
    unlist()
  testthat::expect_true(all(st_ids %in% eur_ids$siteid))
})

context("get_sites()")
test_that("If B is contained in A region,
          get_sites() from B will be contained
          in get_sites() from A", {
            skip_on_cran()
            location <- '{"type": "Polygon",
            "coordinates": [[
                [-169, 24],
                [-169, 75],
                [-52, 75],
                [-52, 24],
                [-169, 24]]]}'
            usa <- get_sites(loc = location, limit = 20000)
            fla <- get_sites(gpid = "Florida", limit = 10000)
            testthat::expect_true(all(getids(fla)$siteid %in% getids(usa)$siteid))
          })

library("testthat")
library("neotoma2")

context("`length()` retrieves correct number of 'sites', 'collection units', and
        'datasets'")
test_that("length of sites is correct", {
  skip_on_cran()
  sts <- get_sites(c(1001, 2001, 15, 24))
  len <- length(sts)
  testthat::expect_equal(len, length(c(1001, 2001, 15, 24)))
})

test_that("length of collection units is correct", {
  skip_on_cran()
  sts <- get_sites(c(1001, 2001, 15, 24))
  cu <- collunits(sts)
  len <- length(cu)
  ids <- getids(sts)$collunitid %>% unique() %>% length()
  testthat::expect_equal(len, ids)
  len <- length(sts[[3]]@collunits)
  stid <- getids(sts[[3]])$siteid %>% unique()
  cuids <- filter(getids(sts), siteid == stid)$collunitid %>% length()
  testthat::expect_equal(len, cuids)
})

test_that("length of datasets is correct", {
  skip_on_cran()
  sts <- get_datasets(c(1001, 2001, 15, 24))
  ds <- datasets(sts)
  len <- length(ds)
  ids <- getids(sts)$datasetid %>% length()
  testthat::expect_equal(len, ids)
  len <- length(
    map(sts[[1]]@collunits, function(y) {
      map(y@datasets, function(z) z$datasetid) %>% unlist()
    }) %>% unlist()
  )
  stid <- getids(sts[[1]])$siteid %>% unique()
  dsids <- filter(getids(sts), siteid == stid)$datasetid %>% length()
  testthat::expect_equal(len, dsids)
})

## load packages
library("testthat")
library("neotoma2")

context("Run Neotoma `test_sites` only when not on CRAN")
test_that("get_sites runs as expected.", {
  skip_on_cran()
  alexander_lake <- get_sites(sitename = "Alexander Lake")
  testthat::expect_identical(alexander_lake[[1]]@sitename, "Alexander Lake")
})

test_that("get_sites runs as expected.", {
  skip_on_cran()
  alex <- get_sites(sitename = "Alex%")
  for (i in seq_len(length(alex))) {
    alex_char <- substring(alex[[i]]@sitename, 1,4)
    alex_char <- tolower(alex_char)
    testthat::expect_identical(alex_char, "alex")
  }
})

test_that("get_sites runs as expected.", {
  skip_on_cran()
  site.1001 <- get_sites(1001)
  testthat::expect_equal(site.1001[[1]]@siteid, 1001)
})

test_that("get_sites runs as expected.", {
  skip_on_cran()
  sites.ob <- get_sites(c(1001, 2001, 15, 24))
  sites.vec <- getids(sites.ob)
  sites.vec <- unique(sites.vec$siteid)
  testthat::expect_setequal(sites.vec, c(1001, 2001, 15, 24))
})

# test_that("All Czech sites work with different spatial bounds:", {
#   skip_on_cran()
#   cz_json <- '{"type": "Polygon",
#         "coordinates": [[
#             [12.40, 50.14],
#             [14.10, 48.64],
#             [16.95, 48.66],
#             [18.91, 49.61],
#             [15.24, 50.99],
#             [12.40, 50.14]]]}'
#   cz_WKT <- 'POLYGON ((12.4 50.14,
#                        14.1 48.64,
#                        16.95 48.66,
#                        18.91 49.61,
#                        15.24 50.99,
#                        12.4 50.14))'
#   cz_bbox <- c(12.4, 48.64, 18.91, 50.99)
# 
#   testthat::expect_true(all.equal(get_sites(loc=cz_json),
#     get_sites(loc=cz_WKT)))
# 
#   # Now, we know that all sites in cz_sites[[1]] should be in cz_sites[[3]],
#   # but the bounding box strategy means that the reverse is not true:
#   cz_ids <- getids(get_sites(loc=cz_json[1]))
#   testthat::expect_true(all(cz_ids$siteid %in%
#     getids(get_sites(loc = cz_bbox, limit = 50))$siteid))
# })

test_that("All Data + loc work", {
  skip_on_cran()
  europe_json <- '{"type": "Polygon",
        "coordinates": [
          [[-23.5546875, 70.8446726342528],
           [-25.3125, 39.36827914916014],
           [61.87499999999999, 39.36827914916014],
           [62.57812500000001, 74.01954331150228],
           [-23.5546875, 70.8446726342528]]
        ]}'
  data <- get_sites(loc = europe_json[1], all_data = TRUE)
  testthat::expect_gt(length(data), 50)

  # Now, we know that all sites in cz_sites[[1]] should be in cz_sites[[3]],
  # but the bounding box strategy means that the reverse is not true:
})

#' @title Check contact information for a record against Neotoma contributors
#' @author Simon Goring \email{goring@wisc.edu}
#' @importFrom purrr map
#' @importFrom dplyr bind_rows
#' @importFrom methods is
#' @description
#'   Uses the Neotoma API to search and access information about individuals
#'   who have contributed to the data in the Neotoma Paleoecology Database
#' @param x contacts A \code{contacts} object associated with a set of names.
#' @param similarity The similarity score between matched records (from 0 - 1).
#' @param ... Additional parameters associated with the call.
#' @return contacts object
#' @export
check_contacts <- function(x, ...) {
  if (class(x) %in% c("contacts")) {
    UseMethod("check_contacts")
  } else {
    stop("check_contacts() must operate on a `contacts` or `contact` object.")
  }
}

#' @rdname check_contacts
#' @export
#' @method check_contacts contacts
check_contacts.contacts <- function(x, similarity = 0.5, ...) {
  checked <- map(x@contacts, function(y) {
    if (is.na(y@contactid) | !is(y@contactid, "numeric")) {
      output <- get_contacts(name = paste0(y@givennames, " ",
                                           y@familyname),
                             similarity = similarity, ...)
    } else {
      output <- y
    }
    return(output)
  })
  return(checked)
}

library("testthat")
library("neotoma2")

context("Contact object is set up properly.")
test_that("check_contacts works properly.", {
  skip_on_cran()
  testthat::expect_error(check_contacts(12))
  testthat::expect_error(check_contacts("party"))
  new_name <- c(set_contact(familyname = "Goring"),
    set_contact(familyname = "Cullen"))
  testthat::expect_is(check_contacts(new_name), "list")
  na_name <- c(set_contact(familyname = "Goring", contactid = NA),
    set_contact(familyname = "Cullen"))
  testthat::expect_is(check_contacts(na_name), "list")
})

#' @title Extract `collunits` from a `sites` object.
#' @param object A `sites` object
#' @importFrom purrr map reduce
#' @returns `collunits` from a `sites` object
#' @md
#' @export
setMethod(f = "collunits",
          signature = "sites",
          definition = function(object) {
            output <- map(object@sites, function(x) {
              x@collunits
            })
            if (length(output) == 0) {
              return(new("collunits", collunits = list()))
            }
            final <- reduce(output, c)
            final <- clean(final)
            return(final)
          })

#' @rdname collunits
#' @export
setMethod(f = "collunits",
          signature = "site",
          definition = function(object) {
            output <- object@collunits
            attr(output, "siteid") <- object$siteid
            return(output)
          })

#' @title Extract `datasets` from a `sites` or `collunits` object.
#' @importFrom purrr map reduce
#' @param object A `sites` or`collunits` object
#' @returns `datasets` object
#' @export
setMethod(f = "datasets",
          signature = "collunits",
          definition = function(object) {
            result <- map(object@collunits,
              function(x) x@datasets)
            if (length(result) == 1) {
              out <- result[[1]]
            } else {
              out <- reduce(result, c)
            }
            return(out)
          })

#' @rdname datasets
#' @export
setMethod(f = "datasets",
          signature = "collunit",
          definition = function(object) {
            result <- object@datasets
            return(result)
          })

#' @rdname datasets
#' @export
setMethod(f = "datasets",
          signature = "sites",
          definition = function(object) {
            clean(datasets(collunits(object)))
          })

#' @rdname datasets
#' @export
setMethod(f = "datasets",
          signature = "site",
          definition = function(object) {
            cunits <- collunits(object)
            result <- map(cunits@collunits,
              function(x) x@datasets)
            if (length(result) == 1) {
              out <- result[[1]]
            } else {
              out <- reduce(result, c)
            }
            return(out)
          })

#' @title Extract `chronologies` from a `site`s or `collunits` object.
#' @importFrom purrr map
#' @param x A `sites` or `collunits` object
#' @returns `chronologies` from a `collunit` object
#' @md
#' @export
setMethod(f = "chronologies",
          signature = "collunit",
          definition = function(x) {
            output <- map(x@chronologies@chronologies, function(y) {
              attr(y, "collunitid") <- x$collectionunitid
              return(y)
            })
            try(
              returner <- new("chronologies", chronologies = output)
            )
            if ("try-error" %in% class(returner)) {
              stop("Cannot create chronology for this colleciton unit.")
            }
            return(returner)
          })

#' @rdname chronologies
#' @export
setMethod(f = "chronologies",
          signature = "collunits",
          definition = function(x) {
            output <- map(x@collunits, function(y) {
              chronologies(y)
            })
            output <- reduce(output, c)
            return(output)
          })

#' @rdname chronologies
#' @export
setMethod(f = "chronologies",
          signature = "site",
          definition = function(x) {
            output <- map(x@collunits@collunits, function(y) {
              chronologies(y)
            })
            output <- purrr::reduce(output, c)
            return(output)
          })

#' @rdname chronologies
#' @export
setMethod(f = "chronologies",
          signature = "sites",
          definition = function(x) {
            output <- map(x@sites, function(y) {
              chronologies(y)
            })
            output <- reduce(output, c)
            return(output)
          })

#' @title get_stats
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @author Simon Goring \email{goring@wisc.edu}
#' @importFrom purrr map map_df
#' @description Returns a count of sites, datasets, publications and other
#' objects added to Neotoma during the requested time period.
#' @details This function returns summaries about the data holdings
#' within Neotoma using the existing Neotoma API's `summary` endpoint. This can
#' provide information about recent uploads (the number of new sites
#' uploaded within the last month, for example), or can be used to provide
#' information about the overall number of sites/datasets (using an arbitrarily
#' high value for `end`).
#' @param start The starting month (from present == 0) for
#' which to generate the summary. Default is 0 (the current month).
#' @param end The ending month (from present == 0) for which
#' to generate the summary. Default is 1 (one month ago).
#' @param type A character string indicating the type of summary
#' to return. Options are \code{dsdbmonth} (the number of datasets
#' in the Neotoma Database added per month), \code{rawbymonth} (the
#' number of datasets, sites, publications, authors, countries and observations
#' added per month), and \code{dstypemonth} (the number of datasets added
#' per dataset type per month). Default is \code{dsdbmonth}.
#' @returns `data.frame` with summary statistics
#' @examples \dontrun{
#' tryCatch({
#' last_month <- get_stats(start = 0, end = 1, type = "dsdbmonth")
#' }, error = function(e) {
#'  message("Neotoma server not responding. Try again later.")
#' })
#' }
#' @md
#' @export
get_stats <- function(start = 0, end = 1, type = "dsdbmonth") {
  if (!(type %in% c("dsdbmonth", "rawbymonth", "dstypemonth"))) {
    stop("type must be either 'dsdbmonth', 'dstypemonth' or 'rawbymonth'.")
  }
  if (type == "dsdbmonth") {
    baseURL <- paste0("data/summary/dsdbmonth?start=", start, "&end=", end)
  } else if (type == "rawbymonth") {
    baseURL <- paste0("data/summary/rawbymonth?start=", start, "&end=", end)
  } else if (type == "dstypemonth") {
    baseURL <- paste0("data/summary/dstypemonth?start=", start, "&end=", end)
  }
  result <- tryCatch(
    parseURL(baseURL),
    error = function(e) {
      message("API call failed: ", e$message)
      NULL
    }
  )
  if (is.null(result) || is.null(result$data)) {
    message("No results returned from Neotoma API.")
    return(NULL)
  }
  if (type == "dsdbmonth") {
    result <- result$data$data
    df <- result %>% map_df(~ data.frame(
      databasename = .$databasename,
      counts = as.integer(.$counts)
    ))
    return(df)
  } else if (type == "dstypemonth") {
    result <- result$data$data
    df <- result %>% map_df(~ data.frame(
      datasettype = .$datasettype,
      counts = as.integer(.$counts)
    ))
    return(df)
  } else if (type == "rawbymonth") {
    result <- result$data$data
    df <- result %>% map_df(~ data.frame(
      datasets = .$datasets,
      sites = .$sites,
      publications = .$publications,
      authors = .$authors,
      countrygpid = .$countrygpid,
      observations = .$observations
    ))
    return(df)
  }
}

library("testthat")
library("neotoma2")

context("`get_stats()` function returns different kinds of summary statistics")
test_that("get_stats with the default values", {
  skip_on_cran()
  stats <- get_stats()
  testthat::expect_is(stats, "data.frame")
  testthat::expect_true(ncol(stats) == 2)
  testthat::expect_true(all(c("databasename", "counts") %in% colnames(stats)))
})

test_that("get_stats rawbymonth", {
  skip_on_cran()
  stats <- get_stats(type="rawbymonth")
  testthat::expect_is(stats, "data.frame")
  testthat::expect_true(nrow(stats) == 1)
  testthat::expect_true(all(c("datasets", "sites", "publications",
                              "authors", "countrygpid", "observations") %in% colnames(stats)))
})

test_that("get_stats dstypemonth", {
  skip_on_cran()
  stats <- get_stats(type="dstypemonth")
  testthat::expect_is(stats, "data.frame")
  testthat::expect_true(all(c("datasettype", "counts") %in% colnames(stats)))
})

#' @title group_data
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom purrr map map_lgl flatten imap
#' @description An internal helper function used to parse the
#' API response into a hierarchical list of
#' *sites* > *collectionunits* > *datasets*
#' @param data API response
#' @returns hierarchical `list` with parsed API request
#' @noRd
group_response <- function(data) {
  # Detect endpoint
  is_dataset  <- all(map_lgl(data, ~ !is.null(.x$site$datasets)))
  is_download <- all(map_lgl(data, function(x) {
    tryCatch(!is.null(x$site$collectionunit$dataset), error = function(e) FALSE)
  }))
  is_site <- all(map_lgl(data, ~ !is.null(.x$collectionunits)))
  if (is_download) {
    return(group_download(data))
  }
  if (is_dataset) {
    return(group_dataset(data))
  }
  if (is_site) {
    return(data)
  } else {
    stop("Unrecognized data structure.")
  }
}

#' @title group_download
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @description An internal function used to parse the 
#' API download object response
#' @param data API response
#' @returns hierarchical `list` with parsed API request
#' @noRd
group_download <- function(data) {
  site_groups <- split(data, sapply(data, function(x) x$site$siteid))
  map(site_groups, function(site_group) {
    site_info <- site_group[[1]]$site
    # collection units
    cu_groups <- split(site_group, sapply(site_group, function(x) {
      x$site$collectionunit$collectionunitid
    }))
    collectionunits <- imap(cu_groups, function(cu_group, cu_id) {
      cu_info <- cu_group[[1]]$site$collectionunit
      # datasets
      cu_info$dataset <- NULL
      cu_info$datasets <- map(cu_group, ~ .x$site$collectionunit$dataset)
      cu_info
    })
    site_info$dataset <- NULL
    site_info$collectionunit <- NULL
    site_info$collectionunits <- collectionunits
    list(site = site_info)
  })
}

#' @title group_download
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @description An internal function used to parse the 
#' API dataset object response
#' @param data API response
#' @returns hierarchical `list` with parsed API request
#' @noRd
group_dataset <- function(data) {
  site_groups <- split(data, sapply(data, function(x) x$site$siteid))
  sites <- map(site_groups, function(site_group) {
    site_info <- site_group[[1]]$site
    # collection units
    cu_groups <- split(site_group, sapply(site_group,
                       function(x) x$site$collectionunitid))
    collunits <- map(cu_groups, function(cu_group) {
      cu_info <- cu_group[[1]]$site
      # datasets in cu
      all_datasets <- map(cu_group, function(x) x$site$datasets) %>%
        flatten()
      cu_info$datasets <- all_datasets
      cu_info
    })
    # Remove unnecessary fields
    site_info$datasets <- NULL
    site_info$handle <- NULL
    site_info$collectionunitid <- NULL
    site_info$collectionunit <- NULL
    site_info$collectionunits <- collunits
    list(site = site_info)
  })
  return(sites)
}

#' @rdname sub-sub
setMethod(f = "[[",
          signature = signature(x = "chronologies", i = "numeric"),
          definition = function(x, i) {
            if (length(i) == 1) {
              out <- new("chronology", x@chronologies[[i]])
            } else {
              out <- map(i, function(z) {
                new("chronology", x@chronologies[[z]])
              })
              out <- new("chronology", chronologies = out)
            }
            return(out)
          })

#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "chronology"),
          definition = function(x, name) {
            slot(x, name)
          })


#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "chronologies"),
          definition = function(x, name) {
            x %>%
              map(function(y) {
                slot(y, name)
              }) %>%
              unlist()
          })

#' @rdname cash-set
setMethod(f = "$<-",
          signature = signature(x = "chronology"),
          definition = function(x, name, value) {
            slot(x, name) <- value
            return(x)
          })

#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("chronology"),
          definition = function(x) {
            data.frame(chronologyid = as.character(x@chronologyid),
                       notes = x@notes,
                       agemodel = x@agemodel,
                       ageboundolder = x@ageboundolder,
                       ageboundyounger = x@ageboundyounger,
                       isdefault = x@isdefault,
                       dateprepared = x@dateprepared,
                       modelagetype = x@modelagetype,
                       chronologyname = x@chronologyname)
          })

#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("chronologies"),
          definition = function(x) {
            x@chronologies %>% map(as.data.frame) %>% bind_rows()
          })

#' @rdname length
setMethod(f = "length",
          signature = signature(x = "chronologies"),
          definition = function(x) {
            length(x@chronologies)
          })

#' @rdname c
setMethod(f = "c",
          signature = signature(x = "chronologies"),
          definition = function(x, y) {
            if ("chronology" %in% class(y)) {
              y <- new("chronologies", chronologies = list(y))
            }
            tryCatch(
              new("chronologies",
                  chronologies = unlist(c(x@chronologies,
                                          y@chronologies),
                                        recursive = FALSE)),
                     error = function(e) {
                       stop("Use `get_downloads()` to fill chronologies details.
                            Current `sites` object comes from `get_sites()` or
                            `get_datasets()` which does not have chronology
                            detail")
                     })
          })

#' @title Change the default age model for a record.
#' @importFrom purrr map
#' @importFrom assertthat assert_that
#' @param x A chronologies object.
#' @param n The particular chronology to be used as the default.
#' @returns `chronologies` object with a new defaulted `chronology`
#' @md
#' @export
setMethod(f = "set_default",
          signature = signature(x = "chronologies"),
          definition = function(x, n) {
            assert_that(class(x) == "chronologies")
            chron_set <- as.data.frame(x)
            assert_that(n %in% chron_set$chronologyid,
                        msg = "The new default chronology 
                        must be a valid chronologyid
                        within the chronologies.")
            which_replace <- chron_set$chronologyid == n
            replacingmodel <- chron_set$modelagetype[which_replace]
            chronout <- map(seq_len(length(x)), function(y) {
              if (x@chronologies[[y]]$chronologyid == n) {
                x@chronologies[[y]]@isdefault <- TRUE
              }
              if (x@chronologies[[y]]$chronologyid != n &
                  x@chronologies[[y]]$modelagetype == replacingmodel) {
                x@chronologies[[y]]@isdefault <- FALSE
              }
              return(x@chronologies[[y]])
            })
            return(new("chronologies", chronologies = chronout))
          })

#' @title Create a new publication (or publication set)
#' @author Simon Goring \email{goring@wisc.edu}
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @param x "publications" object to be updated.
#' @param publicationtype A text string identifying the publication
#'  type within the Neotoma database.
#' @param publicationid ID of publication
#' @param publicationtypeid ID of kind of publication
#' @param author name of the author of publication.
#' @param year The year of publication.
#' @param citation A full text citation for the article.
#' @param articletitle The title of the article.
#' @param journal The journal in which the article was published.
#' @param volume The journal volume.
#' @param issue The journal issue.
#' @param pages The pages of the journal.
#' @param citationnumber How many times has the paper been cited?
#' @param doi A DOI for the record.
#' @param booktitle The title of the book (if the publication is a book)
#' @param numvolumes The number of book volumes (if a series)
#' @param edition The book edition.
#' @param volumetitle The title of the volume (in a published series)
#' @param seriestitle The title of the series.
#' @param seriesvolume The series volume.
#' @param publisher The publisher.
#' @param url Publication URL
#' @param city City of publication.
#' @param state State of publication.
#' @param country Country of publication.
#' @param originallanguage Original language of publication.
#' @param notes Publication notes.
#' @returns `publication` object
#' @description A function to create new publication objects by hand.
#' @md
#' @export
set_publication <- function(
    x = NA,
    publicationid = NA_integer_,
    publicationtypeid = NA_integer_,
    publicationtype = NA_character_,
    year = NA_character_,
    citation = NA_character_,
    articletitle = NA_character_,
    journal = NA_character_,
    volume = NA_character_,
    issue = NA_character_,
    pages = NA_character_,
    citationnumber = NA_character_,
    doi = NA_character_,
    booktitle = NA_character_,
    numvolumes = NA_character_,
    edition = NA_character_,
    volumetitle = NA_character_,
    seriestitle = NA_character_,
    seriesvolume = NA_character_,
    publisher = NA_character_,
    url = NA_character_,
    city = NA_character_,
    state = NA_character_,
    country = NA_character_,
    originallanguage = NA_character_,
    notes = NA_character_,
    author = NULL) {
  if (suppressWarnings(is.na(x))) {
    x <- new("publication")
    if (is.na(publicationid)) {
      hash <- digest(UUIDgenerate(), algo = "xxhash32", serialize = FALSE)
      x@publicationid <- as.integer(strtoi(substr(hash, 1, 7), base = 16L))
    } else {
      x@publicationid <- publicationid
    }
  }
  if (is.null(author)) {
    author <- new("authors", authors = list(new("author")))
  }
  if (is.na(citation)) {
    citation <- paste0(articletitle, journal)
  }
  x@publicationtypeid <- as.numeric(publicationtypeid)
  x@publicationtype <- as.character(publicationtype)
  x@year <- as.character(year)
  x@citation <- as.character(citation)
  x@articletitle <- as.character(articletitle)
  x@journal <- as.character(journal)
  x@volume <- as.character(volume)
  x@issue <- as.character(issue)
  x@pages <- as.character(pages)
  x@citationnumber <- as.character(citationnumber)
  x@doi <- as.character(doi)
  x@booktitle <- as.character(booktitle)
  x@numvolumes <- as.character(numvolumes)
  x@edition <- as.character(edition)
  x@volumetitle <- as.character(volumetitle)
  x@seriestitle <- as.character(seriestitle)
  x@seriesvolume <- as.character(seriesvolume)
  x@publisher <- as.character(publisher)
  x@url <- as.character(url)
  x@city <- as.character(city)
  x@state <- as.character(state)
  x@country <- as.character(country)
  x@originallanguage <- as.character(originallanguage)
  x@notes <- as.character(notes)
  x@author <- author
  return(x)
}

library("testthat")
library("neotoma2")

context("Creating publications from scratch or updating existing objects.")
test_that("`set_publication()` for a new publication", {
  skip_on_cran()
  newPub <- set_publication(publicationid = 56,
                            citation = "Citation for my publication",
                            doi = "fake.doi/10.1234/fake")
  testthat::expect_is(newPub, "publication")
  testthat::expect_equal(newPub@publicationid, 56)
  testthat::expect_equal(newPub@citation, "Citation for my publication")
  testthat::expect_equal(newPub@doi, "fake.doi/10.1234/fake")
})

test_that("Updating existing publication", {
  skip_on_cran()
  # Cannot update over a sites object:
  downloadPub <- get_publications(search = "pollen", limit = 1)
  testthat::expect_error(set_publications(x = downloadPub, doi = "fake.doi/10.1234/fake"))
  testthat::expect_is(set_publication(x = downloadPub[[1]], doi = "fake.doi/10.1234/fake"), "publication")
  updatedPub <- set_publication(x = downloadPub[[1]], doi = "fake.doi/10.1234/fake")
  testthat::expect_equal(updatedPub@doi, "fake.doi/10.1234/fake")
})

#' @title clean Neotoma objects to remove duplicates and empty objects.
#' @name clean
#' @author Simon Goring \email{goring@wisc.edu}
#' @importFrom progress progress_bar
#' @importFrom methods new
#' @param x sites, datasets, collunits that may have duplicates.
#' @param verbose parameter to prints out progress bar
#' @param ... Additional parameters associated with the call.
#' @returns clean `neotoma` objects without duplicates after concatenation
#' @description Function that removes duplicate objects such as sites,
#' datasets, or collection units. When we pull in a large number of objects,
#' or overlapping searches, we can run into a problem where we have multiple
#' instances of the same site, but with different datasets. This function
#' attempts to gather all objects together:
#'   * Before: \{site: 1, dataset: 1\}, \{site: 1, dataset: 2\}
#'   * After: \{site: 1, dataset: \[1, 2\]\}
#' So the site is gathered, and the datasets are now part of an
#' array of datasets.
#' @examples \dontrun{
#' tryCatch({
#'   alex <- get_sites(sitename = "Alex%")
#'   alex2 <- get_sites(24)
#'   c <- c(alex, alex2) #uncleaned
#' }, error = function(e) {
#'   message("Neotoma server not responding. Try again later.")
#' })
#' }
#' @md
#' @export
clean <- function(x, verbose = TRUE, ...) {
  UseMethod("clean")
}

#' @rdname clean
#' @method clean sites
#' @exportS3Method clean sites
clean.sites <- function(x, verbose = TRUE, ...) {
  siteids <- as.data.frame(x)$siteid
  matched <- unique(siteids[duplicated(siteids)])
  non_dupes <- siteids[!duplicated(siteids) & !duplicated(siteids, fromLast = TRUE)]
  if (length(matched) > 0) {
    inter <- intersect(x$siteid, matched)
    clean_sites <- x %>% neotoma2::filter(!(.data$siteid %in% inter))
    messy_sites <- x %>% neotoma2::filter((.data$siteid %in% inter))
    pb <- progress_bar$new(total = length(matched))
    for (i in inter) {
      if (verbose == TRUE) {
        pb$tick()
      }
      messy_site <- neotoma2::filter(messy_sites, .data$siteid == i)
      messy_cus <- clean(collunits(messy_site))
      new_site <- messy_site[1]
      new_site@sites[[1]]@collunits <- messy_cus
      clean_sites <- c(clean_sites, new_site[[1]])
    }
    return(clean_sites)
  } else {
    return(x)
  }
}

#' @rdname clean
#' @method clean collunits
#' @exportS3Method clean collunits
clean.collunits <- function(x, verbose = TRUE, ...) {
  cuids <- x$collectionunitid
  matched <- unique(cuids[duplicated(cuids)])
  non_dupes <- cuids[!duplicated(cuids) & !duplicated(cuids, fromLast = TRUE)]
  if (length(matched) == 0) {
    return(x)
  } else {
    clean_cus <- x[which(!cuids %in% matched)]
    for (i in matched) {
      messy_cus <- new("collunits", collunits = x@collunits[cuids == i])
      cu_ds <- clean(datasets(messy_cus))
      newcu <- messy_cus[1]
      newcu[[1]]@datasets <- cu_ds
      clean_cus <- c(clean_cus, newcu)
    }
  }
  return(clean_cus)
}

#' @rdname clean
#' @method clean datasets
#' @exportS3Method clean datasets
clean.datasets <- function(x, verbose = TRUE, ...) {
  dsids <- as.data.frame(x)$datasetid
  return(x[which(!duplicated(dsids))])
}

#' @title Reformat API author list to Neotoma contacts.
#' @author Simon Goring \email{goring@wisc.edu}
#' @param x A list coming from the `author` element of
#'  the Neotoma publicaiton API
#' @importFrom purrr map
#' @importFrom methods new
#' @returns `author` object
#' @noRd
pubAuthors <- function(x) {
  x <- cleanNULL(x)
  result <- new("authors",
                authors = map(x$author, function(y) {
                  new("author",
                      author = new("contact",
                      familyname = use_na(y$familyname, "char"),
                      givennames = use_na(y$givennames, "char")),
                      order = y$order)
                }))
  return(result)
}

#' @title Build a `site` from the Neotoma API response.
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @param args A list returned from the Neotoma API `data` section.
#' @importFrom assertthat assert_that
#' @importFrom purrr map
#' @importFrom sf st_read
#' @returns A simple `site` object
#' @noRd
build_site <- function(...) {
  args <- list(...)
  args <- cleanNULL(args)
  geo <- if (!is.null(args$geography) && !is.na(args$geography)) {
    st_read(args$geography, quiet = TRUE)
  } else {
    NULL
  }
  assertthat::assert_that(is.list(args),
                          msg = "Parsed object must be a list.")
  site <- set_site(siteid = use_na(args$siteid, "int"),
                   sitename = use_na(args$sitename, "char"),
                   geography = use_na(geo, "sf"),
                   altitude = use_na(args$altitude, "int"),
                   notes = use_na(args$notes, "char"),
                   description = use_na(args$sitedescription, "char"),
                   collunits = args$collunits)
  return(site)
}

#' @title parseURL
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @author Simon Goring \email{goring@wisc.edu}
#' @importFrom httr GET POST add_headers content
#' @importFrom httr stop_for_status modify_url http_error
#' @importFrom jsonlite fromJSON toJSON
#' @importFrom stringr str_extract_all
#' @description An internal helper function used to connect to the Neotoma API
#' in a standard manner, and to provide basic validation of any response.
#' @param x The HTTP/S path for the particular API call.
#' @param use Uses the Neotoma server by default ("neotoma"),
#' but supports either the development API server ("dev"),
#' or a local server ("local").
#' @param all_data If TRUE return all possible API calls
#' @param ... Any query parameters passed from the calling function.
#' @returns `list` with cleaned and parsed data from HTTP request
#' @noRd
parseURL <- function(x, use = "neotoma", all_data = FALSE, ...) {
  if (!Sys.getenv("APIPOINT") == "") {
    use <- Sys.getenv("APIPOINT")
  }
  baseurl <- switch(use,
                    "dev" = "http://api-dev.neotomadb.org/v2.0/",
                    "neotoma" = "https://api.neotomadb.org/v2.0/",
                    "local" = "http://localhost:3001/v2.0/",
                    use)
  query <- list(...)
  get_response <- function(baseurl, x, query) {
    response <-
      tryCatch({
        full_url <-
          modify_url(baseurl,
                     path = file.path("v2.0", x),
                     query = query[setdiff(names(query), "loc")])
        if (nchar(full_url) < 2000 && !("loc" %in% names(query))) {
          GET(paste0(baseurl, x),
              add_headers("User-Agent" = "neotoma2 R package"),
              query = query)
        } else if ("loc" %in% names(query)) {
          query$loc <- parseLocation(query$loc)
          body <- toJSON(query,
                         auto_unbox = TRUE,
                         null = "null")
          parts <- strsplit(x, "/")[[1]]
          path_str <- paste(parts[1:2], collapse = "/")
          baseurl <- paste0(baseurl, path_str)
          response <- POST(paste0(baseurl),
                           body = body,
                           encode = "raw",
                           add_headers("User-Agent" = "neotoma2 R package",
                                       "Content-Type" = "application/json"))
        } else {
          # Numeric Long calls
          parts <- strsplit(x, "/")[[1]]
          path_str <- paste(parts[1:2], collapse = "/")
          parts2 <- sub("\\?.*$", "", path_str)
          baseurl <- paste0(baseurl, parts2)
          if (length(parts) >= 2) {
            resource <- parts[length(parts) - 1]
            value <- parts[length(parts)] # Value limited to Site/DS IDs
            if (grepl("site", parts2)) {
              resource <- "siteid"
            } else if (any(grepl("dataset|download", parts2))) {
              resource <- "datasetid"
            }
          }
          if (length(query) == 0) {
            query$limit <- 50
            value <-  as.numeric(str_extract_all(value, "[0-9.]+")[[1]])
            seq_chunk <- split(value, 
                                ceiling(seq_along(value) / query$limit))
            results <- list()
            for (sequ in seq_chunk) {
              query[[resource]] <- sequ
              body <- toJSON(query,
                              auto_unbox = TRUE,
                              null = "null")
              response <- POST(paste0(baseurl),
                                body = body,
                                encode = "raw",
                                add_headers(
                                "User-Agent" = "neotoma2 R package",
                                "Content-Type" = "application/json"))
              if (http_error(response)) {
                warning("Skipping failed request with status ",
                        response$status_code)
                next
              }
              r <- fromJSON(content(response, as = "text"),
                            flatten = FALSE,
                            simplifyVector = FALSE)
              if (!is.null(r$data)) {
                results <- c(results, r$data)
              }
            }
            response <- list(status = 200,
                              data = results, 
                              message = "Success")
            return(response)
          } else {
            # Convert query to JSON
            if (!resource %in% names(query)) {
              query[[resource]] <- value
            }
            body <- toJSON(query, auto_unbox = TRUE, null = "null")
            POST(paste0(baseurl),
                 body = body,
                 encode = "raw",
                 add_headers("User-Agent" = "neotoma2 R package",
                             "Content-Type" = "application/json"))
          }
        }
      }, error = function(e) {
        if (grepl("SSL certificate", conditionMessage(e), ignore.case = TRUE)) {
          stop("SSL certificate error: ", conditionMessage(e),
               "\nPlease contact the Neotoma team.")
          return(NULL)
        } else {
          stop("API request failed: ", conditionMessage(e),
               "\nCheck that the API path is valid or visit:
               http://data.neotomadb.org")
          return(NULL)
        }
      })

    if (is.null(response) ||
          isTRUE(inherits(response, "response") && http_error(response))) {
      if (is.null(response)) {
        stop("Error: Check your R Code.")
      }
      if (response$status_code == 429) {
        stop("Error 429: Too many requests. Please wait and retry.")
      } else {
        stop_for_status(response,
                        task = "Could not connect to the Neotoma API.
                                Check that the path is valid, and 
                                check the current status of the
                                Neotoma API services at 
                                http://data.neotomadb.org")
      }
    }
    return(response)
  }

  if (all_data == FALSE) {
    response <- get_response(baseurl, x, query)
    if (inherits(response, "response")) {
      result <- fromJSON(content(response, as = "text"),
                         flatten = FALSE,
                         simplifyVector = FALSE)
    } else {
      result <- response
    }
  } else {
    query$offset <- 0
    query$limit <- 2000
    responses <- c()
    while (TRUE) {
      response <- get_response(baseurl, x, query)
      r <- fromJSON(content(response, as = "text"),
                    flatten = FALSE,
                    simplifyVector = FALSE)
      r <- cleanNULL(r)
      if (is.null(r$data) || length(r$data) == 0) {
        break
      }
      responses <- c(responses, r$data)
      query$offset <- query$offset + query$limit
    }
    result <- list(status = 200,
                   data = responses,
                   message = "Success")
  }
  return(result)
}

#' @title c Method - Combine objects, including NULL
setClassUnion("missingOrNULL", c("missing", "NULL"))

#' @aliases add_chronology,collunit-method
#' @rdname add_chronology
setMethod(f = "add_chronology",
          signature = signature(object = "collunit",
                                x = "chronology",
                                y = "data.frame"),
          definition <- function(object, x, y) {
            existingIds <- as.data.frame(object@chronologies)$chronologyid
            existinganalysisIds <- map(object@datasets@datasets,
                                       function(x) {
                                         map(x@samples@samples,
                                             function(y) {
                                               y$analysisunitid
                                             }) %>%
                                           unlist()
                                       })
            if (x$chronologyid %in% existingIds) {
              stop("There is already a chronology with the same ID as your 
                    new chronology.  Please change the new chronologyid.")
            }
            assert_that(all(c("analysisunitid", "age", "agetype", "ageolder",
                              "ageyounger") %in% colnames(y)),
                        msg = "The data.frame for sample ages must
                              contain the columns analysisunitid,
                              age, agetype, ageolder and ageyounger.")
            if (!any(unlist(existinganalysisIds) %in% y$analysisunitid)) {
              stop("There is no overlap between the existing analysis units and
                    the anaysis units in your new sample ages data.frame.")
            } else if (!all(unlist(existinganalysisIds)
                            %in% y$analysisunitid)) {
              warning("Not all of the existing analysis units are 
                          represented in the new sample ages data.frame. 
                          Analysis units without ages will have NA
                          values assigned.")
            }
            object@datasets@datasets <-
              map(object@datasets@datasets,
                  function(z) {
                    z@samples@samples <-
                      map(z@samples@samples,
                          function(a) {
                            auid <- a@analysisunitid
                            if (auid %in% y$analysisunitid) {
                              sampleagerow <- y %>%
                                dplyr::filter(.data$analysisunitid == auid)
                              sample_ages <-
                                data.frame(age = sampleagerow$age,
                                           agetype = sampleagerow$agetype,
                                           ageolder = sampleagerow$ageolder,
                                           ageyounger = sampleagerow$ageyounger,
                                           chronologyid = x$chronologyid,
                                           chronologyname =
                                           x$chronologyname)
                              a@ages <- rbind(sample_ages, a@ages)
                            }
                      return(a)
                    })
                    return(z)
                  })
            object@chronologies <- c(object@chronologies, x)
            if (x@isdefault == 1) {
              object@chronologies <- set_default(object@chronologies,
                                                 x$chronologyid)
            }
            return(object)
          })

#' @aliases show,collunits-method
#' @rdname show
setMethod(f = "show",
          signature = signature(object = "collunits"),
          definition = function(object) {
            result <- map(object@collunits, function(x) {
              as.data.frame(x)
            }) %>%
              bind_rows() %>%
              print(row.names = FALSE)
          })

#' @aliases show,collunit-method
#' @rdname show
setMethod(f = "show",
          signature = signature(object = "collunit"),
          definition = function(object) {
            result <- as.data.frame(object)
            print(result, row.names = FALSE)
          })

#' @aliases sub,collunits-method
#' @rdname sub
setMethod(f = "[",
          signature = signature(x = "collunits", i = "numeric"),
          definition = function(x, i) {
            new("collunits", collunits = x@collunits[i])
          })

#' @aliases sub-sub,collunits-method
#' @rdname sub-sub
setMethod(f = "[[",
          signature = signature(x = "collunits", i = "numeric"),
          definition = function(x, i) {
            if (length(i) == 1) {
              out <- new("collunit", x@collunits[[i]])
            } else {
              out <- map(i, function(z) {
                new("collunit", x@collunits[[z]])
              })
              out <- new("collunits", collunits = out)
            }
            return(out)
          })

#' @aliases names,collunit-method
#' @rdname names
setMethod(f = "names",
          signature = signature(x = "collunit"),
          definition = function(x) {
            slotNames(x)
          })

#' @aliases sub-subset,collunits-method
#' @rdname sub-subset
setMethod(f = "[[<-",
          signature = signature(x = "collunits"),
          definition = function(x, i, value) {
            collunitset <- x@collunits
            collunitset[[i]] <- value
            out <- new("collunits", collunits = collunitset)
            return(out)
          })

#' @aliases subset,collunit-method
#' @rdname subset
setMethod(f = "[<-",
          signature = signature(x = "collunit", i = "character"),
          definition = function(x, i, value) {
            for (idx in 1:length(i)) {
              slot(x, i[idx]) <- value[idx]
            }
            return(x)
          })

#' @aliases subset,collunit-method
#' @rdname subset
setMethod(f = "[<-",
          signature = signature(x = "collunit", i = "numeric"),
          definition = function(x, i, value) {
            slots <- slotNames(x)
            for (idx in seq_along()(i)) {
              slot(x, slots[i[idx]]) <- value[idx]
            }
            return(x)
          })

#' @aliases cash-set,collunit-method
#' @rdname cash-set
setMethod(f = "$<-",
          signature = signature(x = "collunit"),
          definition = function(x, name, value) {
            slot(x, name) <- value
            return(x)
          })

#' @aliases cash,collunit-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "collunit"),
          definition = function(x, name) {
            slot(x, name)
          })

#' @aliases cash,collunits-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "collunits"),
          definition = function(x, name) {
            x %>%
              map(function(y) {
                slot(y, name)
              }) %>%
              unlist()
          })

#' @aliases as.data.frame,collunit-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("collunit"),
          definition = function(x) {
            data.frame(collectionunitid = x@collectionunitid,
                       handle = x@handle,
                       colldate = x@colldate,
                       location = x@location,
                       waterdepth = x@waterdepth,
                       collunittype = x@collunittype,
                       collectiondevice = x@collectiondevice,
                       defaultchronology = x@defaultchronology,
                       collectionunitname = x@collectionunitname,
                       depositionalenvironment = x@depositionalenvironment)
          })

#' @aliases as.data.frame,collunits-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("collunits"),
          definition = function(x) {
            df <- x@collunits %>% map(as.data.frame) %>% bind_rows()
          })

#' @aliases length,collunits-method
#' @rdname length
setMethod(f = "length",
          signature = signature(x = "collunits"),
          definition = function(x) {
            length(x@collunits)
          })

#' @aliases c,collunits-method
#' @rdname c
setMethod(f = "c",
          signature = signature(x = "collunits"),
          definition = function(x, y) {
            if (is(y, "collunits")) {
              cus_l <- c(x@collunits,
                         y@collunits)
              if (is.null(names(cus_l)) || all(names(cus_l) == "")) {
                # Case 1: Remove duplicates by content
                cus_l <- cus_l[!duplicated(cus_l)]
              } else {
                # Case 2: Remove duplicates by name
                cus_l <- cus_l[!duplicated(names(cus_l))]
              }
              out <- new("collunits",
                         collunits = unlist(cus_l,
                                            recursive = FALSE))
            } else if (is(y, "collunit")) {
              collunitset <- c(x@collunits, y)
              out <- new("collunits", collunits = collunitset)
            }
            return(out)
          })

#' @title Set contact information for a new record.
#' @description Create a new contact for a record.
#' Within Neotoma all chronologies have unique numeric identifiers. 
#' Within R, because of the need to use the indentifiers across objects,
#' and because we want to avoid conflicts between naming systems, a 
#' universally unique identifier (UUID) is created for the object ID.This is not
#' to be updated to the database.
#' @importFrom methods is new slot<-
#' @importFrom uuid UUIDgenerate
#' @importFrom digest digest
#' @param x Object to be set as a contact
#' @param contactid An arbitrary Contact identification number.
#' @param familyname Family or surname name of a person.
#' @param leadinginitials Leading initials for given or forenames
#' without spaces (e.g. \code{G.G.}).
#' @param givennames Given or forenames of a person (e.g.
#' \code{George Gaylord}). Initials with spaces are used if full
#' given names are not known (e.g. \code{G. G}).
#' @param contactname Full name of a person (e.g. \code{George Gaylord Simpson}).
#' @param suffix Suffix of a person’s name (e.g. \code{Jr.}, \code{III}).
#' @param ORCID A unique ORCID (see \url{https://orcid.org}).
#' @param title A person’s title (e.g. \code{Dr.}, \code{Prof.},
#' \code{Prof. Dr}).
#' @param institution The institution where an individual works.
#' @param email An individuals email address
#' @param phone Their phone number
#' @param contactstatus Are they "active" or "retired"?
#' @param fax Do people still use fax machines?
#' @param url Their homepage
#' @param address A physical address
#' @param notes Notes about the individual
#' @returns `contact` object
#' @md
#' @export
set_contact <- function(x = NA,
                        contactid = NA_integer_,
                        familyname = NA_character_,
                        leadinginitials = NA_character_,
                        givennames = NA_character_,
                        contactname = NA_character_,
                        suffix = NA_character_,
                        ORCID = NA_character_,
                        title = NA_character_,
                        institution = NA_character_,
                        email = NA_character_,
                        phone = NA_character_,
                        contactstatus = NA_character_,
                        fax = NA_character_,
                        url = NA_character_,
                        address = NA_character_,
                        notes = NA_character_) {
  function_call <- match.call()
  if (suppressWarnings(is.na(x))) {
    x <- new("contact")
    if (is.na(contactid)) {
      hash <- digest(UUIDgenerate(), algo = "xxhash32", serialize = FALSE)
      x@contactid <- as.integer(strtoi(substr(hash, 1, 7), base = 16L))
    } else {
      x@contactid <- contactid
    }
    x@familyname <- familyname
    x@givennames <- givennames
    x@leadinginitials <- leadinginitials
    x@contactname <- contactname
    x@suffix <- suffix
    x@ORCID <- ORCID
    x@title <- title
    x@institution <- institution
    x@email <- email
    x@phone <- phone
    x@contactstatus <- contactstatus
    x@fax <- fax
    x@url <- url
    x@address <- address
    x@notes <- notes
  } else {
    if (is(x, "contact")) {
      if (length(function_call) > 2) {
        for (i in 3:length(function_call)) {
          slot(x, names(function_call)[[i]]) <- eval(function_call[[i]])
        }
        return(x)
      } else {
        return(x)
      }
    } else {
      stop("`x` must be a contact object if it is supplied.")
    }
  }
  return(x)
}

#' Pipe operator
#' See \code{magrittr::\link[magrittr:pipe]{\%>\%}} for details.
#' @name %>%
#' @rdname pipe
#' @keywords internal
#' @export
#' @importFrom magrittr %>%
#' @usage lhs \%>\% rhs
#' @returns NULL; allows using tidyverse pipe operator.
NULL

#' @title set Site Information for Fossil Sites
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @author Simon Goring \email{goring@wisc.edu}
#' @importFrom methods is new slot<-
#' @importFrom sf st_as_sf st_sfc
#' @importFrom uuid UUIDgenerate
#' @importFrom digest digest
#' @param x object to be set as collunit
#' @param collectionunitid collection unit identifier
#' @param notes notes
#' @param handle handle
#' @param colldate collection date
#' @param location location of the collection unit
#' @param waterdepth depth at where the sample is taken
#' @param gpslocation location with GPS
#' @param collunittype type of collection unit
#' @param collectiondevice device used to collect the sample
#' @param collectionunitname name of the collection unit
#' @param depositionalenvironment depositional environment
#' @param datasets datasets that the collection unit has
#' @param chronologies chronologies taken from the collection unit
#' @param defaultchronology best chronology model identifier to
#' be used with this collection unit
#' @param speleothems speleothems associated with the collection unit
#' @returns `collunit` object
#' @description Function to create new collection unit objects for
#' personal analysis.
#' The new object will not be uploaded to the database.
#' @export
#' @md
#' @examples {
#' # Create a collunit
#' my_collunit <- set_collunit(notes = "my lake")
#' }
set_collunit <- function(x = NA,
                         collectionunitid = NA_integer_,
                         notes = NA_character_,
                         handle = NA_character_,
                         colldate = as.Date(character(1)),
                         location = NA_character_,
                         waterdepth = NA_integer_,
                         gpslocation = st_as_sf(st_sfc()),
                         collunittype = NA_character_,
                         collectiondevice = NA_character_,
                         collectionunitname = NA_character_,
                         depositionalenvironment = NA_character_,
                         datasets = NULL,
                         chronologies = NULL,
                         defaultchronology = NA_integer_,
                         speleothems = NULL) {
  function_call <- match.call()
  if (suppressWarnings(is.na(x))) {
    x <- new("collunit")
    if (is.na(collectionunitid)) {
      hash <- digest(UUIDgenerate(), algo = "xxhash32", serialize = FALSE)
      x@collectionunitid <- as.integer(strtoi(substr(hash, 1, 7), base = 16L))
    } else {
      x@collectionunitid <- collectionunitid
    }
    x@notes <- notes
    x@handle <- handle
    x@colldate <- colldate
    x@location <- location
    x@waterdepth <- waterdepth
    x@gpslocation <- gpslocation
    x@collunittype <- collunittype
    x@collectiondevice <- collectiondevice
    x@collectionunitname <- collectionunitname
    x@depositionalenvironment <- depositionalenvironment
    x@datasets <- datasets
    x@chronologies <- chronologies
    x@defaultchronology <- defaultchronology
    x@speleothems <- speleothems
  } else {
    if (is(x, "collunit")) {
      if (length(function_call) > 2) {
        for (i in 3:length(function_call)) {
          slot(x, names(function_call)[[i]]) <- eval(function_call[[i]])
        }
        return(x)
      } else {
        return(x)
      }
    } else {
      stop("`x` must be a collunit object if it is supplied.")
    }
  }
  return(x)
}

#' @title set Sample Information
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom uuid UUIDgenerate
#' @importFrom digest digest
#' @importFrom methods is new slot<-
#' @param x Object to be set as a sample
#' @param ages ages
#' @param igsn IGSN character
#' @param datum dataframe of datum
#' @param depth integer representing depth
#' @param sampleid ID for sample
#' @param thickness thickness of core
#' @param samplename sample's name
#' @param sampleanalyst Analyst's contact name
#' @param analysisunitid Which analysis unit it is
#' @param analysisunitname Analysis Unit's name
#' @returns `sample` object
#' @description Function to create new samples objects for analysis. 
#' The new object will not be uploaded to the database.
#' @export
#' @md
#' @examples {
#' # Set an empty sample
#' my_sample <- set_sample()
#' }
set_sample <- function(x = NA,
                       ages = list(),
                       igsn = NA_character_,
                       datum = data.frame(),
                       depth = NA_integer_,
                       sampleid = NA_integer_,
                       thickness = NA_integer_,
                       samplename = NA_character_,
                       sampleanalyst = list(),
                       analysisunitid = NA_integer_,
                       analysisunitname = NA_character_){
  function_call <- match.call()
  if (suppressWarnings(is.na(x))) {
    x <- new("sample")
    if (is.na(sampleid)) {
      hash <- digest(UUIDgenerate(), algo = "xxhash32", serialize = FALSE)
      x@sampleid <- as.integer(strtoi(substr(hash, 1, 7), base = 16L))
    } else {
      x@sampleid <- sampleid
    }
    x@ages <- ages
    x@igsn <- igsn
    x@datum <- datum
    x@depth <- depth
    x@thickness <- thickness
    x@samplename <- samplename
    x@sampleanalyst <- sampleanalyst
    x@analysisunitid <- analysisunitid
    x@analysisunitname <- analysisunitname
  } else {
    if (is(x, "sample")) {
      if (length(function_call) > 2) {
        for (i in 3:length(function_call)) {
          slot(x, names(function_call)[[i]]) <- eval(function_call[[i]])
        }
        return(x)
      } else {
        return(x)
      }
    } else {
      stop("`x` must be a sample object if it is supplied.")
    }
  }
  return(x)
}

#' @aliases names,contact-method
#' @rdname names
setMethod(f = "names",
          signature = signature(x = "contact"),
          definition = function(x) {
            slotNames(x)
          })


#' @aliases show,contacts-method
#' @rdname show
setMethod(f = "show",
          signature = "contacts",
          definition = function(object) {
            as.data.frame(object) %>%
              bind_rows() %>%
              print(row.names = FALSE)
          })

#' @aliases show,contact-method
#' @rdname show
setMethod(f = "show",
          signature = "contact",
          definition = function(object) {
            as.data.frame(object) %>%
              print(row.names = FALSE)
          })

#' @aliases sub-sub,contacts-method
#' @rdname sub-sub
setMethod(f = "[[",
          signature = signature(x = "contacts", i = "numeric"),
          definition = function(x, i) {
            x@contacts[[i]]
          })

#' @aliases cash,contact-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "contact"),
          definition = function(x, name) {
            slot(x, name)
          })

#' @aliases cash,contacts-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "contacts"),
          definition = function(x, name) {
            x %>%
              map(function(y) {
                slot(y, name)
              }) %>%
              unlist()
          })

#' @aliases as.data.frame,contact-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("contact"),
          definition = function(x) {
            data.frame(contactid = testNull(x@contactid),
                       familyname = testNull(x@familyname),
                       givennames = testNull(x@givennames),
                       contactname = testNull(x@contactname),
                       ORCID = testNull(x@ORCID),
                       url = testNull(x@url),
                       contactstatus = testNull(x@contactstatus),
                       notes = testNull(x@notes))
          })

#' @aliases as.data.frame,contacts-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("contacts"),
          definition = function(x) {
            x@contacts %>%
              map(as.data.frame) %>%
              bind_rows()
          })

#' @aliases length,contacts-method
#' @rdname length
setMethod(f = "length",
          signature = signature(x = "contacts"),
          definition = function(x) {
            length(x@contacts)
          })

#' @aliases c,contacts-method
#' @rdname c
setMethod(f = "c",
          signature = signature(x = "contacts"),
          definition = function(x, y) {
            if (is(y, "contacts")) {
              out <- new("contacts",
                         contacts = unlist(c(x@contacts,
                                             y@contacts),
                                           recursive = FALSE))
            } else if (is(y, "contact")) {
              contactset <- c(x@contacts, y)
              out <- new("contacts", contacts = contactset)
            }
            return(out)
          })

#' @aliases c,contact-method
#' @rdname c
setMethod(f = "c",
          signature = signature(x = "contact"),
          definition = function(x, y) {
            if (is(y, "contact")) {
              out <- new("contacts",
                         contacts = list(x, y))
            } else if (is(y, "contacts")) {
              contactset <- c(x@contacts, y)
              out <- new("contacts", contacts = contactset)
            }
            return(out)
          })

#' @title Set Neotoma API Source or Server
#' @author Simon Goring \email{goring@wisc.edu}
#' @importFrom assertthat assert_that
#' @param server One of \code{local} (when the API is running locally on
#' port 3005), \code{neotoma} or \code{dev}.
#' @examples \dontrun{
#' # The user is running the API locally using the node/express API
#' # cloned from github: https://github.com/NeotomaDB/api_nodetest
#' set_server(server = "local")
#' # The user switches back to the remote api server.
#' set_server(server = "neotoma")
#' }
#' @returns NULL modifies how to talk to the API (local, dev, server)
#' @description
#' Choose to pull Neotoma data from the main Neotoma server, the development
#' server or from a local instance of the API.
#' @export
set_server <- function(server = "neotoma") {
  assert_that(server %in% c("neotoma", "dev", "local"),
              msg = "The parameter `server` must be `local`, 
                    `neotoma` or `dev`.")
  server <- switch(server,
                   neotoma = "neotoma",
                   dev = "dev",
                   local = "local")

  Sys.setenv("APIPOINT" = server)
}

library("testthat")
library("neotoma2")

context("Switching servers works between local, dev and neotoma APIs.")
testthat::test_that("`Switching server with `set_server()`.", {
  skip_on_cran()
  set_server("dev")
  testthat::expect_true(Sys.getenv("APIPOINT") == "dev")
  set_server("local")
  testthat::expect_true(Sys.getenv("APIPOINT") == "local")
  set_server("neotoma")
  testthat::expect_true(Sys.getenv("APIPOINT") == "neotoma")
})

testthat::test_that("Invalid server throws error.", {
  skip_on_cran()
  testthat::expect_error(set_server("invalid_server_name"))
})

#' @title set Site Information for Fossil Sites
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom methods is new slot<-
#' @importFrom uuid UUIDgenerate
#' @importFrom digest digest
#' @param x object to be set as dataset,
#' @param datasetid dataset identifier
#' @param database dataset where the dataset came from
#' @param doi DOI
#' @param recdatecreated date the dataset was created
#' @param datasettype type the dataset belongs to
#' @param datasetname name of the dataset
#' @param age_range_old age range old
#' @param age_range_young age range young
#' @param age_units age units
#' @param specimens specimens slot
#' @param notes notes
#' @param pi_list pi list
#' @param samples taxa objects
#' @export
#' @description Function to create new `dataset` objects for personal analysis. 
#' The new object will not be uploaded to the database.
#' @returns `dataset` object
#' @examples {
#' # Create a dataset
#' my_dataset <- set_dataset(database = "EPD",
#'                     datasettype = "pollen",
#'                     notes = "my lake")
#' }
set_dataset <- function(x = NA,
                        datasetid = NA_integer_,
                        datasetname = NA_character_,
                        database = NA_character_,
                        doi = NA,
                        recdatecreated = as.Date(character(1)),
                        datasettype = NA_character_,
                        age_range_old = NA_integer_,
                        age_range_young = NA_integer_,
                        age_units = NA_character_,
                        notes = NA_character_,
                        pi_list = NA,
                        samples = NULL,
                        specimens = NULL) {
  function_call <- match.call()
  if (suppressWarnings(is.na(x))) {
    x <- new("dataset")
    if (is.na(datasetid)) {
      hash <- digest(UUIDgenerate(), algo = "xxhash32", serialize = FALSE)
      x@datasetid <- as.integer(strtoi(substr(hash, 1, 7), base = 16L))
    } else {
      x@datasetid <- datasetid
    }
    x@datasetname <- datasetname
    x@database <- database
    x@doi <- doi
    x@recdatecreated <- recdatecreated
    x@datasettype <- datasettype
    x@age_range_old <- age_range_old
    x@age_range_young <- age_range_young
    x@age_units <- age_units
    x@notes <- notes
    x@pi_list <- pi_list
    x@samples <- samples
  } else {
    if (is(x, "dataset")) {
      if(length(function_call)>2){
        for (i in 3:length(function_call)) {
          slot(x, names(function_call)[[i]]) <- eval(function_call[[i]])
        }
        return(x)
      } else {
        return(x)
      }
    } else {
      stop("`x` must be a dataset object if it is supplied.")
    }
  }
  return(x)
}

#' @title Build a samples `data.frame` from Neotoma API JSON
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom purrr map
#' @importFrom dplyr bind_rows
#' @importFrom methods new
#' @description
#' Helper function to build a sample from the API input (list formatted)
#' coming from the Neotoma API.
#' @param x sample list
#' @returns A simple `sample` object
#' @noRd
build_sample <- function(x) {
  df <- x$datum %>%
          map(function(y) {
            y <- map(y, testNull)
          }) %>%
          bind_rows()
  df_age <- x$ages %>%
              map(function(y) {
                y <- map(y, testNull)
                as.data.frame(y)
              }) %>%
              bind_rows()
  analyst_list_helper <- x$sampleanalyst %>%
                            map(function(y) {
                              y$contactname
                            })
  new_sample <- new("sample",
                    ages = df_age,
                    igsn = use_na(x$igsn, "char"),
                    datum = df,
                    depth = use_na(x$depth, "int"),
                    sampleid = use_na(x$sampleid, "int"),
                    thickness = use_na(x$thickness, "int"),
                    samplename = use_na(x$samplename, "char"),
                    sampleanalyst = analyst_list_helper,
                    analysisunitid = use_na(x$analysisunitid, "int"),
                    analysisunitname = use_na(x$analysisunitname, "char"))
  return(new_sample)
}

#' @aliases show,taxon-method
#' @rdname show
setMethod(f = "show",
          signature = "taxon",
          definition = function(object) {
            print(data.frame(taxonid = as.character(object@taxonid),
                             taxoncode = object@taxoncode,
                             taxonname = object@taxonname,
                             author =  object@author,
                             ecolgroup =  object@ecolgroup,
                             highertaxonid = object@highertaxonid,
                             status = object@status,
                             taxagroupid = object@taxagroupid,
                             publicationid = object@publicationid,
                             publication = object@publication),
                  row.names = FALSE)
          })

#' @aliases show,taxa-method
#' @rdname show
setMethod(f = "show",
          signature = "taxa",
          definition = function(object) {
            map(object@taxa, function(y) {
              data.frame(taxonid = as.character(y@taxonid),
                         taxoncode = y@taxoncode,
                         taxonname = y@taxonname,
                         author =  y@author,
                         ecolgroup =  y@ecolgroup,
                         highertaxonid = y@highertaxonid,
                         status = y@status,
                         taxagroupid = y@taxagroupid,
                         publicationid = y@publicationid,
                         publication = y@publication)
            }) %>%
              bind_rows() %>%
              print(row.names = FALSE)
          })

#' @aliases sub-sub,taxa-method
#' @rdname sub-sub
setMethod(f = "[[",
          signature = signature(x = "taxa", i = "numeric"),
          definition = function(x, i) {
            if (length(i) == 1) {
              out <- new("taxon", x@taxa[[i]])
            } else {
              out <- purrr::map(i, function(z) {
                new("taxon", x@taxa[[z]])
              })
              out <- new("taxa", taxa = out)
            }
            return(out)
          })

#' @aliases names,taxon-method
#' @rdname names
setMethod(f = "names",
          signature = signature(x = "taxon"),
          definition = function(x) {
            slotNames(x)
          })

#' @aliases sub-subset,taxa-method
#' @rdname sub-subset
setMethod(f = "[[<-",
          signature = signature(x = "taxa"),
          definition = function(x, i, value) {
            taxaset <- x@taxa
            taxaset[[i]] <- value
            out <- new("taxa", taxa = taxaset)
            return(out)
          })

#' @aliases subset,taxon-method
#' @rdname subset
setMethod(f = "[<-",
          signature = signature(x = "taxon", i = "character"),
          definition = function(x, i, value) {
            for (idx in seq_along(i)) {
              slot(x, i[idx]) <- value[idx]
            }
            return(x)
          })

#' @aliases subset,taxon-method
#' @rdname subset
setMethod(f = "[<-",
          signature = signature(x = "taxon", i = "numeric"),
          definition = function(x, i, value) {
            slots <- slotNames(x)
            for (idx in seq_along(i)) {
              slot(x, slots[i[idx]]) <- value[idx]
            }
            return(x)
          })

#' @aliases cash-set,taxon-method
#' @rdname cash-set
setMethod(f = "$<-",
          signature = signature(x = "taxon"),
          definition = function(x, name, value) {
            slot(x, name) <- value
            return(x)
          })

#' @aliases sub,taxa-method
#' @rdname sub
setMethod(f = "[",
          signature = signature(x = "taxa", i = "numeric"),
          definition = function(x, i) {
            new("taxa", taxa = x@taxa[i])
          })

#' @aliases cash,taxon-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "taxon"),
          definition = function(x, name) {
            slot(x, name)
          })

#' @aliases cash,taxa-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "taxa"),
          definition = function(x, name) {
            x %>%
              map(function(y) {
                slot(y, name)
              }) %>%
              unlist()
          })

#' @aliases as.data.frame,taxon-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("taxon"),
          definition = function(x) {
            data.frame(taxonid = as.character(x@taxonid),
                       taxoncode = x@taxoncode,
                       taxonname = x@taxonname,
                       author =  x@author,
                       ecolgroup =  x@ecolgroup,
                       highertaxonid = x@highertaxonid,
                       status = x@status,
                       taxagroupid = x@taxagroupid,
                       publicationid = x@publicationid,
                       publication = x@publication)
          })

#' @aliases as.data.frame,taxa-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
  signature = "taxa",
  definition = function(x) {
    df <- map(x@taxa, function(y) {
      data.frame(
        taxonid = as.character(y@taxonid),
        taxoncode = y@taxoncode,
        taxonname = y@taxonname,
        author = y@author,
        ecolgroup = y@ecolgroup,
        highertaxonid = y@highertaxonid,
        status = y@status,
        taxagroupid = y@taxagroupid,
        publicationid = y@publicationid,
        publication = y@publication
      )
    }) %>%
      bind_rows()
    return(df)
  }
)

#' @aliases length,taxa-method
#' @rdname length
setMethod(f = "length",
          signature = signature(x = "taxa"),
          definition = function(x) {
            length(x@taxa)
          })

#' @aliases c,taxa-method
#' @rdname c
setMethod(f = "c",
          signature = signature(x = "taxa"),
          definition = function(x, y) {
            new("taxa",
                taxa = unlist(c(x@taxa,
                                y@taxa),
                              recursive = FALSE))
          })

#' @title parse_speleothem
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom purrr map
#' @importFrom methods new
#' @description An internal helper function used to parse the Neotoma API
#' content of speleothems into `neotoma2R` objects.
#' @param data The API response.
#' @returns `list` with cleaned and parsed data from HTTP request
#' @keywords internal
#' @noRd
parse_speleothem <- function(data) {
  speleothems <- map(data, function(x) {
    x <- x$speleothem
    if (x$entrancedistanceunits == 37) {
      x$entrancedistanceunits <- "m"
    }
    sp <-
      list(entityid = x$entityid,
           entityname = x$entityname,
           siteid = x$siteid,
           collectionunitid = x$collectionunitid,
           datasetid = x$datasetid,
           dripheight = use_na(x$dripheight, "int"),
           monitoring = use_na(x$monitoring, "bool"),
           geology = use_na(x$geology, "char"),
           relativeage = use_na(x$rockage, "char"),
           speleothemtype = use_na(x$speleothemtype, "char"),
           dripheightunits = use_na(x$dripheightunits, "char"),
           entitycovertype = use_na(x$entitycovertype, "char"),
           entrancedistance = use_na(x$entrancedistance, "int"),
           entrancedistanceunits = use_na(x$entrancedistanceunits, "char"),
           landusecovertype = use_na(x$landusecovertype, "char"),
           speleothemdriptype = use_na(x$speleothemdriptype, "char"),
           landusecoverpercent = use_na(x$landusecoverpercent, "int"),
           vegetationcovertype = use_na(x$vegetationcovertype, "char"),
           entitycoverthickness = use_na(x$entitycoverthickness, "int"),
           vegetationcoverpercent = use_na(x$vegetationcoverpercent, "int"))
    do.call(build_speleothem, sp)
  })
  if (is.null(speleothems) || all(sapply(speleothems, is.null))) {
    speleo <- new("speleothems", speleothems = list())
  } else {
    speleo <- new("speleothems", speleothems = speleothems)
  }
  return(speleo)
}

#' @importFrom rlang .data
NULL

setClassUnion("id", c("character", "integer", "numeric"))
#' @title An S4 class for multi-contact information from the Neotoma
#' Paleoecology Database.
#' @name contacts_classes
#' @description An unordered list of individual S4 `contact` objects.
#' @export
#' @examples {
#' new("contact", familyname = "Goring", givennames = "Simon J.")
#' }
#' @returns object of class `contact`
#' @aliases contact-class
#' @md
#' @export
setClass("contact",
         representation(contactid = "id",
                        familyname = "character",
                        leadinginitials = "character",
                        givennames = "character",
                        contactname = "character",
                        suffix = "character",
                        ORCID = "character",
                        title = "character",
                        institution = "character",
                        email = "character",
                        phone = "character",
                        contactstatus = "character",
                        fax = "character",
                        url = "character",
                        address = "character",
                        notes = "character"),
         prototype(contactid = NA_integer_,
                   familyname = NA_character_,
                   leadinginitials = NA_character_,
                   givennames = NA_character_,
                   contactname = NA_character_,
                   suffix = NA_character_,
                   ORCID = NA_character_,
                   title = NA_character_,
                   institution = NA_character_,
                   email = NA_character_,
                   phone = NA_character_,
                   contactstatus = NA_character_,
                   fax = NA_character_,
                   url = NA_character_,
                   address = NA_character_,
                   notes = NA_character_))

#' @aliases contacts-class
#' @rdname contacts_classes
#' @export
setClass("contacts",
         representation(contacts  = "list"),
         validity = function(object) {
           all(map(object@contacts, function(x) {
             class(x) == "contact"
           }) %>%
             unlist())
         })

#' @title An S4 class for the authors of a Neotoma publication.
#' @description This class combines the S4 class `contact` with a numeric
#' author order. This allows us to reuse `contact` objects, and to assign
#' the authorship order within a publication. The full set of authors for
#' a publication are represented by the `authors` object.
#' @importFrom purrr map
#' @examples {
#' simon <- new("contact", familyname = "Goring", givennames = "Simon J.")
#' firstauthor <- new("author", author = simon, order = 1)
#' }
#' @returns object of class `author`
#' @aliases author-class
#' @md
#' @name author_classes
#' @export
setClass("author",
         representation(author = "contact",
                        order = "numeric"),
         prototype(author = new("contact"),
                   order = NA_integer_))

#' @aliases authors-class
#' @rdname author_classes
#' @export
setClass("authors",
         representation(authors = "list"),
         validity = function(object) {
           all(map(object@authors,
                   function(x) {
                     class(x) == "author"}) %>%
             unlist())
         })

#' @title An S4 class for Neotoma `publications`.
#' @name publications_classes
#' @description A publication is liked to an individual Neotoma dataset object
#' They are grouped using an S4 `publications` class.
#' This `publications` class allows a single dataset to have one
#' or more publication classes associated with it.
#' @examples {
#' simon <- new("contact", familyname = "Goring", givennames = "Simon J.")
#' socorro <- new("contact", familyname = "Dominguez", givennames = "Socorro")
#' first_author <- new("author", author = simon, order = 1)
#' second_author <- new("author", author = socorro, order = 2)
#' author_list <- new("authors", authors = list(first_author, second_author))
#' pub <- new("publication",
#'            articletitle = "Top or bottom: Best toast spreading surfaces.",
#'            journal = "Peanut Butter Science",
#'            year = "2022",
#'            volume = "2",
#'            author = author_list)
#'            }
#' @returns object of class `publication`
#' @aliases publication-class
#' @md
#' @export
setClass("publication",
         representation(publicationid = "id",
                        publicationtypeid = "numeric",
                        publicationtype = "character",
                        year = "character",
                        citation = "character",
                        articletitle = "character",
                        journal = "character",
                        volume = "character",
                        issue = "character",
                        pages = "character",
                        citationnumber = "character",
                        doi = "character",
                        booktitle = "character",
                        numvolumes = "character",
                        edition = "character",
                        volumetitle = "character",
                        seriestitle = "character",
                        seriesvolume = "character",
                        publisher = "character",
                        url = "character",
                        city = "character",
                        state = "character",
                        country = "character",
                        originallanguage = "character",
                        notes = "character",
                        author = "authors"),
         prototype(publicationid = NA_integer_,
                   publicationtypeid = NA_integer_,
                   publicationtype = NA_character_,
                   year = NA_character_,
                   citation = NA_character_,
                   articletitle = NA_character_,
                   journal = NA_character_,
                   volume = NA_character_,
                   issue = NA_character_,
                   pages = NA_character_,
                   citationnumber = NA_character_,
                   doi = NA_character_,
                   booktitle = NA_character_,
                   numvolumes = NA_character_,
                   edition = NA_character_,
                   volumetitle = NA_character_,
                   seriestitle = NA_character_,
                   seriesvolume = NA_character_,
                   publisher = NA_character_,
                   url = NA_character_,
                   city = NA_character_,
                   state = NA_character_,
                   country = NA_character_,
                   originallanguage = NA_character_,
                   notes = NA_character_,
                   author = new("authors")))

#' @aliases publications-class
#' @rdname publications_classes
#' @export
setClass("publications",
         representation(publications  = "list"),
         validity = function(object) {
           all(map(object@publications,
                   function(x) {
                     class(x) == "publication"
                   }) %>%
             unlist())
         })

#' @title S4 class for `chronologies` information
#' @name chronologies_classes
#' @description The class for chronologies from the
#' Neotoma Paleoecology Database. A single collection unit may
#' have one or more chronology. The individual chronology
#' classes are grouped into an S4 `chronologies` class.
#' @returns object of class `chronologies`
#' @aliases chronology-class
#' @md
#' @export
setClass("chronology",
         representation(chronologyid = "id",
                        notes = "character",
                        contact = "ANY",
                        agemodel = "character",
                        ageboundolder = "numeric",
                        ageboundyounger = "numeric",
                        isdefault = "logical",
                        dateprepared = "Date",
                        modelagetype = "character",
                        chronologyname = "character",
                        chroncontrols = "ANY"),
         prototype(chronologyid = NA_integer_,
                   notes = NA_character_,
                   contact = list(),
                   agemodel = NA_character_,
                   ageboundolder = NA_integer_,
                   ageboundyounger = NA_integer_,
                   isdefault = FALSE,
                   dateprepared = as.Date(character(1)),
                   modelagetype = NA_character_,
                   chronologyname = NA_character_,
                   chroncontrols = data.frame()))

#' @aliases chronologies-class
#' @rdname chronologies_classes
#' @export
setClass("chronologies",
         representation(chronologies = "list"),
         validity = function(object) {
           all(object@chronologies %>%
                 lapply(class) %>%
                 unlist(recursive = FALSE) ==  "chronology")
         })

#' @title S4 class for `samples` information
#' @name samples_classes
#' @description The standard object class for `samples`
#'  in the Neotoma Paleoecology Database.
#' @returns object of class `sample`
#' @aliases sample-class
#' @md
#' @export
setClass("sample",
         representation(ages = "ANY",
                        igsn = "character",
                        datum = "ANY",
                        depth = "numeric",
                        sampleid = "id",
                        thickness = "numeric",
                        samplename = "character",
                        sampleanalyst = "ANY",
                        analysisunitid = "id",
                        analysisunitname = "character"),
         prototype(ages = list(),
                   igsn = NA_character_,
                   datum = data.frame(),
                   depth = NA_integer_,
                   sampleid = NA_integer_,
                   thickness = NA_integer_,
                   samplename = NA_character_,
                   sampleanalyst = list(),
                   analysisunitid = NA_integer_,
                   analysisunitname = NA_character_))

#' @aliases samples-class
#' @rdname samples_classes
#' @export
setClass("samples",
         representation(samples = "list"))

#' @title S4 class for `repository` information
#' @name repositories_classes
#' @description The standard object class for `repository`
#'  from the Neotoma Paleoecology Database.
#' @returns object of class `repository`
#' @aliases repository-class
#' @md
#' @export
setClass("repository",
         representation(notes = "character",
                        acronym = "character",
                        repository = "character",
                        repositoryid = "id",
                        repositorynotes = "character"),
         prototype(notes = NA_character_,
                   acronym = NA_character_,
                   repository = NA_character_,
                   repositoryid = NA_integer_,
                   repositorynotes = NA_character_))

#' @aliases repositories-class
#' @rdname repositories_classes
#' @export
setClass("repositories",
         representation(repositories = "list"))

#' @title S4 class for `specimens` information
#' @name specimens_classes
#' @description The standard object class for `specimens`
#'  from the Neotoma Paleoecology Database.
#' @returns object of class `specimens`
#' @aliases specimen-class
#' @md
#' @export
setClass("specimen",
         representation(datasetid = "id",
                        sampleid = "id",
                        specimenid = "numeric",
                        repository = "repository",
                        taxonid = "id",
                        taxonname = "character",
                        elementtype = "character",
                        symmetry = "character",
                        portion = "character",
                        sex = "character",
                        domesticstatus = "character",
                        taphonomictype = "character",
                        nisp = "numeric",
                        preservative = "character",
                        maturity = "character",
                        samplenotes = "character"),
         prototype(datasetid = NA_integer_,
                   sampleid = NA_integer_,
                   specimenid = NA_integer_,
                   repository = NULL,
                   taxonid = NA_integer_,
                   taxonname = NA_character_,
                   elementtype = NA_character_,
                   symmetry = NA_character_,
                   portion = NA_character_,
                   sex = NA_character_,
                   domesticstatus = NA_character_,
                   taphonomictype = NA_character_,
                   nisp = NA_integer_,
                   preservative = NA_character_,
                   maturity = NA_character_,
                   samplenotes = NA_character_))

#' @aliases specimens-class
#' @rdname specimens_classes
#' @export
setClass("specimens",
         representation(specimens = "list"))

setClassUnion("samplesOrNULL", c("samples", "NULL"))
setClassUnion("specimensOrNULL", c("specimens", "NULL"))

#' @title S4 class for `datasets` information
#' @aliases dataset-class
#' @rdname datasets_classes
#' @description The standard object class for `datasets`
#'  from the Neotoma Paleoecology Database.
#' @export
#' @returns object of class `datasets`
setClass("dataset",
  representation(datasetid = "id",
                 database = "character",
                 doi = "ANY",
                 recdatecreated = "Date",
                 datasettype = "character",
                 datasetname = "character",
                 age_range_old = "numeric",
                 age_range_young = "numeric",
                 age_units = "character",
                 notes = "character",
                 pi_list = "ANY",
                 samples = "samplesOrNULL",
                 specimens = "specimensOrNULL"),
  prototype(datasetid = NA_integer_,
            database = NA_character_,
            doi = list(),
            recdatecreated = as.Date(character(1)),
            datasettype = NA_character_,
            datasetname = NA_character_,
            age_range_old =  NA_integer_,
            age_range_young =  NA_integer_,
            age_units = NA_character_,
            notes = NA_character_,
            pi_list = list(),
            samples = NULL,
            specimens = NULL)
)

#' @aliases datasets-class
#' @rdname datasets_classes
#' @export
setClass("datasets",
         representation(datasets = "list"))

#' @title S4 class for `speleothem` information
#' @name speleothems_classes
#' @description The S4 class for `speleothem` data.
#' @returns object of class `speleothems`
#' @aliases speleothem-class
#' @md
#' @export
setClass("speleothem",
         representation(entityid = "numeric",
                        entityname = "character",
                        siteid = "numeric",
                        collectionunitid = "numeric",
                        dripheight = "numeric",
                        monitoring = "logical",
                        geology = "character",
                        relativeage = "character",
                        speleothemtype = "character",
                        dripheightunits = "character",
                        entitycovertype = "character",
                        entrancedistance = "numeric",
                        entrancedistanceunits = "character",
                        landusecovertype = "character",
                        speleothemdriptype = "character",
                        landusecoverpercent = "numeric",
                        vegetationcovertype = "character",
                        entitycoverthickness = "numeric",
                        vegetationcoverpercent = "numeric"),
         prototype(entityid = NA_integer_,
                   entityname = NA_character_,
                   siteid = NA_integer_,
                   collectionunitid = NA_integer_,
                   dripheight = NA_integer_,
                   monitoring = FALSE,
                   geology = NA_character_,
                   relativeage = NA_character_,
                   speleothemtype = NA_character_,
                   dripheightunits = NA_character_,
                   entitycovertype = NA_character_,
                   entrancedistance = NA_integer_,
                   entrancedistanceunits = NA_character_,
                   landusecovertype = NA_character_,
                   speleothemdriptype = NA_character_,
                   landusecoverpercent = NA_integer_,
                   vegetationcovertype = NA_character_,
                   entitycoverthickness = NA_integer_,
                   vegetationcoverpercent = NA_integer_))

#' @aliases speleothems-class
#' @rdname speleothems_classes
#' @export
setClass("speleothems", representation(speleothems = "list"),
         validity = function(object) {
           all(map(object, function(x) {
             class(x) == "speleothem"
           }) %>%
             unlist())
         })

setClassUnion("datasetsOrNULL", c("datasets", "NULL"))
setClassUnion("speleothemsOrNULL", c("speleothems", "NULL"))
setClassUnion("chronologiesOrNULL", c("chronologies", "NULL"))
#' @title S4 class for `collection units` information.
#' @name collunits_classes
#' @description A `collection unit` represents a collection event from within
#' a `site`. For example, a lake sediment core, or a single dig site within an
#' archaeological site.
#' @returns object of class `collunits`
#' @aliases collunit-class
#' @md
#' @export
setClass("collunit",
         representation(collectionunitid = "id",
                        notes = "character",
                        handle = "character",
                        colldate = "Date",
                        location = "character",
                        waterdepth = "numeric",
                        gpslocation = "sf",
                        collunittype = "character",
                        collectiondevice = "character",
                        collectionunitname = "character",
                        depositionalenvironment = "character",
                        datasets = "datasetsOrNULL",
                        chronologies = "chronologiesOrNULL",
                        defaultchronology = "integer",
                        speleothems = "speleothemsOrNULL"),
         prototype(collectionunitid = NA_integer_,
                   notes = NA_character_,
                   handle = NA_character_,
                   colldate = as.Date(character(1)),
                   location = NA_character_,
                   waterdepth = NA_integer_,
                   gpslocation = sf::st_as_sf(sf::st_sfc()),
                   collunittype = NA_character_,
                   collectiondevice = NA_character_,
                   collectionunitname = NA_character_,
                   depositionalenvironment = NA_character_,
                   datasets = NULL,
                   chronologies = NULL,
                   defaultchronology = NA_integer_,
                   speleothems = NULL))

#' @aliases collunits-class
#' @rdname collunits_classes
#' @export
setClass("collunits",
         representation(collunits = "list"),
         validity = function(object) {
           all(map(object@collunits,
                   function(x) {
                     class(x) == "collunit"
                   }) %>%
             unlist())
         })

setClassUnion("collunitsOrNULL", c("collunits", "NULL"))
#' @title An S4 class for `sites` information
#' @name sites_classes
#' @description The S4 class for sites in the Neotoma Paleoecology Database.
#' @returns object of class `sites`
#' @aliases site-class
#' @md
#' @export
setClass("site",
         representation(siteid = "id",
                        sitename = "character",
                        geography = "sf",
                        altitude = "numeric",
                        geopolitical = "list",
                        area = "numeric",
                        notes = "character",
                        description = "character",
                        collunits = "collunitsOrNULL"),
         prototype(siteid = NA_integer_,
                   sitename = NA_character_,
                   geography = sf::st_sf(sf::st_sfc()),
                   geopolitical = list(),
                   altitude = NA_integer_,
                   area = NA_integer_,
                   notes = NA_character_,
                   description = NA_character_,
                   collunits = NULL))

#' @aliases sites-class
#' @rdname sites_classes
#' @export
setClass("sites",
         representation(sites = "list"),
         validity = function(object) {
           all(map(object@sites, function(x) {
             class(x) == "site"
           }) %>%
             unlist())
         })

#' @title S4 class for taxa information
#' @name taxa_classes
#' @description Taxa details from the Neotoma Paleoecology Database.
#' @returns object of class `taxon`
#' @aliases taxon-class
#' @md
#' @export
setClass("taxon",
         representation(taxonid = "numeric",
                        taxoncode = "character",
                        taxonname = "character",
                        author = "character",
                        ecolgroup = "character",
                        highertaxonid = "numeric",
                        status = "character",
                        taxagroupid = "character",
                        publicationid = "numeric",
                        publication = "character"),
         prototype(taxonid = NA_integer_,
                   taxoncode = NA_character_,
                   taxonname = NA_character_,
                   author = NA_character_,
                   ecolgroup = NA_character_,
                   highertaxonid = NA_integer_,
                   status = NA_character_,
                   taxagroupid = NA_character_,
                   publicationid = NA_integer_,
                   publication = NA_character_))

#' @aliases taxa-class
#' @rdname taxa_classes
#' @export
setClass("taxa", representation(taxa = "list"),
         validity = function(object) {
           all(map(object, function(x) {
             class(x) == "taxon"
           }) %>%
             unlist())
         })

#' @title Set Speleothem
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom methods is new slot<-
#' @importFrom uuid UUIDgenerate
#' @importFrom digest digest
#' @param x Object to be set as a speleothem
#' @param entityid The speleothem entity ID.
#' @param entityname Name of the speleothem entity.
#' @param siteid The unique site ID for a site.
#' @param collectionunitid The unique collection unit ID for a collection unit.
#' @param dripheight drip height
#' @param monitoring Is the speleothem being monitored? TRUE/FALSE
#' @param geology rock type of the speleothem
#' @param relativeage relative age of the speleothem
#' @param speleothemtype type of speleothem
#' @param dripheightunits Units for drip height
#' @param entitycovertype type of cover around the speleothem
#' @param entrancedistance distance from cave entrance
#' @param entrancedistanceunits Units for distance from cave entrance
#' @param landusecovertype type of land use cover around the speleothem
#' @param speleothemdriptype type of speleothem drip
#' @param landusecoverpercent land use cover percent
#' @param vegetationcovertype type of vegetation cover around the speleothem
#' @param vegetationcoverpercent vegetation cover percent
#' @param entitycoverthickness thickness of the entity cover
#' @returns `speleothem` object
#' @export
set_speleothem <- function(x = NA,
                           entityid = NA_integer_,
                           entityname = NA_character_,
                           siteid = NA_integer_,
                           collectionunitid = NA_integer_,
                           dripheight = NA_integer_,
                           monitoring = FALSE,
                           geology = NA_character_,
                           relativeage = NA_character_,
                           speleothemtype = NA_character_,
                           dripheightunits = NA_character_,
                           entitycovertype = NA_character_,
                           entrancedistance = NA_integer_,
                           landusecovertype = NA_character_,
                           speleothemdriptype = NA_character_,
                           landusecoverpercent = NA_integer_,
                           vegetationcovertype = NA_character_,
                           entitycoverthickness = NA_integer_,
                           entrancedistanceunits = NA_character_,
                           vegetationcoverpercent = NA_integer_){
  function_call <- match.call()
  if (suppressWarnings(is.na(x))) {
    x <- new("speleothem")
    if (is.na(entityid)) {
      hash <- digest(UUIDgenerate(), algo = "xxhash32", serialize = FALSE)
      x@entityid <- as.integer(strtoi(substr(hash, 1, 7), base = 16L))
    } else {
      x@entityid <- entityid
    }
    x$entityname <- entityname
    x$siteid <- siteid
    x$collectionunitid <- collectionunitid
    x$dripheight <- dripheight
    x$monitoring <- monitoring
    x$geology <- geology
    x$relativeage <- relativeage
    x$speleothemtype <- speleothemtype
    x$dripheightunits <- dripheightunits
    x$entitycovertype <- entitycovertype
    x$entrancedistance <- entrancedistance
    x$landusecovertype <- landusecovertype
    x$speleothemdriptype <- speleothemdriptype
    x$landusecoverpercent <- landusecoverpercent
    x$vegetationcovertype <- vegetationcovertype
    x$entitycoverthickness <- entitycoverthickness
    x$entrancedistanceunits <- entrancedistanceunits
    x@vegetationcoverpercent <- vegetationcoverpercent
  } else {
    if (is(x, "speleothem")) {
      if (length(function_call) > 2) {
        for (i in 3:length(function_call)) {
          slot(x, names(function_call)[[i]]) <- eval(function_call[[i]])
        }
        return(x)
      } else {
        return(x)
      }
    } else {
      stop("`x` must be a site object if it is supplied.")
    }
  }
  return(x)
}

#' @title Change NA values from logic to a prescribed type.
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @author Simon Goring \email{goring@wisc.edu}
#' @description Pass an object and convert all
#' \code{NA} elements to particular \code{NA} types.
#' @param x An element that may or may not have NA values.
#' @param type A character string with values either \code{char} or \code{int}.
#' @returns object converted to `NA_character` or `NA_integer`
#' @keywords internal
#' @noRd
use_na <- function(x, type) {
  tryCatch({
    if (type == "sf") {
      if (is.null(x) || nrow(x) == 0) {
        return(sf::st_as_sf(sf::st_sfc()))
      } else {
        return(x)
      }
    }
    if (is.null(x) || (is.atomic(x) && all(is.na(x)))) {
      return(switch(type,
                    "char" = NA_character_,
                    "int"  = NA_integer_,
                    "list" = list(),
                    "date" = as.Date(NA_character_),
                    "bool" = NA,
                    "df"   = data.frame(),
                    NA))
    }
    x
  }, error = function(e) {
    # Fallback
    switch(type,
           "char" = NA_character_,
           "int"  = NA_integer_,
           "sf"   = sf::st_as_sf(sf::st_sfc()),
           "list" = list(),
           "date" = as.Date(NA_character_),
           "bool" = NA,
           "df"   = data.frame(),
           NA)
  })
}

setClassUnion("missingOrNULL", c("missing", "NULL"))
setClassUnion("sitesOrsite", c("site", "sites"))

#' @title Display a `sites` object or nested slots.
#' @name show
#' @importFrom sf st_coordinates
#' @importFrom purrr map
#' @importFrom dplyr bind_rows
#' @param object `sites`, `datasets`, `collunits`, `contacts` object
#' @returns NULL
#' @aliases show,site-method
#' @importMethodsFrom methods show
setMethod(f = "show",
          signature = "site",
          definition = function(object) {
            print(data.frame(siteid = object@siteid,
                             sitename = object@sitename,
                             lat = mean(st_coordinates(object@geography)[, 2]),
                             long = mean(st_coordinates(object@geography)[, 1]),
                             altitude = object@altitude), row.names = FALSE) %>%
              unique()
          })

#' @aliases show,sites-method
#' @rdname show
setMethod(f = "show",
          signature = "sites",
          definition = function(object) {
            map(object@sites, function(x) {
              data.frame(
                siteid = x@siteid,
                sitename = x@sitename,
                lat = mean(st_coordinates(x@geography)[, 2]),
                long = mean(st_coordinates(x@geography)[, 1]),
                altitude = x@altitude
              )
            }) %>%
              bind_rows() %>%
              unique() %>%
              print(row.names = FALSE)
          })

#' @title sub-sub
#' @name sub-sub
#' @importFrom purrr map
#' @param x Neotoma2 nested object
#' @param i iteration in nested list
#' @description Obtain one of the elements within a `sites`,
#' `collectionunits`, `datasets`, etc... Neotoma objects.
#' @returns sliced `site` object
#' @examples \dontrun{
#' some_site <- get_sites(sitename = "Site%", limit=3)
#' some_site[[2]]
#' }
#' @aliases [[,sites,numeric-method
#' @exportMethod [[
setMethod(f = "[[",
          signature = signature(x = "sites", i = "numeric"),
          definition = function(x, i) {
            if (length(i) == 1) {
              out <- new("site", x@sites[[i]])
            } else {
              out <- map(i, function(z) {
                new("site", x@sites[[z]])
              })
              out <- new("sites", sites = out)
            }
            return(out)
          })

#' @title sub
#' @name [
#' @param x The `sites` object
#' @param i The numeric index
#' @returns sliced `site` object
#' @md
#' @aliases  [,sites,numeric-method
#' @exportMethod [
setMethod(f = "[",
          signature = signature(x = "sites", i = "numeric"),
          definition = function(x, i) {
            new("sites", sites = x@sites[i])
          })

#' @aliases [,site,numeric-method
#' @rdname sub
setMethod(f = "[",
          signature = signature(x = "site", i = "numeric"),
          definition = function(x, i) {
            slots <- slotNames(x)[i]
            as.data.frame(sapply(slots, function(y) slot(x, y)))
          })

#' @aliases [,site,character-method
#' @rdname sub
setMethod(f = "[",
          signature = signature(x = "site", i = "character"),
          definition = function(x, i) {
            out <- as.data.frame(lapply(i, function(y) slot(x, y)))
            colnames(out) <- i
            return(out)
          })

#' @title Get a `neotoma2` object's slot names
#' @name names
#' @param x A `neotoma2` object.
#' @returns NULL.
#' @description Get all names for elements' slots within a `collunit` object.
#' @md
#' @aliases names,site-method
#' @exportMethod names
setMethod(f = "names",
          signature = signature(x = "site"),
          definition = function(x) {
            slotNames(x)
          })

#' @aliases names,sites-method
#' @rdname names
setMethod(f = "names",
          signature = signature(x = "sites"),
          definition = function(x) {
            slotNames("site")
          })

#' @title sub-subset
#' @name sub-subset
#' @param x `neotoma2` object
#' @param i iteration in `neotoma2` object
#' @param value The value to be used
#' @description Obtain one of the elements within a nested `neotoma2` object
#' @returns `sites` object with reassigned values
#' @md
#' @aliases [[<-,sites-method
#' @exportMethod [[<-
setMethod(f = "[[<-",
          signature = signature(x = "sites"),
          definition = function(x, i, value) {
            siteset <- x@sites
            siteset[[i]] <- value
            out <- new("sites", sites = siteset)
            return(out)
          })

#' @title subset
#' @name subset
#' @param x A `neotoma2` object.
#' @param i The column indicator.
#' @param value The value to be used.
#' @returns `site` object with reassigned character values
#' @md
#' @aliases [<-,site,character-method
#' @exportMethod [<-
setMethod(f = "[<-",
          signature = signature(x = "site", i = "character"),
          definition = function(x, i, value) {
            for (idx in seq_along(length(i))) {
              slot(x, i[idx]) <- value[idx]
            }
            return(x)
          })

#' @aliases [<-,site,numeric-method
#' @rdname subset
setMethod(f = "[<-",
          signature = signature(x = "site", i = "numeric"),
          definition = function(x, i, value) {
            slots <- slotNames(x)
            for (idx in seq_along(length(i))) {
              slot(x, slots[i[idx]]) <- value[idx]
            }
            return(x)
          })

#' @title cash-set
#' @name cash-set
#' @param x A `neotoma2` object.
#' @param name name of the slot
#' @param value The value to be used.
#' @returns `neotoma2` object with reassigned values
#' @md
#' @aliases $<-,site-method
#' @exportMethod $<-
setMethod(f = "$<-",
          signature = signature(x = "site"),
          definition = function(x, name, value) {
            slot(x, name) <- value
            return(x)
          })

#' @title cash
#' @name cash
#' @param x `neotoma2` object
#' @param name name of the slot
#' @description Obtain slots of a `neotoma2` without using at-mark
#' @returns value at chosen slot in the `site` object
#' @md
#' @aliases $,site-method
#' @exportMethod $
setMethod(f = "$",
          signature = signature(x = "site"),
          definition = function(x, name) {
            slot(x, name)
          })

#' @aliases $,sites-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "sites"),
          definition = function(x, name) {
            vals <- lapply(x@sites, function(y) slot(y, name))
            out <- unlist(vals, recursive = FALSE)
            return(out)
          })

#' @title as.data.frame
#' @name as.data.frame
#' @importFrom sf st_coordinates
#' @importFrom dplyr bind_cols
#' @param x `neotoma2` object
#' @returns `data.frame` object
#' @description Returns `neotoma2` object's data as a `data.frame`.
#' @md
#' @aliases as.data.frame,site-method
#' @exportMethod as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("site"),
          definition = function(x) {
            data.frame(siteid = x@siteid,
                       sitename = x@sitename,
                       lat = mean(st_coordinates(x@geography)[, 2]),
                       long = mean(st_coordinates(x@geography)[, 1]),
                       area = x@area,
                       notes = x@notes,
                       description = x@description,
                       elev = x@altitude)
          })

#' @aliases as.data.frame,sites-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("sites"),
          definition = function(x) {
            x@sites %>% map(as.data.frame) %>% bind_rows() #%>% unique()
          })

#' @title  as.list sites
#' @param x sites object
#' @returns `list` object with sites
#' @noRd
setMethod(f = "as.list",
          signature = signature("sites"),
          definition = function(x) {
            as.list(x@sites)
          })

#' @title length
#' @name length
#' @param x `neotoma2` object
#' @returns `int` representing length of a `neotoma2` object
#' @md
#' @aliases length,sites-method
#' @exportMethod length
setMethod(f = "length",
          signature = signature(x = "sites"),
          definition = function(x) {
            length(x@sites)
          })

#' @title c - Combine `neotoma2` objects
#' @name c
#' @importFrom methods is
#' @param x `neotoma2` object or NULL
#' @param y `neotoma2` object or NULL
#' @returns concatenated and cleaned `sites` object
#' @md
#' @rdname c
#' @aliases c,sites-method
#' @exportMethod c
setMethod(f = "c",
          signature = "sites",
          definition = function(x, y) {
            if (is(y, "sites")) {
              x@sites <- unlist(c(x@sites, y@sites), recursive = FALSE)
            } else if (is(y, "site")) {
              x@sites <- c(x@sites, y)
            }
            x <- clean(x)
            if (length(x) == 0){
              x <- NULL
            } else {
              x <- new("sites", sites = x@sites)
            }
            return(x)
          }
)

#' @rdname c
setMethod(f = "c",
          signature = "missingOrNULL",
          definition = function(x = "missingORNULL", y) {
            y
          })

#' @title Return the latitude and longitude of sites
#' @param obj A sites object
#' @param ... Additional parameters associated with the call.
#' @returns `data.frame` object with site coordinates.
#' @aliases coordinates,sites-method
#' @exportMethod coordinates
setMethod(f = "coordinates",
          signature = "sites",
          definition = function(obj, ...) {
            coords <- as.data.frame(obj)[, c("siteid", "long", "lat")]
            return(coords)
          })

#' @title Plot site coordinates using a basic plot.
#' @name plot
#' @param x sites object
#' @param y *Ignored.*
#' @param ... Additional parameters associated with the call.
#' @returns `plot` object with site coordinates.
#' @aliases plot,sites-method
#' @exportMethod plot
setMethod(f = "plot",
          signature = "sites",
          definition = function(x, y, ...) {
            coords <- as.data.frame(x)[, c("long", "lat")]
            plot(coords, ...)
          })

#' @aliases plot,site-method
#' @rdname plot
setMethod(f = "plot",
          signature = "site",
          definition = function(x, y, ...) {
            coords <- as.data.frame(x)[, c("long", "lat")]
            plot(coords, ...)
          })

#' @title Summary of objects within a sites object.
#' @param object sites object
#' @param ... additional properties passed to \code{summary}
#' @description This function summarizes a sites object, from \code{site} level
#' and returns a \code{data.frame} that contains the site ID, sitename,
#' collectionunit ID, count of chronologies, count of datasets
#' and types of datasets within the site.
#' @importFrom dplyr bind_rows mutate select
#' @returns `data.frame` object with site summary information
#' @aliases summary,sites-method
#' @exportMethod summary
setMethod(f = "summary",
          signature = "sites",
          definition = function(object, ...) {
            datasettype <- lapply(object@sites, function(x) {
              collunits <- length(x@collunits@collunits)
              if (length(x) > 0) {
                collunits <- lapply(x@collunits@collunits,
                                    function(y) {
                                      chrons <- length(y@chronologies)
                                      datasets <- length(y@datasets)
                                      if (datasets > 0) {
                                        types <- sapply(y@datasets@datasets,
                                                        function(r) {
                                                          r@datasettype
                                                        }) %>%
                                          paste0(collapse = ",")
                                      } else {
                                        types <- NA
                                      }
                                      data.frame(collectionunit = y@handle,
                                                 chronologies = chrons,
                                                 datasets = datasets,
                                                 types = types)
                                    }) %>%
                  bind_rows() %>%
                  na.omit()
              } else {
                collunits <- data.frame(collectionunit = NA,
                                        chronologies = 0,
                                        datasets = 0,
                                        types = NA)
              }
              data.frame(siteid = x$siteid,
                         sitename = x$sitename,
                         collunits = collunits)
            }) %>%
              bind_rows() %>%
              rename(collunit_name = .data$collunits.collectionunit,
                     n_chronologies = .data$collunits.chronologies,
                     n_datasets = .data$collunits.datasets,
                     dataset_types = .data$collunits.types)
            return(datasettype)
          })

#' @title Obtain dataset DOIs from records.
#' @description Given complete dataset objects in Neotoma (must have used
#' \code{get_datasets()} or \code{get_downloads()}), return the dataset
#' DOI for the record.
#' @param x a Neotoma2 \code{site} object
#' @importFrom purrr map
#' @importFrom dplyr bind_rows full_join select arrange filter 
#' @importFrom dplyr mutate group_by row_number
#' @returns `data.frame` object with DOIs information.
#' @examples \dontrun{
#' # Get datasets metadata from API and retrieve DOIs
#' ds <- get_datasets(1)
#' doi(ds)
#' }
#' @aliases doi,sites-method
#' @exportMethod doi
setMethod(f = "doi",
          signature = "sites",
          definition = function(x) {
            map(x@sites, function(y) {
              doi(y)
            }) %>%
              bind_rows()
          })

#' @aliases doi,site-method
#' @rdname doi
setMethod(f = "doi",
          signature = "site",
          definition = function(x) {
            ids <- getids(x)
            ds <- get_datasets(x)
            dois <-
              map(datasets(ds)@datasets,
                  function(x) {
                    doi <- x@doi
                    doi <- unlist(doi, recursive = TRUE, use.names = FALSE)
                    doi <- use_na(testNull(doi), "char")
                    data.frame(datasetid = rep(x$datasetid, length(doi)),
                               doi = doi)
                  }) %>%
              bind_rows()

            dois <- dois %>%
              full_join(ids, by = "datasetid") %>%
              select(.data$siteid, .data$collunitid,
                     .data$datasetid, .data$doi) %>%
              arrange(.data$siteid, .data$collunitid, .data$datasetid)
            return(dois)
          })

#' @title Obtain data citations from multiple records.
#' @description Given complete dataset objects in Neotoma (must have used
#' \code{get_datasets()} or \code{get_downloads()}), return a formatted
#' citation for the record, including the dataset DOI.
#' @param x sites object
#' @importFrom purrr map
#' @importFrom dplyr bind_rows full_join select arrange filter
#' @returns `data.frame` object with citation information.
#' @examples \dontrun{
#' # Get datasets metadata from API
#' ds <- get_datasets(1)
#' }
#' @aliases cite_data,sites-method
#' @exportMethod cite_data
setMethod(f = "cite_data",
          signature = "sites",
          definition = function(x) {
            map(x@sites,
                function(y) {
                  cite_data(y)
                }) %>%
              bind_rows()
          })

#' @aliases cite_data,site-method
#' @rdname cite_data
setMethod(f = "cite_data",
          signature = "site",
          definition = function(x) {
            if (is.null(x)) {
              warning("No sites to filter")
              return(NULL)
            }
            strn <- paste0("%s. %s; %s dataset. ",
                           "In %s. Neotoma Paleoecology Database. doi:%s")
            ids <- getids(x)
            sitenames <- as.data.frame(x)  %>%
              select(.data$siteid, .data$sitename)
            ds <- get_datasets(x)
            ds_df <- as.data.frame(datasets(ds)) %>%
              select(.data$datasetid,
                     .data$datasettype,
                     .data$database)
            dois <-
              map(datasets(ds)@datasets,
                  function(x) {
                    doi <- x@doi
                    doi <- unlist(doi, recursive = TRUE, use.names = FALSE)
                    doi <- use_na(testNull(doi), "char")
                    pi_list <- testNull(unlist(x@pi_list))
                    data.frame(datasetid = rep(x$datasetid, length(doi)),
                               doi = doi,
                               pi_list = rep(paste0(sort(pi_list),
                                                    collapse = "; "),
                                             length(doi)))
                  }) %>%
              bind_rows()

            citations <- dois %>%
              full_join(ids, by = "datasetid") %>%
              full_join(sitenames, by = "siteid") %>%
              full_join(ds_df, by = "datasetid") %>%
              select(.data$siteid, .data$sitename, .data$collunitid,
                     .data$datasetid, .data$datasettype, .data$database,
                     .data$doi, .data$pi_list) %>%
              group_by(.data$siteid, .data$collunitid, .data$datasetid) %>%
              arrange(.data$doi) %>%
              dplyr::filter(row_number() == 1) %>%
              as.data.frame() %>%
              mutate(citation = sprintf(strn, .data$pi_list,
                                        .data$sitename, .data$datasettype,
                                        .data$database, .data$doi)) %>%
              select(.data$datasetid, .data$citation)
            return(citations)
          })

#' @aliases cite_data,NULL-method
#' @rdname cite_data
setMethod(f = "cite_data",
          signature = "NULL",
          definition = function(x) {
            if (is.null(x)) {
              warning("No sites to cite.")
              return(NULL)
            }
          })

#' @title Clear NULL values and replace with value predefined value.
#' @description Similar to concatenate, checks to see if a value is NULL
#' and replace it with a known value provided in \code{out}.
#' @param val The value to be checked.
#' @param out A default value to be returned if \code{val} is NULL.
#' @returns The values passed in \code{val} or \code{out} if
#'   \code{val} is \code{NULL}.
#' @examples {
#' # Passing a null value into the function returns 12:
#' a <- testNull(val = NULL, out = 12)
#' # Passing a non-NULL value returns that value:
#' b <- testNull(val = 11, out = 12)
#' }
#' @keywords internal
#' @noRd
testNull <- function(val, out = NA) {
  if (is.null(val)) {
    return(out)
  } else {
    return(val)
  }
}

#' @title Build a specimen objects.
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom methods new
#' @description
#' A helper function to build a specimen object from a list returned by
#' the Neotoma API call. The function is not exported, but called from
#' the get_specimens() call.
#' @param x specimen list
#' @returns A simple `specimen` object
#' @noRd
build_specimen <- function(x) {
  x <- cleanNULL(x)
  repo <- x$repository %>% cleanNULL()
  sp <- new("specimen",
            datasetid = use_na(x$datasetid, "int"),
            sampleid = use_na(x$sampleid, "int"),
            specimenid = use_na(x$specimenid, "int"),
            repository = new("repository",
                             notes = use_na(repo$notes, "char"),
                             acronym = use_na(repo$acronym, "char"),
                             repository = use_na(repo$repository, "char"),
                             repositoryid =  use_na(repo$repositoryid, "int"),
                             repositorynotes = use_na(repo$repositorynotes,
                                                      "char")),
            taxonid = use_na(x$taxonid, "int"),
            taxonname = use_na(x$taxonname, "char"),
            elementtype = use_na(x$elementtype, "char"),
            symmetry = use_na(x$symmetry, "char"),
            portion = use_na(x$portion, "char"),
            sex = use_na(x$sex, "char"),
            domesticstatus = use_na(x$domesticstatus, "char"),
            taphonomictype = use_na(x$taphonomictype, "char"),
            nisp = use_na(x$nisp, "int"),
            preservative = use_na(x$preservative, "char"),
            maturity = use_na(x$maturity, "char"),
            samplenotes = use_na(x$samplenotes, "char"))
  return(sp)
}

#' @title Build a collection unit from the API response
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom assertthat assert_that
#' @param args The structured JSON from a Neotoma API v2.0 response that
#'   returns a collection unit in any form.
#' @returns An simple `collunit` object
#' @noRd
build_collunits <- function(...) {
  args <- list(...)
  assert_that(is.list(args), msg = "Parsed object must be a list.")
  args <- cleanNULL(args)
  cu <- set_collunit(
    collectionunitid = use_na(args$collectionunitid, "int"),
    collunittype = use_na(args$collunittype, "char"),
    handle = use_na(args$handle, "char"),
    collectiondevice = use_na(args$collectiondevice, "char"),
    collectionunitname = use_na(args$collectionunitname, "char"),
    waterdepth = use_na(args$waterdepth, "int"),
    colldate = use_na(args$colldate, "date"),
    depositionalenvironment = use_na(args$depositionalenvironment, "char"),
    location = use_na(args$location, "char"),
    gpslocation = use_na(args$gpslocation, "sf"),
    notes = use_na(args$notes, "char"),
    datasets = testNull(args$datasets, NULL),
    defaultchronology = use_na(args$defaultchronology, "int"),
    chronologies = testNull(args$chronologies, NULL),
    speleothems = testNull(args$speleothems, NULL)
  )
  return(cu)
}

#' @title toWide
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom dplyr filter mutate arrange select group_by_at
#' @importFrom dplyr group_by desc all_of case_when
#' @importFrom tidyr pivot_wider
#' @param x dataframe object with samples
#' @param variablenames Optional vector to filter by specific variable names.
#' @param ecologicalgroups Vector stating the ecological groups to be
#' filtered by, e.g "DIAT", "TRSH"
#' @param elementtypes Label of element type to filter by,
#' e.g. "pollen", "valve"
#' @param unit Label stating which units to filter by, e.g. "NISP"
#' @param groupby Group by 'age' or 'depth'
#' @param operation label or vector of operations to be chosen from:
#' 'prop', 'sum', 'presence'.
#' @returns wide `data.frame` obtained from long `samples` `data.frame`
#' @examples \dontrun{
#' fc_sites <- neotoma2::get_datasets(limit=5, datasettype = "vertebrate fauna")
#' fc_ds <- fc_sites %>%
#'    neotoma2::get_downloads()
#' fc_dl1 <- fc_dl[[1]]
#' fc_smp <- samples(fc_dl1)
#' toWide(fc_smp, ecologicalgroups=c('AVES', 'RODE'),
#'        elementtypes='bone/tooth', unit='present/absent')
#' }
#' @description Obtain a wide table with information regarding of
#' samples grouped by variablename and depth/age.
#' @export
toWide <- function(x,
                   variablenames = NA,
                   ecologicalgroups = NA,
                   elementtypes = NA,
                   unit = NA,
                   groupby = "age",
                   operation = "prop") {
  if (!(length(variablenames) == 1 && is.na(variablenames)))  {
    x <- x %>% filter(.data$variablename %in% variablenames)
  }
  if (!(length(ecologicalgroups) == 1 && is.na(ecologicalgroups)))  {
    x <- x %>% filter(.data$ecologicalgroup %in% ecologicalgroups)
  }
  if (!(length(elementtypes) == 1 && is.na(elementtypes)))  {
    x <- x %>% filter(.data$elementtype %in% elementtypes)
  }
  if (!(length(unit) == 1 && is.na(unit)))  {
    x <- x %>% filter(units %in% unit)
  }
  # Get proportion values
  onesite <- x %>%
    group_by_at(groupby) %>%
    mutate(counter = sum(.data$value, na.rm = TRUE)) %>%
    group_by(.data$variablename) %>%
    mutate(prop = .data$value / .data$counter,
           n = .data$value) %>%
    arrange(desc(groupby))
  if (unit == "present/absent") {
    if (operation != "presence") {
      warning("Unit is `present/absent`, operation 'presence' will be applied.")
    }
    operation <- "presence"
  }
  widetable <- onesite %>%
    mutate(prop = as.numeric(.data$prop),
           sum = as.numeric(.data$value),
           presence = case_when(counter > 0 ~ 1,
                                counter == 0 ~ 0)) %>%
    select(all_of(groupby), .data$variablename, all_of(operation))
  counts <- pivot_wider(widetable,
                        id_cols = all_of(groupby),
                        names_from = .data$variablename,
                        values_from = operation,
                        values_fill = 0,
                        values_fn = sum)
  return(counts)
}

library("testthat")
library("neotoma2")

context("Apply a similar version to `tidyr::toWide` for
        `neotoma2` samples dataframes.")
test_that("`toWide()` on a samples dataframe.", {
  skip_on_cran()
  fauna1 <- samples(get_downloads(4716))
  fauna1t <- toWide(fauna1,
                 ecologicalgroups = c("AVES", "CARN", "PRIM", "RODE"), 
                 elementtype = c("bone/tooth/shell", "bone/bill", "bone/tooth"), 
                 unit = "present/absent", 
                 operation="presence")
  testthat::expect_true(is.data.frame(fauna1t), single_w)
  fauna2 <- samples(get_downloads(21007))
  fauna2t <- toWide(fauna2,
                        ecologicalgroups = c("RODE", "ARTI", "SORI"), 
                        elementtype = c("bone/tooth"), 
                        unit = "present/absent", operation="presence")
  testthat::expect_true(is.data.frame(fauna2t), TRUE)
})

test_that("`toWide()` proportions calculation.", {
  skip_on_cran()
  alex <- samples(get_downloads(24))
  alext <- toWide(alex,
                  unit = "NISP",
                  operation = "prop")
  nuph4129 <- alex %>%
    dplyr::filter(variablename == "Nuphar", age == 4129)
  # calculate the proportion manually from alex
  total_nisp_4129 <- sum(alex$value[alex$age == 4129 & alex$unit == "NISP"], na.rm = TRUE)
  nuphprop_4129 <- sum(nuph4129$value, na.rm = TRUE) / total_nisp_4129
  # extract the proportion from the wide data frame
  nuphprop_wide_4129 <- alext$Nuphar[alext$age == 4129]
  testthat::expect_equal(nuphprop_4129, nuphprop_wide_4129)
})

test_that("`toWide()` sum operation works", {
  skip_on_cran()
  alex <- samples(get_downloads(24))
  alext <- toWide(alex,
                  unit = "NISP",
                  operation = "sum")
  nuph4129 <- alex %>%
    dplyr::filter(variablename == "Nuphar", age == 4129)
  # calculate the sum manually from alex
  nuph_sum_4129 <- sum(nuph4129$value, na.rm = TRUE)
  # extract the sum from the wide data frame
  nuph_sum_wide_4129 <- alext$Nuphar[alext$age == 4129]
  testthat::expect_equal(nuph_sum_4129, nuph_sum_wide_4129)
})

#' @title set chronology information for a new record.
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @author Simon Goring \email{goring@wisc.edu}
#' @description Create a new chronology for a record.
#' Within Neotoma all chronologies have unique numeric
#' identifiers. Within R, because of the need to use the 
#' identifiers across objects, and because we want to avoid
#'  conflicts between naming systems, a universally unique 
#' identifier (UUID) is created for the object ID.
#' @importFrom methods is new slot<-
#' @importFrom digest digest
#' @importFrom uuid UUIDgenerate
#' @param x Object to be set as a chronology
#' @param chronologyid An optional value. Will be assigned a
#' unique identifier if not provided.
#' @param contact A contacts object, identifying the individual(s)
#' who created the chronology
#' @param agemodel A string representing the age model name, for
#' example "Crummy linear interpolation".
#' @param ageboundolder The `ageboundolder` is assigned the oldest sample
#' age rounded up to the nearest 10
#' @param ageboundyounger The `ageboundyounger` is assigned the oldest
#' sample age rounded up to the nearest 10
#' @param isdefault Defines whether the model is the default for
#' the collection unit for a particular model age type.
#' @param notes Additional notes about the chronology. For more
#' modern models, often the function call to Bacon or Bchron is added here.
#' @param dateprepared The date at which the age model was prepared.
#' @param modelagetype The age type for the model. For validation, the models
#' should be one of the valid Neotoma \code{agetypes}:
#' \url{https://api.neotomadb.org/v2.0/data/dbtables?table=agetypes}
#' @param chronologyname A valid name for the chronology.
#' @param chroncontrols A data.frame containing the chronological controls
#' for the age model.
#' @description Function to create new chronology objects for
#' personal analysis. 
#' The new object will not be uploaded to the database.
#' @returns `chronology` object
#' @md
#' @export
set_chronology <- function(x = NA,
                           chronologyid = NA_integer_,
                           notes = NA_character_,
                           contact = list(),
                           agemodel = NA_character_,
                           ageboundolder = NA_integer_,
                           ageboundyounger = NA_integer_,
                           isdefault = NA_integer_,
                           dateprepared = as.Date(character(0)),
                           modelagetype = NA_character_,
                           chronologyname = NA_character_,
                           chroncontrols = data.frame(0)) {
  function_call <- match.call()
  if (suppressWarnings(is.na(x))) {
    x <- new("chronology")
    if (is.na(x@chronologyid)) {
      if (is.na(chronologyid)) {
        hash <- digest(UUIDgenerate(), algo = "xxhash32", serialize = FALSE)
        x@chronologyid <- as.integer(strtoi(substr(hash, 1, 7), base = 16L))
      } else {
        x@chronologyid <- chronologyid
      }
      x@contact <- contact
      x@agemodel <- agemodel
      x@ageboundolder <- ceiling(ageboundolder / 10) * 10
      x@ageboundyounger <- floor(ageboundyounger / 10) * 10
      x@isdefault <- isdefault
      x@notes <- notes
      x@dateprepared <- dateprepared
      x@modelagetype <- modelagetype
      x@chronologyname <- chronologyname
      x@chroncontrols <- chroncontrols
    } else {
      if (is(x, "chronology")) {
        if (length(function_call) > 2) {
          for (i in 3:length(function_call)) {
            slot(x, names(function_call)[[i]]) <- eval(function_call[[i]])
          }
          return(x)
        } else {
          return(x)
        }
      } else {
        stop("`x` must be a chronology object if it is supplied.")
      }
    }
    return(x)
  }
}

library("testthat")
library("neotoma2")
library("dplyr")

test_that("Add a new chronology to a record:", {
  skip_on_cran()
  stara <- get_downloads(24238)
  stara_chron <- chronologies(stara)
  controls <- chroncontrols(stara) %>%
    dplyr::filter(chronologyid == 14591) %>%
    arrange(depth)
  controls$chroncontrolage[1] <- 0
  controls$agelimityounger[1] <- -2
  controls$agelimitolder[1] <- 2
  controls$thickness[1] <- 1
  predict_depths <- samples(stara) %>%
    select(depth, analysisunitid) %>%
    unique() %>%
    arrange(depth)
  new_chron <- Bchron::Bchronology(ages = controls$chroncontrolage,
                                   ageSds = abs(controls$agelimityounger -
                                                  controls$chroncontrolage),
                                   calCurves = c("normal", rep("intcal20", 4)),
                                   positionThicknesses = controls$thickness,
                                   positions = controls$depth,
                                   allowOutside = TRUE,
                                   ids = controls$chroncontrolid)
  newpredictions <- predict(new_chron, predict_depths$depth)
  new_chron_stara <- set_chronology(agemodel = "Bchron model",
                                    isdefault = TRUE,
                                    ageboundolder = max(newpredictions),
                                    ageboundyounger = min(newpredictions),
                                    dateprepared = Sys.Date(),
                                    modelagetype = "Calibrated radiocarbon years BP",
                                    chronologyname = "Simon's example chronology",
                                    chroncontrols = controls)
  new_sample_ages <- data.frame(predict_depths,
                                age = colMeans(newpredictions),
                                ageolder = colMeans(newpredictions) +
                                  apply(newpredictions, 2, sd),
                                ageyounger = colMeans(newpredictions) -
                                  apply(newpredictions, 2, sd),
                                agetype = "Calibrated radiocarbon years")
  stara[[1]]$collunits[[1]] <- add_chronology(stara[[1]]$collunits[[1]],
                                              new_chron_stara,
                                              new_sample_ages)
  last_age <- samples(stara) %>%
    dplyr::filter(analysisunitid == 194633) %>%
    select(age) %>%
    unique()
  testthat::expect_true(last_age == new_sample_ages$age[nrow(new_sample_ages)])
})

#' @title Parse Taxon Results
#' @importFrom purrr map
#' @param result The result object from the Neotoma API call
#' @returns A `taxa` object with the parsed taxon information
#' @noRd
parse_taxon <- function(data) {
  data <- data$data %>%
    cleanNULL()
  newTaxa <- map(data, function(x) {
    new("taxon",
        taxonid = use_na(testNull(x$taxonid, NA), "int"),
        taxoncode = use_na(testNull(x$taxoncode, NA), "char"),
        taxonname = use_na(testNull(x$taxonname, NA), "char"),
        author = use_na(testNull(x$author, NA), "char"),
        ecolgroup = use_na(testNull(x$ecolgroup, NA), "char"),
        highertaxonid = use_na(testNull(x$highertaxonid, NA), "int"),
        status = use_na(testNull(x$status, NA), "char"),
        taxagroupid = use_na(testNull(x$taxagroupid, NA), "int"),
        publicationid = use_na(testNull(x$publicationid, NA), "int"),
        publication = use_na(testNull(x$publication, NA), "char"))
  })
  taxa <- new("taxa", taxa = newTaxa)
  return(taxa)
}

#' @title get_taxon
#' @name get_taxon
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @description a `taxon` object with detailed information
#' @param x taxon ID
#' @param ... accepted arguments, see **details** for more information.
#' @returns A Neotoma2 `taxa` object with datasets with the requested taxa.
#' @details
#' A `taxa` may have one or more taxa associated with it.
#' The `get_taxon` function searches for `taxa` detail data within Neotoma
#' while the `get_taxa` function searches for sites that contain the requested
#' taxa. The function searches for each taxon by using a SQL query which
#' accepts any one of the following parameters:
#' * `taxonid`  The unique taxon ID (integer) in Neotoma. Can be passed as a
#' vector of taxa IDs.
#'  * `taxonname` Taxon name or partial name. You can pass wildcard characters
#' using %.
#'  * `taxagroup` The taxonomic grouping the taxon belongs to, from the Neotoma
#' taxagrouptypes table.
#'  * `ecolgroup` The ecological group of interest for the Neotoma taxon (from
#' the Neotoma ecolgrouptypes table)
#'  * `status` The taxonomic extinction status of the taxon, either extinct 
#' (1, True) or extant (0, False).
#' @md
#' @export
get_taxon <- function(x = NA, ...) {
  if (missing(x)) {
    UseMethod("get_taxon", "default")
  } else {
    UseMethod("get_taxon", x)
  }
}

#' @rdname get_taxon
#' @export
#' @method get_taxon default
get_taxon.default <- function(...) {
  oo <- options(scipen = 9999)
  on.exit(options(oo))
  cl <- as.list(match.call())
  cl[[1]] <- NULL
  cl <- lapply(cl, eval, envir = parent.frame())
  if ("all_data" %in% cl) {
    all_data <- cl$all_data
  } else {
    all_data <- TRUE
  }
  params <- get_params("taxa")
  if (!all(names(cl) %in% params)) {
    warning("Some parameters seem invalid. The current accepted parameters
             are: ", paste(unlist(params), collapse = ", "))
  }
  baseURL <- paste0("data/taxa")
  result <- tryCatch(
    parseURL(baseURL, ...),
    error = function(e) {
      stop("API call failed: ", e$message)
      NULL
    }
  )
  if (is.null(result)) {
    return(NULL)
  } else {
    result <- result %>%
      cleanNULL()
    if (is.null(result$data[1][[1]]) || is.null(result[1][[1]])) {
      message("No taxon data found with the provided parameters.")
      return(NULL)
    } else {
      output <- parse_taxon(result)
      return(output)
    }
  }
}

#' @rdname get_taxon
#' @export
#' @method get_taxon numeric
get_taxon.numeric <- function(x, ...) {
  if (length(x) > 0) {
    taxa_id <- paste0(x, collapse = ",")
  }
  baseURL <- paste0("data/taxa/", taxa_id)
  result <- tryCatch(
    parseURL(baseURL, ...),
    error = function(e) {
      stop("API call failed: ", e$message)
      NULL
    }
  )
  result_length <- length(result[2]$data)
  if (result_length > 0) {
    output <- parse_taxon(result)
    return(output)
  } else {
    message("No taxon data found for taxon ID: ", taxa_id)
    return(NULL)
  }
}

library("testthat")
library("neotoma2")

test_that("`get_taxon` yields specific taxon information", {
  skip_on_cran()
  abies_num <- get_taxon(1)
  testthat::expect_is(abies_num, "taxa")
  testthat::expect_is(abies_num[[1]], "taxon")
  
  abies_name <- get_taxon(taxonname = "abies")
  testthat::expect_is(abies_name, "taxa")
  testthat::expect_is(abies_name[[1]], "taxon")
  testthat::expect_equal(as.data.frame(abies_num), as.data.frame(abies_name))
})

library("testthat")
library("neotoma2")

context("`get_taxa()` and `get_taxon()` functions")
test_that("get_taxa() sites object", {
  skip_on_cran()
  abies <- get_taxa(1)
  testthat::expect_is(abies, "sites")
  abies_df <- abies %>% get_downloads() %>% taxa() %>% suppressWarnings()
  testthat::expect_true(any(grepl("Abies", abies_df$variablename)))
  abies <- get_taxa(taxonname = 'abies')
  testthat::expect_is(abies, "sites")
  abies_df <- abies %>% get_downloads() %>% taxa() %>% suppressWarnings()
  testthat::expect_true(any(grepl("Abies", abies_df$variablename)))
})

#' @aliases names,publication-method
#' @rdname names
setMethod(f = "names",
          signature = signature(x = "publication"),
          definition = function(x) {
            slotNames("publication")
          })

#' @aliases names,publications-method
#' @rdname names
setMethod(f = "names",
          signature = signature(x = "publications"),
          definition = function(x) {
            slotNames("publication")
          })

#' @aliases show,publications-method
#' @rdname show
setMethod(f = "show",
          signature = signature(object = "publications"),
          definition = function(object) {
            map(object@publications, function(x) {
              data.frame(publicationid = x@publicationid,
                         citation = x@citation,
                         doi = x@doi)
            }) %>%
              bind_rows() %>%
              print()
          })

#' @aliases cash,publication-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "publication"),
          definition = function(x, name) {
            slot(x, name)
          })

#' @aliases sub-sub,publications-method
#' @rdname sub-sub
setMethod(f = "[[",
          signature = signature(x = "publications", i = "numeric"),
          definition = function(x, i) {
            if (length(i) == 1) {
              out <- new("publication", x@publications[[i]])
            } else {
              out <- map(i, function(z) {
                new("publication", x@publications[[z]])
              })
              out <- new("publications", publications = out)
            }
            return(out)
          })

#' @aliases sub-subset,publications-method
#' @rdname sub-subset
setMethod(f = "[[<-",
          signature = signature(x = "publications"),
          definition = function(x, i, value) {
            if (length(i) == 1) {
              x@publications[[i]] <- new("publication", value)
              out <- x
              return(out)
            } else {
              warning("You can only reassign one publication at a time.")
            }
          })

#' @aliases length,publications-method
#' @rdname length
setMethod(f = "length",
          signature = signature(x = "publications"),
          definition = function(x) {
            length(x@publications)
          })

#' @aliases c,publications-method
#' @rdname c
setMethod(f = "c",
          signature = signature(x = "publications"),
          definition = function(x, y) {
            new("publications",
                publications = unlist(c(x@publications,
                                        y@publications),
                                      recursive = FALSE))
          })

#' @aliases show,publication-method
#' @rdname show
setMethod(f = "show",
          signature = signature(object = "publication"),
          definition = function(object) {
            print(data.frame(publicationid = object@publicationid,
                             citation = object@citation,
                             doi = object@doi))
          })

#' @title Show matched publication objects.
#' @param x A \code{publication} object.
#' @returns NULL
#' @noRd
#' @keywords internal
setMethod(f = "showMatch",
          signature = signature(x = "publication"),
          definition = function(x) {
            if (!is.null(attr(x, "matches"))) {
              print(attr(x, "matches"))
            }
          })

#' @aliases doi,publication-method
#' @rdname doi
setMethod(f = "doi",
          signature = signature(x = "publication"),
          definition = function(x) {
            x@doi
          })

#' @aliases as.data.frame,publication-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature(x = "authors"),
          definition = function(x) {
            authors <- x@authors %>%
              map(function(y) {
                paste0(y@author@familyname, ", ",
                       y@author@givennames)
              }) %>%
              paste0(collapse = "; ")
            return(authors)
          })

#' @aliases as.data.frame,publication-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature(x = "publication"),
          definition = function(x) {
            slots <- slotNames(x)
            slots <- slots[!slots == "author"]
            table <- slots %>%
              map(function(s) {
                out <- data.frame(slot(x, s),
                                  stringsAsFactors = FALSE)
                colnames(out) <- s
                return(out)
              }) %>%
              bind_cols()
            table$authors <- as.data.frame(x@author)
            return(table)
          })

#' @aliases as.data.frame,publication-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature(x = "publications"),
          definition = function(x) {
            full <- x@publications %>%
              map(function(y) as.data.frame(y)) %>%
              bind_rows()
            return(full)
          })

#' @title Select the best match (between a local record and a Neotoma match)
#' @param x A \code{publication} object
#' @param n The match number.
#' @returns the best match to the selected publication.
#' @noRd
#' @keywords internal
setMethod(f = "selectMatch",
          signature = signature(x = "publication", n = "numeric"),
          definition = function(x, n) {
            if (is.null(attr(x, "matches"))) {
              stop("There are no existing matches.")
            } else if (n > length(attr(x, "matches"))) {
              stop("The requested match is not in the current list.")
            } else if (n <= length(attr(x, "matches"))) {
              return(attr(x, "matches")[[n]])
            }
          })

#' @noRd
#' @keywords internal
setMethod(f = "selectMatch",
          signature = signature(x = "publication", n = "logical"),
          definition = function(x, n) {
            attr(x, "matches") <- NULL
            return(x)
          })

#' @title plotLeaflet
#' @description Plot sites on a leaflet map
#' @importFrom purrr map
#' @importFrom leaflet leaflet addTiles addCircleMarkers 
#' @importFrom leaflet markerOptions markerClusterOptions
#' @param object Sites object to plot
#' @examples \dontrun{
#' # Note that by default the limit for queries is 25 records:
#' tryCatch({
#' modernSites <- get_sites(keyword = "Modern")
#' plotLeaflet(modernSites)
#' }, error = function(e) {
#'  message("Neotoma server not responding. Try again later.")
#' })
#' }
#' @returns `leaflet` map
#' @export
setMethod(f = "plotLeaflet",
          signature = "sites",
          definition = function(object) {
            explorerURL <- "http://apps.neotomadb.org/explorer/"
            df1 <- as.data.frame(object)
            map1 <- leaflet(df1) %>%
              addTiles() %>%
              addCircleMarkers(lng = df1$long,
                               lat = df1$lat,
                               popup = paste0("<b>", df1$sitename,
                                              "</b><br><b>Description:</b> ",
                                              df1$description,
                                              "<br><a href=", 
                                              explorerURL, "?siteids=",
                                              df1$siteid,
                                              ">Explorer Link</a>"),
                               clusterOptions = markerClusterOptions(),
                               options = markerOptions(riseOnHover = TRUE))
            return(map1)
          })

#' @rdname plotLeaflet
setMethod(f = "plotLeaflet",
          signature = "site",
          definition = function(object) {
            explorerURL <- "http://apps.neotomadb.org/explorer/"
            df1 <- as.data.frame(object)
            map1 <- leaflet(df1) %>%
              addTiles() %>%
              addCircleMarkers(lng = df1$long,
                               lat = df1$lat,
                               popup = paste0("<b>", df1$sitename,
                                              "</b><br><b>Description:</b> ",
                                              df1$description,
                                              "<br><a href=", 
                                              explorerURL, "?siteids=",
                                              df1$siteid,
                                              ">Explorer Link</a>"),
                               clusterOptions = markerClusterOptions(),
                               options = markerOptions(riseOnHover = TRUE))
            return(map1)
          })

#' @rdname plotLeaflet
setMethod(f = "plotLeaflet",
          signature = "ANY",
          definition = function(object) {
            if (is.null(object)) {
              warning("No sites to plot")
              return(NULL)
            }
          })

#' @title parseLocation
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @author Simon Goring \email{goring@wisc.edu}
#' @importFrom jsonlite fromJSON toJSON
#' @importFrom geojsonsf geojson_sf sfc_geojson sf_geojson
#' @importFrom sf st_as_sfc st_bbox st_crs st_sfc
#' @importFrom assertthat assert_that
#' @importFrom stats na.omit
#' @description An internal helper function used to connect to the Neotoma API
#' in a standard manner, and to provide basic validation of any response.
#' @param x location parameter
#' @returns `geojson` object to be used for API requests.
#' @keywords internal
#' @noRd
parseLocation <- function(x) {
  if (is.numeric(x)) {
    assert_that(length(na.omit(x)) == 4,
                msg = "Numeric coordinates need to be an array of
                       4 units, c(xmin, xmax, ymax, ymin)")
    bbox <- st_bbox(c(xmin = x[1], xmax = x[3],
                      ymax = x[4], ymin = x[2]),
                    crs = st_crs(4326))
    sfc <- st_as_sfc(bbox)
    geojson <- sfc_geojson(sfc)
    geojson <- fromJSON(geojson, simplifyVector = FALSE)
  } else if (inherits(x, "sf")) {
    geojson <- sf_geojson(x)
    geojson <- fromJSON(geojson, simplifyVector = FALSE)
  } else if (inherits(x, "sfg")) {
    geojson <- sfc_geojson(st_sfc(x))
    geojson <- fromJSON(geojson, simplifyVector = FALSE)
  } else if (is.character(x)) {
    geojson <- tryCatch({
      parsed <- geojson_sf(x)
      if (inherits(parsed, "sf")) {
        gj <- sf_geojson(parsed)
        fromJSON(gj, simplifyVector = FALSE)
      } else {
        stop("Input string is not valid GeoJSON.")
      }
    }, error = function(e) {
      stop("Error parsing GeoJSON string: ", e$message)
    })
  }
  body <- list(toJSON(geojson, auto_unbox = TRUE))
  return(body)
}
#add wkt

testthat::test_that("Parse location fails with an incomplete bounding box:", {
  skip_on_cran()
  testthat::expect_error(parseLocation(c(1, 2)))
})

# testthat::test_that("We can pass in an sf object and return the WKT:", {
#   g <- sf::st_sfc(st_point(1:2))
#   sf_obj <- sf::st_sf(a = 3, g)
#   testthat::expect_is(parseLocation(sf_obj), "geojson")
# })
# 
# testthat::test_that("Passing in a WKT works:", {
#   poly <- parse_location("POLYGON ((40 40, 20 45, 45 30, 40 40))")
#   testthat::expect_is(parseLocation(poly), "geojson")
# })

#' @title get_params
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom httr GET content
#' @importFrom stringr str_match str_match_all
#' @keywords internal
get_params <- function(obj) {
  url <- "https://api.neotomadb.org/api-docs/swagger-ui-init.js"
  r <- tryCatch({
    resp <- GET(url)
    result <- content(resp, "text", encoding = "UTF-8")
    sw <- str_match(result,
                    paste0("v2.0/data/", obj,
                           "\"\\s*:\\s*(\\{[\\s\\S]+?\\})\\s*\\n"))[, 2]
    matches <- str_match_all(
      sw, "#/components/parameters/([a-zA-Z0-9_]+)Query\\\""
    )[[1]][, 2]
    matches <- unique(matches)
    matches
  }, error = function(e) {
    warning("Swagger with available parameters not available in API: ",
            conditionMessage(e))
    NULL
  })
  r <- append(r, list("all_data", "limit", "offset"))
  r <- lapply(r, tolower)
  return(r)
}

#' @title get_taxa
#' @name get_taxa
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @description a `sites` object with sites that contain the requested `taxa.`
#' @param x A `taxon` ID to extract site information
#' @param ... accepted arguments, see details for more information.
#' @details
#' The `get_taxa` function searches for a sites within Neotoma 
#' that contain the requested taxa.
#' The `get_taxa` command wraps the Neotoma API`.
#' The call itself accepts any one of the following
#' parameters:
#'  * `taxonid`  The unique taxon ID (integer) in Neotoma. Can be passed as a
#' vector of site IDs.
#' @returns A Neotoma2 sites object with datasets with the requested taxa.
#' @md
#' @export
get_taxa <- function(x = NA, ...) {
  if (missing(x)) {
    UseMethod("get_taxa", "default")
  } else {
    UseMethod("get_taxa", x)
  }
}

#' @rdname get_taxa
#' @method get_taxa numeric
#' @exportS3Method get_taxa numeric
get_taxa.numeric <- function(x, ...) {
  oo <- options(scipen = 9999)
  on.exit(options(oo))
  taxa_data <- get_taxon(x, ...)
  taxa_names <- unique(as.data.frame(taxa_data)$taxonname)
  cl <- as.list(match.call())
  cl[[1]] <- NULL
  cl <- lapply(cl, eval, envir = parent.frame())
  params <- suppressWarnings(get_params("datasets"))
  params <- c(params, "limit", "offset", "all_data")
  ds_params <- list()
  for (name in names(cl)) {
    if (name %in% params){
      if (name != "taxa") {
        ds_params[name] <- cl[[name]]
      } else {
        ds_params[['taxa']] <- list(taxa_names)
      }
    }
  }
  if (is.null(ds_params)) {
    ds_params <- list()
  }
  sites <- do.call(get_datasets, ds_params)
  return(sites)
}

#' @rdname get_taxa
#' @method get_taxa default
#' @exportS3Method get_taxa default
get_taxa.default <- function(...) {
  cl <- as.list(match.call())
  cl[[1]] <- NULL
  cl <- lapply(cl, eval, envir = parent.frame())
  if ("all_data" %in% cl) {
    all_data <- cl$all_data
  } else {
    all_data <- FALSE
  }
  if ("taxa" %in% names(cl)){
    sites <- get_datasets(taxa = taxa, all_data = all_data)
  } else if ("taxonname" %in% names(cl)) {
    taxon_data <- get_taxon(taxonname = cl$taxonname)
    taxon_names <- unique(as.data.frame(taxon_data)$taxonname)
    sites <- get_datasets(taxa = taxon_names, all_data = all_data)
  }  else {
    stop("Please provide a taxon ID, taxon name, 
          or taxon object to extract site information.")
    return(NULL)
  }
  return(sites)
}

#' @title get_datasets
#' @name get_datasets
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom methods new
#' @importFrom dplyr select filter
#' @description
#' The `get_datasets()` function is a wrapper for the Neotoma `datasets` API
#' endpoint.
#' The function takes parameters defined by the user and returns dataset
#' information supplied by the Neotoma Paleoecological Database.
#' The user may define all or none of the possible fields.
#' @param x A single datasetid, or a vector of unique dataset ids.
#' @param ... accepted arguments, see details for more information.
#' @returns The function returns either a single item of class `"try-error"`
#' describing the reason for failure (either misdefined parameters or an error
#' from the Neotoma API), or list of site objects, each containing one or more
#' `collunit` objects, with fully populated `datasets` elements.
#' @details
#' A `dataset` is an element nested within `neotoma2` site objects. The
#' `get_datasets()` call returns a list of individual `site` objects with
#' `collunits` (collection units) that contain valid, matching `dataset`
#' elements.
#' So, `get_sites()` returns only site metadata. `get_datasets()` returns
#' site metadata, plus metadata about the individual datasets present at that
#' site.
#' The `get_datasets()` function searches for each site within Neotoma
#' that matches the query parameters, and returns them as a `sites` object,
#' a list of `site` objects, plus returns all the additional metadata for
#' the datasets at that site.
#' The `get_datasets()` command wraps the Neotoma API
#' ([api.neotomadb.org](https://api.neotomadb.org)) call for `datasets`.
#' The call itself uses a SQL query which accepts any one of the following
#' parameters:
#'  * `siteid`  The unique site ID (integer) in Neotoma. Can be passed as a
#' vector of site IDs.
#'  * `sitename`  The site name, or approximate match using the % wildcard.
#'  * `database`  The constituent database for the record. See 
#' `get_table("constituentdatabases")`
#'  * `datasettype` Neotoma contains data for a number of dataset types. 
#'  This returns a subset of data types. For a complete list of available 
#'  dataset types, run `neotoma2::get_table('datasettypes')`
#'  * `altmin`  The minimum altitude range for site elevation (in meters).
#'  *  `altmax`  The maximum altitude range for site elevation (in meters).
#'  *  `datasetid`  The unique dataset ID (integer) in Neotoma. Can be passed
#' as a vector of dataset IDs.
#'  * `doi`  The dataset DOI for a dataset contained within a site. Can be
#' passed as a vector of DOIs.
#'  * `gpid`  The geopolitical name or identifier containing a site. Can be
#' passed as a vector of names.
#'  * `keywords`  Keywords for samples within a set of sites. For example
#' "modern" indicates a sample within the record uses the keyword "modern".
#'  * `contacts`  Contact names or IDs associated with a site.
#'  * `ageyoung`  A minimum spanning age for the record, in years before
#' radiocarbon present (1950).
#'  * `ageold`  A maximum spanning age for the record, in years before
#' radiocarbon present (1950).
#'  * `ageof`  An age which must be contained within the range of sample ages
#' for a site.
#'  * `taxa`  The names of taxa which must be present within samples in a
#' record.
#'  * `all_data` The API only downloads the first 25 records of the query. 
#'  For the complete records, use `all_data=TRUE`
#' @examples \dontrun{
#' tryCatch({
#'   random_sites <- get_sites(1)
#'   allds <- get_datasets(random_sites, limit=3)
#' }, error = function(e) {
#'    message("Neotoma server not responding. Try again later.")
#' })
#' # To find all datasets with a min altitude of 12 and a max altitude of 25:
#' tryCatch({
#'   sites_12to25 <- get_datasets(altmin=12, altmax=25)
#' }, error = function(e) {
#'   message("Neotoma server not responding. Try again later.")
#' })
#' # To find all datasets in Brazil
#' brazil <- '{"type": "Polygon",
#' "coordinates": [[
#'  [-73.125, -9.102096738726443],
#'  [-56.953125,-33.137551192346145],
#'  [-36.5625,-7.710991655433217],
#'  [-68.203125,13.923403897723347],
#'  [-73.125,-9.102096738726443]]]}'
#' tryCatch({
#'   brazil_datasets <- get_datasets(loc = brazil[1], limit=2)
#' }, error = function(e) {
#'   message("Neotoma server not responding. Try again later.")
#' })
#' }
#' @md
#' @export
get_datasets <- function(x = NA, ...) {
  if (missing(x)) {
    UseMethod("get_datasets", "default")
  } else {
    UseMethod("get_datasets", x)
  }
}

#' @rdname get_datasets
#' @method get_datasets numeric
#' @exportS3Method get_datasets numeric
get_datasets.numeric <- function(x, ...) {
  if (length(x) > 0) {
    dataset <- paste0(x, collapse = ",")
  }
  baseURL <- paste0("data/datasets/", dataset)
  result <- tryCatch(
    parseURL(baseURL, ...),
    error = function(e) {
      stop("API call failed: ", e$message)
      NULL
    }
  )
  if (length(result[2]$data) > 0) {
    output <- parse_site(result)
    return(output)
  } else {
    return(NULL)
  }
}

#' @rdname get_datasets
#' @method get_datasets default
#' @exportS3Method get_datasets default
get_datasets.default <- function(x, ...) {
  params <- get_params("datasets")
  oo <- options(scipen = 9999)
  on.exit(options(oo))
  cl <- as.list(match.call())
  if ('doidata' %in% params) {
    params <- append(params, 'doi')
  }
  cl[[1]] <- NULL
  cl <- lapply(cl, eval, envir = parent.frame())
  if (!all(names(cl) %in% params)) {
    warning("Some parameters seem invalid. 
    The current accepted parameters are: ",
    paste(unlist(params), collapse = ", "))
  }
  base_url <- "data/datasets/"
  result <- tryCatch(
    parseURL(base_url, ...),
    error = function(e) {
      message("API call failed: ", e$message)
      NULL
    }
  )
  if (is.null(result$data[1][[1]]) || is.null(result[1][[1]])) {
    return(NULL)
  } else {
    output <- parse_site(result)
    return(output)
  }
}

#' @rdname get_datasets
#' @exportS3Method get_datasets sites
get_datasets.sites <- function(x, ...) {
  ids <- getids(x)
  cl <- as.list(match.call())
  cl[[1]] <- NULL
  ids <- ids %>%
    select(.data$datasetid) %>%
    unique() %>%
    unlist() %>%
    as.numeric()
  if ("all_data" %in% cl) {
    all_data <- cl$all_data
  } else {
    all_data <- TRUE
  }
  dots <- list(...)
  if ("all_data" %in% names(dots)) {
    dots$all_data <- NULL
  }
  output <- get_datasets(x = ids, all_data = all_data)
  return(output)
}

#' @rdname get_datasets
#' @exportS3Method get_datasets site
get_datasets.site <- function(x, ...) {
  # List of datasets ids
  cl <- as.list(match.call())
  cl[[1]] <- NULL
  ids1 <- getids(x)
  ids <- ids1 %>%
    filter(!is.na(suppressWarnings(as.numeric(.data$siteid))),
           !is.na(suppressWarnings(as.numeric(.data$datasetid))))
  ids2 <- getids(x) %>% 
    filter(is.na(suppressWarnings(as.numeric(.data$siteid))) |
             is.na(suppressWarnings(as.numeric(.data$datasetid))))
  if (nrow(ids2) != 0) {
    warnsite <- sprintf("SiteID %s or DatasetID %s does not exist in the
                         Neotoma DB yet or it has been removed. 
                        It will be removed from your search.",
                        paste0(ids2$siteid, collapse = ", "),
                        paste0(ids2$datasetid, collapse = ", "))
    warning(warnsite)
  }
  dataset_list <- ids$datasetid
  dataset_list <- as.numeric(unlist(dataset_list))
  if ("all_data" %in% cl) {
    all_data <- cl$all_data
  } else {
    all_data <- TRUE
  }
  dots <- list(...)
  if ("all_data" %in% names(dots)) {
    dots$all_data <- NULL
  }
  output <- get_datasets(dataset_list, all_data = all_data)
  return(output)
}

#' @rdname get_datasets
#' @exportS3Method get_datasets NULL
get_datasets.NULL <- function(x, ...) {
  message("Input is NULL, returning NULL")
  return(NULL)
}

library("testthat")
library("neotoma2")

context("`doi()` function")
test_that("", {
  skip_on_cran()
  sts <- get_datasets(c(24, 100, 101))
  dois <- doi(sts)
  testthat::expect_is(dois, "data.frame")
  st24 <- sts[[1]]
  doi24 <- purrr::map(datasets(st24)@datasets,
                      function(x)  {
                        doi <- x@doi
                        unlist(doi, recursive = TRUE, use.names = FALSE)
                        }) %>% unlist()
  dois24_df <- dois %>% dplyr::filter(siteid == 24) %>% dplyr::pull(doi) %>% unlist()
  testthat::expect_true(all(doi24 %in% dois24_df))
  testthat::expect_true(all(dois24_df %in% doi24))
  st100 <- sts[[2]]
  doi100 <- purrr::map(datasets(st100)@datasets,
                       function(x)  {
                         doi <- x@doi
                         unlist(doi, recursive = TRUE, use.names = FALSE)
                       }) %>% unlist()
  dois100_df <- dois %>% dplyr::filter(siteid == 100) %>% dplyr::pull(doi) %>% unlist()
  testthat::expect_true(all(doi100 %in% dois100_df))
  testthat::expect_true(all(dois100_df %in% doi100))
  st101 <- sts[[3]]
  doi101 <- purrr::map(datasets(st101)@datasets,
                          function(x)  {
                            doi <- x@doi
                            unlist(doi, recursive = TRUE, use.names = FALSE)
                          }) %>% unlist()
  dois101_df <- dois %>% dplyr::filter(siteid == 101) %>% dplyr::pull(doi) %>% unlist()
  testthat::expect_true(all(doi101 %in% dois101_df))
  testthat::expect_true(all(dois101_df %in% doi101))
})

#' @title Build a `speleothem` from the Neotoma API response.
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom assertthat assert_that
#' @param args A list returned from the Neotoma API `data` section.
#' @returns A simple `speleothem` object
#' @noRd
build_speleothem <- function(...) {
  args <- list(...)
  args <- cleanNULL(args)
  assert_that(is.list(args), msg = "Parsed object must be a list.")
  speleothem <- set_speleothem(
                  entityid = use_na(args$entityid, "int"),
                  entityname = use_na(args$entityname, "char"),
                  siteid = use_na(args$siteid, "int"),
                  collectionunitid = use_na(args$collectionunitid, "int"),
                  dripheight = use_na(args$dripheight, "int"),
                  dripheightunits = use_na(args$dripheightunits, "char"),
                  monitoring = use_na(args$monitoring, "logic"),
                  geology = use_na(args$geology, "char"),
                  relativeage = use_na(args$relativeage, "char"),
                  speleothemtype = use_na(args$speleothemtype, "char"),
                  entitycovertype = use_na(args$entitycovertype, "char"),
                  entrancedistance = use_na(args$entrancedistance, "int"),
                  landusecovertype = use_na(args$landusecovertype, "char"),
                  speleothemdriptype = use_na(args$speleothemdriptype, "char"),
                  landusecoverpercent = use_na(args$landusecoverpercent, "int"),
                  vegetationcovertype = use_na(args$vegetationcovertype,
                                               "char"),
                  entitycoverthickness = use_na(args$entitycoverthickness,
                                                "int"),
                  entrancedistanceunits = use_na(args$entrancedistanceunits,
                                                 "char"),
                  vegetationcoverpercent = use_na(args$vegetationcoverpercent,
                                                  "int"))
  return(speleothem)
}

#' @title Clean NULL values
#' @author Simon Goring \email{goring@wisc.edu}
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @description Pass an object and convert all
#' \code{NULL} elements to \code{NA}.
#' @param x An element that may or may not have NULL values.
#' @returns parsed `list` where NULL values are changed to NA
#' @noRd
cleanNULL <- function(x) {
  if (is.list(x)) {
    lapply(x, cleanNULL)
  } else if (is.null(x)) {
    NA
  } else {
    x
  }
}

#' @title speleothemdetails
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom dplyr bind_rows distinct mutate rename 
#' @importFrom dplyr left_join select arrange
#' @importFrom purrr map
#' @param x site object
#' @returns `data.frame` with speleothem records
#' @description Obtain elements on the speleothems level
#' Experimental function: API and behavior may change.
#' @examples \dontrun{
#' kesang <- get_sites(sitename = "Kesang cave") %>%
#'   get_datasets() %>%
#'   filter(datasettype == "pollen") %>%
#'   get_speleothems()
#' sp <- speleothemdetails(kesang)
#' }
#' @md
#' @export
setMethod(f = "speleothemdetails",
  signature = "sites",
  definition = function(x) {
    output <- map(x@sites,
                  function(y) speleothemdetails(y)) %>%
      bind_rows() %>%
      distinct() %>%
      select(.data$siteid, .data$sitename, .data$collectionunitid,
             .data$datasetid, .data$entityid, .data$entityname,
             .data$depth, .data$thickness, .data$chronologyid,
             .data$chronologyname, .data$agetype, .data$ageolder,
             .data$age, .data$ageyounger, .data$age_units, .data$sampleid,
             .data$samplename, .data$taxongroup, .data$ecologicalgroup,
             .data$taxonid, .data$variablename, .data$value, .data$units,
             # speleothem details
             .data$speleothemtype, .data$geology,
             .data$relativeage, .data$monitoring,
             .data$speleothemdriptype, .data$dripheight, .data$dripheightunits,
             .data$covertype, .data$entitycoverthickness,
             .data$entrancedistance, .data$entrancedistanceunits, 
             .data$landusecovertype, .data$landusecoverpercent,
             .data$vegetationcovertype, .data$vegetationcoverpercent) %>%
      arrange(.data$entityid, .data$taxonid, .data$depth, .data$age)
    if (nrow(output) == 0) {
      warnsite <- sprintf("No assigned speleothems. Is it a speleothem 
                           dataset? \nDid you run get_speleothems()?")
      warning(warnsite)
    }
    return(output)
  }
)

#' @rdname speleothemdetails
#' @export
setMethod(f = "speleothemdetails",
  signature = "site",
  definition = function(x) {
    map(x@collunits@collunits,
        function(y) {
          speleothemdetails(y) %>%
            bind_rows()
        })
  }
)

#' @rdname speleothemdetails
#' @export
setMethod(f = "speleothemdetails",
  signature = "collunits",
  definition = function(x) {
    df <- map(x@collunits, function(x) speleothemdetails(x)) %>%
      bind_rows()
    return(df)
  }
)

#' @rdname speleothemdetails
#' @export
setMethod(f = "speleothemdetails",
          signature = "collunit",
          definition = function(x) {
            speleothemset <- speleothems(x)
            if (nrow(speleothemset) == 0) {
              msg <- "No assigned speleothems. Is it a speleothem dataset? \n
                                  Did you run get_speleothems()?"
              warnsite <- sprintf(msg)
              warning(warnsite)
              return(data.frame())
            } else {
              x <- get_downloads(speleothemset$datasetid)
              sp_samples <- samples(x) %>% distinct()
              # join speleothemset and sp_samples on datasetid
              df <- speleothemset %>%
                left_join(sp_samples,
                          by = "datasetid") %>%
                distinct()
              return(df)
            }
          })

# #' @title get_specimens
# ##' @name get_specimens
# #' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
# #' @importFrom purrr map
# #' @importFrom dplyr bind_rows select filter
# #' @description
# #' Information for Specimens
# #' @param x Use a single specimenid
# #' @param ... Additional terms passed to get_specimens, most common datasetid
# #' @returns The function returns a specimens list
# #' @keywords internal
# #' @noRd
# get_specimens <- function(x, ...) {
#   UseMethod("get_specimens")
# }

# parse_specimen <- function(result, ds) {
#   sps <- result$data %>%
#     cleanNULL()
#   sp_index <- map(sps,
#                   function(x) {
#                     data.frame(datasetid = x$datasetid)
#                   }) %>%
#                 bind_rows()

#   # Move from get_downloads to a different helper function script
#   check_match <- function(sp_row, ids) {
#     apply(ids, 1, function(x) sum(sp_row == x))
#   }
#   for (i in seq_along(sps)) {
#     ids <- getids(ds, order = FALSE)
#     matches <- check_match(sp_index[i, ], ids)
#     if (max(matches) == 1) {
#       # Retrieve IDs for site and collectionunit based on datasetID
#       st <- match(ids$siteid[which.max(matches)], unique(ids$siteid))
#       cuids <- ids %>%
#         filter(siteid == unique(ids$siteid)[st], .preserve = TRUE)
#       cuid <- which(unique(
#                     cuids$collunitid) == ids$collunitid[which.max(matches)])
#       # Filter based on datasetID
#       dsids <- cuids %>%
#         filter(collunitid == unique(cuids$collunitid)[cuid], .preserve = TRUE)
#       dsid <- which(unique(dsids$datasetid) == sp_index$datasetid[i])
#       newsp <- build_specimen(sps[[i]])

#       # Attach built specimen slot to datasets
#       datasets <- ds[[st]]@collunits@collunits[[cuid]]@datasets@datasets[[dsid]]
#       datasets@specimens@specimens <- c(datasets@specimens@specimens,
#                                         newsp)
#       datasets@samples@samples <-
#         ds[[st]]@collunits@collunits[[
#           cuid]]@datasets@datasets[[dsid]]@samples@samples
#       ds[[st]]@collunits@collunits[[cuid]]@datasets@datasets[[dsid]] <- datasets
#     }
#   }
#   return(ds)
# }

# ##' @rdname get_specimens
# ##' @export
# # remove meanwhile
# get_specimens.numeric <- function(x, ...) {
#   if (length(x) > 0) {
#     specimenid <- paste0(x, collapse = ",")
#   }

#   baseURL <- paste0("data/specimens/", specimenid)
#   result <- tryCatch(
#     parseURL(baseURL),
#     error = function(e) {
#       message("API call failed: ", e$message)
#       NULL
#     }
#   )
#   if ((is.null(result)) | (length(result$data) == 0)) {
#     stop("Specimen ID not found. 
#          If you meant dataset ID, use parameter datasetid")
#   }
#   sps <- result$data %>%
#     cleanNULL()
#   #if sps is empty list, exit and return error No Specimen ID available.
#   sp_index <- map(sps, function(x) {
#     data.frame(datasetid = x$datasetid)}) %>%
#     bind_rows()
#   dw <- get_downloads(sp_index$datasetid)
#   ds <- parse_specimen(result, dw)
#   return(ds)

# }

# ##' @rdname get_specimens
# ##' @export
# get_specimens.default <- function(...) {
#   cl <- as.list(match.call())
#   cl[[1]] <- NULL
#   cl <- lapply(cl, eval, envir = parent.frame())
#   dsid <- as.numeric(cl$datasetid)
#   if (length(dsid) > 0) {
#     dsid <- paste0(dsid, collapse = ",")
#   }
#   baseURL <- paste0("data/datasets/", as.character(dsid), "/specimens")
#   result <- tryCatch(
#     parseURL(base_url, ...),
#     error = function(e) {
#       message("API call failed: ", e$message)
#       NULL
#     }
#   )
#   if(length(result$data) == 0) {
#     warning("No specimens found for the provided dataset ID.")
#     return(NULL)
#   } else {
#     dw <- get_downloads(cl$datasetid)
#     ds <- parse_specimen(result, dw)
#     return(ds)
#   }
# }

# ##' @rdname get_specimens
# ##' @export
# get_specimens.sites <- function(x,...) {
#   output <- getids(x) %>%
#     select(datasetid) %>%
#     stats::na.omit() %>%
#     unique() %>%
#     unlist()
#   if (length(output) > 0) {
#     output <- paste0(output, collapse = ",")
#   }
#   baseURL <- paste0("data/datasets/", output, "/specimens/")
#   result <- tryCatch(
#     parseURL(baseURL, ...),
#     error = function(e) {
#       message("API call failed: ", e$message)
#       NULL
#     }
#   )
#   if (length(result$data) == 0) {
#     warning("No specimens found for the provided dataset ID.")
#     return(NULL)
#   } else {
#     df <- suppressWarnings(samples(x))
#     if(dim(df)[1] == 0){
#       x <- get_downloads(x)
#     }
#     ds <- parse_specimen(result, x)
#     return(ds)
#   }
# }

#' @title Extract datasets from a sites object.
#' @description If the sites object contains datasets, then the datasets
#' will be returned. If the sites object does not contain datasets then
#' the user can apply `get_datasets()` to the object.
#' @param object A sites object
#' @returns datasets object specific to the metadata contained in datasets
#' @export
setGeneric("datasets",
           function(object) {
             standardGeneric(f = "datasets")
           })

#' @title Obtain coordinates from a sites object.
#' @param obj A sites object
#' @param ... Additional parameters associated with the call.
#' @returns dataframe with coordinate values
#' @export
setGeneric("coordinates", function(obj, ...) {
  standardGeneric(f = "coordinates")
})

#' @title plotLeaflet
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @description Plot sites on a leaflet map
#' @param object Sites object to plot
#' @returns leaflet map with site markers
#' @export
setGeneric("plotLeaflet", function(object) {
  standardGeneric(f = "plotLeaflet")
})

#' @title Show matches for objects.
#' @param x object to show matches for
#' @returns data.frame that marks if a `site` exists in another `sites` object
#' @noRd
#' @keywords internal
setGeneric("showMatch", function(x) {
  standardGeneric(f = "showMatch")
})

#' @title Obtain samples from a record or multiple records.
#' @param x sites object
#' @returns data.frame with record information at sample level
#' @export
setGeneric("samples", function(x) {
  standardGeneric(f = "samples")
  })

#' @title Obtain speleothems from a record or multiple records.
#' @param x sites object
#' @returns data.frame with record information at speleothem level
#' @export
setGeneric("speleothems", function(x) {
  standardGeneric(f = "speleothems")
})

#' @title Obtain speleothems and samples from a record or multiple records.
#' @param x sites object
#' @returns data.frame with record information at speleothem level
#' @export
setGeneric("speleothemdetails", function(x) {
  standardGeneric(f = "speleothemdetails")
})

#' @title Obtain the chronology from a record or multiple records.
#' @param x sites object that contains chronologies
#' @returns chronologies object with all chronologies used.
#' @export
setGeneric("chronologies", function(x) {
  standardGeneric(f = "chronologies")
})

#' @title Obtain the DOI for publications or datasets.
#' @param x Object with DOIs associated to it.
#' @returns doi object with DOI information
#' @export
setGeneric("doi", function(x) {
  standardGeneric(f = "doi")
})

#' @title Generate a data citation from a Neotoma2 object.
#' @description The function, applied to a data object with a valid dataset, 
#' will return a properly formatted data citation for the record.
#' @param x Object with DOIs associated to it.
#' @returns data.frame with citation data
#' @export
setGeneric("cite_data", function(x) {
  standardGeneric(f = "cite_data")
})

#' @title Select the best match for an object.
#' @param x object
#' @param n n elements that are a best match
#' @returns attr Select the match between a local record and a Neotoma match
#' @noRd
#' @keywords internal
setGeneric("selectMatch", function(x, n) {
  standardGeneric(f = "selectMatch")
})

#' @title chroncontrols
#' @description Show the samples table
#' @param x Sites object to extract chroncontrols table from
#' @returns data.frame with chroncontrols information
#' @export
setGeneric("chroncontrols", function(x) {
  standardGeneric(f = "chroncontrols")
})

#' @title taxa
#' @description Show the samples table
#' @param object Sites object to extract taxa table from
#' @returns data.frame with taxa records
#' @export
setGeneric("taxa", function(object) {
  standardGeneric(f = "taxa")
})

#' @title Add a new chronology into an existing collectionunit.
#' @name add_chronology
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom purrr map
#' @importFrom assertthat assert_that
#' @importFrom dplyr filter
#' @param object A collection unit object
#' @param x A chronology object generated using \code{set_chronology()}
#' @param y A data.frame of sample ages, with required columns:
#'   `"analysisunitid"`, `"age"`, `"agetype"`, `"ageolder"`, and `"ageyounger"`.
#' @returns `chronologies` with new added chronology
#' @description Given a collunit, add a new chronology object to the unit
#' with both the chronology metadata and the age information (as `y`)
#' @details When undertaking analysis we may wish to add a new chronology to
#' existing records within Neotoma. To do this we must first build the
#' chronology, but also link it to existing analysis units within the
#' collection unit.
#' For examples from this function, see the
#' https://open.neotomadb.org/EPD_binder/complex_workflow.html
#' documentation online.
#' @md
#' @exportMethod add_chronology
setGeneric("add_chronology",
           function(object, x, y) {
             standardGeneric(f = "add_chronology")
           })

#' @title Extract collection units from a sites object
#' @param object A sites object
#' @importFrom methods slotNames slot
#' @returns collunits detail from a sites object
#' @export
setGeneric("collunits",
           function(object) {
             standardGeneric(f = "collunits")
           })

#' @title Set the default chronology within a collectionunit.
#' @param x A chronologies object.
#' @param n The particular chronology to be used as the default.
#' @returns sites object with new default chronology
#' @export
setGeneric("set_default",
           function(x, n) {
             standardGeneric(f = "set_default")
           })
#TODO
# #' @title Obtain specimens from a record or multiple records.
# #' @param x sites object
# #' @returns data.frame with record information regarding specimens
# #' @export
# setGeneric("specimens", function(x) {
#   standardGeneric(f = "specimens")
#   })

#' @title Get table record from Neotoma
#' @description Call Neotoma and return a table
#'  (with limits & offsets for large tables)
#' @param x Table name (consult \url{https://open.neotomadb.org/dbschema/}
#'  for a complete list of table names.
#' @param limit Default 25 records
#' @param offset Default 0.
#' @returns selected `table` values from the Database.
#' @examples \dontrun{
#' # Returns only the first 25 specimen records.
#' someSpec <- get_table('specimens')
#' }
#' @importFrom dplyr bind_rows
#' @importFrom purrr map
#' @export
get_table <- function(x, limit = 25, offset = 0) {
  result <- tryCatch(
    parseURL("dbtables/table", table = x, limit = limit, offset = offset),
    error = function(e) {
      message("API call failed: ", e$message)
      NULL
    }
  )
  if (is.null(result$data)) {
    return(NULL)
  } else {
    result <- result$data$data 
    output <- result %>%
      cleanNULL() %>%
      map(data.frame) %>%
      bind_rows()
    return(output)
  }
}

library("testthat")
library("neotoma2")

context("Working with general data from the database:")
test_that("Call a single database table:", {
  skip_on_cran()
  tb <- get_table("agetypes")
  testthat::expect_is(tb, "data.frame")
})

test_that("Apply limits for get_table()", {
  skip_on_cran()
  tb <- get_table("agetypes", limit = 1)
  testthat::expect_equal(nrow(tb), 1)
  tb <- get_table("agetypes", limit = 3)
  testthat::expect_equal(nrow(tb), 3)
})

test_that("Limit and offsets work as expected.
          Offsetting by one returns a different result.", {
            skip_on_cran()
            tb1 <- get_table("agetypes", limit = 1)
            tb2 <- get_table("agetypes", limit = 1, offset = 1)
            testthat::expect_false(rlang::hash(tb1) ==
                                     rlang::hash(tb2))
          })

test_that("Correct table", {
  skip_on_cran()
  tb1 <- get_table("agetypes", limit = 1)
  cols <- c("agetypeid", "agetype", "precedence",
            "shortagetype", "recdatecreated", "recdatemodified")
  testthat::expect_identical(colnames(tb1), cols)
})

#' @aliases show,dataset-method
#' @rdname show
setMethod(f = "show",
          signature = "dataset",
          definition = function(object) {
            print(data.frame(datasetid = object@datasetid,
                             database = object@database,
                             datasettype = object@datasettype,
                             age_range_old =  object@age_range_old,
                             age_range_young =  object@age_range_young,
                             age_units = object@age_units,
                             recdatecreated = object@recdatecreated,
                             notes = object@notes), row.names = FALSE)
          })

#' @aliases show,datasets-method
#' @rdname show
setMethod(f = "show",
          signature = "datasets",
          definition = function(object) {
            map(object@datasets, function(y) {
              data.frame(datasetid = y@datasetid,
                         database = y@database,
                         datasettype = y@datasettype,
                         age_range_old =  y@age_range_old,
                         age_range_young =  y@age_range_young,
                         age_units = y@age_units,
                         recdatecreated = y@recdatecreated,
                         notes = y@notes)
            }) %>%
              bind_rows() %>%
              print(row.names = FALSE)
          })

#' @aliases sub-sub,datasets-method
#' @rdname sub-sub
setMethod(f = "[[",
          signature = signature(x = "datasets", i = "numeric"),
          definition = function(x, i) {
            if (length(i) == 1) {
              out <- new("dataset", x@datasets[[i]])
            } else {
              out <- map(i, function(z) {
                new("dataset", x@datasets[[z]])
              })
              out <- new("datasets", datasets = out)
            }
            return(out)
          })

#' @aliases names,dataset-method
#' @rdname names
setMethod(f = "names",
          signature = signature(x = "dataset"),
          definition = function(x) {
            slotNames(x)
          })

#' @aliases sub-subset,datasets-method
#' @rdname sub-subset
setMethod(f = "[[<-",
          signature = signature(x = "datasets"),
          definition = function(x, i, value) {
            datasetset <- x@datasets
            datasetset[[i]] <- value
            out <- new("datasets", datasets = datasetset)
            return(out)
          })

#' @aliases subset,dataset-method
#' @rdname subset
setMethod(f = "[<-",
          signature = signature(x = "dataset", i = "character"),
          definition = function(x, i, value) {
            for (idx in 1:length(i)) {
              slot(x, i[idx]) <- value[idx]
            }
            return(x)
          })

#' @aliases subset,dataset-method
#' @rdname subset
setMethod(f = "[<-",
          signature = signature(x = "dataset", i = "numeric"),
          definition = function(x, i, value) {
            slots <- slotNames(x)
            for (idx in 1:length(i)) {
              slot(x, slots[i[idx]]) <- value[idx]
            }
            return(x)
          })

#' @aliases cash-set,dataset-method
#' @rdname cash-set
setMethod(f = "$<-",
          signature = signature(x = "dataset"),
          definition = function(x, name, value) {
            slot(x, name) <- value
            return(x)
          })

#' @aliases sub,datasets-method
#' @rdname sub
setMethod(f = "[",
          signature = signature(x = "datasets", i = "numeric"),
          definition = function(x, i) {
            new("datasets", datasets = x@datasets[i])
          })

#' @aliases cash,dataset-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "dataset"),
          definition = function(x, name) {
            slot(x, name)
          })

#' @aliases cash,datasets-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "datasets"),
          definition = function(x, name) {
            x %>%
              map(function(y) {
                slot(y, name)
              }) %>%
              unlist()
          })

#' @aliases as.data.frame,dataset-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("dataset"),
          definition = function(x) {
            data.frame(datasetid = x@datasetid,
                       database = x@database,
                       datasettype = x@datasettype,
                       age_range_old =  x@age_range_old,
                       age_range_young =  x@age_range_young,
                       age_units = x@age_units,
                       recdatecreated = x@recdatecreated,
                       notes = x@notes)
          })

#' @aliases as.data.frame,datasets-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("datasets"),
          definition = function(x) {
            x@datasets %>% map(as.data.frame) %>% bind_rows()
          })

#' @aliases length,datasets-method
#' @rdname length
setMethod(f = "length",
          signature = signature(x = "datasets"),
          definition = function(x) {
            length(x@datasets)
          })

#' @aliases c,datasets-method
#' @rdname c
setMethod(f = "c",
          signature = signature(x = "datasets"),
          definition = function(x, y) {
            new("datasets",
                datasets = unlist(c(x@datasets,
                                    y@datasets), recursive = FALSE))
          })

#' @title Get object IDs
#' @name getids
#' @author Simon Goring \email{goring@wisc.edu}
#' @description This function parses a site object, from \code{site} to
#' \code{dataset} level and returns a \code{data.frame} that contains the
#' site, collectionunit and dataset IDs for each element within the site.
#' @importFrom purrr map
#' @importFrom dplyr arrange bind_rows mutate
#' @param x A Neotoma2 \code{sites} or \code{collunits} object.
#' @param order sort items by `siteid`, `collunitid`, `datasetid`
#' @returns `data.frame` containing `siteid`, `datasetid`, and `collunitid`
#' @examples \dontrun{
#' marion <- get_sites(sitename = "Marion Lake")
#' collunitids <- getids(collunits(marion))
#' }
#' @md
#' @export
getids <- function(x, order = TRUE) {
  UseMethod("getids")
}

#' @rdname getids
#' @method getids sites
#' @export getids.sites
#' @exportS3Method getids sites
getids.sites <- function(x, order = TRUE) {
  siteids <- map(x@sites, function(y) {
    siteid <- y@siteid
    if (length(y@collunits) > 0) {
      cuids <- map(y@collunits@collunits, function(z) {
        collunitid <- z@collectionunitid
        if (length(z@datasets) > 0) {
          datasetids <- map(z@datasets@datasets, function(a) {
            a@datasetid
          })
        } else {
          datasetids <- NA
        }
        return(data.frame(collunitid = collunitid,
                          datasetid = unlist(datasetids)))
      }) %>%
        bind_rows()
    } else {
      cuids <- data.frame(collunitid = NA, datasetid = NA)
    }
    return(data.frame(siteid = siteid, cuids))
  })
  siteids <- do.call("rbind.data.frame", args = siteids)
  rownames(siteids) <- seq_len(nrow(siteids))
  if (order) {
    siteids <- siteids %>%
      arrange(.data$siteid, .data$collunitid, .data$datasetid)
  }
  # Guaranteeing that future joins all work out
  siteids <- siteids %>%
    mutate(siteid = .data$siteid,
           collunitid = .data$collunitid,
           datasetid = .data$datasetid)
  siteids <- siteids[!is.na(siteids$datasetid), ]
  siteids <- unique(siteids)
  return(siteids)
}

#' @rdname getids
#' @method getids site
#' @exportS3Method neotoma2::getids site
getids.site <- function(x, order = TRUE) {
  siteid <- x@siteid
  if (length(x@collunits) > 0) {
    cuids <- map(x@collunits@collunits, function(z) {
      collunitid <- z@collectionunitid
      if (length(z@datasets) > 0) {
        datasetids <- map(z@datasets@datasets, function(a) {
          a@datasetid
        })
      } else {
        datasetids <- NA
      }
      return(data.frame(collunitid = (collunitid),
                        datasetid = (unlist(datasetids))))
    }) %>%
      bind_rows()
  } else {
    cuids <- data.frame(collunitid = NA, datasetid = NA)
  }
  return(data.frame(siteid = (siteid), cuids))
}

#' @rdname getids
#' @method getids collunits
#' @exportS3Method getids collunits
getids.collunits <- function(x, order = TRUE) {
  siteid <- NA
  if (length(x) > 0) {
    cuids <- map(x@collunits, function(z) {
      collunitid <- z@collectionunitid
      if (length(z@datasets) > 0) {
        datasetids <- map(z@datasets@datasets, function(a) {
          a@datasetid
        })
      } else {
        datasetids <- NA
      }
      return(data.frame(collunitid = (collunitid),
                        datasetid = (unlist(datasetids))))
    }) %>%
      bind_rows()
  } else {
    cuids <- data.frame(collunitid = NA, datasetid = NA)
  }
  return(data.frame(siteid = (siteid), cuids))
}

#' @rdname getids
#' @method getids collunit
#' @exportS3Method getids collunit
getids.collunit <- function(x, order = TRUE) {
  siteid <- NA
  collunitid <- x@collectionunitid
  if (length(x@datasets) > 0) {
    datasetids <- map(x@datasets@datasets, function(a) {
      a@datasetid
    })
  } else {
    datasetids <- NA
  }
  df <- data.frame(siteid = siteid,
                   collunitid = collunitid,
                   datasetid = datasetids)
  df <- df[!is.na(df$datasetid), ]
  return(df)
}

#' @title parse_site
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @author Simon Goring \email{goring@wisc.edu}
#' @importFrom purrr map
#' @importFrom methods new
#' @description An internal helper function used to parse site information into
#' `neotoma2R` objects.
#' @param result The API response.
#' @param verbose If TRUE print progress to console in bar form.
#' @returns `list` with cleaned and parsed data from HTTP request
#' @noRd
parse_site <- function(result, verbose = FALSE) {
  data <- result$data
  data <- group_response(data)
  new_sites <- map(data, function(x) {
    if (verbose) {
      cat(".")
    }
    # Collection units
    if (is.null(x$site$collectionunits)) {
      call <- x$collectionunits
    } else {
      call <- x$site$collectionunits
    }
    cus <- map(call, function(y){
      # Datasets
      ds <- map(y$datasets, function(z) {
        samp <- map(z$samples, build_sample)
        samp <- new("samples", samples = samp)
        z$agerange <- normalize_agerange(z$agerange)
        ds_l <- list(datasetid = z$datasetid,
                     database = use_na(z$database, "char"),
                     doi = z$doi,
                     recdatecreated = use_na(as.Date(z$recdatecreated), "date"),
                     datasettype = use_na(z$datasettype, "char"),
                     datasetname = use_na(z$datasetname, "char"),
                     age_range_old = use_na(z$agerange$ageold, "int"),
                     age_range_young = use_na(z$agerange$ageyoung, "int"),
                     age_units = use_na(z$agerange$units, "int"),
                     notes = use_na(z$datasetnotes, "char"),
                     pi_list = z$pi_list,
                     samples = samp,
                     specimens = NULL)
        do.call(build_dataset, ds_l)
      })
      ds <- new("datasets", datasets = ds)
      # Chronologies
      chronologies <- map(y$chronologies, function(z) {
        if (!is.null(z$chronology$chronologyid)) {
          ch <- z$chronology
          ch_l <-
            list(chronologyid = use_na(ch$chronologyid, "int"),
                 notes = use_na(ch$chronolgy$notes, "char"),
                 contact = use_na(ch$chronolgy$contact, "char"),
                 agemodel = use_na(ch$chronolgy$agemodel, "char"),
                 ageboundolder = use_na(ch$chronolgy$ageboundolder, "int"),
                 ageboundyounger = use_na(ch$chronolgy$ageboundyounger, "int"),
                 isdefault = use_na(ch$chronology$isdefault, "bool"),
                 dateprepared = use_na(as.Date(ch$chronology$dateprepared),
                                       "date"),
                 modelagetype = use_na(ch$chronology$modelagetype, "char"),
                 chronologyname = use_na(ch$chronology$chronologyname, "char"),
                 chroncontrols = ch$chroncontrols)
          do.call(build_chron, ch_l)
        } else {
          NULL
        }
      })
      if (is.null(chronologies) || all(sapply(chronologies, is.null))) {
        chron <- new("chronologies", chronologies = list())
      } else {
        chron <- new("chronologies", chronologies = chronologies)
      }
      # Speleothems
      speleothems <- tryCatch(
        parse_speleothem(y$speleothems),
        error = function(e) {
          NULL # Speleothems does not exist in the regular API calls
        }
      )
      # Build collection unit
      cu_l <-
        list(collectionunitid = y$collectionunitid,
             colldate = as.Date(use_na(y$colldate, "date")),
             handle = use_na(y$handle, "char"),
             datasets = ds,
             chronologies = chron,
             location = use_na(y$location, "char"),
             waterdepth = use_na(y$waterdepth, "int"),
             gpslocation = use_na(y$gpslocation, "sf"),
             collunittype = use_na(y$collectionunittype, "char"),
             collectiondevice = use_na(y$collectiondevice, "char"),
             collectionunitname = use_na(y$collectionunit, "char"),
             depositionalenvironment = use_na(y$depositionalenvironment,
                                              "char"),
             defaultchronology = use_na(y$defaultchronology, "int"),
             speleothems = speleothems)
      do.call(build_collunits, cu_l)
    })
    cu <- new("collunits", collunits = cus)
    st <- if (!is.null(x$site)) x$site else x
    st_l <- list(sitename = st$sitename,
                 siteid = st$siteid,
                 geography = st$geography,
                 altitude = st$altitude,
                 description = st$sitedescription,
                 notes = st$sitenotes,
                 collunits = cu)
    do.call(build_site, st_l)
  })
  new_sites <- new("sites", sites = new_sites)
  new_sites <- clean(new_sites)
  return(new_sites)
}

#' @title normalize_agerange
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @description An internal helper function used to parse age range information
#' @param agerange age range information from API response
#' @returns list with normalized age range information
#' @noRd
normalize_agerange <- function(agerange) {
  if (is.null(agerange) || length(agerange) == 0) {
    list(ageold = NA, ageyoung = NA, units = NA)
  } else {
    list(ageold = agerange[[1]]$ageold,
         ageyoung = agerange[[1]]$ageyoung,
         units = agerange[[1]]$units)
  }
}

#' @title build_chron
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom assertthat assert_that
#' @importFrom purrr map
#' @importFrom dplyr bind_rows
#' @description
#' A helper function to build a new chronology object from the
#' Neotoma API response.
#' @param x A chronology element from the API JSON output.
#' @returns A single `chronology` object.
#' @details This function is an internal function called from
#' `build_collunit()` to help support the translation between the JSON
#' representation of data in the API and the R implementation.
#' @noRd
build_chron <- function(...) {
  args <- list(...)
  assert_that(is.list(args), msg = "Parsed object must be a list.")
  df <- map(args$chroncontrols, function(x) {
    as.data.frame(list(
      depth = use_na(x$depth, "int"),
      thickness = use_na(x$thickness, "int"),
      agelimityounger = use_na(x$agelimityounger, "int"),
      agelimitolder = use_na(x$agelimitolder, "int"),
      chroncontrolid = use_na(x$chroncontrolid, "int"),
      chroncontrolage = use_na(x$chroncontrolage, "int"),
      chroncontroltype = use_na(x$chroncontroltype, "char")
    ))
  }) %>%
    bind_rows()
  chron_table <- df[!duplicated(df), ]
  chronology <- set_chronology(chronologyid = use_na(args$chronologyid, "int"),
                               notes = use_na(args$notes, "char"),
                               contact = use_na(args$contact, "list"),
                               agemodel = use_na(args$agemodel, "char"),
                               ageboundolder = use_na(args$ageboundolder,
                                                      "int"),
                               ageboundyounger = use_na(args$ageboundyounger,
                                                        "int"),
                               isdefault = use_na(args$isdefault, "bool"),
                               dateprepared = use_na(args$dateprepared, "date"),
                               modelagetype = use_na(args$modelagetype, "char"),
                               chronologyname = use_na(args$chronologyname,
                                                       "char"),
                               chroncontrols = chron_table)
  return(chronology)
}

#' @title Recover information about the chron controls for a collectionunit.
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom dplyr bind_rows mutate
#' @importFrom purrr map
#' @description For all sites that includes collection units with chronologies
#' return the chronological controls that are used in building the chronology.
#' @param x sites object
#' @returns data.frame with chronological controls
#' @export
setMethod(f = "chroncontrols",
  signature = "sites",
  definition = function(x) {
    output <- map(x@sites, function(y) chroncontrols(y)) %>%
      bind_rows()
    return(output)
  }
)

#' @title Recover information about the chron controls for a collectionunit.
#' @description For a site that includes collection units with chronologies
#' return the chronological controls that are used in building the chronology.
#' @importFrom dplyr bind_rows select mutate everything
#' @importFrom purrr map
#' @param x site object
#' @returns data.frame with chronological controls
#' @export
setMethod(f = "chroncontrols",
  signature = "site",
  definition = function(x) {
    siteid <- as.data.frame(x)$siteid
    chrons <- chronologies(x)
    chronset <- map(chrons@chronologies, function(y) {
      data.frame(chronologyid = y@chronologyid,
                 y@chroncontrols)
    }) %>%
      bind_rows() %>%
      mutate(siteid = siteid)
    chronset <- chronset %>%
      select(siteid, everything())
    return(chronset)
  }
)

#' @title get_downloads
#' @name get_downloads
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom dplyr select
#' @importFrom methods new
#' @description
#' Download `sites` objects up to datum and chronology detail level.
#' @param x Use a single number to extract site information
#' @param verbose Status bar of items being downloaded
#' @param ... accepted arguments: sites, datasets
#' @details
#' The `get_downloads()` command wraps the Neotoma API
#' ([api.neotomadb.org](https://api.neotomadb.org)) call for `downloads`.
#' The call itself uses a SQL query which accepts any one of the following
#' parameters:
#'  * `datasetid` The unique dataset ID (integer) in Neotoma. Can be passed
#' as a vector of dataset IDs.
#'  * `all_data` The API only downloads the first 25 records of the query. 
#'  For the complete records, use `all_data=TRUE`
#' @returns The function returns either a single item of class
#' \code{"try-error"} describing the reason for failure
#' (either misdefined parameters or an error from the Neotoma API),
#' or a table of sites, with rows corresponding to the number of
#' individual sites and datasets returned by the Neotoma API.
#' Each "site" object contains 6 parameters that can be accessed as well:
#' \item{ \code{siteid} }{site ID number}
#' \item{ \code{sitename} }{site's name}
#' \item{ \code{location} }{sf object that describes site's location}
#' \item{ \code{description} }{}
#' \item{ \code{collunits} }{limited information on collunits}
#' Each "collection unit" embedded in the "sites" object contains
#' 6 parameters that can be accessed as well:
#' \item{ \code{collunitid}}{collection unit ID number}
#' \item{ \code{handle} }{collection unit's handle}
#' \item{ \code{collunitname} }{collection unit's name}
#' \item{ \code{colldate} }{date in collection unit}
#' \item{ \code{substrate} }{substrate}
#' \item{ \code{location} }{sf object that describes site's location}
#' \item{ \code{datasets} }{detailed information regarding dataset}
#' Each "dataset" nested in the "collection unit" contains the
#' following detail of information:
#' \item{ \code{datasetid} }{dataset ID number}
#' \item{ \code{datasetname} }{site's name}
#' \item{ \code{datasettype} }{type of data found}
#' \item{ \code{location} }{sf object that describes site's location}
#' \item{ \code{notes} }{notes on the dataset}
#' \item{ \code{taxa table} }{taxa table}
#' \item{ \code{pi list} }{P.I. info}
#' \item{ \code{analyst} }{analyst info}
#' \item{ \code{metadata} }{dataset metadata}
#' @examples \dontrun{
#' # To find the downloads object of dataset 24:
#' tryCatch({
#'   downloads24 <- get_downloads(24)
#' }, error = function(e) {
#'  message("Neotoma server not responding. Try again later.")
#' })
#' # To find all downloads in Brazil
#' brazil <- '{"type": "Polygon",
#' "coordinates": [[
#'  [-73.125, -9.102096738726443],
#'  [-56.953125,-33.137551192346145],
#'  [-36.5625,-7.710991655433217],
#'  [-68.203125,13.923403897723347],
#'  [-73.125,-9.102096738726443]]]}'
#' tryCatch({
#'   brazil_datasets <- get_datasets(loc = brazil[1])
#'   brazil_downloads <- get_downloads(brazil_datasets)
#' }, error = function(e) {
#'   message("Neotoma server not responding. Try again later.")
#' })
#' }
#' @md
#' @export
get_downloads <- function(x = NA, ...) {
  UseMethod("get_downloads", x)
}

#' @rdname get_downloads
#' @exportS3Method get_downloads numeric
get_downloads.numeric <- function(x, ...) {
  if (length(x) > 0) {
    dataset <- paste0(x, collapse = ",")
  }
  baseURL <- paste0("data/downloads?datasetid=", dataset)
  result <- tryCatch(
    parseURL(baseURL, ...),
    error = function(e1) {
      tryCatch(
        parseURL(baseURL),
        error = function(e2) {
          stop("API call failed: ", e2$message)
        }
      )
    }
  )
  if (!is.null(result[2]$data) || (length(result[2]$data) > 0)) {
    output <- parse_site(result)
    return(output)
  } else {
    return(NULL)
  }
}

#' @rdname get_downloads
#' @exportS3Method get_downloads sites
get_downloads.sites <- function(x, verbose = TRUE, ...) {
  ids <- getids(x)$datasetid
  cl <- as.list(match.call())
  cl[[1]] <- NULL
  if ("all_data" %in% cl) {
    all_data <- cl$all_data
  } else {
    all_data <- TRUE
  }
  output <- get_downloads(x = ids, all_data = all_data)
  return(output)
}

## for null x
#' @rdname get_downloads
#' @exportS3Method get_downloads NULL
get_downloads.NULL <- function(x, ...) {
  return(NULL)
}

library("testthat")
library("neotoma2")

context("`chronologies()` function displays all chronologies for a download object")
test_that("`get_downloads()` fills up chronologies' slots.", {
  skip_on_cran()
  dl <- get_downloads(4716)
  chron <- dl %>% chronologies() %>% as.data.frame()

  testthat::expect_is(dl[[1]]@collunits[[1]]@chronologies, "chronologies")
  testthat::expect_true(nrow(chron) > 1)
  testthat::expect_true(any(chron$chronologyid == 2195))
  # Only one isdefault value
  # Careful, if the DB is wrong, so will the API and this test will fail
  testthat::expect_equal(sum(chron$isdefault), 1)
  testthat::expect_is(chron, "data.frame")
})

library("testthat")
library("neotoma2")

test_that("Set and change the default chronology.", {
  skip_on_cran()
  site <- get_downloads(24238)
  chrono <- chronologies(site)
  newchron <- set_default(chrono, 14590)
  df_default <- as.data.frame(chrono)$isdefault
  df_newchron <- as.data.frame(newchron)$isdefault
  tester <- all(df_default == df_newchron)
  testthat::expect_false(tester)
})

library("testthat")
library("neotoma2")

context("`taxa()` function")
test_that("taxa() returns only unique results", {
  skip_on_cran()
  mydataset <- get_downloads(c(1435, 24238))
  df <- taxa(mydataset)
  testthat::expect_false(any(duplicated(df)))
  testthat::expect_is(df, "data.frame")
})

library("testthat")
library("neotoma2")
library("dplyr")

context("Chronology controls functions work as expected.")
test_that("Chroncontrols gets record", {
  skip_on_cran()
  single <- chroncontrols(get_downloads(4716))
  multi <- chroncontrols(get_downloads(21007))
  mamchron <- chroncontrols(get_downloads(4564))
  
  testthat::expect_is(single, "data.frame")
  testthat::expect_equal(length(unique(single$chronologyid)), 3)
  testthat::expect_is(multi, "data.frame")
  testthat::expect_gt(length(unique(multi$chronologyid)), 4)
  testthat::expect_is(mamchron, "data.frame")
})

#' @title speleothems
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom dplyr bind_rows distinct select filter
#' @importFrom dplyr arrange left_join rename
#' @importFrom purrr map
#' @param x sites object
#' @returns `data.frame` with sample records
#' @description Obtain all speleothems within a sites object
#' Experimental function: API and behavior may change.
#' @examples \dontrun{
#' ds <- get_datasets(37302)
#' sp <- speleothems(ds)
#' }
#' @md
#' @export
setMethod(f = "speleothems",
  signature = "sites",
  definition = function(x) {
    output <- map(x@sites,
                  function(y) speleothems(y)) %>%
      bind_rows()
    if (output$datasetid %>% is.null()) {
      return(data.frame())
    } else {
      output <- output %>%
        select(.data$siteid, .data$sitename, .data$collectionunitid,
               .data$datasetid, .data$entityid, .data$entityname,
               .data$speleothemtype, .data$geology, .data$relativeage,
               .data$monitoring, .data$speleothemdriptype, .data$dripheight,
               .data$dripheightunits, .data$covertype,
               .data$entitycoverthickness, .data$entrancedistance,
               .data$entrancedistanceunits, .data$landusecovertype,
               .data$landusecoverpercent, .data$vegetationcovertype,
               .data$vegetationcoverpercent) %>%
        distinct() %>%
        arrange(.data$entityid, .data$siteid, .data$collectionunitid)
    }
    if (nrow(output) == 0) {
      msg <- "No assigned speleothems. Is it a speleothem dataset? \n
                          Did you run get_speleothems()?"
      warnsite <- sprintf(msg)
      warning(warnsite)
    }
    return(output)
  }
)

#' @rdname speleothems
#' @export
setMethod(f = "speleothems",
  signature = "site",
  definition = function(x) {
    allids <- getids(x) %>%
      distinct() %>%
      select(.data$siteid, .data$datasetid)
    dsids <- as.data.frame(datasets(x)) %>%
      filter(.data$datasettype == "speleothem")
    dsids <- dsids$datasetid
    if (length(dsids) == 0) {
      warning(sprintf("No speleothem datasets for siteid %s", x@siteid))
      return(data.frame())
    } else {
      allids <- allids %>%
        filter(.data$datasetid %in% dsids)
      siteinfo <- as.data.frame(x) %>%
        distinct() %>%
        left_join(allids, by = "siteid")
      sampset <- map(x@collunits@collunits,
                    function(y) speleothems(y)) %>%
        bind_rows() %>%
        left_join(siteinfo, by = "datasetid") %>%
        rename(sitenotes = .data$notes)
      return(sampset)
    }
  }
)

#' @rdname speleothems
#' @export
setMethod(f = "speleothems",
  signature = "collunits",
  definition = function(x) {
    df <- map(x@collunits, function(x) speleothems(x)) %>%
      bind_rows()
    return(df)
  }
)

#' @rdname speleothems
#' @export
setMethod(f = "speleothems",
  signature = "collunit",
  definition = function(x) {
    dsids <- as.data.frame(datasets(x)) %>%
      filter(.data$datasettype == "speleothem") %>%
      mutate(collectionunitid = x@collectionunitid) %>%
      select(.data$collectionunitid, .data$datasetid)
    if (length(x@speleothems@speleothems) == 0) {
      warning(sprintf("No assigned speleothems. Is it a speleothems dataset?
              Did you run `get_speleothems()`?"))
      return(data.frame())
    } else {
      speleothemset <-
        map(x@speleothems@speleothems,
            function(y) {
              y <- as.data.frame(y)
              if (!is.null(y) && nrow(y) > 0) {
                df <- data.frame(
                  collectionunitid = y$collectionunitid,
                  entityid = y$entityid,
                  entityname = y$entityname,
                  speleothemtype = y$speleothemtype,
                  speleothemdriptype = y$speleothemdriptype,
                  dripheight = y$dripheight,
                  dripheightunits = y$dripheightunits,
                  monitoring = y$monitoring,
                  geology = y$geology,
                  relativeage = y$relativeage,
                  covertype = y$entitycovertype,
                  entitycoverthickness = y$entitycoverthickness,
                  entrancedistance = y$entrancedistance,
                  entrancedistanceunits = y$entrancedistanceunits,
                  landusecovertype = y$landusecovertype,
                  landusecoverpercent = y$landusecoverpercent,
                  vegetationcovertype = y$vegetationcovertype,
                  vegetationcoverpercent = y$vegetationcoverpercent
                ) %>%
                  left_join(dsids, by = "collectionunitid")
              } else {
                df <- data.frame()
              }
              return(df)
            }) %>%
        bind_rows()
      speleothemset  <- speleothemset %>%
        distinct()
      return(speleothemset)
    }
  }
)

#' @title Helper function for `get_speleothems`
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom purrr map
#' @param sites A `sites` object
#' @return Processed speleothem data in the `sites` object
#' @noRd
speleo_helper <- function(sites) {
  ids <- getids(sites)
  cuids <- ids$collunitid
  if (length(cuids) > 0) {
    cuids <- paste0(cuids, collapse = ",")
  }
  baseURL <- paste0("data/speleothems/", cuids)
  result <- tryCatch(
    parseURL(baseURL),
    error = function(e) {
      message("API call failed: ", e$message)
      NULL
    }
  )
  if (length(result$data) > 0) {
    speleo <- parse_speleothem(result$data)
    speleo <- speleo %>% cleanNULL()
  } else {
    speleo <- NULL
  }
  if (!is.null(speleo)) {
    pared_ds <- map(sites@sites, function(x) {
      ycu <- map(x@collunits, function(z) {
        yds <- speleo[
          which(as.data.frame(speleo)$collectionunitid %in% z$collectionunitid)
        ]
        z@speleothems <- yds
        z
      })
      x@collunits@collunits <- ycu
      x
    })
  } else {
    pared_ds <- NULL
  }
  pared_ds
}

#' @title get_speleothems
#' @name get_speleothems
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom methods new
#' @param x A dataset ID or vector of dataset IDs
#' @param ... accepted arguments
#' @returns `sites` object with speleothem data
#' @details
#' Experimental function: API and behavior may change.
#' The `get_speleothems()` command wraps the Neotoma API
#' ([api.neotomadb.org](https://api.neotomadb.org)) call for `speleothems`.
#' The call itself uses a SQL query which accepts any one of the following
#' parameters:
#'  * `x`  The unique dataset ID (integer) in Neotoma. Can be passed as a
#' vector of dataset IDs.
#'  * `sites`  A `sites` R object.
#' @examples \dontrun{
#' ## Find speleothems by numeric datasetid:
#' speleo <- get_speleothems(c(2,5))
#' }
#' @md
#' @export
get_speleothems <- function(x = NA, ...) {
  if (missing(x)) {
    UseMethod("get_speleothems", "default")
  } else {
    UseMethod("get_speleothems", x)
  }
}


#' @rdname get_speleothems
#' @method get_speleothems numeric
#' @exportS3Method get_speleothems numeric
get_speleothems.numeric <- function(x, ...) {
  st <- get_datasets(x)
  pared_ds <- speleo_helper(st)
  if (is.null(pared_ds)) {
    st
  } else {
    new("sites", sites = pared_ds)
  }
}

#' @rdname get_speleothems
#' @method get_speleothems sites
#' @exportS3Method get_speleothems sites
get_speleothems.sites <- function(x, ...) {
  pared_ds <- speleo_helper(x)
  if (is.null(pared_ds)) {
    x
  } else {
    new("sites", sites = pared_ds)
  }
}

#' @title pingNeotoma
#' @importFrom httr HEAD
#' @importFrom stringr str_detect
#' @importFrom assertthat assert_that
#' @description A quick function to test whether or not the Neotoma Database
#' API is currently running.
#' @param server One of \code{localhost:PORT} (where \code{PORT} is a valid
#' numeric port), \code{neotoma} or \code{dev}.
#' @returns A valid HTTP status code or returns an error if a connection
#' is refused.
#' @examples \dontrun{
#' test_connection <- pingNeotoma("neotoma")
#' }
#' @export
pingNeotoma <- function(server = "neotoma") {
  valid_local <- str_detect(server,
                            "(http?:////){0,1}localhost:\\d{1,5}$")
  assert_that(valid_local | (server %in% c("neotoma", "dev")),
              msg = "The parameter `server` must be a valid 
                    localhost (e.g., localhost:3005), or `neotoma` or `dev`.")
  server <- switch(server,
                   neotoma = "https://api.neotomadb.org",
                   dev = "https://api-dev.neotomadb.org",
                   server)
  status <- HEAD(server)
  return(status)
}

library("testthat")
library("neotoma2")

context("Ping Neotoma API to check availability")
test_that("The Pings work", {
  skip_on_cran()
  ping_head <- try(httr::HEAD("https://api.neotomadb.org"))
  ping_call <- try(pingNeotoma("neotoma"))
  testthat::expect_equal(ping_head$status_code,
    ping_call$status_code)
  
  ping_head <- try(httr::HEAD("https://api-dev.neotomadb.org"))
  ping_call <- try(pingNeotoma("dev"))
  testthat::expect_equal(ping_head$status_code,
                         ping_call$status_code)
})

#' @title Apply a `filter` for Neotoma sites objects.
#' @name filter
#' @author Simon Goring \email{goring@wisc.edu}
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @description The \code{filter} function takes a \code{sites} object
#' and allows a user to filter on a number of properties. Since a sites object
#' is a nested object (it contains collection units, datasets, samples, etc.)
#' the degree to which filtering occurs depends on the amount of data contained
#' within the sites object. Filtering parameters include:
#'  * `siteid` A numeric site identifier from the Neotoma Database.
#'  * `sitename` The character string sitename.
#'  * `lat` A numeric latitude value.
#'  * `long` A numeric longitude value.
#'  * `altitude` The elevation of the site. Note that some sites do not
#'   include elevation information. For these an NA value appears, and they
#'   would be removed when using an elevation filter.
#'  * `datasetid` A numeric datasetid from Neotoma.
#'  * `database` A character string naming the constituent database
#'   from which the dataset is drawn.
#'  * `datasettype` A character string representing one of the many
#'   dataset types within Neotoma.
#'  * `age_range_old` A dataset-level parameter indicating the oldest
#'   date covered by the dataset chronology.
#'  * `age_range_young` A dataset-level parameter indicating the youngest
#'   date covered by the dataset chronology.
#'  * `notes` Free-form dataset notes provided by the dataset PI(s),
#'   analysts or data stewards.
#'  * `collectionunitid` A numeric collection unit identifier from
#'   Neotoma.
#'  * `handle` A character string identifying the collection unit. These
#'   are often shorter form names (originally a default 8 character length).
#'  * `collectionunitname` A character string identifying the collection
#'   unit name.
#'  * `colldate` The date on which the collection unit was sampled. Many
#'   of these are empty.
#'  * `location` A free-form character string indicating the location of
#'   the collection unit within the site.
#'  * `waterdepth` A numeric depth at which the core was obtained.
#'  * `collunittype` A character string for the collection unit type.
#'  * `collectiondevice` A fixed vocabulary term for the collection
#'   device.
#'  * `depositionalenvironment` A fixed vocabulary name for the
#'   depositional environment.
#' @importFrom dplyr filter inner_join
#' @importFrom purrr map
#' @importFrom stringr str_detect
#' @param .data A site, dataset, download, or data frame
#' @param ... Additional arguments passed to `filter()`
#' @param .by (only used for filtering `data.frame` objects)
#' @param .preserve (only used for filtering `data.frame` objects)
#' @returns filtered `sites` object
#' @examples \dontrun{
#' # Download 10 sites, but only keep the sites that are close to sea level.
#' tryCatch({
#'  some_sites <- get_sites(sitename = "Lake%", limit = 3)
#'   site_subset <- some_sites %>% filter(altitude < 100)
#' }, error = function(e) {
#'   message("Neotoma server not responding. Try again later.")
#' })
#' # Download 10 sites, get all associated datasets, but keep only
#' # sites/datasets that are of datasettype "pollen":
#' tryCatch({
#'   sites <- get_sites(limit = 10) %>%
#'     get_datasets()
#'   pollen_subset <- sites %>% filter(datasettype == "pollen")
#' }, error = function(e) {
#'   message("Neotoma server not responding. Try again later.") 
#' })
#' }
#' @md
#' @export
filter <- function(.data, ..., .by = NULL, .preserve = FALSE) {
  UseMethod("filter")
}

#' @rdname filter
#' @exportS3Method filter NULL
filter.NULL <- function(.data, ...) {
  warning("No sites to filter")
  return(NULL)
}

#' @rdname filter
#' @exportS3Method filter sites
filter.sites <- function(.data, ...) {
  x <- .data
  ellipsis <- as.list(substitute(list(...), environment()))[-1L][[1]] %>%
    as.character()
  sitecols <- c("sitename", "lat", "long", "altitude") %>%
    map(function(x) any(stringr::str_detect(ellipsis, x))) %>%
    unlist() %>%
    any()
  datasetcols <- c("datasetid", "database", "datasettype", "age_range_old",
                   "age_range_young", "notes") %>%
    map(function(x) any(stringr::str_detect(ellipsis, x))) %>%
    unlist() %>%
    any()
  collunitcols <- c("collectionunitid", "handle", "colldate",
                    "location", "waterdepth", "collunittype",
                    "collectiondevice", "defaultchronology",
                    "collectionunitname", "depositionalenvironment") %>%
    map(function(x) any(stringr::str_detect(ellipsis, x))) %>%
    unlist() %>%
    any()
  ids <- getids(x)
  if (sitecols == TRUE) {
    ids <- ids %>%
      inner_join(as.data.frame(x), by = "siteid") %>%
      rename(altitude = .data$elev,
             sitenotes = .data$notes)
  }
  if (collunitcols == TRUE) {
    ids <- ids %>%
      inner_join(as.data.frame(collunits(x)),
                 by = c("collunitid" = "collectionunitid"))
  }
  if (datasetcols == TRUE) {
    ids <- ids %>%
      inner_join(mutate(as.data.frame(datasets(x)),
                        datasetid = as.numeric(.data$datasetid)),
                 by = "datasetid")
  }
  cleanids <- ids %>%
    dplyr::filter(...)
  if (nrow(cleanids) == 0) {
    return(new("sites"))
  }
  siteids <- unique(as.data.frame(x)$siteid)
  pared_sites <- x[which(siteids %in% cleanids$siteid)]
  # Clear datasets:
  good_dsid <- unique(cleanids$datasetid)
  good_cuid <- unique(cleanids$collunitid)
  pared_ds <- purrr::map(pared_sites@sites, function(x) {
    ycu <- collunits(x)
    ycu <- ycu[which(as.data.frame(ycu)$collectionunitid %in% good_cuid)]
    xcu <- purrr::map(ycu@collunits, function(y) {
      yds <- datasets(y)
      yds <- yds[which(as.data.frame(yds)$datasetid %in% good_dsid)]
      y@datasets <- yds
      return(y)
    })
    x@collunits@collunits <- xcu
    return(x)
  })
  return(new("sites", sites = pared_ds))
}

#' This is a re-export of \code{dplyr::filter} for data frames.
#' @rdname filter
#' @exportS3Method filter data.frame
filter.data.frame <- getS3method("filter",
                                 "data.frame",
                                 envir = asNamespace("dplyr"))

#' @aliases show,speleothem-method
#' @rdname show
setMethod(f = "show",
          signature = "speleothem",
          definition = function(object) {
            print(data.frame(entityid = object@entityid,
                    entityname = object@entityname,
                    siteid = object@siteid,
                    collectionunitid = object@collectionunitid,
                    speleothemdriptype = object@speleothemdriptype,
                    dripheight = object@dripheight,
                    dripheightunits = object@dripheightunits,
                    monitoring = object@monitoring,
                    geology = object@geology,
                    relativeage = object@relativeage,
                    speleothemtype = object@speleothemtype,
                    entrancedistance = object@entrancedistance,
                    entrancedistanceunits = object@entrancedistanceunits,
                    landusecovertype = object@landusecovertype,
                    landusecoverpercent = object@landusecoverpercent,
                    vegetationcovertype = object@vegetationcovertype,
                    vegetationcoverpercent = object@vegetationcoverpercent,
                    entitycovertype = object@entitycovertype,
                    entitycoverthickness = object@entitycoverthickness
                  ), row.names = FALSE)
          })

#' @aliases show,speleothems-method
#' @rdname show
setMethod(f = "show",
          signature = "speleothems",
          definition = function(object) {
            map(object, function(y) {
              data.frame(entityid = y@entityid,
                         entityname = y@entityname,
                         siteid = y@siteid,
                         collectionunitid = y@collectionunitid,
                         speleothemdriptype = y@speleothemdriptype,
                         dripheight = y@dripheight,
                         dripheightunits = y@dripheightunits,
                         monitoring = y@monitoring,
                         geology = y@geology,
                         relativeage = y@relativeage,
                         speleothemtype = y@speleothemtype,
                         entrancedistance = y@entrancedistance,
                         entrancedistanceunits = y@entrancedistanceunits,
                         landusecovertype = y@landusecovertype,
                         landusecoverpercent = y@landusecoverpercent,
                         vegetationcovertype = y@vegetationcovertype,
                         vegetationcoverpercent = y@vegetationcoverpercent,
                         entitycovertype = y@entitycovertype,
                         entitycoverthickness = y@entitycoverthickness)
            }) %>%
              bind_rows() %>%
              print(row.names = FALSE)
          })

#' @aliases sub-sub,speleothems-method
#' @rdname sub-sub
setMethod(f = "[[",
          signature = signature(x = "speleothems", i = "numeric"),
          definition = function(x, i) {
            if (length(i) == 1) {
              out <- new("speleothem", x@speleothems[[i]])
            } else {
              out <- map(i, function(z) {
                new("dataset", x@speleothems[[z]])
              })
              out <- new("speleothems", speleothems = out)
            }
            return(out)
          })

#' @aliases names,speleothem-method
#' @rdname names
setMethod(f = "names",
          signature = signature(x = "speleothem"),
          definition = function(x) {
            slotNames(x)
          })

#' @aliases sub-subset,speleothems-method
#' @rdname sub-subset
setMethod(f = "[[<-",
          signature = signature(x = "speleothems"),
          definition = function(x, i, value) {
            speleothemset <- x@speleothems
            speleothemset[[i]] <- value
            out <- new("speleothems", speleothems = speleothemset)
            return(out)
          })

#' @aliases subset,speleothem-method
#' @rdname subset
setMethod(f = "[<-",
          signature = signature(x = "speleothem", i = "character"),
          definition = function(x, i, value) {
            for (idx in seq_along(i)) {
              slot(x, i[idx]) <- value[idx]
            }
            return(x)
          })

#' @aliases subset,speleothem-method
#' @rdname subset
setMethod(f = "[<-",
          signature = signature(x = "speleothem", i = "numeric"),
          definition = function(x, i, value) {
            slots <- slotNames(x)
            for (idx in seq_along(i)) {
              slot(x, slots[i[idx]]) <- value[idx]
            }
            return(x)
          })

#' @aliases cash-set,speleothem-method
#' @rdname cash-set
setMethod(f = "$<-",
          signature = signature(x = "speleothem"),
          definition = function(x, name, value) {
            slot(x, name) <- value
            return(x)
          })

#' @aliases sub,speleothems-method
#' @rdname sub
setMethod(f = "[",
          signature = signature(x = "speleothems", i = "numeric"),
          definition = function(x, i) {
            new("speleothems", speleothems = x@speleothems[i])
          })

#' @aliases cash,speleothem-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "speleothem"),
          definition = function(x, name) {
            slot(x, name)
          })

#' @aliases cash,speleothems-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "speleothems"),
          definition = function(x, name) {
            x %>%
              map(function(y) {
                slot(y, name)
              }) %>%
              unlist()
          })

#' @aliases as.data.frame,speleothem-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("speleothem"),
          definition = function(x) {
            data.frame(entityid = x@entityid,
                       entityname = x@entityname,
                       siteid = x@siteid,
                       collectionunitid = x@collectionunitid,
                       speleothemdriptype = x@speleothemdriptype,
                       dripheight = x@dripheight,
                       dripheightunits = x@dripheightunits,
                       monitoring = x@monitoring,
                       geology = x@geology,
                       relativeage = x@relativeage,
                       speleothemtype = x@speleothemtype,
                       entrancedistance = x@entrancedistance,
                       entrancedistanceunits = x@entrancedistanceunits,
                       landusecovertype = x@landusecovertype,
                       landusecoverpercent = x@landusecoverpercent,
                       vegetationcovertype = x@vegetationcovertype,
                       vegetationcoverpercent = x@vegetationcoverpercent,
                       entitycoverthickness = x@entitycoverthickness,
                       entitycovertype = x@entitycovertype)
          })

#' @aliases as.data.frame,speleothems-method
#' @rdname as.data.frame
setMethod(f = "as.data.frame",
          signature = signature("speleothems"),
          definition = function(x) {
            x@speleothems %>% map(as.data.frame) %>% bind_rows()
          })

#' @aliases length,speleothems-method
#' @rdname length
setMethod(f = "length",
          signature = signature(x = "speleothems"),
          definition = function(x) {
            length(x@speleothems)
          })

#' @aliases c,speleothems-method
#' @rdname c
setMethod(f = "c",
          signature = signature(x = "speleothems"),
          definition = function(x, y) {
            new("speleothems",
                speleothems = unlist(c(x@speleothems,
                                       y@speleothems),
                                     recursive = FALSE))
          })

#' @title samples
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @author Simon Goring \email{goring@wisc.edu}
#' @param x sites object
#' @description Obtain all samples within a sites object
#' @examples \dontrun{
#' # Get full data download from API and create a long table with samples data.
#' dw <- get_downloads(1)
#' pollen <- samples(dw)
#' }
#' @importFrom dplyr bind_rows left_join rename mutate
#' @importFrom purrr map
#' @returns `data.frame` with sample records
#' @md
#' @export
setMethod(f = "samples",
  signature = "sites",
  definition = function(x) {
    output <- map(x@sites, function(y) samples(y)) %>%
      bind_rows() # %>%
      # Handle NAs to allow distinct to work properly
      #distinct(.data$sampleid, .keep_all = TRUE)
    if (nrow(output) == 0) {
      warnsite <- sprintf("No assigned samples. Did you run get_downloads()?")
      warning(warnsite)
    }
    return(output)
  }
)

#' @rdname samples
#' @export
setMethod(f = "samples",
  signature = "site",
  definition = function(x) {
    allids <- getids(x)
    siteinfo <- as.data.frame(x) %>%
      left_join(allids, by = "siteid")
    sampset <- map(x@collunits@collunits,
                   function(y) samples(y)) %>%
      bind_rows() %>%
      bind_rows() %>%
      left_join(siteinfo, by = "datasetid") %>%
      rename(sitenotes = .data$notes)
    return(sampset)
  }
)

#' @rdname samples
#' @export
setMethod(f = "samples",
  signature = "collunits",
  definition = function(x) {
    map(x@collunits, function(x) samples(x)) %>%
      bind_rows()
  }
)

#' @rdname samples
#' @export
setMethod(f = "samples",
  signature = "collunit",
  definition = function(x) {
    precedence <- c("Calendar years BP",
                    "Calibrated radiocarbon years BP",
                    "Radiocarbon years BP", "Varve years BP")
    ids <- getids(x)
    # Check the chronologies to make sure everything is okay:
    if (length(chronologies(x)) > 0) {
      # This pulls the chronology IDs, then applies the Neotoma
      # age model precedence (see get_table('agetypes')).
      # It returns a value that is larger when your age reporting is
      # better.
      defaultchron <- map(chronologies(x)@chronologies,
                          function(y) {
                            data.frame(chronologyid = y@chronologyid,
                                       isdefault = y@isdefault,
                                       modelagetype = y@modelagetype,
                                       chronologyname = y@chronologyname,
                                       dateprepared = y@dateprepared)
                          }) %>%
        bind_rows() %>%
        mutate(modelrank = match(.data$modelagetype, rev(precedence)),
               order = .data$isdefault * match(.data$modelagetype,
                                               rev(precedence)))
      # Validation of default chrons, we want to check whether there
      # exists either multiple default chronologies for the same
      # time-frame or, alternately, no default chronology.
      all_na <- all(is.na(defaultchron$order))
      max_order <- max(defaultchron$order, na.rm = TRUE)
      if (sum(defaultchron$order == max_order, na.rm = TRUE) > 1) {
        if (any(is.na(defaultchron$dateprepared))) {
          high_chron <- defaultchron$order == max_order
          newmax_order <- which.max(defaultchron$chronologyid[high_chron])
          defaultchron$order[high_chron][newmax_order] <- max_order + 1
        } else {
          newmax_order <- which.max(defaultchron$dateprepared[
                                      defaultchron$order == max_order])
          defaultchron$order[defaultchron$order == max_order][
                                      newmax_order] <- max_order + 1
        }
      }
      if (all_na == TRUE) {
        warnsite <- sprintf("The dataset %s has no default chronologies.",
                            ids$datasetid[1])
        warning(warnsite)
      } else if (sum(defaultchron$order == max_order, na.rm = TRUE) > 1) {
        warnsite <- sprintf("The dataset %s has multiple default chronologies.
                             Chronology %s has been used.", ids$datasetid[1],
                            defaultchron$chronologyid[
                              which.max(defaultchron$order)])
        warning(warnsite)
        defaultchron <- defaultchron[which.max(defaultchron$order), ]
      } else {
        defaultchron <- defaultchron[which.max(defaultchron$order), ]
      }
    } else {
      defaultchron <- data.frame(chronologyid = NULL)
    }
    sampset <- map(datasets(x)@datasets,
                   function(y) {
                     dsid <- y$datasetid
                     allsamp <-
                       map(y@samples@samples,
                           function(z) {
                             whichage <-
                               which(z@ages$chronologyid ==
                                     defaultchron$chronologyid)
                             if (length(whichage) == 0) {
                               whichage <- 1
                             }
                             if (dim(z@datum)[1] > 0) {
                               df <-
                                 data.frame(z@ages[whichage,],
                                            z@datum,
                                            analysisunitid = z@analysisunitid,
                                            sampleanalyst =
                                            toString(unique(unlist(
                                                          z@sampleanalyst,
                                                          use.names = FALSE))),
                                            sampleid = z@sampleid,
                                            depth = z@depth,
                                            thickness = z@thickness,
                                            samplename = z@samplename,
                                            row.names = NULL)
                             } else {
                               df <- data.frame()
                             }
                             return(df)
                           }) %>%
                       bind_rows() %>%
                       mutate(datasetid = dsid)
                     return(allsamp)
                   }) %>%
      bind_rows() %>%
      left_join(as.data.frame(datasets(x)), by = "datasetid") %>%
      rename(datasetnotes = .data$notes)
    return(sampset)
  }
)

#' @title set Site Information for Fossil Sites
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom sf st_as_sf st_sfc
#' @importFrom uuid UUIDgenerate
#' @importFrom digest digest
#' @importFrom methods is new slot<-
#' @param x Object to be set as a site
#' @param siteid The unique `site id` for a site.
#' If this site is new to Neotoma then leave the ID as NA.
#' @param sitename Name of a site.
#' @param geography A \code{sf} object representing the site location,
#' either as a polygon or point.
#' @param altitude altitude/elevation of the site.
#' @param geopolitical The geopolitical unit in which the site is located.
#' @param area The area of the site or depositional basin in *ha*.
#' Can be calculated from the polygon.
#' @param description Function to create new site objects.
#' The new object will not be uploaded to the database.
#' @param notes additional information of the site
#' @param collunits Collection units in the site
#' @returns `site` object
#' @examples {
#' # Create a site called "My Lake", to
#' x <- sf::st_as_sf(sf::st_sfc(sf::st_point(c(5,5))))
#' my_site <- set_site(sitename = "My Lake",
#'                     geography = x,
#'                     description = "my lake",
#'                     altitude = 30)
#' }
#' @md
#' @export
set_site <- function(x = NA,
                     siteid = NA_integer_,
                     sitename = NA_character_,
                     geography = sf::st_as_sf(sf::st_sfc()),
                     altitude = NA_integer_,
                     geopolitical = list(),
                     area = NA_integer_,
                     notes = NA_character_,
                     description = NA_character_,
                     collunits = NULL) {
  function_call <- match.call()
  if (suppressWarnings(is.na(x))) {
    x <- new("site")
    if (is.na(siteid)) {
      hash <- digest(UUIDgenerate(), algo = "xxhash32", serialize = FALSE)
      x@siteid <- as.integer(strtoi(substr(hash, 1, 7), base = 16L))
    } else {
      x@siteid <- siteid
    }
    x@sitename <- sitename
    x@geography <- geography
    x@altitude <- altitude
    x@geopolitical <- geopolitical
    x@notes <- notes
    x@description <- description
    x@collunits <- collunits
  } else {
    if (is(x, "site")) {
      if (length(function_call) > 2) {
        for (i in 3:length(function_call)) {
          slot(x, names(function_call)[[i]]) <- eval(function_call[[i]])
        }
        return(x)
      } else {
        return(x)
      }
    } else {
      stop("`x` must be a site object if it is supplied.")
    }
  }
  return(x)
}

#' @aliases sub-sub,samples-method
#' @rdname sub-sub
setMethod(f = "[[",
          signature = signature(x = "samples", i = "numeric"),
          definition = function(x, i) {
            if (length(i) == 1) {
              out <- new("sample", x@samples[[i]])
            } else {
              out <- map(i, function(z) {
                new("sample", x@samples[[z]])
              })
              out <- new("samples", samples = out)
            }
            return(out)
          })

#' @aliases cash,samples-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "samples"),
          definition = function(x, name) {
            x %>%
              map(function(y) {
                slot(y, name)
              }) %>%
              unlist()
          })

#' @aliases cash,sample-method
#' @rdname cash
setMethod(f = "$",
          signature = signature(x = "sample"),
          definition = function(x, name) {
            slot(x, name)
          })

#' @aliases length,samples-method
#' @rdname length
setMethod(f = "length",
          signature = signature(x = "samples"),
          definition = function(x) {
            length(x@samples)
          })

#' @aliases c,samples-method
#' @rdname c
setMethod(f = "c",
          signature = signature(x = "samples"),
          definition = function(x, y) {
            samp <- new("samples",
                        samples = unlist(c(x@samples,
                                           y@samples),
                                         recursive = FALSE))
            return(samp)
          })

#' @aliases sub-subset,samples-method
#' @rdname sub-subset
setMethod(f = "[[<-", 
          signature = signature(x = "samples"),
          definition = function(x, i, value) {
            samples <- x@samples
            samples[[i]] <- value
            out <- new("samples", sites = samples)
            return(out)
          })

#' @aliases subset,sample-method
#' @rdname subset
setMethod(f = "[<-",
          signature = signature(x = "sample", i = "character"),
          definition = function(x, i, value) {
            for (idx in seq_along(length(i))) {
              slot(x, i[idx]) <- value[idx]
            }
            return(x)
          })

#' @title get_documentation
#' @author Socorro Dominguez \email{dominguezvid@@wisc.edu}
#' @description Open up the Neotoma R homepage.
#' @importFrom utils browseURL
#' @importFrom rlang is_interactive
#' @returns NULL
#' @examples \dontrun{
#' if (interactive()) {
#'  get_documentation()
#' }
#' }
#' @md
#' @export
get_documentation <- function() {
  if (is_interactive()) {
    browseURL(url = "https://open.neotomadb.org/neotoma2/")
  } else {
    stop("Browser window will not open if interactive() mode is FALSE.")
  }
  return(NULL)
}

#' @title Build a dataset object from a JSON list representation.
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom purrr map
#' @importFrom assertthat assert_that
#' @description
#' Helper function to build a dataset from the API JSON response.
#' @param args a JSON dataset object passed from the Neotoma API.
#' @returns A simple `dataset` object.
#' @noRd
build_dataset <- function(...) {
  args <- list(...)
  assert_that(is.list(args), msg = "Parsed object must be a list.")
  # PI Information
  pi_list <- testNull(args$datasetpi, list())
  if (length(pi_list) != 0) {
    pi_list <- pi_list %>%
      map(function(y) {
        if (is.na(y[1])) {
          NA_character_
        } else {
          y$contactname
        }
      })
  }
  args <- args %>% cleanNULL()
  ds <- set_dataset(datasetid = use_na(args$datasetid, "int"),
                    database = use_na(args$database, "char"),
                    doi = list(args$doi),
                    recdatecreated = use_na(args$recdatecreated, "date"),
                    datasettype = use_na(args$datasettype, "char"),
                    datasetname = use_na(args$datasetname, "char"),
                    age_range_old = use_na(args$age_range_old, "int"),
                    age_range_young = use_na(args$age_range_young, "int"),
                    age_units = use_na(args$units, "char"),
                    notes = use_na(args$datasetnotes, "char"),
                    pi_list = pi_list,
                    samples = testNull(args$samples, NULL),
                    specimens = testNull(args$specimens, NULL))
  return(ds)
}

#' @title get_manual
#' @author Simon Goring \email{goring@@wisc.edu}
#' @description Open up the Neotoma manual homepage.
#' @importFrom utils browseURL
#' @importFrom rlang is_interactive
#' @examples \dontrun{
#' # This call does not work from `source()` calls or in testing.
#' # interactive() just lets us know you are interacting with the console:
#' if (interactive()) {
#'  get_manual()
#' }
#' }
#' @returns NULL side effect for opening browser with the manual
#' @md
#' @export
get_manual <- function() {
  if (is_interactive()) {
    browseURL(url = "https://open.neotomadb.org/manual/")
  } else {
    stop("Browser window will not open if interactive() mode is FALSE.")
  }
  return(NULL)
}

library("testthat")
library("neotoma2")

context("Retrieves general Neotoma manual")
test_that("Manual loading fails if interactive is false.", {
  skip_on_cran()
  testthat::expect_error(get_manual())
})

test_that("Manual loading succeeds when interactive is true:", {
  skip_on_cran()
  testthat::expect_null(rlang::with_interactive(get_manual(),
                                                value=TRUE))
})

#' @title Extract taxonomic data from a set of sites.
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom dplyr bind_rows distinct group_by summarise n
#' @importFrom purrr map
#' @param object A \code{sites} object.
#' @returns A \code{data.frame} reporting the taxa/data objects, units,
#' elements and other features within a set of records.
#' @description Extract taxonomic data from a set of sites.
#' @examples \dontrun{
#' somesites <- get_sites(datasettype = "diatom") %>%
#'   get_downloads()
#' diatomtaxa <- taxa(somesites)
#' }
#' @md
#' @export
setMethod(f = "taxa",
  signature = "sites",
  definition = function(object) {
    output <- map(object@sites, function(y) taxa(y)) %>%
      bind_rows() %>%
      group_by(.data$units,
               .data$context,
               .data$element,
               .data$taxonid,
               .data$symmetry,
               .data$taxongroup,
               .data$elementtype,
               .data$variablename,
               .data$ecologicalgroup) %>%
      summarise(samples = sum(samples),
                sites = sum(.data$sites), .groups = "keep")
    if (nrow(output) == 0) {
      warnsite <- sprintf("No assigned samples. Did you run get_downloads()?")
      warning(warnsite)
    }
    return(output)
  }
)

#' @rdname taxa
#' @export
setMethod(f = "taxa",
          signature = "site",
          definition = function(object) {
            samples <- samples(object)
            tx_table <- samples %>%
              group_by(.data$units,
                       .data$context,
                       .data$element,
                       .data$taxonid,
                       .data$symmetry,
                       .data$taxongroup,
                       .data$elementtype,
                       .data$variablename,
                       .data$ecologicalgroup,
                       .data$siteid) %>%
              summarise(samples = n(), .groups = "keep") %>%
              group_by(.data$units,
                       .data$context,
                       .data$element,
                       .data$taxonid,
                       .data$symmetry,
                       .data$taxongroup,
                       .data$elementtype,
                       .data$variablename,
                       .data$ecologicalgroup) %>%
              summarise(sites = n(), samples = sum(samples), .groups = "keep")
            return(tx_table)
          })

#' @rdname taxa
#' @export
setMethod(f = "taxa",
          signature = "collunits",
          definition = function(object) {
            samples <- samples(object)
            tx_table <- samples %>%
              group_by(.data$units,
                       .data$context,
                       .data$element,
                       .data$taxonid,
                       .data$symmetry,
                       .data$taxongroup,
                       .data$elementtype,
                       .data$variablename,
                       .data$ecologicalgroup,
                       .data$siteid) %>%
              summarise(samples = n(), .groups = "keep") %>%
              group_by(.data$units,
                       .data$context,
                       .data$element,
                       .data$taxonid,
                       .data$symmetry,
                       .data$taxongroup,
                       .data$elementtype,
                       .data$variablename,
                       .data$ecologicalgroup) %>%
              summarise(sites = n(), samples = sum(samples), .groups = "keep")
            return(tx_table)
          })

#' @rdname taxa
#' @export
setMethod(f = "taxa",
          signature = "collunit",
          definition = function(object) {
            samples <- samples(object)
            tx_table <- samples %>%
              group_by(.data$units,
                       .data$context,
                       .data$element,
                       .data$taxonid,
                       .data$symmetry,
                       .data$taxongroup,
                       .data$elementtype,
                       .data$variablename,
                       .data$ecologicalgroup) %>%
              summarise(samples = n(), .groups = "keep") %>%
              group_by(.data$units,
                       .data$context,
                       .data$element,
                       .data$taxonid,
                       .data$symmetry,
                       .data$taxongroup,
                       .data$elementtype,
                       .data$variablename,
                       .data$ecologicalgroup) %>%
              summarise(sites = n(), samples = sum(samples), .groups = "keep")
            return(tx_table)
          })

#' @title Get `contact` information for Neotoma contributors
#' @name get_contacts
#' @author Simon Goring \email{goring@wisc.edu}
#' @author Socorro Dominguez \email{dominguezvid@wisc.edu}
#' @importFrom methods new
#' @param x integer A contact ID
#' @param ...
#' (\code{contactname})  A full or partial name for an individual
#'  contributor to the database.
#' (\code{familyname})  The full or partial last name for an
#'  individual contributor to the database.
#' (\code{status}) The current status of the contributor
#' (\code{active} or \code{retired})
#' @returns `contacts` object
#' @description Uses the Neotoma API to search and access
#'  information about individuals who have contributed to
#'  the data in the Neotoma Paleoecology Database
#' @md
#' @export
get_contacts <- function(x = NA, ...) {
  if (missing(x)) {
    UseMethod("get_contacts", "default")
  } else {
    UseMethod("get_contacts", x)
  }
}


#' @rdname get_contacts
#' @method get_contacts numeric
#' @exportS3Method get_contacts numeric
get_contacts.numeric <- function(x, ...) {
  if (length(x) > 0) {
    contactname <- paste0(x, collapse = ",")
  }
  baseURL <- paste0("data/contacts/", contactname)
  result <- tryCatch(
    parseURL(baseURL, ...),
    error = function(e) {
      stop("API call failed: ", e$message)
      NULL
    }
  )
  result <- result %>% cleanNULL()
  contact <- map(result$data,
                 function(x) {
                   x[is.null(x)] <- NA_character_
                   new("contact",
                       contactid = use_na(x$contactid, "int"),
                       familyname = use_na(x$familyname, "char"),
                       leadinginitials = use_na(x$leadinginitials,
                                                "char"),
                       givennames = use_na(x$givennames, "char"),
                       contactname = use_na(x$contactname, "char"),
                       suffix = use_na(x$suffix, "char"),
                       ORCID = use_na(x$ORCID, "char"),
                       title = use_na(x$title, "char"),
                       institution = use_na(x$institution, "char"),
                       email = use_na(x$email, "char"),
                       phone = use_na(x$phone, "char"),
                       contactstatus = use_na(x$contactstatus,
                                              "char"),
                       fax = use_na(x$fax, "char"),
                       url = use_na(x$url, "char"),
                       address = use_na(x$address, "char"),
                       notes = use_na(x$notes, "char"))
                 })
  contacts <- new("contacts", contacts = contact)
  return(contacts)
}

#' @rdname get_contacts
#' @method get_contacts default
#' @exportS3Method get_contacts default
get_contacts.default <- function(x, ...) {
  baseURL <- paste0("data/contacts")
  result <- tryCatch(
    parseURL(baseURL, ...),
    error = function(e) {
      stop("API call failed: ", e$message)
      NULL
    }
  )
  params <- get_params("contacts")
  cl <- as.list(match.call())
  cl[[1]] <- NULL
  cl <- lapply(cl, eval, envir = parent.frame())
  if (!all(names(cl) %in% params)) {
    warning("Some parameters seem invalid. 
             The current accepted parameters are: ",
            paste(unlist(params), collapse = ", "))
  }
  result <- result %>% cleanNULL()
  contact <- map(result$data,
                 function(x) {
                   new("contact",
                       contactid = use_na(x$contactid, "int"),
                       familyname = use_na(x$familyname, "char"),
                       leadinginitials = use_na(x$leadinginitials,
                                                "char"),
                       givennames = use_na(x$givennames, "char"),
                       contactname = use_na(x$contactname, "char"),
                       suffix = use_na(x$suffix, "char"),
                       ORCID = use_na(x$ORCID, "char"),
                       title = use_na(x$title, "char"),
                       institution = use_na(x$institution, "char"),
                       email = use_na(x$email, "char"),
                       phone = use_na(x$phone, "char"),
                       contactstatus = use_na(x$contactstatus,
                                              "char"),
                       fax = use_na(x$fax, "char"),
                       url = use_na(x$url, "char"),
                       address = use_na(x$address, "char"),
                       notes = use_na(x$notes, "char"))
                 })
  contacts <- new("contacts", contacts = contact)
  return(contacts)
}

library("testthat")
library("neotoma2")

context("`get_contacts()` tests")
test_that("`get_contacts()` numeric", {
  skip_on_cran()
  single <- get_contacts(x = 1)
  testthat::expect_equal(length(single), 1)
  testthat::expect_true(class(single) == "contacts")
  testthat::expect_true(class(single[[1]]) == "contact")
  
  multiple <- get_contacts(x = c(1, 2, 3))
  testthat::expect_equal(length(multiple), 3)
  testthat::expect_true(class(multiple) == "contacts")
  testthat::expect_true(all(sapply(multiple@contacts, function(x) {
    class(x) == "contact"
  })))
})

test_that("`get_contacts()` query", {
  skip_on_cran()
  goring <- get_contacts(familyname ="Goring")
  testthat::expect_gt(length(goring), 0)
  testthat::expect_true(class(goring) == "contacts")
  testthat::expect_true(class(goring[[1]]) == "contact")
})

#' @title Get publication information for Neotoma records
#' @name get_publications
#' @description Uses the Neotoma API to search and access information
#' about publications associated with data in the Neotoma Paleoecology Database
#' @importFrom methods new
#' @importFrom purrr pluck
#' @importFrom dplyr coalesce
#' @param x integer A contact ID
#' @param ...
#' `publicationid`
#'    The unique numeric identifier associated with a publication in Neotoma.
#' `datasetid`
#'    A unique identifier for a Neotoma dataset that is associated
#'    with a publication.
#' `familyname` The full or partial last name for an individual author.
#' `pubtype` The publication type, from `get_tables("publicationtypes")`.
#' `year` The year the publication was released.
#' `search` A plain text search string used to search the citation.
#' @returns `publications` object
#' @examples \dontrun{
#' # How old are the papers in Neotoma that include the term "mammut"?
#' tryCatch({
#'   mammoth_papers <- get_publications(search="mammut") %>%
#'     as.data.frame()
#'   hist(as.numeric(mammoth_papers$year))
#' }, error = function(e) {
#'  message("Neotoma server not responding. Try again later.")
#' })
#' # We want the paper identified in Neotoma as 666:
#' tryCatch({
#' get_publications(666)
#' }, error = function(e) {
#' message("Neotoma server not responding. Try again later.")
#' })
#' # Take a publication object and purposely degrade the metadata:
#' tryCatch({
#'   bad_pub <- get_publications(666)
#'   # Note this only changes the reported year, not the citation string.
#'   bad_pub[[1]]@year <- "1923"
#'   bad_pub[[1]]@publicationid <- NA_integer_
#'   updated_pubs <- get_publications(bad_pub[[1]])
#'   attr(updated_pubs, "matches")
#'   # we see the proper citation in the record:
#'   updated_pubs <- attr(updated_pubs, "matches")[[3]]
#' }, error = function(e) {
#'  message("Neotoma server not responding. Try again later.")
#' })
#' }
#' @md
#' @export
get_publications <- function(x = NA, ...) {
  if (missing(x)) {
    UseMethod("get_publications", "default")
  } else {
    UseMethod("get_publications", x)
  }
}

#' @rdname get_publications
#' @export
get_publications.default <- function(...) {
  . <- ""
  cl <- as.list(match.call())
  cl[[1]] <- NULL
  cl <- lapply(cl, eval, envir = parent.frame())
  get_params("publications")
  baseURL <- paste0("data/publications")
  result <- tryCatch(
    parseURL(baseURL, ...),
    error = function(e) {
      stop("API call failed: ", e$message)
      NULL
    }
  )
  if (is.null(result$data)) {
    return(NULL)
  } else {
    result <- result$data %>% cleanNULL() %>% pluck("result")
    pubs <- map(result, function(x) {
      if ("match" %in% names(x)) {
        match <- x$match
      } else {
        match <- NULL
      }
      x <- x$publication
      x[is.null(x)] <- NA_character_
      output <- new("publication",
                    publicationtype = use_na(testNull(x$pubtype), "char"),
                    publicationid = use_na(testNull(x$publicationid), "int"),
                    articletitle = use_na(testNull(x$articletitle), "char"),
                    year = use_na(testNull(x$year), "char"),
                    journal = use_na(testNull(x$journal), "char"),
                    volume = use_na(testNull(x$volume), "char"),
                    issue = use_na(testNull(x$issue), "char"),
                    pages = use_na(testNull(x$pages), "char"),
                    citation = use_na(testNull(x$citation), "char"),
                    doi = use_na(testNull(x$doi), "char"),
                    author = pubAuthors(x))
      attr(output, "match") <- match
      return(output)
    }) %>%
      new("publications", publications = .)
    return(pubs)
  }
}

#' @rdname get_publications
#' @export
get_publications.numeric <- function(x, ...) {
  . <- ""
  if (length(x) > 0) {
    pubids <- paste0(x, collapse = ",")
  }
  testNull <- function(val, out) { # nolint
    if (is.null(val)) {
      return(out)
    } else {
      return(val)
    }
  }
  baseURL <- paste0("data/publications/", pubids) # nolint
  result <- tryCatch(
    parseURL(baseURL, ...),
    error = function(e) {
      stop("API call failed: ", e$message)
      NULL
    }
  )
  if (is.null(result$data)) {
    return(NULL)
  } else {
    result <- result %>% cleanNULL() # nolint
    pubs <- map(result$data, function(x) {
      x <- x$publication
      x[is.null(x)] <- NA_character_
      new("publication",
          publicationtype = use_na(testNull(x$pubtype), "char"),
          publicationid = use_na(testNull(x$publicationid), "int"),
          articletitle = use_na(testNull(x$articletitle), "char"),
          year = use_na(testNull(x$year), "char"),
          journal = use_na(testNull(x$journal), "char"),
          volume = use_na(testNull(x$volume), "char"),
          issue = use_na(testNull(x$issue), "char"),
          pages = use_na(testNull(x$pages), "char"),
          citation = use_na(testNull(x$citation), "char"),
          doi = use_na(testNull(x$doi), "char"),
          author = pubAuthors(x))
    }) %>%
      new("publications", publications = .)
    return(pubs)
  }
}

#' @rdname get_publications
#' @export
get_publications.publication <- function(x, ...) {
  if (is.na(x@publicationid)) {
    if (!is.na(x@citation)) {
      test <- get_publications(search = x@citation, limit = 3)
      attr(x, "matches") <- test
    } else {
      searchString <- coalesce(x@citation, x@articletitle, x@booktitle) # nolint
      test <- get_publications(search = searchString, limit = 3)
      attr(x, "matches") <- test
    }
  }
  return(x)
}

#' @rdname get_publications
#' @export
#' @method get_publications publications
get_publications.publications <- function(x, ...) {
  for (i in seq_len(length(x))) {
    pub <- x[[i]]
    if (is.na(x[[i]]@publicationid)) {
      if (!is.na(pub@citation)) {
        test <- get_publications(search = pub@citation, limit = 3, ...)
        attr(pub, "matches") <- test
        x@publications[[i]] <- pub
      }
    }
  }
  return(x)
}

#' @rdname get_publications
#' @export
get_publications.sites <- function(x, ...) {
  ds_ids <- getids(x)$datasetid %>%
    unique() %>%
    unlist() %>%
    as.numeric()
  l <- nrow(ds_ids)
  ds_ids <- paste0(ds_ids, collapse = ",")
  output <- get_publications(datasetid = ds_ids, limit=l)
  return(output)
}

library("testthat")
library("neotoma2")

context("get_publications tests")
test_that("`get_publications` numeric", {
  skip_on_cran()
  empty <- get_publications()
  testthat::expect_is(empty, "publications")
  # From numeric vector
  counts <- get_publications(c(1, 2, 3, 4))
  testthat::expect_is(counts, "publications")
  testthat::expect_is(counts[[1]], "publication")
  testthat::expect_length(counts, 4)
})

test_that("`get_publications` default", {
  skip_on_cran()
  pollen <- get_publications(search = "pollen")
  testthat::expect_is(pollen, "publications")
  testthat::expect_equal(length(pollen), 25)
})

test_that("`get_publications` publications", {
  skip_on_cran()
  counts <- get_publications(c(1, 2, 3, 4))
  frompubs <- get_publications(counts)
  testthat::expect_identical(counts, frompubs)
})

testthat::skip("Skipping all tests in this file")
library("testthat")
library("neotoma2")
library("tools")

context("Test that all examples run without error")
csv_path <- testthat::test_path("all_aliases.csv")
topics <- read.csv(csv_path, stringsAsFactors = FALSE)$alias
for (topic in topics) {
  topic <- sub(",.*$", "", topic)
  local({
    t <- topic
    test_that(paste0("Example for '", topic, "' runs without error"), {
      skip_on_cran()
      testthat::expect_error(
        example(topic = t,
                package = "neotoma2",
                ask = FALSE,
                character.only = TRUE,
                run.dontrun = TRUE,
                echo = FALSE),
        NA)
    })
  })
}

---
title: "The neotoma2 R Package"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{The neotoma2 R Package}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include=FALSE}
library(sf)
library(geojsonsf)
library(dplyr)
library(neotoma2)
```

> **Note**
> This package is an interface to an external web API.
> To comply with CRAN policies on Internet access, code examples in this vignette
> are shown but not executed during CRAN checks.
> Users can run all examples locally after installation.

```{r setOpts, include = FALSE}
knitr::opts_chunk$set(eval = FALSE,
                      error = TRUE)
```

```{r setup_md, include=FALSE}
safe_eval <- function(expr, fallback = "N/A") {
  tryCatch(eval(expr, envir = .GlobalEnv), error = function(e) fallback)
}
```



![closeup of several Neotoma sites in the Caribbean.](images/siteshot.png){width=100%}

## Neotoma Resources

The [Neotoma Paleoecology Database](https://www.neotomadb.org) is a domain-specific data resource containing millions of fossil records from around the globe, covering the last 5.4 million years. The `neotoma2` R package simplifies some of the data structures and concepts to facilitate statistical analysis and visualization. Users may wish to gain a deeper understanding of the resource itself, or build more complex data objects and relationships. For those users a partial list is provided here, including a table of code examples focusing on different geographic regions, languages and dataset types.

### Resources

* [Neotoma Homepage](https://www.neotomadb.org)
* The Neotoma homepage, with links to contacts, news and other tools and resources.
* [Neotoma Database Manual](https://open.neotomadb.org/manual/)
* Documentation for the database itself, with examples of SQL queries and descriptions of Neotoma tables.
* [Neotoma (JSON) API](https://api.neotomadb.org/)
* A tool to obtain data in JSON format directly through calls to Neotoma.
* [Neotoma GitHub Organization](https://github.com/NeotomaDB)
* Open code repositories to see how folks are using Neotoma and what kinds of projects we're working on.
* Workshops and Code Examples ([see the section below](#workshops-and-code-examples))

## Neotoma Data Structure

![Three panels showing context for Neotoma’s geographic representation of sites. In panel a a site is defined by the boundaries of a lake. The site also has a bounding box, and the core location is defined by a collection unit within the site that is defined with precise coordinates. In panel b a site is defined as a single point, for example, from a textual reference indicating the site is at the intersection of two roads. Here the site and collection unit share the unique point location. In panel c we show how that site location may be obfuscated using a bounding box as the site delimiter. In this case the collection unit would not be defined (but is represented as the triangle for illustration). Figure obtained from the Neotoma Database Manual.](images/siteboundarydiagram.svg){width=100%}

Data in Neotoma is associated with sites, specific locations with lat/long coordinates. Within a site, there may be one or more [**collection units**](https://open.neotomadb.org/manual/dataset-collection-related-tables-1.html#CollectionUnits) -- locations at which samples are physically collected within the site. For example, an archaeological **site** may have one or more **collection units**, pits within a broader dig site; a pollen sampling **site** on a lake may have multiple **collection units** -- core sites within the lake basin. Collection units may have higher resolution GPS locations, but are considered to be part of the broader site. Within a **collection unit** data is collected at various [**analysis units**] from which **samples** are obtained.

Because Neotoma is made up of a number of constituent databases (e.g., the Indo-Pacific Pollen Database, NANODe, FAUNMAP), a set of **sample**s associated with a **collection unit** are assigned to a single **dataset** associated with a particular **dataset type** (e.g., pollen, diatom, vertebrate fauna) and **constituent database**.

![**Figure**. *The structure of sites, collection units and datasets within Neotoma. A site contains one or more collection units. Chronologies are associated with collection units. Data of a common type (pollen, diatoms, vertebrate fauna) are assigned to a dataset.*](images/site_collunit_dataset.svg){width=50%}

Researchers often begin by searching for sites within a particular study area, whether that is defined by geographic or political boundaries. From there they interrogate the available datasets for their particular dataset type of interest.  When they find records of interest, they will then often call for the data and associated chronologies.

The `neotoma2` R package is intended to act as the intermediary to support these research activities using the Neotoma Paleoecology Database. Because R is not a relational database, we needed to modify the data structures of the objects. To do this the package uses a set of S4 objects to represent different elements within the database.

![A diagram showing the different major classes within the `neotoma2` R package, and the way the elements are related to one another. Individual boxes represent the major classes (sites, site, collectionunits, etc.). Each box then has a list of the specific metadata contained within the class, and the variable type (e.g., *siteid: integer*). Below these are the functions that can be applied to the object (e.g., *`[[<-`*). ](images/neotomaUML_as.svg){width=100%}

It is important to note, here and elsewhere: **Almost everything you will interact with is a `sites` object**. A `sites` object is the general currency of this package. `sites` may have more or less metadata associated with them, but they are the primary object, and, as you can see in the diagram above, they have the most functions associated with them.

### Package Requirements

The earlier `neotoma` package tried to use base R as much as possible.  The `neotoma2` package now draws primarily on `dplyr` and `purrr` packages from the `tidyverse`, and on the `sf` spatial data package. The choice to integrate `tidyverse` packages was made largely because of the current ubiquity of the `tidyverse` in R education.

## Site Searches

The highest level object in Neotoma is the **site**.  Sites have spatial coordinates and, in many cases, additional metadata related to lake parameters, or other site-specific properties.

Sites can be searched using the `get_sites()` function, or, can be created using the `set_site()` function. A single `site` object is a special object in R, that can be combined with other sites into a `sites` object.  A `sites` object is effectively a `list()` of `site` objects with special methods for printing, plotting and exporting information.

### Finding Sites

All sites in Neotoma have a unique numeric identifier.  With the `neotoma2` package you can search for a site using the `get_sites()` function by its unique site id (`siteid`), by name (`sitename`), by altitude (`altmin`, `altmax`), by geopolitical name (`gpid`), location (`loc`) or age bounds.

If we're looking for a site and we know its specific identifier, we can use the simplest implementation of `get_sites()`.  Here we are searching for a site (Alexander Lake), where we know that the siteid for the record in Neotoma is `24`. We can get these siteids using the [Neotoma Explorer web application](https://apps.neotomadb.org/explorer/), or if we have some familiarity with the site records already.

```{r getSiteBySiteID}
# Search for site by a single numeric ID:
alex <- get_sites(24)
alex

# Search for sites with multiple IDs using c():
multiple_sites <- get_sites(c(24, 47))
multiple_sites
```

Once you search for a site, the `neotoma2` R package makes a call to the Neotoma Database, and returns a structured `sites` object that contains metadata about the sites, and some additional metadata about collection units and datasets at those sites.  This limited metadata helps speed up further searches, but is not complete, for the purposes of analysis.

![The result of a hypothetical `get_sites()` call, is a `sites` object containing two individual `site` objects. Each `site` object contains a `collunits` object with some limited metadata. The top `site` appears to have two collection units, while the lower site has only a single collection unit. Each of the top two collection units appear to contain two datasets, while the bottom site has only the one collection unit with only one dataset.](images/understanding_S4_structure_get_sites.svg){width=100%}

#### Searching for Sites by Name

Often we do not know the particular `siteid`.  If we're looking for a site and we know its name or a part of its name, we can search using the function with the `sitename` argument, `get_site(sitename = 'XXX')`, where `'XXX'` is the site name.  This does not support multiple text strings (i.e., you can't use `c()`).

```{r getsitename}
alex <- get_sites(sitename = "Alexander Lake")
alex
```

Neotoma uses a Postgres Database to manage data.  Postgres uses the `%` sign as a general wildcard, so we can use the `%` in the `sitename` argument operator to help us find sites when we're not sure the exact match.  Note that the search is case **insensitive** so a search for `alex%` or `Alex%` will return the same results.

```{r sitewithwildcardname}
alex <- get_sites(sitename = 'Alex%')
alex
```

Since this new `sites` object has `r tryCatch(length(alex@sites), error = function(e) "N/A")`. elements that belong to `site`, we may want to access only one of the objects, or sets of metadata from our variable `alex`.

#### Searching for Sites by Age

There are several ways of searching for sites using age parameters. These are represented below:

![Site searches using age parameters including `ageof`, `ageyoung`, `ageold`, `maxage` and `minage`.](images/understandingAgeRanges.svg){width=100%}

We offer several methods of searching because different users have different requirements. A user might be only interested in one specific point in time in the past, for example the 8.2ka event. In this instance they would search `get_sites(ageof = 8200)`. They may want sites with records that completely span a time period, for example the Atlantic chronozone of the Holocene: `get_sites(ageyounger = 5000, ageolder = 8000)`. These sites would have samples both within and outside the defined age range, so that the user could track change into and out of the time period. A user may also be interested in any record within a time bin, regardless of whether the site spans that time zone or not. They would query `get_sites(minage = 5000, maxage = 8000)`.

We can see how these age bounds differ:

```{r agebounds}
# Note, we are using the `all_data = TRUE` flag here to avoid the default limit of 25 records, discussed below.
# Because these queries are searching through every record they are slow and and are not
# run in knitting this vignette.
get_sites(ageof = 8200, all_data = TRUE) %>% length()
get_sites(ageyounger = 5000, ageolder = 8000, all_data = TRUE) %>% length()
get_sites(minage = 5000, maxage = 8000, all_data = TRUE) %>% length()
```

It is possible to pass all parameters (`ageof`, `minage`, `maxage`, `ageyounger`, . . . ), but it is likely that these will conflict and result in an empty set of records. To avoid this, be aware of the relationships among these search parameters, and how they might affect your search window.

### Accessing `sites` metadata

Although the `sites` are structured using S4 objects (see [Hadley Wickham's S4 documentation](http://adv-r.had.co.nz/S4.html)), we've added helper functions to make accessing elements easier for users.  

The `alex` object is composed of several smaller objects of class `site`.  We can call any individual site using `[[ ]]`, placing the index of the desired object between the brackets.  Then we can also call the particular variable we want using the `$` symbol.

```{r extractElement}
alex <- get_sites(sitename = "Alexander Lake")
alex[[1]]$siteid
```

The elements within a `site` are the same as the defined columns within the Neotoma [`ndb.sites`](https://open.neotomadb.org/dbschema/tables/sites.html) table, with the exception of the `collunits` slot, which contains the collection units and associated datasets that are found within a site.  You can see all the `site` slots using the `names()` function.  You can select individual elements of a `site`, and you can assign values to these parameters:

```{r showallNamesSite}
names(alex[[1]])

# Modify a value using $<- assignment:
alex[[1]]$area
alex[[1]]$area <- 100
alex[[1]]$area

# Modify a value using [<- assignment:
alex[[1]]["area"] <- 30
alex[[1]]$area
# alex[[1]][7] <- 30  This fails because the `Notes` field expects a character string.
```

Using assignment, we can add information programmatically, for example, by working interactively with a digital elevation model or hydrographic data to obtain lake area measurements. Although not currently implemented, the goal is to support direct upload of updated information by users.

### Creating a Site

As explained above, a `site` is the fundamental unit of the Neotoma Database. If you are working with your own data, you might want to create a `site` object to allow it to interact with other data within Neotoma.  You can create a site with the `set_site()` function. It will ask you to provide important information such as `sitename`, `lat`, and `long` attributes.

```{r setsitefunction}
my_site <- set_site(sitename = "My Lake", 
                    geography = st_sf(a = 3, st_sfc(st_point(1:2))), 
                    description = "my lake", 
                    altitude = 30)
my_site
```

If we have a set of sites that we are analyzing, we can add the new site to the set of sites, either by appending it to the end, using `c()`, or by replacing a particular element using `[[<-`.

This method allows us to begin modifying site information for existing sites if we have updated knowledge about site properties.

```{r addtosites}
# Add a new site that's been edited using set_site()
longer_alex <- c(alex, my_site)
# Or replace an element within the existing list of sites
# with the newly created site.
longer_alex[[2]] <- my_site

# Or append to the `sites` list with assignment:
longer_alex[[3]] <- my_site
```

We can also use `set_sites()` as a tool to update the metadata associated with an existing site object:

```r
# Update a value within an existing `sites` object:
longer_alex[[3]] <- set_site(longer_alex[[3]],
altitude = 3000)
longer_alex
```

## Datasets

If you need to get to a deeper level of the sites object, you may want to look at the `get_datasets()` function.  You can use `get_datasets()` using search parameters, or you can use it on an existing `sites` object, such as our prior `alex` dataset.

`get_datasets()` adds additional metadata to the `site` objects, letting us know which `datasettypes` are associated with a site, and the dataset sample locations at the site.

![Using `get_datasets()` provides more complete metadata about a record, including the addition of chronological information, and more complete metadata about the datasets, compared to the `get_sites()` call, shown above.  The objects here are the same as above, but now have chronology metadata, and contact metadata for the records. Note that there is still no sample or taxonomic information about these records.  This comes from the `get_downloads()` function.](images/understanding_S4_structure_get_datasets.svg){width=100%}

Getting the datasets by id is the easiest call, you can also pass a vector of IDs or, if you already have a `sites` object, you can pass a sites object.

```{r getdatasetsbyid}
# Getting datasets by ID
my_datasets <- get_datasets(c(5, 10, 15, 20))
my_datasets
```

You can also retrieve datasets by type directly from the API.

```{r getdatasetsbytype}
# Getting datasets by type
my_pollen_datasets <- get_datasets(datasettype = "pollen", limit = 25)
my_pollen_datasets
```

It can be computationally intensive to obtain the full set of records for `sites` or `datasets`. By default the `limit` for all queries is `25`.  The default `offset` is `0`.  To capture all results we can use the `all_data = TRUE` flag in our calls.  **However**, this is hard on the Neotoma servers. We tend to prefer that users use `all_data = TRUE` once their analytic workflow is mostly complete.

We can use that `all_data = TRUE` in R in the following way:

```{r all_data}
allSites_dt <- get_sites(datasettype = "diatom")
allSites_dt_all <- get_sites(datasettype = "diatom", all_data = TRUE)

# Because we used the `all_data = TRUE` flag, there will be more sites
# in allSites_dt_all, because it represents all sites containing diatom datasets.
length(allSites_dt_all) > length(allSites_dt)
```

### Spatial Searches

You can get the coordinates to create a GeoJson bounding box from [here](https://geojson.io/#map=2/20.0/0.0), or you can use pre-existing objects within R, for example, country-level data within the `spData` package:

Accessing datasets by bounding box:

```{r boundingBox}
brazil <- '{"type": "Polygon", 
            "coordinates": [[
                [-73.125, -9.102],
                [-56.953, -33.138],
                [-36.563, -7.711],
                [-68.203, 13.923],
                [-73.125, -9.102]
              ]]}'

# We can make the geojson a spatial object if we want to use the
# functionality of the `sf` package.
brazil_sf <- geojsonsf::geojson_sf(brazil)


brazil_datasets <- get_datasets(loc = brazil_sf)
```

Now we have an object called `brazil_datasets` that contains `r tryCatch(length(brazil_datasets), error = function(e) "N/A")`. 

You can plot these findings!

```{r leafletBrazil}
plotLeaflet(brazil_datasets)
```

## Filtering Records

Sometimes we take a large number of records, do some analysis, and then choose to select a subset.  For example, we may want to select all sites in a region, and then subset those by dataset type.  If we want to look at only the geochronological datasets from Brazil, we can start with the set of records returned from our `get_datasets()` query, and then use the `filter` function in `neotoma2` to select only those datasets that are geochronologic:

```{r filterBrazil}

brazil_dates <- neotoma2::filter(brazil_datasets,
                                   datasettype == "geochronologic")
# or:
brazil_dates <- brazil_datasets %>%
    neotoma2::filter(datasettype == "geochronologic")

# With boolean operators:
brazil_space <- brazil_datasets %>% neotoma2::filter(lat > -18 & lat < -16)
```

The `filter()` function takes as the first argument, a datasets object, followed by the criteria we want to use to filter. Current supported criteria includes:

* `lat`
* `long`
* `elev`
* `datasettype`

You also need to make sure that you accompany any of these terms with the following boolean operators: `<`, `>` or `==`, `!=`. `datasettype` has to be of type string, while the other terms must be numeric.  If you need to filter by the same argument, let's say, you need to filter "geochronologic" and "pollen data types, then you will also make use of `&` and `|` operators.

## Sample and Taxonomic data

Once we have the set of records we wish to examine, we then want to recover the actual sample data. This will provide us with information about the kinds of elements found at the site, within the dataset, their sample ages, and their counts or measurements. To do this we use the `get_downloads()` call.  Note, as before, we are returning a `sites` objects, but this time with the most complete metadata.

![Using `get_downloads()` returns a `sites` object, but one that contains dataset objects with filled `samples` slots. The `samples` slot is often very large relative to the other metadata associated with `sites`, and so it is commonly held back until a direct request is provided. Helper functions at the `sites` level can pull out `sample` data once `get_downloads()` has been called.](images/understanding_S4_structure.svg){width=100%}

Assuming we continue with our example from Brazil, we want to extract records from the country, filter to only pollen records with samples covering the last 10,000 years, and then look at the relative frequency of taxa across sites.  We might do something like this:

```{r filterAndShowTaxa}
brazil <- '{"type": "Polygon", 
            "coordinates": [[
                [-73.125, -9.102],
                [-56.953, -33.138],
                [-36.563, -7.711],
                [-68.203, 13.923],
                [-73.125, -9.102]
              ]]}'

# We can make the geojson a spatial object if we want to use the
# functionality of the `sf` package.
brazil_sf <- geojsonsf::geojson_sf(brazil)

brazil_records <- get_datasets(loc = brazil_sf, all_data=TRUE) %>%
    neotoma2::filter(datasettype == "pollen" & age_range_young <= 1000 & age_range_old >= 10000) %>%
    get_downloads()


count_by_site <- samples(brazil_records) %>%
  dplyr::filter(elementtype == "pollen" & units == "NISP") %>%
  group_by(siteid, variablename) %>%
  summarise(n = n()) %>%
  group_by(variablename) %>%
  summarise(n = n()) %>%
  arrange(desc(n))

```

In this code chunk we define the bounding polygon for our sites, filter by time and dataset type, and then return the full records for those sites. We get a `sites` object with dataset and sample information (because we used `get_downloads()`). We execute the `samples()` function to extract all the samples from the `sites` objects, and then filter the resulting `data.frame` to pull only pollen (a pollen dataset may contain spores and other elements that are not, strictly speaking, pollen) that are counted using the number of identified specimens (or NISP). We then `group_by()` the unique site identifiers (`siteid`) and the taxa (`variablename`) to get a count of the number of times each taxon appears in each site. We then want to `summarize()` to a higher level, just trying to understand how many sites each taxon appears in. After that we `arrange()` so that the records show the most common taxa first in the resulting variable `count_by_site`.

## Publications

Many Neotoma records have publications associated with them.  The `publication` object (and the `publications` collection) provide the opportunity to do this.  The [`publication`](https://open.neotomadb.org/dbschema/tables/publications.html) table in Neotoma contains an extensive number of fields.  The methods for `publications` in the `neotoma2` package provide us with tools to retrieve publication data from Neotoma, to set and manipulate publication data locally, and to retrieve publication data from external sources (e.g., using a DOI).

### `get_publications()` from Neotoma

The most simple case is a search for a publication based on one or more publication IDs.  Most people do not know the unique publication ID of individual articles, but this provides a simple method to highlight the way Neotoma retrieves and presents publication information.

#### Get Publication By ID

We can use a single publication ID or multiple IDs.  In either case the API returns the publication(s) and creates a new `publications` object (which consists of multiple individual `publication`s).  

```{r pubsbyid}
one <- get_publications(12)
two <- get_publications(c(12, 14))
```

From there we can then then subset and extract elements from the list using the standard `[[` format.  For example:

```{r showSinglePub}
two[[2]]
```

Will return the second publication in the list, corresponding to the publication with `publicationid` 14 in this case.

#### Get Publication using Search

We can also use search elements to search for publications.  The `get_publication` method uses the Neotoma API to search for publications.  We can search using the following properties:

* `publicationid`
* `datasetid`
* `siteid`
* `familyname`
* `pubtype`
* `year`
* `search`
* `limit`
* `offset`

```{r fulltestPubSearch}
michPubs <- get_publications(search = "Michigan", limit = 2)
```

This results in a set of `r tryCatch(length(michPubs), error = function(e) "N/A")` publications from Neotoma, equal to the `limit`.  If the number of matching publications is less than the limit then the `length()` will be smaller.

Text matching in Neotoma is approximate, meaning it is a measure of the overall similarity between the search string and the set of article titles.  This means that using a nonsense string may still return results results:

```{r nonsenseSearch}
noise <- get_publications(search = "Canada Banada Nanada", limit = 5)
```

This returns a result set of length `r tryCatch(length(noise), error = function(e) "N/A")`.

This returns the (Neotoma) ID, the citation and the publication DOI (if that is stored in Neotoma).  We can get the first publication using the standard `[[` nomenclature:

```{r getSecondPub}
two[[1]]
```

The output will look similar to the output for `two` above, however you will see that only a single result will be returned and the class (for a single publication) will be of type `publication` (as opposed to `publications`).  

We can select an array of `publication` objects using the `[[` method, either as a sequence (`1:10`, or as a numeric vector (`c(1, 2, 3)`)):

```{r subsetPubs}
# Select publications with Neotoma Publication IDs 1 - 10.
pubArray <- get_publications(1:10)
# Select the first five publications:
subPub <- pubArray[[1:5]]
subPub
```

#### Create (or Import) New Publications

Just as we can use the `set_sites()` function to set new site information, we can also create new publication information using `set_publications()`.  With `set_publications()` you can enter as much or as little of the article metadata as you'd like, but it's designed (in part) to use the CrossRef API to return information from a DOI.

```{r setNewPub}
new_pub <- set_publications(
articletitle = "Myrtle Lake: a late- and post-glacial pollen diagram from northern Minnesota",
journal = "Canadian Journal of Botany",
volume = 46)
```

A `publication` has a large number of slots that can be defined.  These may be left blank, they may be set directly after the publication is defined:

```{r setPubValue}
new_pub@pages <- "1397-1410"
```

## Workshops and Code Examples

* 2022 International AL/IPA Meeting; Bariloche, Argentina
* [English Language Simple Workflow](https://open.neotomadb.org/Workshops/2022_IAL-IPA/simple_workflow.html)
* Topics: Simple search, climate gradients, stratigraphic plotting
* Spatial Domain: South America
* Dataset Types: Diatoms
* [Spanish Language Simple Workflow](https://open.neotomadb.org/Workshops/2022_IAL-IPA/simple_workflow_ES.html)
* Topics: Simple search, climate gradients, stratigraphic plotting
* Spatial Domain: South America
* Dataset Types: Diatoms
* [English Language Complex Workflow](https://open.neotomadb.org/Workshops/2022_IAL-IPA/complex_workflow.html)
* Topics: Chronology building, Bchron
* Spatial Domain: South America
* Dataset Types: Diatoms
* [Spanish Language Complex Workflow](https://open.neotomadb.org/Workshops/2022_IAL-IPA/complex_workflow_ES.html)
* Topics: Chronology building, Bchron
* Spatial Domain: South America
* Dataset Types: Diatoms
* 2022 European Pollen Database Meeting; Prague, Czech Republic
* [English Language Simple Workflow](https://open.neotomadb.org/Workshops/2022_EPD/simple_workflow.html)
* Topics: Simple search, climate gradients, stratigraphic plotting, taxonomic harmonization
* Spatial Domain: Europe/Czech Republic
* Dataset Types: Pollen
* [English Language Complex Workflow](https://open.neotomadb.org/Workshops/2022_EPD/complex_workflow.html)
* Topics: Chronology building, Bchron
* Spatial Domain: Europe/Czech Republic
* Dataset Types: Pollen
* 2022 American Quaternary Association Meeting
* [English Language Simple Workflow](https://open.neotomadb.org/Workshops/2022_AMQUA/simple_workflow.html)
* Topics: Simple search, climate gradients, stratigraphic plotting
* Spatial Domain: North America
* Dataset Types: Pollen
* [English Language Complex Workflow](https://open.neotomadb.org/Workshops/2022_AMQUA/complex_workflow.html)
* Topics: Chronologies
* Spatial Domain: North America
* Dataset Types: Pollen
* Neotoma-charcoal Workshop, Göttingen, Germany. Authors: Petr Kuneš & Thomas Giesecke
* [English Language Workflow](https://rpubs.com/petrkunes/neotoma-charcoal)
* Topics: Simple Search, PCA, DCA, Charcoal/Pollen Correlation
* Spatial Domain: Global/Czech Republic
* Dataset Types: Pollen, Charcoal

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R, R/speleothems.R
\name{speleothems}
\alias{speleothems}
\alias{speleothems,site-method}
\alias{speleothems,collunits-method}
\alias{speleothems,collunit-method}
\title{Obtain speleothems from a record or multiple records.}
\usage{
speleothems(x)

\S4method{speleothems}{site}(x)

\S4method{speleothems}{collunits}(x)

\S4method{speleothems}{collunit}(x)
}
\arguments{
\item{x}{sites object}
}
\value{
data.frame with record information at speleothem level
}
\description{
Obtain speleothems from a record or multiple records.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxa.R
\name{taxa,sites-method}
\alias{taxa,sites-method}
\title{Extract taxonomic data from a set of sites.}
\usage{
\S4method{taxa}{sites}(object)
}
\arguments{
\item{object}{A \code{sites} object.}
}
\value{
A \code{data.frame} reporting the taxa/data objects, units,
elements and other features within a set of records.
}
\description{
Extract taxonomic data from a set of sites.
}
\examples{
\dontrun{
somesites <- get_sites(datasettype = "diatom") \%>\%
  get_downloads()
diatomtaxa <- taxa(somesites)
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/speleothemdetails.R
\name{speleothemdetails,sites-method}
\alias{speleothemdetails,sites-method}
\title{speleothemdetails}
\usage{
\S4method{speleothemdetails}{sites}(x)
}
\arguments{
\item{x}{site object}
}
\value{
\code{data.frame} with speleothem records
}
\description{
Obtain elements on the speleothems level
Experimental function: API and behavior may change.
}
\examples{
\dontrun{
kesang <- get_sites(sitename = "Kesang cave") \%>\%
  get_datasets() \%>\%
  filter(datasettype == "pollen") \%>\%
  get_speleothems()
sp <- speleothemdetails(kesang)
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{sites_classes}
\alias{sites_classes}
\alias{site-class}
\alias{sites-class}
\title{An S4 class for \code{sites} information}
\value{
object of class \code{sites}
}
\description{
The S4 class for sites in the Neotoma Paleoecology Database.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R, R/samples.R
\name{samples}
\alias{samples}
\alias{samples,site-method}
\alias{samples,collunits-method}
\alias{samples,collunit-method}
\title{Obtain samples from a record or multiple records.}
\usage{
samples(x)

\S4method{samples}{site}(x)

\S4method{samples}{collunits}(x)

\S4method{samples}{collunit}(x)
}
\arguments{
\item{x}{sites object}
}
\value{
data.frame with record information at sample level
}
\description{
Obtain samples from a record or multiple records.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_site.R
\name{set_site}
\alias{set_site}
\title{set Site Information for Fossil Sites}
\usage{
set_site(
  x = NA,
  siteid = NA_integer_,
  sitename = NA_character_,
  geography = sf::st_as_sf(sf::st_sfc()),
  altitude = NA_integer_,
  geopolitical = list(),
  area = NA_integer_,
  notes = NA_character_,
  description = NA_character_,
  collunits = NULL
)
}
\arguments{
\item{x}{Object to be set as a site}

\item{siteid}{The unique \verb{site id} for a site.
If this site is new to Neotoma then leave the ID as NA.}

\item{sitename}{Name of a site.}

\item{geography}{A \code{sf} object representing the site location,
either as a polygon or point.}

\item{altitude}{altitude/elevation of the site.}

\item{geopolitical}{The geopolitical unit in which the site is located.}

\item{area}{The area of the site or depositional basin in \emph{ha}.
Can be calculated from the polygon.}

\item{notes}{additional information of the site}

\item{description}{Function to create new site objects.
The new object will not be uploaded to the database.}

\item{collunits}{Collection units in the site}
}
\value{
\code{site} object
}
\description{
set Site Information for Fossil Sites
}
\examples{
{
# Create a site called "My Lake", to
x <- sf::st_as_sf(sf::st_sfc(sf::st_point(c(5,5))))
my_site <- set_site(sitename = "My Lake",
                    geography = x,
                    description = "my lake",
                    altitude = 30)
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chronology-methods.R
\name{set_default,chronologies-method}
\alias{set_default,chronologies-method}
\title{Change the default age model for a record.}
\usage{
\S4method{set_default}{chronologies}(x, n)
}
\arguments{
\item{x}{A chronologies object.}

\item{n}{The particular chronology to be used as the default.}
}
\value{
\code{chronologies} object with a new defaulted \code{chronology}
}
\description{
Change the default age model for a record.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{collunits_classes}
\alias{collunits_classes}
\alias{collunit-class}
\alias{collunits-class}
\title{S4 class for \verb{collection units} information.}
\value{
object of class \code{collunits}
}
\description{
A \verb{collection unit} represents a collection event from within
a \code{site}. For example, a lake sediment core, or a single dig site within an
archaeological site.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_sample.R
\name{set_sample}
\alias{set_sample}
\title{set Sample Information}
\usage{
set_sample(
  x = NA,
  ages = list(),
  igsn = NA_character_,
  datum = data.frame(),
  depth = NA_integer_,
  sampleid = NA_integer_,
  thickness = NA_integer_,
  samplename = NA_character_,
  sampleanalyst = list(),
  analysisunitid = NA_integer_,
  analysisunitname = NA_character_
)
}
\arguments{
\item{x}{Object to be set as a sample}

\item{ages}{ages}

\item{igsn}{IGSN character}

\item{datum}{dataframe of datum}

\item{depth}{integer representing depth}

\item{sampleid}{ID for sample}

\item{thickness}{thickness of core}

\item{samplename}{sample's name}

\item{sampleanalyst}{Analyst's contact name}

\item{analysisunitid}{Which analysis unit it is}

\item{analysisunitname}{Analysis Unit's name}
}
\value{
\code{sample} object
}
\description{
Function to create new samples objects for analysis.
The new object will not be uploaded to the database.
}
\examples{
{
# Set an empty sample
my_sample <- set_sample()
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_contacts.R
\name{get_contacts}
\alias{get_contacts}
\alias{get_contacts.numeric}
\alias{get_contacts.default}
\title{Get \code{contact} information for Neotoma contributors}
\usage{
get_contacts(x = NA, ...)

\method{get_contacts}{numeric}(x, ...)

\method{get_contacts}{default}(x, ...)
}
\arguments{
\item{x}{integer A contact ID}

\item{...}{(\code{contactname})  A full or partial name for an individual
contributor to the database.
(\code{familyname})  The full or partial last name for an
individual contributor to the database.
(\code{status}) The current status of the contributor
(\code{active} or \code{retired})}
}
\value{
\code{contacts} object
}
\description{
Uses the Neotoma API to search and access
information about individuals who have contributed to
the data in the Neotoma Paleoecology Database
}
\author{
Simon Goring \email{goring@wisc.edu}

Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_stats.R
\name{get_stats}
\alias{get_stats}
\title{get_stats}
\usage{
get_stats(start = 0, end = 1, type = "dsdbmonth")
}
\arguments{
\item{start}{The starting month (from present == 0) for
which to generate the summary. Default is 0 (the current month).}

\item{end}{The ending month (from present == 0) for which
to generate the summary. Default is 1 (one month ago).}

\item{type}{A character string indicating the type of summary
to return. Options are \code{dsdbmonth} (the number of datasets
in the Neotoma Database added per month), \code{rawbymonth} (the
number of datasets, sites, publications, authors, countries and observations
added per month), and \code{dstypemonth} (the number of datasets added
per dataset type per month). Default is \code{dsdbmonth}.}
}
\value{
\code{data.frame} with summary statistics
}
\description{
Returns a count of sites, datasets, publications and other
objects added to Neotoma during the requested time period.
}
\details{
This function returns summaries about the data holdings
within Neotoma using the existing Neotoma API's \code{summary} endpoint. This can
provide information about recent uploads (the number of new sites
uploaded within the last month, for example), or can be used to provide
information about the overall number of sites/datasets (using an arbitrarily
high value for \code{end}).
}
\examples{
\dontrun{
tryCatch({
last_month <- get_stats(start = 0, end = 1, type = "dsdbmonth")
}, error = function(e) {
 message("Neotoma server not responding. Try again later.")
})
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}

Simon Goring \email{goring@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R, R/subset-methods.R
\name{chronologies}
\alias{chronologies}
\alias{chronologies,collunits-method}
\alias{chronologies,site-method}
\alias{chronologies,sites-method}
\title{Obtain the chronology from a record or multiple records.}
\usage{
chronologies(x)

\S4method{chronologies}{collunits}(x)

\S4method{chronologies}{site}(x)

\S4method{chronologies}{sites}(x)
}
\arguments{
\item{x}{sites object that contains chronologies}
}
\value{
chronologies object with all chronologies used.
}
\description{
Obtain the chronology from a record or multiple records.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R, R/subset-methods.R
\name{datasets}
\alias{datasets}
\alias{datasets,collunit-method}
\alias{datasets,sites-method}
\alias{datasets,site-method}
\title{Extract datasets from a sites object.}
\usage{
datasets(object)

\S4method{datasets}{collunit}(object)

\S4method{datasets}{sites}(object)

\S4method{datasets}{site}(object)
}
\arguments{
\item{object}{A sites object}
}
\value{
datasets object specific to the metadata contained in datasets
}
\description{
If the sites object contains datasets, then the datasets
will be returned. If the sites object does not contain datasets then
the user can apply \code{get_datasets()} to the object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_speleothem.R
\name{set_speleothem}
\alias{set_speleothem}
\title{Set Speleothem}
\usage{
set_speleothem(
  x = NA,
  entityid = NA_integer_,
  entityname = NA_character_,
  siteid = NA_integer_,
  collectionunitid = NA_integer_,
  dripheight = NA_integer_,
  monitoring = FALSE,
  geology = NA_character_,
  relativeage = NA_character_,
  speleothemtype = NA_character_,
  dripheightunits = NA_character_,
  entitycovertype = NA_character_,
  entrancedistance = NA_integer_,
  landusecovertype = NA_character_,
  speleothemdriptype = NA_character_,
  landusecoverpercent = NA_integer_,
  vegetationcovertype = NA_character_,
  entitycoverthickness = NA_integer_,
  entrancedistanceunits = NA_character_,
  vegetationcoverpercent = NA_integer_
)
}
\arguments{
\item{x}{Object to be set as a speleothem}

\item{entityid}{The speleothem entity ID.}

\item{entityname}{Name of the speleothem entity.}

\item{siteid}{The unique site ID for a site.}

\item{collectionunitid}{The unique collection unit ID for a collection unit.}

\item{dripheight}{drip height}

\item{monitoring}{Is the speleothem being monitored? TRUE/FALSE}

\item{geology}{rock type of the speleothem}

\item{relativeage}{relative age of the speleothem}

\item{speleothemtype}{type of speleothem}

\item{dripheightunits}{Units for drip height}

\item{entitycovertype}{type of cover around the speleothem}

\item{entrancedistance}{distance from cave entrance}

\item{landusecovertype}{type of land use cover around the speleothem}

\item{speleothemdriptype}{type of speleothem drip}

\item{landusecoverpercent}{land use cover percent}

\item{vegetationcovertype}{type of vegetation cover around the speleothem}

\item{entitycoverthickness}{thickness of the entity cover}

\item{entrancedistanceunits}{Units for distance from cave entrance}

\item{vegetationcoverpercent}{vegetation cover percent}
}
\value{
\code{speleothem} object
}
\description{
Set Speleothem
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collunits-methods.R
\docType{class}
\name{missingOrNULL-class}
\alias{missingOrNULL-class}
\title{c Method - Combine objects, including NULL}
\description{
c Method - Combine objects, including NULL
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/site-methods.R
\name{plot}
\alias{plot}
\alias{plot,sites-method}
\alias{plot,site-method}
\title{Plot site coordinates using a basic plot.}
\usage{
\S4method{plot}{sites}(x, y, ...)

\S4method{plot}{site}(x, y, ...)
}
\arguments{
\item{x}{sites object}

\item{y}{\emph{Ignored.}}

\item{...}{Additional parameters associated with the call.}
}
\value{
\code{plot} object with site coordinates.
}
\description{
Plot site coordinates using a basic plot.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{contacts_classes}
\alias{contacts_classes}
\alias{contact-class}
\alias{contacts-class}
\title{An S4 class for multi-contact information from the Neotoma
Paleoecology Database.}
\value{
object of class \code{contact}
}
\description{
An unordered list of individual S4 \code{contact} objects.
}
\examples{
{
new("contact", familyname = "Goring", givennames = "Simon J.")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collunits-methods.R, R/dataset-methods.R,
%   R/site-methods.R, R/speleothem-methods.R, R/taxon-methods.R
\name{[,collunits,numeric-method}
\alias{[,collunits,numeric-method}
\alias{sub,collunits-method}
\alias{[,datasets,numeric-method}
\alias{sub,datasets-method}
\alias{[}
\alias{[,sites,numeric-method}
\alias{[,site,numeric-method}
\alias{[,site,character-method}
\alias{[,speleothems,numeric-method}
\alias{sub,speleothems-method}
\alias{[,taxa,numeric-method}
\alias{sub,taxa-method}
\title{sub}
\usage{
\S4method{[}{collunits,numeric}(x, i)

\S4method{[}{datasets,numeric}(x, i)

\S4method{[}{sites,numeric}(x, i)

\S4method{[}{site,numeric}(x, i)

\S4method{[}{site,character}(x, i)

\S4method{[}{speleothems,numeric}(x, i)

\S4method{[}{taxa,numeric}(x, i)
}
\arguments{
\item{x}{The \code{sites} object}

\item{i}{The numeric index}
}
\value{
sliced \code{site} object
}
\description{
sub
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_dataset.R
\name{set_dataset}
\alias{set_dataset}
\title{set Site Information for Fossil Sites}
\usage{
set_dataset(
  x = NA,
  datasetid = NA_integer_,
  datasetname = NA_character_,
  database = NA_character_,
  doi = NA,
  recdatecreated = as.Date(character(1)),
  datasettype = NA_character_,
  age_range_old = NA_integer_,
  age_range_young = NA_integer_,
  age_units = NA_character_,
  notes = NA_character_,
  pi_list = NA,
  samples = NULL,
  specimens = NULL
)
}
\arguments{
\item{x}{object to be set as dataset,}

\item{datasetid}{dataset identifier}

\item{datasetname}{name of the dataset}

\item{database}{dataset where the dataset came from}

\item{doi}{DOI}

\item{recdatecreated}{date the dataset was created}

\item{datasettype}{type the dataset belongs to}

\item{age_range_old}{age range old}

\item{age_range_young}{age range young}

\item{age_units}{age units}

\item{notes}{notes}

\item{pi_list}{pi list}

\item{samples}{taxa objects}

\item{specimens}{specimens slot}
}
\value{
\code{dataset} object
}
\description{
Function to create new \code{dataset} objects for personal analysis.
The new object will not be uploaded to the database.
}
\examples{
{
# Create a dataset
my_dataset <- set_dataset(database = "EPD",
                    datasettype = "pollen",
                    notes = "my lake")
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{speleothems_classes}
\alias{speleothems_classes}
\alias{speleothem-class}
\alias{speleothems-class}
\title{S4 class for \code{speleothem} information}
\value{
object of class \code{speleothems}
}
\description{
The S4 class for \code{speleothem} data.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_params.R
\name{get_params}
\alias{get_params}
\title{get_params}
\usage{
get_params(obj)
}
\description{
get_params
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R, R/collunits-methods.R
\name{add_chronology}
\alias{add_chronology}
\alias{add_chronology,collunit,chronology,data.frame-method}
\alias{add_chronology,collunit-method}
\title{Add a new chronology into an existing collectionunit.}
\usage{
add_chronology(object, x, y)

\S4method{add_chronology}{collunit,chronology,data.frame}(object, x, y)
}
\arguments{
\item{object}{A collection unit object}

\item{x}{A chronology object generated using \code{set_chronology()}}

\item{y}{A data.frame of sample ages, with required columns:
\code{"analysisunitid"}, \code{"age"}, \code{"agetype"}, \code{"ageolder"}, and \code{"ageyounger"}.}
}
\value{
\code{chronologies} with new added chronology
}
\description{
Given a collunit, add a new chronology object to the unit
with both the chronology metadata and the age information (as \code{y})
}
\details{
When undertaking analysis we may wish to add a new chronology to
existing records within Neotoma. To do this we must first build the
chronology, but also link it to existing analysis units within the
collection unit.
For examples from this function, see the
https://open.neotomadb.org/EPD_binder/complex_workflow.html
documentation online.
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_publications.R
\name{get_publications}
\alias{get_publications}
\alias{get_publications.default}
\alias{get_publications.numeric}
\alias{get_publications.publication}
\alias{get_publications.publications}
\alias{get_publications.sites}
\title{Get publication information for Neotoma records}
\usage{
get_publications(x = NA, ...)

\method{get_publications}{default}(...)

\method{get_publications}{numeric}(x, ...)

\method{get_publications}{publication}(x, ...)

\method{get_publications}{publications}(x, ...)

\method{get_publications}{sites}(x, ...)
}
\arguments{
\item{x}{integer A contact ID}

\item{...}{\code{publicationid}
The unique numeric identifier associated with a publication in Neotoma.
\code{datasetid}
A unique identifier for a Neotoma dataset that is associated
with a publication.
\code{familyname} The full or partial last name for an individual author.
\code{pubtype} The publication type, from \code{get_tables("publicationtypes")}.
\code{year} The year the publication was released.
\code{search} A plain text search string used to search the citation.}
}
\value{
\code{publications} object
}
\description{
Uses the Neotoma API to search and access information
about publications associated with data in the Neotoma Paleoecology Database
}
\examples{
\dontrun{
# How old are the papers in Neotoma that include the term "mammut"?
tryCatch({
  mammoth_papers <- get_publications(search="mammut") \%>\%
    as.data.frame()
  hist(as.numeric(mammoth_papers$year))
}, error = function(e) {
 message("Neotoma server not responding. Try again later.")
})
# We want the paper identified in Neotoma as 666:
tryCatch({
get_publications(666)
}, error = function(e) {
message("Neotoma server not responding. Try again later.")
})
# Take a publication object and purposely degrade the metadata:
tryCatch({
  bad_pub <- get_publications(666)
  # Note this only changes the reported year, not the citation string.
  bad_pub[[1]]@year <- "1923"
  bad_pub[[1]]@publicationid <- NA_integer_
  updated_pubs <- get_publications(bad_pub[[1]])
  attr(updated_pubs, "matches")
  # we see the proper citation in the record:
  updated_pubs <- attr(updated_pubs, "matches")[[3]]
}, error = function(e) {
 message("Neotoma server not responding. Try again later.")
})
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotLeaflet.R
\name{plotLeaflet,sites-method}
\alias{plotLeaflet,sites-method}
\title{plotLeaflet}
\usage{
\S4method{plotLeaflet}{sites}(object)
}
\arguments{
\item{object}{Sites object to plot}
}
\value{
\code{leaflet} map
}
\description{
Plot sites on a leaflet map
}
\examples{
\dontrun{
# Note that by default the limit for queries is 25 records:
tryCatch({
modernSites <- get_sites(keyword = "Modern")
plotLeaflet(modernSites)
}, error = function(e) {
 message("Neotoma server not responding. Try again later.")
})
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{author_classes}
\alias{author_classes}
\alias{author-class}
\alias{authors-class}
\title{An S4 class for the authors of a Neotoma publication.}
\value{
object of class \code{author}
}
\description{
This class combines the S4 class \code{contact} with a numeric
author order. This allows us to reuse \code{contact} objects, and to assign
the authorship order within a publication. The full set of authors for
a publication are represented by the \code{authors} object.
}
\examples{
{
simon <- new("contact", familyname = "Goring", givennames = "Simon J.")
firstauthor <- new("author", author = simon, order = 1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R,
%   R/publication-methods.R, R/site-methods.R
\name{doi}
\alias{doi}
\alias{doi,publication-method}
\alias{doi,site-method}
\title{Obtain the DOI for publications or datasets.}
\usage{
doi(x)

\S4method{doi}{publication}(x)

\S4method{doi}{site}(x)
}
\arguments{
\item{x}{Object with DOIs associated to it.}
}
\value{
doi object with DOI information
}
\description{
Obtain the DOI for publications or datasets.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_documentation.R
\name{get_documentation}
\alias{get_documentation}
\title{get_documentation}
\usage{
get_documentation()
}
\description{
Open up the Neotoma R homepage.
}
\examples{
\dontrun{
if (interactive()) {
 get_documentation()
}
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_collunit.R
\name{set_collunit}
\alias{set_collunit}
\title{set Site Information for Fossil Sites}
\usage{
set_collunit(
  x = NA,
  collectionunitid = NA_integer_,
  notes = NA_character_,
  handle = NA_character_,
  colldate = as.Date(character(1)),
  location = NA_character_,
  waterdepth = NA_integer_,
  gpslocation = st_as_sf(st_sfc()),
  collunittype = NA_character_,
  collectiondevice = NA_character_,
  collectionunitname = NA_character_,
  depositionalenvironment = NA_character_,
  datasets = NULL,
  chronologies = NULL,
  defaultchronology = NA_integer_,
  speleothems = NULL
)
}
\arguments{
\item{x}{object to be set as collunit}

\item{collectionunitid}{collection unit identifier}

\item{notes}{notes}

\item{handle}{handle}

\item{colldate}{collection date}

\item{location}{location of the collection unit}

\item{waterdepth}{depth at where the sample is taken}

\item{gpslocation}{location with GPS}

\item{collunittype}{type of collection unit}

\item{collectiondevice}{device used to collect the sample}

\item{collectionunitname}{name of the collection unit}

\item{depositionalenvironment}{depositional environment}

\item{datasets}{datasets that the collection unit has}

\item{chronologies}{chronologies taken from the collection unit}

\item{defaultchronology}{best chronology model identifier to
be used with this collection unit}

\item{speleothems}{speleothems associated with the collection unit}
}
\value{
\code{collunit} object
}
\description{
Function to create new collection unit objects for
personal analysis.
The new object will not be uploaded to the database.
}
\examples{
{
# Create a collunit
my_collunit <- set_collunit(notes = "my lake")
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}

Simon Goring \email{goring@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_sites.R
\name{get_sites}
\alias{get_sites}
\alias{get_sites.numeric}
\alias{get_sites.default}
\alias{get_sites.NULL}
\title{get_sites}
\usage{
get_sites(x = NA, ...)

\method{get_sites}{numeric}(x, ...)

\method{get_sites}{default}(...)

\method{get_sites}{`NULL`}(x, ...)
}
\arguments{
\item{x}{Use a single integer or vector of integers representing
unique Neotoma site identifiers (siteids) to extract site information.}

\item{...}{accepted arguments, see details for more information.}
}
\value{
The function returns either a single item of class \code{"try-error"}
describing the reason for failure (either misdefined parameters or an error
from the Neotoma API), or a table of sites, with rows corresponding to the
number of individual sites returned by the Neotoma API.
Each "site" object contains 6 parameters that can be accessed as well:
siteid, sitename, location, altitude, description,
limited collection units information.
\itemize{
\item \code{loc} An \code{sf} object that describes site's location.
\item \code{description}
\item \code{collunits} limited information on collunits
}
}
\description{
The \code{get_sites} function is a wrapper for the Neotoma \code{sites} API
endpoint.
The function takes parameters defined by the user and returns a list
of site information supplied by the Neotoma Paleoecological Database.
The user may define all or none of the possible fields.
}
\details{
A \code{site} object in Neotoma is a physical location at which one or more
collection units are located. Each collection unit may have one or more
datasets within it, defined by the dataset type. The \code{get_sites()}
function searches for each site within Neotoma that matches the query
parameters, and returns them as a \code{sites} object, a list of \code{site}
objects. The \code{get_sites()} command wraps the Neotoma API
(\href{https://api.neotomadb.org}{api.neotomadb.org}) call for \code{sites}.
The call itself uses a SQL query which accepts any one of the following
parameters:
\itemize{
\item \code{siteid}  The unique site ID (integer) in Neotoma. Can be passed as a
vector of site IDs.
\item \code{sitename}  The site name, or approximate match using the \% wildcard.
\item \code{database}  The constituent database for the record.
See \code{get_table("constituentdatabases")}
\item \code{altmin}  The minimum altitude range for site elevation (in meters).
\item \code{altmax}  The maximum altitude range for site elevation (in meters).
\item \code{datasetid}  The unique dataset ID (integer) in Neotoma. Can be passed
as a vector of dataset IDs.
\item \code{datasettype}  Neotoma contains data for a number of datasettypes.
This returns a subset of data types. For a complete list of available
datasettypes, run \code{neotoma2::get_table('datasettypes')}
\item \code{doi}  The dataset DOI for a dataset contained within a site. Can be
passed as a vector of DOIs.
\item \code{gpid}  The geopolitical name or identifier containing a site. Can be
passed as a vector of names.
\item \code{keywords}  Keywords for samples within a set of sites. For example
"modern" indicates a sample within the record uses the keyword "modern".
\item \code{contacts}  Contact names or IDs associated with a site.
\item \code{ageyoung}  A minimum spanning age for the record, in years before
radiocarbon present (1950).
\item \code{ageold}  A maximum spanning age for the record, in years before
radiocarbon present (1950).
\item \code{ageof}  An age which must be contained within the range of sample ages
for a site.
\item \code{taxa}  The names of taxa which must be present within samples in a
record.
\item \code{all_data}  The API only downloads the first 25 records of the query.
For the complete records, use \code{all_data=TRUE}
This call will then return a data object that contains site metadata for one
or more sites, along with limited metadata describing the collection units
and datasets located at that site.
}
}
\examples{
\dontrun{
## Find sites with a min altitude of 12m and a max altitude of 25m
sites_12to25 <- get_sites(altmin=12, altmax=25)
sites_2500 <- get_sites(altmin=2500, all_data = TRUE)
## To find sites in Brazil
brazil <- '{"type": "Polygon",
"coordinates": [[
 [-73.125, -9.102096738726443],
 [-56.953125,-33.137551192346145],
 [-36.5625,-7.710991655433217],
 [-68.203125,13.923403897723347],
 [-73.125,-9.102096738726443]]]}'
brazil_sites <- get_sites(loc = brazil[1])
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/samples.R
\name{samples,sites-method}
\alias{samples,sites-method}
\title{samples}
\usage{
\S4method{samples}{sites}(x)
}
\arguments{
\item{x}{sites object}
}
\value{
\code{data.frame} with sample records
}
\description{
Obtain all samples within a sites object
}
\examples{
\dontrun{
# Get full data download from API and create a long table with samples data.
dw <- get_downloads(1)
pollen <- samples(dw)
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}

Simon Goring \email{goring@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collunits-methods.R, R/dataset-methods.R,
%   R/publication-methods.R, R/samples-methods.R, R/site-methods.R,
%   R/speleothem-methods.R, R/taxon-methods.R
\name{[[<-,collunits-method}
\alias{[[<-,collunits-method}
\alias{sub-subset,collunits-method}
\alias{[[<-,datasets-method}
\alias{sub-subset,datasets-method}
\alias{[[<-,publications-method}
\alias{sub-subset,publications-method}
\alias{[[<-,samples-method}
\alias{sub-subset,samples-method}
\alias{sub-subset}
\alias{[[<-,sites-method}
\alias{[[<-,speleothems-method}
\alias{sub-subset,speleothems-method}
\alias{[[<-,taxa-method}
\alias{sub-subset,taxa-method}
\title{sub-subset}
\usage{
\S4method{[[}{collunits}(x, i) <- value

\S4method{[[}{datasets}(x, i) <- value

\S4method{[[}{publications}(x, i) <- value

\S4method{[[}{samples}(x, i) <- value

\S4method{[[}{sites}(x, i) <- value

\S4method{[[}{speleothems}(x, i) <- value

\S4method{[[}{taxa}(x, i) <- value
}
\arguments{
\item{x}{\code{neotoma2} object}

\item{i}{iteration in \code{neotoma2} object}

\item{value}{The value to be used}
}
\value{
\code{sites} object with reassigned values
}
\description{
Obtain one of the elements within a nested \code{neotoma2} object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_contacts.R
\name{check_contacts}
\alias{check_contacts}
\alias{check_contacts.contacts}
\title{Check contact information for a record against Neotoma contributors}
\usage{
check_contacts(x, ...)

\method{check_contacts}{contacts}(x, similarity = 0.5, ...)
}
\arguments{
\item{x}{contacts A \code{contacts} object associated with a set of names.}

\item{...}{Additional parameters associated with the call.}

\item{similarity}{The similarity score between matched records (from 0 - 1).}
}
\value{
contacts object
}
\description{
Uses the Neotoma API to search and access information about individuals
who have contributed to the data in the Neotoma Paleoecology Database
}
\author{
Simon Goring \email{goring@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_taxa.R
\name{get_taxa}
\alias{get_taxa}
\alias{get_taxa.numeric}
\alias{get_taxa.default}
\title{get_taxa}
\usage{
get_taxa(x = NA, ...)

\method{get_taxa}{numeric}(x, ...)

\method{get_taxa}{default}(...)
}
\arguments{
\item{x}{A \code{taxon} ID to extract site information}

\item{...}{accepted arguments, see details for more information.}
}
\value{
A Neotoma2 sites object with datasets with the requested taxa.
}
\description{
a \code{sites} object with sites that contain the requested \code{taxa.}
}
\details{
The \code{get_taxa} function searches for a sites within Neotoma
that contain the requested taxa.
The \code{get_taxa} command wraps the Neotoma API`.
The call itself accepts any one of the following
parameters:
\itemize{
\item \code{taxonid}  The unique taxon ID (integer) in Neotoma. Can be passed as a
vector of site IDs.
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/site-methods.R
\name{summary,sites-method}
\alias{summary,sites-method}
\title{Summary of objects within a sites object.}
\usage{
\S4method{summary}{sites}(object, ...)
}
\arguments{
\item{object}{sites object}

\item{...}{additional properties passed to \code{summary}}
}
\value{
\code{data.frame} object with site summary information
}
\description{
This function summarizes a sites object, from \code{site} level
and returns a \code{data.frame} that contains the site ID, sitename,
collectionunit ID, count of chronologies, count of datasets
and types of datasets within the site.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collunits-methods.R, R/dataset-methods.R,
%   R/samples-methods.R, R/site-methods.R, R/speleothem-methods.R,
%   R/taxon-methods.R
\name{[<-,collunit,character-method}
\alias{[<-,collunit,character-method}
\alias{subset,collunit-method}
\alias{[<-,collunit,numeric-method}
\alias{[<-,dataset,character-method}
\alias{subset,dataset-method}
\alias{[<-,dataset,numeric-method}
\alias{[<-,sample,character-method}
\alias{subset,sample-method}
\alias{subset}
\alias{[<-,site,character-method}
\alias{[<-,site,numeric-method}
\alias{[<-,speleothem,character-method}
\alias{subset,speleothem-method}
\alias{[<-,speleothem,numeric-method}
\alias{[<-,taxon,character-method}
\alias{subset,taxon-method}
\alias{[<-,taxon,numeric-method}
\title{subset}
\usage{
\S4method{[}{collunit,character}(x, i) <- value

\S4method{[}{collunit,numeric}(x, i) <- value

\S4method{[}{dataset,character}(x, i) <- value

\S4method{[}{dataset,numeric}(x, i) <- value

\S4method{[}{sample,character}(x, i) <- value

\S4method{[}{site,character}(x, i) <- value

\S4method{[}{site,numeric}(x, i) <- value

\S4method{[}{speleothem,character}(x, i) <- value

\S4method{[}{speleothem,numeric}(x, i) <- value

\S4method{[}{taxon,character}(x, i) <- value

\S4method{[}{taxon,numeric}(x, i) <- value
}
\arguments{
\item{x}{A \code{neotoma2} object.}

\item{i}{The column indicator.}

\item{value}{The value to be used.}
}
\value{
\code{site} object with reassigned character values
}
\description{
subset
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collunits-methods.R, R/contact-methods.R,
%   R/dataset-methods.R, R/publication-methods.R, R/site-methods.R,
%   R/speleothem-methods.R, R/taxon-methods.R
\name{show,collunits-method}
\alias{show,collunits-method}
\alias{show,collunit-method}
\alias{show,contacts-method}
\alias{show,contact-method}
\alias{show,dataset-method}
\alias{show,datasets-method}
\alias{show,publications-method}
\alias{show,publication-method}
\alias{show}
\alias{show,site-method}
\alias{show,sites-method}
\alias{show,speleothem-method}
\alias{show,speleothems-method}
\alias{show,taxon-method}
\alias{show,taxa-method}
\title{Display a \code{sites} object or nested slots.}
\usage{
\S4method{show}{collunits}(object)

\S4method{show}{collunit}(object)

\S4method{show}{contacts}(object)

\S4method{show}{contact}(object)

\S4method{show}{dataset}(object)

\S4method{show}{datasets}(object)

\S4method{show}{publications}(object)

\S4method{show}{publication}(object)

\S4method{show}{site}(object)

\S4method{show}{sites}(object)

\S4method{show}{speleothem}(object)

\S4method{show}{speleothems}(object)

\S4method{show}{taxon}(object)

\S4method{show}{taxa}(object)
}
\arguments{
\item{object}{\code{sites}, \code{datasets}, \code{collunits}, \code{contacts} object}
}
\description{
Display a \code{sites} object or nested slots.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{taxa_classes}
\alias{taxa_classes}
\alias{taxon-class}
\alias{taxa-class}
\title{S4 class for taxa information}
\value{
object of class \code{taxon}
}
\description{
Taxa details from the Neotoma Paleoecology Database.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R
\name{coordinates}
\alias{coordinates}
\title{Obtain coordinates from a sites object.}
\usage{
coordinates(obj, ...)
}
\arguments{
\item{obj}{A sites object}

\item{...}{Additional parameters associated with the call.}
}
\value{
dataframe with coordinate values
}
\description{
Obtain coordinates from a sites object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R
\name{set_default}
\alias{set_default}
\title{Set the default chronology within a collectionunit.}
\usage{
set_default(x, n)
}
\arguments{
\item{x}{A chronologies object.}

\item{n}{The particular chronology to be used as the default.}
}
\value{
sites object with new default chronology
}
\description{
Set the default chronology within a collectionunit.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pingNeotoma.r
\name{pingNeotoma}
\alias{pingNeotoma}
\title{pingNeotoma}
\usage{
pingNeotoma(server = "neotoma")
}
\arguments{
\item{server}{One of \code{localhost:PORT} (where \code{PORT} is a valid
numeric port), \code{neotoma} or \code{dev}.}
}
\value{
A valid HTTP status code or returns an error if a connection
is refused.
}
\description{
A quick function to test whether or not the Neotoma Database
API is currently running.
}
\examples{
\dontrun{
test_connection <- pingNeotoma("neotoma")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{samples_classes}
\alias{samples_classes}
\alias{sample-class}
\alias{samples-class}
\title{S4 class for \code{samples} information}
\value{
object of class \code{sample}
}
\description{
The standard object class for \code{samples}
in the Neotoma Paleoecology Database.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_server.R
\name{set_server}
\alias{set_server}
\title{Set Neotoma API Source or Server}
\usage{
set_server(server = "neotoma")
}
\arguments{
\item{server}{One of \code{local} (when the API is running locally on
port 3005), \code{neotoma} or \code{dev}.}
}
\value{
NULL modifies how to talk to the API (local, dev, server)
}
\description{
Choose to pull Neotoma data from the main Neotoma server, the development
server or from a local instance of the API.
}
\examples{
\dontrun{
# The user is running the API locally using the node/express API
# cloned from github: https://github.com/NeotomaDB/api_nodetest
set_server(server = "local")
# The user switches back to the remote api server.
set_server(server = "neotoma")
}
}
\author{
Simon Goring \email{goring@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset-methods.R
\name{collunits,sites-method}
\alias{collunits,sites-method}
\title{Extract \code{collunits} from a \code{sites} object.}
\usage{
\S4method{collunits}{sites}(object)
}
\arguments{
\item{object}{A \code{sites} object}
}
\value{
\code{collunits} from a \code{sites} object
}
\description{
Extract \code{collunits} from a \code{sites} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chroncontrols.R
\name{chroncontrols,sites-method}
\alias{chroncontrols,sites-method}
\title{Recover information about the chron controls for a collectionunit.}
\usage{
\S4method{chroncontrols}{sites}(x)
}
\arguments{
\item{x}{sites object}
}
\value{
data.frame with chronological controls
}
\description{
For all sites that includes collection units with chronologies
return the chronological controls that are used in building the chronology.
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset-methods.R
\name{chronologies,collunit-method}
\alias{chronologies,collunit-method}
\title{Extract \code{chronologies} from a \code{site}s or \code{collunits} object.}
\usage{
\S4method{chronologies}{collunit}(x)
}
\arguments{
\item{x}{A \code{sites} or \code{collunits} object}
}
\value{
\code{chronologies} from a \code{collunit} object
}
\description{
Extract \code{chronologies} from a \code{site}s or \code{collunits} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R, R/site-methods.R
\name{cite_data}
\alias{cite_data}
\alias{cite_data,site-method}
\alias{cite_data,NULL-method}
\title{Generate a data citation from a Neotoma2 object.}
\usage{
cite_data(x)

\S4method{cite_data}{site}(x)

\S4method{cite_data}{NULL}(x)
}
\arguments{
\item{x}{Object with DOIs associated to it.}
}
\value{
data.frame with citation data
}
\description{
The function, applied to a data object with a valid dataset,
will return a properly formatted data citation for the record.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_taxon.R
\name{get_taxon}
\alias{get_taxon}
\alias{get_taxon.default}
\alias{get_taxon.numeric}
\title{get_taxon}
\usage{
get_taxon(x = NA, ...)

\method{get_taxon}{default}(...)

\method{get_taxon}{numeric}(x, ...)
}
\arguments{
\item{x}{taxon ID}

\item{...}{accepted arguments, see \strong{details} for more information.}
}
\value{
A Neotoma2 \code{taxa} object with datasets with the requested taxa.
}
\description{
a \code{taxon} object with detailed information
}
\details{
A \code{taxa} may have one or more taxa associated with it.
The \code{get_taxon} function searches for \code{taxa} detail data within Neotoma
while the \code{get_taxa} function searches for sites that contain the requested
taxa. The function searches for each taxon by using a SQL query which
accepts any one of the following parameters:
\itemize{
\item \code{taxonid}  The unique taxon ID (integer) in Neotoma. Can be passed as a
vector of taxa IDs.
\item \code{taxonname} Taxon name or partial name. You can pass wildcard characters
using \%.
\item \code{taxagroup} The taxonomic grouping the taxon belongs to, from the Neotoma
taxagrouptypes table.
\item \code{ecolgroup} The ecological group of interest for the Neotoma taxon (from
the Neotoma ecolgrouptypes table)
\item \code{status} The taxonomic extinction status of the taxon, either extinct
(1, True) or extant (0, False).
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chronology-methods.R, R/collunits-methods.R,
%   R/contact-methods.R, R/dataset-methods.R, R/publication-methods.R,
%   R/samples-methods.R, R/site-methods.R, R/speleothem-methods.R,
%   R/taxon-methods.R
\name{$,chronology-method}
\alias{$,chronology-method}
\alias{$,chronologies-method}
\alias{$,collunit-method}
\alias{cash,collunit-method}
\alias{$,collunits-method}
\alias{cash,collunits-method}
\alias{$,contact-method}
\alias{cash,contact-method}
\alias{$,contacts-method}
\alias{cash,contacts-method}
\alias{$,dataset-method}
\alias{cash,dataset-method}
\alias{$,datasets-method}
\alias{cash,datasets-method}
\alias{$,publication-method}
\alias{cash,publication-method}
\alias{$,samples-method}
\alias{cash,samples-method}
\alias{$,sample-method}
\alias{cash,sample-method}
\alias{cash}
\alias{$,site-method}
\alias{$,sites-method}
\alias{$,speleothem-method}
\alias{cash,speleothem-method}
\alias{$,speleothems-method}
\alias{cash,speleothems-method}
\alias{$,taxon-method}
\alias{cash,taxon-method}
\alias{$,taxa-method}
\alias{cash,taxa-method}
\title{cash}
\usage{
\S4method{$}{chronology}(x, name)

\S4method{$}{chronologies}(x, name)

\S4method{$}{collunit}(x, name)

\S4method{$}{collunits}(x, name)

\S4method{$}{contact}(x, name)

\S4method{$}{contacts}(x, name)

\S4method{$}{dataset}(x, name)

\S4method{$}{datasets}(x, name)

\S4method{$}{publication}(x, name)

\S4method{$}{samples}(x, name)

\S4method{$}{sample}(x, name)

\S4method{$}{site}(x, name)

\S4method{$}{sites}(x, name)

\S4method{$}{speleothem}(x, name)

\S4method{$}{speleothems}(x, name)

\S4method{$}{taxon}(x, name)

\S4method{$}{taxa}(x, name)
}
\arguments{
\item{x}{\code{neotoma2} object}

\item{name}{name of the slot}
}
\value{
value at chosen slot in the \code{site} object
}
\description{
Obtain slots of a \code{neotoma2} without using at-mark
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_table.R
\name{get_table}
\alias{get_table}
\title{Get table record from Neotoma}
\usage{
get_table(x, limit = 25, offset = 0)
}
\arguments{
\item{x}{Table name (consult \url{https://open.neotomadb.org/dbschema/}
for a complete list of table names.}

\item{limit}{Default 25 records}

\item{offset}{Default 0.}
}
\value{
selected \code{table} values from the Database.
}
\description{
Call Neotoma and return a table
(with limits & offsets for large tables)
}
\examples{
\dontrun{
# Returns only the first 25 specimen records.
someSpec <- get_table('specimens')
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{dataset-class}
\alias{dataset-class}
\alias{datasets-class}
\title{S4 class for \code{datasets} information}
\value{
object of class \code{datasets}
}
\description{
The standard object class for \code{datasets}
from the Neotoma Paleoecology Database.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{chronologies_classes}
\alias{chronologies_classes}
\alias{chronology-class}
\alias{chronologies-class}
\title{S4 class for \code{chronologies} information}
\value{
object of class \code{chronologies}
}
\description{
The class for chronologies from the
Neotoma Paleoecology Database. A single collection unit may
have one or more chronology. The individual chronology
classes are grouped into an S4 \code{chronologies} class.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{specimens_classes}
\alias{specimens_classes}
\alias{specimen-class}
\alias{specimens-class}
\title{S4 class for \code{specimens} information}
\value{
object of class \code{specimens}
}
\description{
The standard object class for \code{specimens}
from the Neotoma Paleoecology Database.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getids.r
\name{getids}
\alias{getids}
\alias{getids.sites}
\alias{getids.site}
\alias{getids.collunits}
\alias{getids.collunit}
\title{Get object IDs}
\usage{
getids(x, order = TRUE)

\method{getids}{sites}(x, order = TRUE)

\method{getids}{site}(x, order = TRUE)

\method{getids}{collunits}(x, order = TRUE)

\method{getids}{collunit}(x, order = TRUE)
}
\arguments{
\item{x}{A Neotoma2 \code{sites} or \code{collunits} object.}

\item{order}{sort items by \code{siteid}, \code{collunitid}, \code{datasetid}}
}
\value{
\code{data.frame} containing \code{siteid}, \code{datasetid}, and \code{collunitid}
}
\description{
This function parses a site object, from \code{site} to
\code{dataset} level and returns a \code{data.frame} that contains the
site, collectionunit and dataset IDs for each element within the site.
}
\examples{
\dontrun{
marion <- get_sites(sitename = "Marion Lake")
collunitids <- getids(collunits(marion))
}
}
\author{
Simon Goring \email{goring@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{publications_classes}
\alias{publications_classes}
\alias{publication-class}
\alias{publications-class}
\title{An S4 class for Neotoma \code{publications}.}
\value{
object of class \code{publication}
}
\description{
A publication is liked to an individual Neotoma dataset object
They are grouped using an S4 \code{publications} class.
This \code{publications} class allows a single dataset to have one
or more publication classes associated with it.
}
\examples{
{
simon <- new("contact", familyname = "Goring", givennames = "Simon J.")
socorro <- new("contact", familyname = "Dominguez", givennames = "Socorro")
first_author <- new("author", author = simon, order = 1)
second_author <- new("author", author = socorro, order = 2)
author_list <- new("authors", authors = list(first_author, second_author))
pub <- new("publication",
           articletitle = "Top or bottom: Best toast spreading surfaces.",
           journal = "Peanut Butter Science",
           year = "2022",
           volume = "2",
           author = author_list)
           }
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R, R/speleothemdetails.R
\name{speleothemdetails}
\alias{speleothemdetails}
\alias{speleothemdetails,site-method}
\alias{speleothemdetails,collunits-method}
\alias{speleothemdetails,collunit-method}
\title{Obtain speleothems and samples from a record or multiple records.}
\usage{
speleothemdetails(x)

\S4method{speleothemdetails}{site}(x)

\S4method{speleothemdetails}{collunits}(x)

\S4method{speleothemdetails}{collunit}(x)
}
\arguments{
\item{x}{sites object}
}
\value{
data.frame with record information at speleothem level
}
\description{
Obtain speleothems and samples from a record or multiple records.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chronology-methods.R, R/collunits-methods.R,
%   R/dataset-methods.R, R/site-methods.R, R/speleothem-methods.R,
%   R/taxon-methods.R
\name{$<-,chronology-method}
\alias{$<-,chronology-method}
\alias{$<-,collunit-method}
\alias{cash-set,collunit-method}
\alias{$<-,dataset-method}
\alias{cash-set,dataset-method}
\alias{cash-set}
\alias{$<-,site-method}
\alias{$<-,speleothem-method}
\alias{cash-set,speleothem-method}
\alias{$<-,taxon-method}
\alias{cash-set,taxon-method}
\title{cash-set}
\usage{
\S4method{$}{chronology}(x, name) <- value

\S4method{$}{collunit}(x, name) <- value

\S4method{$}{dataset}(x, name) <- value

\S4method{$}{site}(x, name) <- value

\S4method{$}{speleothem}(x, name) <- value

\S4method{$}{taxon}(x, name) <- value
}
\arguments{
\item{x}{A \code{neotoma2} object.}

\item{name}{name of the slot}

\item{value}{The value to be used.}
}
\value{
\code{neotoma2} object with reassigned values
}
\description{
cash-set
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/speleothems.R
\name{speleothems,sites-method}
\alias{speleothems,sites-method}
\title{speleothems}
\usage{
\S4method{speleothems}{sites}(x)
}
\arguments{
\item{x}{sites object}
}
\value{
\code{data.frame} with sample records
}
\description{
Obtain all speleothems within a sites object
Experimental function: API and behavior may change.
}
\examples{
\dontrun{
ds <- get_datasets(37302)
sp <- speleothems(ds)
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/01_classDefinitions.R
\docType{class}
\name{repositories_classes}
\alias{repositories_classes}
\alias{repository-class}
\alias{repositories-class}
\title{S4 class for \code{repository} information}
\value{
object of class \code{repository}
}
\description{
The standard object class for \code{repository}
from the Neotoma Paleoecology Database.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_datasets.R
\name{get_datasets}
\alias{get_datasets}
\alias{get_datasets.numeric}
\alias{get_datasets.default}
\alias{get_datasets.sites}
\alias{get_datasets.site}
\alias{get_datasets.NULL}
\title{get_datasets}
\usage{
get_datasets(x = NA, ...)

\method{get_datasets}{numeric}(x, ...)

\method{get_datasets}{default}(x, ...)

\method{get_datasets}{sites}(x, ...)

\method{get_datasets}{site}(x, ...)

\method{get_datasets}{`NULL`}(x, ...)
}
\arguments{
\item{x}{A single datasetid, or a vector of unique dataset ids.}

\item{...}{accepted arguments, see details for more information.}
}
\value{
The function returns either a single item of class \code{"try-error"}
describing the reason for failure (either misdefined parameters or an error
from the Neotoma API), or list of site objects, each containing one or more
\code{collunit} objects, with fully populated \code{datasets} elements.
}
\description{
The \code{get_datasets()} function is a wrapper for the Neotoma \code{datasets} API
endpoint.
The function takes parameters defined by the user and returns dataset
information supplied by the Neotoma Paleoecological Database.
The user may define all or none of the possible fields.
}
\details{
A \code{dataset} is an element nested within \code{neotoma2} site objects. The
\code{get_datasets()} call returns a list of individual \code{site} objects with
\code{collunits} (collection units) that contain valid, matching \code{dataset}
elements.
So, \code{get_sites()} returns only site metadata. \code{get_datasets()} returns
site metadata, plus metadata about the individual datasets present at that
site.
The \code{get_datasets()} function searches for each site within Neotoma
that matches the query parameters, and returns them as a \code{sites} object,
a list of \code{site} objects, plus returns all the additional metadata for
the datasets at that site.
The \code{get_datasets()} command wraps the Neotoma API
(\href{https://api.neotomadb.org}{api.neotomadb.org}) call for \code{datasets}.
The call itself uses a SQL query which accepts any one of the following
parameters:
\itemize{
\item \code{siteid}  The unique site ID (integer) in Neotoma. Can be passed as a
vector of site IDs.
\item \code{sitename}  The site name, or approximate match using the \% wildcard.
\item \code{database}  The constituent database for the record. See
\code{get_table("constituentdatabases")}
\item \code{datasettype} Neotoma contains data for a number of dataset types.
This returns a subset of data types. For a complete list of available
dataset types, run \code{neotoma2::get_table('datasettypes')}
\item \code{altmin}  The minimum altitude range for site elevation (in meters).
\item \code{altmax}  The maximum altitude range for site elevation (in meters).
\item \code{datasetid}  The unique dataset ID (integer) in Neotoma. Can be passed
as a vector of dataset IDs.
\item \code{doi}  The dataset DOI for a dataset contained within a site. Can be
passed as a vector of DOIs.
\item \code{gpid}  The geopolitical name or identifier containing a site. Can be
passed as a vector of names.
\item \code{keywords}  Keywords for samples within a set of sites. For example
"modern" indicates a sample within the record uses the keyword "modern".
\item \code{contacts}  Contact names or IDs associated with a site.
\item \code{ageyoung}  A minimum spanning age for the record, in years before
radiocarbon present (1950).
\item \code{ageold}  A maximum spanning age for the record, in years before
radiocarbon present (1950).
\item \code{ageof}  An age which must be contained within the range of sample ages
for a site.
\item \code{taxa}  The names of taxa which must be present within samples in a
record.
\item \code{all_data} The API only downloads the first 25 records of the query.
For the complete records, use \code{all_data=TRUE}
}
}
\examples{
\dontrun{
tryCatch({
  random_sites <- get_sites(1)
  allds <- get_datasets(random_sites, limit=3)
}, error = function(e) {
   message("Neotoma server not responding. Try again later.")
})
# To find all datasets with a min altitude of 12 and a max altitude of 25:
tryCatch({
  sites_12to25 <- get_datasets(altmin=12, altmax=25)
}, error = function(e) {
  message("Neotoma server not responding. Try again later.")
})
# To find all datasets in Brazil
brazil <- '{"type": "Polygon",
"coordinates": [[
 [-73.125, -9.102096738726443],
 [-56.953125,-33.137551192346145],
 [-36.5625,-7.710991655433217],
 [-68.203125,13.923403897723347],
 [-73.125,-9.102096738726443]]]}'
tryCatch({
  brazil_datasets <- get_datasets(loc = brazil[1], limit=2)
}, error = function(e) {
  message("Neotoma server not responding. Try again later.")
})
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/site-methods.R
\name{doi,sites-method}
\alias{doi,sites-method}
\title{Obtain dataset DOIs from records.}
\usage{
\S4method{doi}{sites}(x)
}
\arguments{
\item{x}{a Neotoma2 \code{site} object}
}
\value{
\code{data.frame} object with DOIs information.
}
\description{
Given complete dataset objects in Neotoma (must have used
\code{get_datasets()} or \code{get_downloads()}), return the dataset
DOI for the record.
}
\examples{
\dontrun{
# Get datasets metadata from API and retrieve DOIs
ds <- get_datasets(1)
doi(ds)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_manual.R
\name{get_manual}
\alias{get_manual}
\title{get_manual}
\usage{
get_manual()
}
\value{
NULL side effect for opening browser with the manual
}
\description{
Open up the Neotoma manual homepage.
}
\examples{
\dontrun{
# This call does not work from `source()` calls or in testing.
# interactive() just lets us know you are interacting with the console:
if (interactive()) {
 get_manual()
}
}
}
\author{
Simon Goring \email{goring@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subset-methods.R
\name{datasets,collunits-method}
\alias{datasets,collunits-method}
\title{Extract \code{datasets} from a \code{sites} or \code{collunits} object.}
\usage{
\S4method{datasets}{collunits}(object)
}
\arguments{
\item{object}{A \code{sites} or\code{collunits} object}
}
\value{
\code{datasets} object
}
\description{
Extract \code{datasets} from a \code{sites} or \code{collunits} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_downloads.R
\name{get_downloads}
\alias{get_downloads}
\alias{get_downloads.numeric}
\alias{get_downloads.sites}
\alias{get_downloads.NULL}
\title{get_downloads}
\usage{
get_downloads(x = NA, ...)

\method{get_downloads}{numeric}(x, ...)

\method{get_downloads}{sites}(x, verbose = TRUE, ...)

\method{get_downloads}{`NULL`}(x, ...)
}
\arguments{
\item{x}{Use a single number to extract site information}

\item{...}{accepted arguments: sites, datasets}

\item{verbose}{Status bar of items being downloaded}
}
\value{
The function returns either a single item of class
\code{"try-error"} describing the reason for failure
(either misdefined parameters or an error from the Neotoma API),
or a table of sites, with rows corresponding to the number of
individual sites and datasets returned by the Neotoma API.
Each "site" object contains 6 parameters that can be accessed as well:
\item{ \code{siteid} }{site ID number}
\item{ \code{sitename} }{site's name}
\item{ \code{location} }{sf object that describes site's location}
\item{ \code{description} }{}
\item{ \code{collunits} }{limited information on collunits}
Each "collection unit" embedded in the "sites" object contains
6 parameters that can be accessed as well:
\item{ \code{collunitid}}{collection unit ID number}
\item{ \code{handle} }{collection unit's handle}
\item{ \code{collunitname} }{collection unit's name}
\item{ \code{colldate} }{date in collection unit}
\item{ \code{substrate} }{substrate}
\item{ \code{location} }{sf object that describes site's location}
\item{ \code{datasets} }{detailed information regarding dataset}
Each "dataset" nested in the "collection unit" contains the
following detail of information:
\item{ \code{datasetid} }{dataset ID number}
\item{ \code{datasetname} }{site's name}
\item{ \code{datasettype} }{type of data found}
\item{ \code{location} }{sf object that describes site's location}
\item{ \code{notes} }{notes on the dataset}
\item{ \code{taxa table} }{taxa table}
\item{ \code{pi list} }{P.I. info}
\item{ \code{analyst} }{analyst info}
\item{ \code{metadata} }{dataset metadata}
}
\description{
Download \code{sites} objects up to datum and chronology detail level.
}
\details{
The \code{get_downloads()} command wraps the Neotoma API
(\href{https://api.neotomadb.org}{api.neotomadb.org}) call for \code{downloads}.
The call itself uses a SQL query which accepts any one of the following
parameters:
\itemize{
\item \code{datasetid} The unique dataset ID (integer) in Neotoma. Can be passed
as a vector of dataset IDs.
\item \code{all_data} The API only downloads the first 25 records of the query.
For the complete records, use \code{all_data=TRUE}
}
}
\examples{
\dontrun{
# To find the downloads object of dataset 24:
tryCatch({
  downloads24 <- get_downloads(24)
}, error = function(e) {
 message("Neotoma server not responding. Try again later.")
})
# To find all downloads in Brazil
brazil <- '{"type": "Polygon",
"coordinates": [[
 [-73.125, -9.102096738726443],
 [-56.953125,-33.137551192346145],
 [-36.5625,-7.710991655433217],
 [-68.203125,13.923403897723347],
 [-73.125,-9.102096738726443]]]}'
tryCatch({
  brazil_datasets <- get_datasets(loc = brazil[1])
  brazil_downloads <- get_downloads(brazil_datasets)
}, error = function(e) {
  message("Neotoma server not responding. Try again later.")
})
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R, R/taxa.R
\name{taxa}
\alias{taxa}
\alias{taxa,site-method}
\alias{taxa,collunits-method}
\alias{taxa,collunit-method}
\title{taxa}
\usage{
taxa(object)

\S4method{taxa}{site}(object)

\S4method{taxa}{collunits}(object)

\S4method{taxa}{collunit}(object)
}
\arguments{
\item{object}{Sites object to extract taxa table from}
}
\value{
data.frame with taxa records
}
\description{
Show the samples table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chronology-methods.R, R/collunits-methods.R,
%   R/contact-methods.R, R/dataset-methods.R, R/publication-methods.R,
%   R/samples-methods.R, R/site-methods.R, R/speleothem-methods.R,
%   R/taxon-methods.R
\name{length,chronologies-method}
\alias{length,chronologies-method}
\alias{length,collunits-method}
\alias{length,contacts-method}
\alias{length,datasets-method}
\alias{length,publications-method}
\alias{length,samples-method}
\alias{length}
\alias{length,sites-method}
\alias{length,speleothems-method}
\alias{length,taxa-method}
\title{length}
\usage{
\S4method{length}{chronologies}(x)

\S4method{length}{collunits}(x)

\S4method{length}{contacts}(x)

\S4method{length}{datasets}(x)

\S4method{length}{publications}(x)

\S4method{length}{samples}(x)

\S4method{length}{sites}(x)

\S4method{length}{speleothems}(x)

\S4method{length}{taxa}(x)
}
\arguments{
\item{x}{\code{neotoma2} object}
}
\value{
\code{int} representing length of a \code{neotoma2} object
}
\description{
length
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/site-methods.R
\name{coordinates,sites-method}
\alias{coordinates,sites-method}
\title{Return the latitude and longitude of sites}
\usage{
\S4method{coordinates}{sites}(obj, ...)
}
\arguments{
\item{obj}{A sites object}

\item{...}{Additional parameters associated with the call.}
}
\value{
\code{data.frame} object with site coordinates.
}
\description{
Return the latitude and longitude of sites
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_speleothems.R
\name{get_speleothems}
\alias{get_speleothems}
\alias{get_speleothems.numeric}
\alias{get_speleothems.sites}
\title{get_speleothems}
\usage{
get_speleothems(x = NA, ...)

\method{get_speleothems}{numeric}(x, ...)

\method{get_speleothems}{sites}(x, ...)
}
\arguments{
\item{x}{A dataset ID or vector of dataset IDs}

\item{...}{accepted arguments}
}
\value{
\code{sites} object with speleothem data
}
\description{
get_speleothems
}
\details{
Experimental function: API and behavior may change.
The \code{get_speleothems()} command wraps the Neotoma API
(\href{https://api.neotomadb.org}{api.neotomadb.org}) call for \code{speleothems}.
The call itself uses a SQL query which accepts any one of the following
parameters:
\itemize{
\item \code{x}  The unique dataset ID (integer) in Neotoma. Can be passed as a
vector of dataset IDs.
\item \code{sites}  A \code{sites} R object.
}
}
\examples{
\dontrun{
## Find speleothems by numeric datasetid:
speleo <- get_speleothems(c(2,5))
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chronology-methods.R, R/collunits-methods.R,
%   R/contact-methods.R, R/dataset-methods.R, R/publication-methods.R,
%   R/samples-methods.R, R/site-methods.R, R/speleothem-methods.R,
%   R/taxon-methods.R
\name{[[,chronologies,numeric-method}
\alias{[[,chronologies,numeric-method}
\alias{[[,collunits,numeric-method}
\alias{sub-sub,collunits-method}
\alias{[[,contacts,numeric-method}
\alias{sub-sub,contacts-method}
\alias{[[,datasets,numeric-method}
\alias{sub-sub,datasets-method}
\alias{[[,publications,numeric-method}
\alias{sub-sub,publications-method}
\alias{[[,samples,numeric-method}
\alias{sub-sub,samples-method}
\alias{sub-sub}
\alias{[[,sites,numeric-method}
\alias{[[,speleothems,numeric-method}
\alias{sub-sub,speleothems-method}
\alias{[[,taxa,numeric-method}
\alias{sub-sub,taxa-method}
\title{sub-sub}
\usage{
\S4method{[[}{chronologies,numeric}(x, i)

\S4method{[[}{collunits,numeric}(x, i)

\S4method{[[}{contacts,numeric}(x, i)

\S4method{[[}{datasets,numeric}(x, i)

\S4method{[[}{publications,numeric}(x, i)

\S4method{[[}{samples,numeric}(x, i)

\S4method{[[}{sites,numeric}(x, i)

\S4method{[[}{speleothems,numeric}(x, i)

\S4method{[[}{taxa,numeric}(x, i)
}
\arguments{
\item{x}{Neotoma2 nested object}

\item{i}{iteration in nested list}
}
\value{
sliced \code{site} object
}
\description{
Obtain one of the elements within a \code{sites},
\code{collectionunits}, \code{datasets}, etc... Neotoma objects.
}
\examples{
\dontrun{
some_site <- get_sites(sitename = "Site\%", limit=3)
some_site[[2]]
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/filter.R
\name{filter}
\alias{filter}
\alias{filter.NULL}
\alias{filter.sites}
\alias{filter.data.frame}
\title{Apply a \code{filter} for Neotoma sites objects.}
\usage{
filter(.data, ..., .by = NULL, .preserve = FALSE)

\method{filter}{`NULL`}(.data, ...)

\method{filter}{sites}(.data, ...)

\method{filter}{data.frame}(.data, ..., .by = NULL, .preserve = FALSE)
}
\arguments{
\item{.data}{A site, dataset, download, or data frame}

\item{...}{Additional arguments passed to \code{filter()}}

\item{.by}{(only used for filtering \code{data.frame} objects)}

\item{.preserve}{(only used for filtering \code{data.frame} objects)}
}
\value{
filtered \code{sites} object
}
\description{
The \code{filter} function takes a \code{sites} object
and allows a user to filter on a number of properties. Since a sites object
is a nested object (it contains collection units, datasets, samples, etc.)
the degree to which filtering occurs depends on the amount of data contained
within the sites object. Filtering parameters include:
\itemize{
\item \code{siteid} A numeric site identifier from the Neotoma Database.
\item \code{sitename} The character string sitename.
\item \code{lat} A numeric latitude value.
\item \code{long} A numeric longitude value.
\item \code{altitude} The elevation of the site. Note that some sites do not
include elevation information. For these an NA value appears, and they
would be removed when using an elevation filter.
\item \code{datasetid} A numeric datasetid from Neotoma.
\item \code{database} A character string naming the constituent database
from which the dataset is drawn.
\item \code{datasettype} A character string representing one of the many
dataset types within Neotoma.
\item \code{age_range_old} A dataset-level parameter indicating the oldest
date covered by the dataset chronology.
\item \code{age_range_young} A dataset-level parameter indicating the youngest
date covered by the dataset chronology.
\item \code{notes} Free-form dataset notes provided by the dataset PI(s),
analysts or data stewards.
\item \code{collectionunitid} A numeric collection unit identifier from
Neotoma.
\item \code{handle} A character string identifying the collection unit. These
are often shorter form names (originally a default 8 character length).
\item \code{collectionunitname} A character string identifying the collection
unit name.
\item \code{colldate} The date on which the collection unit was sampled. Many
of these are empty.
\item \code{location} A free-form character string indicating the location of
the collection unit within the site.
\item \code{waterdepth} A numeric depth at which the core was obtained.
\item \code{collunittype} A character string for the collection unit type.
\item \code{collectiondevice} A fixed vocabulary term for the collection
device.
\item \code{depositionalenvironment} A fixed vocabulary name for the
depositional environment.
}
}
\examples{
\dontrun{
# Download 10 sites, but only keep the sites that are close to sea level.
tryCatch({
 some_sites <- get_sites(sitename = "Lake\%", limit = 3)
  site_subset <- some_sites \%>\% filter(altitude < 100)
}, error = function(e) {
  message("Neotoma server not responding. Try again later.")
})
# Download 10 sites, get all associated datasets, but keep only
# sites/datasets that are of datasettype "pollen":
tryCatch({
  sites <- get_sites(limit = 10) \%>\%
    get_datasets()
  pollen_subset <- sites \%>\% filter(datasettype == "pollen")
}, error = function(e) {
  message("Neotoma server not responding. Try again later.") 
})
}
}
\author{
Simon Goring \email{goring@wisc.edu}

Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chronology-methods.R, R/collunits-methods.R,
%   R/contact-methods.R, R/dataset-methods.R, R/publication-methods.R,
%   R/site-methods.R, R/speleothem-methods.R, R/taxon-methods.R
\name{as.data.frame,chronology-method}
\alias{as.data.frame,chronology-method}
\alias{as.data.frame,chronologies-method}
\alias{as.data.frame,collunit-method}
\alias{as.data.frame,collunits-method}
\alias{as.data.frame,contact-method}
\alias{as.data.frame,contacts-method}
\alias{as.data.frame,dataset-method}
\alias{as.data.frame,datasets-method}
\alias{as.data.frame,authors-method}
\alias{as.data.frame,publication-method}
\alias{as.data.frame,publications-method}
\alias{as.data.frame}
\alias{as.data.frame,site-method}
\alias{as.data.frame,sites-method}
\alias{as.data.frame,speleothem-method}
\alias{as.data.frame,speleothems-method}
\alias{as.data.frame,taxon-method}
\alias{as.data.frame,taxa-method}
\title{as.data.frame}
\usage{
\S4method{as.data.frame}{chronology}(x)

\S4method{as.data.frame}{chronologies}(x)

\S4method{as.data.frame}{collunit}(x)

\S4method{as.data.frame}{collunits}(x)

\S4method{as.data.frame}{contact}(x)

\S4method{as.data.frame}{contacts}(x)

\S4method{as.data.frame}{dataset}(x)

\S4method{as.data.frame}{datasets}(x)

\S4method{as.data.frame}{authors}(x)

\S4method{as.data.frame}{publication}(x)

\S4method{as.data.frame}{publications}(x)

\S4method{as.data.frame}{site}(x)

\S4method{as.data.frame}{sites}(x)

\S4method{as.data.frame}{speleothem}(x)

\S4method{as.data.frame}{speleothems}(x)

\S4method{as.data.frame}{taxon}(x)

\S4method{as.data.frame}{taxa}(x)
}
\arguments{
\item{x}{\code{neotoma2} object}
}
\value{
\code{data.frame} object
}
\description{
Returns \code{neotoma2} object's data as a \code{data.frame}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_contact.R
\name{set_contact}
\alias{set_contact}
\title{Set contact information for a new record.}
\usage{
set_contact(
  x = NA,
  contactid = NA_integer_,
  familyname = NA_character_,
  leadinginitials = NA_character_,
  givennames = NA_character_,
  contactname = NA_character_,
  suffix = NA_character_,
  ORCID = NA_character_,
  title = NA_character_,
  institution = NA_character_,
  email = NA_character_,
  phone = NA_character_,
  contactstatus = NA_character_,
  fax = NA_character_,
  url = NA_character_,
  address = NA_character_,
  notes = NA_character_
)
}
\arguments{
\item{x}{Object to be set as a contact}

\item{contactid}{An arbitrary Contact identification number.}

\item{familyname}{Family or surname name of a person.}

\item{leadinginitials}{Leading initials for given or forenames
without spaces (e.g. \code{G.G.}).}

\item{givennames}{Given or forenames of a person (e.g.
\code{George Gaylord}). Initials with spaces are used if full
given names are not known (e.g. \code{G. G}).}

\item{contactname}{Full name of a person (e.g. \code{George Gaylord Simpson}).}

\item{suffix}{Suffix of a person’s name (e.g. \code{Jr.}, \code{III}).}

\item{ORCID}{A unique ORCID (see \url{https://orcid.org}).}

\item{title}{A person’s title (e.g. \code{Dr.}, \code{Prof.},
\code{Prof. Dr}).}

\item{institution}{The institution where an individual works.}

\item{email}{An individuals email address}

\item{phone}{Their phone number}

\item{contactstatus}{Are they "active" or "retired"?}

\item{fax}{Do people still use fax machines?}

\item{url}{Their homepage}

\item{address}{A physical address}

\item{notes}{Notes about the individual}
}
\value{
\code{contact} object
}
\description{
Create a new contact for a record.
Within Neotoma all chronologies have unique numeric identifiers.
Within R, because of the need to use the indentifiers across objects,
and because we want to avoid conflicts between naming systems, a
universally unique identifier (UUID) is created for the object ID.This is not
to be updated to the database.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/collunits-methods.R, R/contact-methods.R,
%   R/dataset-methods.R, R/publication-methods.R, R/site-methods.R,
%   R/speleothem-methods.R, R/taxon-methods.R
\name{names,collunit-method}
\alias{names,collunit-method}
\alias{names,contact-method}
\alias{names,dataset-method}
\alias{names,publication-method}
\alias{names,publications-method}
\alias{names}
\alias{names,site-method}
\alias{names,sites-method}
\alias{names,speleothem-method}
\alias{names,taxon-method}
\title{Get a \code{neotoma2} object's slot names}
\usage{
\S4method{names}{collunit}(x)

\S4method{names}{contact}(x)

\S4method{names}{dataset}(x)

\S4method{names}{publication}(x)

\S4method{names}{publications}(x)

\S4method{names}{site}(x)

\S4method{names}{sites}(x)

\S4method{names}{speleothem}(x)

\S4method{names}{taxon}(x)
}
\arguments{
\item{x}{A \code{neotoma2} object.}
}
\value{
NULL.
}
\description{
Get all names for elements' slots within a \code{collunit} object.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/chronology-methods.R, R/collunits-methods.R,
%   R/contact-methods.R, R/dataset-methods.R, R/publication-methods.R,
%   R/samples-methods.R, R/site-methods.R, R/speleothem-methods.R,
%   R/taxon-methods.R
\name{c,chronologies-method}
\alias{c,chronologies-method}
\alias{c,collunits-method}
\alias{c,contacts-method}
\alias{c,contact-method}
\alias{c,datasets-method}
\alias{c,publications-method}
\alias{c,samples-method}
\alias{c}
\alias{c,sites-method}
\alias{c,missingOrNULL-method}
\alias{c,speleothems-method}
\alias{c,taxa-method}
\title{c - Combine \code{neotoma2} objects}
\usage{
\S4method{c}{chronologies}(x, y)

\S4method{c}{collunits}(x, y)

\S4method{c}{contacts}(x, y)

\S4method{c}{contact}(x, y)

\S4method{c}{datasets}(x, y)

\S4method{c}{publications}(x, y)

\S4method{c}{samples}(x, y)

\S4method{c}{sites}(x, y)

\S4method{c}{missingOrNULL}(x = "missingORNULL", y)

\S4method{c}{speleothems}(x, y)

\S4method{c}{taxa}(x, y)
}
\arguments{
\item{x}{\code{neotoma2} object or NULL}

\item{y}{\code{neotoma2} object or NULL}
}
\value{
concatenated and cleaned \code{sites} object
}
\description{
c - Combine \code{neotoma2} objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R
\name{chroncontrols}
\alias{chroncontrols}
\title{chroncontrols}
\usage{
chroncontrols(x)
}
\arguments{
\item{x}{Sites object to extract chroncontrols table from}
}
\value{
data.frame with chroncontrols information
}
\description{
Show the samples table
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_chronology.R
\name{set_chronology}
\alias{set_chronology}
\title{set chronology information for a new record.}
\usage{
set_chronology(
  x = NA,
  chronologyid = NA_integer_,
  notes = NA_character_,
  contact = list(),
  agemodel = NA_character_,
  ageboundolder = NA_integer_,
  ageboundyounger = NA_integer_,
  isdefault = NA_integer_,
  dateprepared = as.Date(character(0)),
  modelagetype = NA_character_,
  chronologyname = NA_character_,
  chroncontrols = data.frame(0)
)
}
\arguments{
\item{x}{Object to be set as a chronology}

\item{chronologyid}{An optional value. Will be assigned a
unique identifier if not provided.}

\item{notes}{Additional notes about the chronology. For more
modern models, often the function call to Bacon or Bchron is added here.}

\item{contact}{A contacts object, identifying the individual(s)
who created the chronology}

\item{agemodel}{A string representing the age model name, for
example "Crummy linear interpolation".}

\item{ageboundolder}{The \code{ageboundolder} is assigned the oldest sample
age rounded up to the nearest 10}

\item{ageboundyounger}{The \code{ageboundyounger} is assigned the oldest
sample age rounded up to the nearest 10}

\item{isdefault}{Defines whether the model is the default for
the collection unit for a particular model age type.}

\item{dateprepared}{The date at which the age model was prepared.}

\item{modelagetype}{The age type for the model. For validation, the models
should be one of the valid Neotoma \code{agetypes}:
\url{https://api.neotomadb.org/v2.0/data/dbtables?table=agetypes}}

\item{chronologyname}{A valid name for the chronology.}

\item{chroncontrols}{A data.frame containing the chronological controls
for the age model.}
}
\value{
\code{chronology} object
}
\description{
Create a new chronology for a record.
Within Neotoma all chronologies have unique numeric
identifiers. Within R, because of the need to use the
identifiers across objects, and because we want to avoid
conflicts between naming systems, a universally unique
identifier (UUID) is created for the object ID.

Function to create new chronology objects for
personal analysis.
The new object will not be uploaded to the database.
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}

Simon Goring \email{goring@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R, R/subset-methods.R
\name{collunits}
\alias{collunits}
\alias{collunits,site-method}
\title{Extract collection units from a sites object}
\usage{
collunits(object)

\S4method{collunits}{site}(object)
}
\arguments{
\item{object}{A sites object}
}
\value{
collunits detail from a sites object
}
\description{
Extract collection units from a sites object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/site-methods.R
\name{cite_data,sites-method}
\alias{cite_data,sites-method}
\title{Obtain data citations from multiple records.}
\usage{
\S4method{cite_data}{sites}(x)
}
\arguments{
\item{x}{sites object}
}
\value{
\code{data.frame} object with citation information.
}
\description{
Given complete dataset objects in Neotoma (must have used
\code{get_datasets()} or \code{get_downloads()}), return a formatted
citation for the record, including the dataset DOI.
}
\examples{
\dontrun{
# Get datasets metadata from API
ds <- get_datasets(1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/set_publication.R
\name{set_publication}
\alias{set_publication}
\title{Create a new publication (or publication set)}
\usage{
set_publication(
  x = NA,
  publicationid = NA_integer_,
  publicationtypeid = NA_integer_,
  publicationtype = NA_character_,
  year = NA_character_,
  citation = NA_character_,
  articletitle = NA_character_,
  journal = NA_character_,
  volume = NA_character_,
  issue = NA_character_,
  pages = NA_character_,
  citationnumber = NA_character_,
  doi = NA_character_,
  booktitle = NA_character_,
  numvolumes = NA_character_,
  edition = NA_character_,
  volumetitle = NA_character_,
  seriestitle = NA_character_,
  seriesvolume = NA_character_,
  publisher = NA_character_,
  url = NA_character_,
  city = NA_character_,
  state = NA_character_,
  country = NA_character_,
  originallanguage = NA_character_,
  notes = NA_character_,
  author = NULL
)
}
\arguments{
\item{x}{"publications" object to be updated.}

\item{publicationid}{ID of publication}

\item{publicationtypeid}{ID of kind of publication}

\item{publicationtype}{A text string identifying the publication
type within the Neotoma database.}

\item{year}{The year of publication.}

\item{citation}{A full text citation for the article.}

\item{articletitle}{The title of the article.}

\item{journal}{The journal in which the article was published.}

\item{volume}{The journal volume.}

\item{issue}{The journal issue.}

\item{pages}{The pages of the journal.}

\item{citationnumber}{How many times has the paper been cited?}

\item{doi}{A DOI for the record.}

\item{booktitle}{The title of the book (if the publication is a book)}

\item{numvolumes}{The number of book volumes (if a series)}

\item{edition}{The book edition.}

\item{volumetitle}{The title of the volume (in a published series)}

\item{seriestitle}{The title of the series.}

\item{seriesvolume}{The series volume.}

\item{publisher}{The publisher.}

\item{url}{Publication URL}

\item{city}{City of publication.}

\item{state}{State of publication.}

\item{country}{Country of publication.}

\item{originallanguage}{Original language of publication.}

\item{notes}{Publication notes.}

\item{author}{name of the author of publication.}
}
\value{
\code{publication} object
}
\description{
A function to create new publication objects by hand.
}
\author{
Simon Goring \email{goring@wisc.edu}

Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clean.R
\name{clean}
\alias{clean}
\alias{clean.sites}
\alias{clean.collunits}
\alias{clean.datasets}
\title{clean Neotoma objects to remove duplicates and empty objects.}
\usage{
clean(x, verbose = TRUE, ...)

\method{clean}{sites}(x, verbose = TRUE, ...)

\method{clean}{collunits}(x, verbose = TRUE, ...)

\method{clean}{datasets}(x, verbose = TRUE, ...)
}
\arguments{
\item{x}{sites, datasets, collunits that may have duplicates.}

\item{verbose}{parameter to prints out progress bar}

\item{...}{Additional parameters associated with the call.}
}
\value{
clean \code{neotoma} objects without duplicates after concatenation
}
\description{
Function that removes duplicate objects such as sites,
datasets, or collection units. When we pull in a large number of objects,
or overlapping searches, we can run into a problem where we have multiple
instances of the same site, but with different datasets. This function
attempts to gather all objects together:
\itemize{
\item Before: \{site: 1, dataset: 1\}, \{site: 1, dataset: 2\}
\item After: \{site: 1, dataset: [1, 2]\}
So the site is gathered, and the datasets are now part of an
array of datasets.
}
}
\examples{
\dontrun{
tryCatch({
  alex <- get_sites(sitename = "Alex\%")
  alex2 <- get_sites(24)
  c <- c(alex, alex2) #uncleaned
}, error = function(e) {
  message("Neotoma server not responding. Try again later.")
})
}
}
\author{
Simon Goring \email{goring@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/02_genericDefinitions.R, R/plotLeaflet.R
\name{plotLeaflet}
\alias{plotLeaflet}
\alias{plotLeaflet,site-method}
\alias{plotLeaflet,ANY-method}
\title{plotLeaflet}
\usage{
plotLeaflet(object)

\S4method{plotLeaflet}{site}(object)

\S4method{plotLeaflet}{ANY}(object)
}
\arguments{
\item{object}{Sites object to plot}
}
\value{
leaflet map with site markers
}
\description{
Plot sites on a leaflet map
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/toWide.R
\name{toWide}
\alias{toWide}
\title{toWide}
\usage{
toWide(
  x,
  variablenames = NA,
  ecologicalgroups = NA,
  elementtypes = NA,
  unit = NA,
  groupby = "age",
  operation = "prop"
)
}
\arguments{
\item{x}{dataframe object with samples}

\item{variablenames}{Optional vector to filter by specific variable names.}

\item{ecologicalgroups}{Vector stating the ecological groups to be
filtered by, e.g "DIAT", "TRSH"}

\item{elementtypes}{Label of element type to filter by,
e.g. "pollen", "valve"}

\item{unit}{Label stating which units to filter by, e.g. "NISP"}

\item{groupby}{Group by 'age' or 'depth'}

\item{operation}{label or vector of operations to be chosen from:
'prop', 'sum', 'presence'.}
}
\value{
wide \code{data.frame} obtained from long \code{samples} \code{data.frame}
}
\description{
Obtain a wide table with information regarding of
samples grouped by variablename and depth/age.
}
\examples{
\dontrun{
fc_sites <- neotoma2::get_datasets(limit=5, datasettype = "vertebrate fauna")
fc_ds <- fc_sites \%>\%
   neotoma2::get_downloads()
fc_dl1 <- fc_dl[[1]]
fc_smp <- samples(fc_dl1)
toWide(fc_smp, ecologicalgroups=c('AVES', 'RODE'),
       elementtypes='bone/tooth', unit='present/absent')
}
}
\author{
Socorro Dominguez \email{dominguezvid@wisc.edu}
}
